#include "integ.h"

/* Table of constant values */

static int c__1 = 1;
static int c__0 = 0;
static int c_false = FALSE;
static int c__2 = 2;

static int nsp_ode_ixsav (int *ipar, int *ivalue, int *iset);

/*
 *  Write error message with values. 
 * AUTHOR  Hindmarsh, Alan C., (LLNL) 
 *
 * 
 * Subroutines XERRWD, XSETF1, XSETUN1, and the function routine IXSAV, 
 * as given here, constitute a simplified version of the SLATEC error 
 * handling package. 
 * 
 * All arguments are input arguments. 
 * 
 * MSG    = The message (character array). 
 * NMES   = The length of MSG (number of characters). 
 * NERR   = The error number (not used). 
 * LEVEL  = The error level.. 
 *          0 or 1 means recoverable (control returns to caller). 
 *          2 means fatal (run is aborted--see note below). 
 * NI     = Number of ints (0, 1, or 2) to be printed with message. 
 * I1,I2  = Ints to be printed, depending on NI. 
 * NR     = Number of reals (0, 1, or 2) to be printed with message. 
 * R1,R2  = Reals to be printed, depending on NR. 
 * 
 * Note..  this routine is machine-dependent and specialized for use 
 * in limited context, in the following ways.. 
 * 1. The argument MSG is assumed to be of type CHARACTER, and 
 *    the message is printed with a format of (1X,A). 
 * 2. The message is assumed to take only one line. 
 *    Multi-line messages are generated by repeated calls. 
 * 3. If LEVEL = 2, control passes to the statement   STOP 
 *    to abort the run.  This statement may be machine-dependent. 
 * 4. R1 and R2 are assumed to be in double precision and are printed 
 *    in D21.13 format. 
 * 
 * 
 *For a different default int unit number, IXSAV (or a subsidiary 
 *routine that it calls) will need to be modified. 
 *For a different run-abort command, change the statement following 
 *statement 100 at the end. 
 *----------------------------------------------------------------------- 
 *Subroutines called by XERRWD.. None 
 *Function routine called by XERRWD.. IXSAV 
 *----------------------------------------------------------------------- 
 */

int nsp_ode_xerrwd (char *msg, int *nmes, int *nerr, int *level, int *ni,
		       int *i1, int *i2, int *nr, double *r1, double *r2,
		       long int msg_len)
{
  int mesflg;
  /* int lunit = nsp_ode_ixsav (&c__1, &c__0, &c_false); */
  mesflg = nsp_ode_ixsav (&c__2, &c__0, &c_false);
  if (mesflg == 0)
    {
      goto L100;
    }
  
  /* 
   * Write the message. 
   * 
   */
  Sciprintf("%s\n",msg);
  if (*ni == 1)
    {
      Sciprintf("In above message,  I1 =%d)\n",*i1);
    }
  if (*ni == 2)
    {
      Sciprintf("In above message,  I1 =%d, I2=%d)\n",*i1,*i2);
    }
  if (*nr == 1)
    {
      Sciprintf("In above message,  R1 =%f)\n",*r1);
    }
  if (*nr == 2)
    {
      Sciprintf("In above message,  R1 =%f, R2=%f)\n",*r1,*r2);
    }
  /* 
   * Abort the run if LEVEL = 2. 
   * 
   */
 L100:
  if (*level != 2)
    {
      return 0;
    }
  /* XXX 
     s_stop ("", 0L);
  */

  return 0;
}


/*
 * Reset the error print control flag. 
 * 
 *  XSETF1 sets the error print control flag to MFLAG: 
 *     MFLAG=1 means print all messages (the default). 
 *     MFLAG=0 means no printing. 
 * 
 */

int nsp_ode_xsetf1 (int *mflag)
{
  if (*mflag == 0 || *mflag == 1)
    {
      int c_true = TRUE; 
      nsp_ode_ixsav (&c__2, mflag, &c_true);
    }
  return 0;
}


/*
 * Reset the int unit number for error messages. 
 * AUTHOR  Hindmarsh, Alan C., (LLNL) 
 */

int nsp_ode_xsetun1 (int *lun)
{
  int c_true = TRUE; 
  if (*lun > 0)
    {
      nsp_ode_ixsav (&c__1, lun, &c_true);
    }
  return 0;
}

/*
 * Save and recall error message control parameters. 
 * AUTHOR  Hindmarsh, Alan C., (LLNL) 
 * 
 * IXSAV saves and recalls one of two error message parameters: 
 *   LUNIT, the int unit number to which messages are printed, and 
 *   MESFLG, the message print flag. 
 * This is a modification of the SLATEC library routine J4SAVE. 
 * 
 * Saved local variables.. 
 *  LUNIT  = Int unit number for messages. 
 *  LUNDEF = Default int unit number, data-loaded to 6 below 
 *           (may be machine-dependent). 
 *  MESFLG = Print control flag.. 
 *           1 means print all messages (the default). 
 *           0 means no printing. 
 * 
 * On input.. 
 *   IPAR   = Parameter indicator (1 for LUNIT, 2 for MESFLG). 
 *   IVALUE = The value to be set for the parameter, if ISET = .TRUE. 
 *   ISET   = Int flag to indicate whether to read or write. 
 *            If ISET = .TRUE., the parameter will be given 
 *            the value IVALUE.  If ISET = .FALSE., the parameter 
 *            will be unchanged, and IVALUE is a dummy argument. 
 * 
 * On return.. 
 *   IXSAV = The (old) value of the parameter. 
 * 
 */

static int nsp_ode_ixsav (int *ipar, int *ivalue, int *iset)
{
  static int lunit = -1;
  static int lundef = 6;
  static int mesflg = 1;
  /* System generated locals */
  int ret_val=0;
  if (*ipar == 1)
    {
      if (lunit == -1)
	{
	  lunit = lundef;
	}
      ret_val = lunit;
      if (*iset)
	{
	  lunit = *ivalue;
	}
    }
  /* 
   */
  if (*ipar == 2)
    {
      ret_val = mesflg;
      if (*iset)
	{
	  mesflg = *ivalue;
	}
    }
  return ret_val;
}
