// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MBC_PID(job,arg1,arg2)

  function MBC_PID_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    b=xget('color','blue');
    w=xget('color','white');
    g=xget('color','gray');
    options=hash(background=w,color=b);
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,options(:));
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,options(:));
    end;
    if orient then
      xstringb(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,label,sz(1)*1.5,sz(2)*0.2,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.25-1.5),orig(2)+sz(2)*1.05,label,sz(1)*1.5,sz(2)*0.2,"fill");
    end;
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.89;0.05],color=g);
    xpoly(xx+ww*[0.1;0.06;0.14;0.1],yy+hh*[0.95;0.84;0.84;0.95],color=g);
    xpoly(xx+ww*[0.05;0.91],yy+hh*[0.1;0.1],color=g);
    xpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.1;0.14;0.06;0.1],color=g);
    xpoly(xx+ww*[0.1;0.1;0.1;0.8],yy+hh*[0.1;0.75;0.4;0.9],color=b);
    if orient then
      xstringb(orig(1)+sz(1)*0.4,orig(2)+sz(2)*0.2,"PID",sz(1)*0.5,sz(2)*0.2,"fill");
    else
      xstringb(orig(1)+sz(1)*(1-0.4-0.5),orig(2)+sz(2)*0.2,"PID",sz(1)*0.5,sz(2)*0.2,"fill");
    end;
    if orient then
      xstringb(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.25,"Ti="+string(Ti),sz(1)*1.5,sz(2)*0.2,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.25-1.5),orig(2)+sz(2)*-0.25,"Ti="+string(Ti),sz(1)*1.5,sz(2)*0.2,"fill");
    end;
  endfunction


  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    k=arg1.graphics.exprs(1);
    Ti=arg1.graphics.exprs(2);
    Td=arg1.graphics.exprs(3);
    Nd=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_MBI_SISO_dp);
   case 'getinputs' then
    [x,y,typ]=_MBI_SISO_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MBI_SISO_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,k,Ti,Td,Nd,exprs]=...
	  getvalue(['';'MBC_PID';'';'PID-controller in additive description form';''],...
		   [' k [-] : Gain',' Ti [s] : Time Constant of Integrator',' Td [s] : Time Constant of Derivative block',' Nd [-] : The higher Nd, the more ideal the derivative block'],...
		   list('vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(k,Ti,Td,Nd)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    k=1;
    Ti=0.5;
    Td=0.1;
    Nd=10;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Blocks.Continuous.PID';
    mo.inputs=['u'];
    mo.outputs=['y'];
    mo.parameters=list(['k','Ti','Td','Nd'],...
		       list(k,Ti,Td,Nd),...
		       [0,0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(k);sci2exp(Ti);sci2exp(Td);sci2exp(Nd)];
    gr_i="MBC_PID_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['E'];
    x.graphics.out_implicit=['E'];
  end
endfunction
