// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAI_IdealDiode(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    
    blue=xget('color','blue');
    white=xget('color','white');
    black=xget('color','black');
    gray=xget('color','gray');
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xfpoly(xx+ww*[0.65;0.35;0.35;0.65],yy+hh*[0.5;0.7;0.3;0.5],color=black,fill_color=white);
    xpoly(xx+ww*[0.05;0.7],yy+hh*[0.5;0.5],color=blue);
    xpoly(xx+ww*[0.7;0.95],yy+hh*[0.5;0.5],color=blue);
    xpoly(xx+ww*[0.65;0.65],yy+hh*[0.7;0.3],color=blue);
    if orient then
      xstringb(orig(1)+sz(1)*-0.28,orig(2)+sz(2)*0.805,label,sz(1)*1.5,sz(2)*0.2,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.28-1.5),orig(2)+sz(2)*0.805,label,sz(1)*1.5,sz(2)*0.2,"fill");
    end
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    Ron=arg1.graphics.exprs(1);
    Goff=arg1.graphics.exprs(2);
    Vknee=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_MEAI_OnePort_dp);
   case 'getinputs' then
    [x,y,typ]=_MEAI_OnePort_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MEAI_OnePort_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Ron,Goff,Vknee,exprs]=...
	  getvalue(['';'CEAI_IdealDiode';'';'Ideal diode';''],...
		   [' Ron [Ohm] : Forward state-on differential resistance (closed diode resistance >=0)',' Goff [S] : Backward state-off conductance (opened diode conductance >= 0)',' Vknee [V] : Forward threshold voltage (>= 0'],...
		   list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Ron,Goff,Vknee)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    Ron=0.00001;
    Goff=0.00001;
    Vknee=0;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Coselica.Electrical.Analog.Ideal.IdealDiode';
    mo.inputs=['p'];
    mo.outputs=['n'];
    mo.parameters=list(['Ron','Goff','Vknee'],...
		       list(Ron,Goff,Vknee),...
		       [0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(Ron);sci2exp(Goff);sci2exp(Vknee)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
