// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CMT_Stop(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    green=xget('color','green');
    white=xget('color','white');
    black=xget('color','black');
    gray=xget('color','gray');

    
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xfpoly(xx+ww*[0.75;0.6;0.6;0.75],yy+hh*[0.05;0.1;0;0.05],color=gray,fill_color=gray);
    
    xpoly(xx+ww*[0.2;0.6],yy+hh*[0.05;0.05],color=black);
    if orient then
      xrect(orig(1)+sz(1)*0.35,orig(2)+sz(2)*0.65,sz(1)*0.325,sz(2)*0.325,color=black,background=white);
    else
      xrect(orig(1)+sz(1)*(1-0.35-0.325),orig(2)+sz(2)*0.65,sz(1)*0.325,sz(2)*0.325,color=black,background=white);
    end
    //e.line_style=0;
    xpoly(xx+ww*[0.05;0.35],yy+hh*[0.5;0.5],color=green);
    if orient then
      xrect(orig(1)+sz(1)*0.15,orig(2)+sz(2)*0.275,sz(1)*0.72,sz(2)*0.075,color=black,background=gray);
    else
      xrect(orig(1)+sz(1)*(1-0.15-0.72),orig(2)+sz(2)*0.275,sz(1)*0.72,sz(2)*0.075,color=black,background=gray);
    end
    xpoly(xx+ww*[0.675;0.95],yy+hh*[0.5;0.5],color=green);
    if orient then
      xstringb(orig(1)+sz(1)*0,orig(2)+sz(2)*0.7,label,sz(1)*1,sz(2)*0.3,"fill");
    else
      xstringb(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*0.7,label,sz(1)*1,sz(2)*0.3,"fill");
    end
    
    if orient then
      xrect(orig(1)+sz(1)*0.185,orig(2)+sz(2)*0.425,sz(1)*0.04,sz(2)*0.15,color=black,background=black);
    else
      xrect(orig(1)+sz(1)*(1-0.185-0.04),orig(2)+sz(2)*0.425,sz(1)*0.04,sz(2)*0.15,color=black,background=black);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.8,orig(2)+sz(2)*0.42,sz(1)*0.045,sz(2)*0.145,color=black,background=black);
    else
      xrect(orig(1)+sz(1)*(1-0.8-0.045),orig(2)+sz(2)*0.42,sz(1)*0.045,sz(2)*0.145,color=black,background=black);
    end
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    smax=arg1.graphics.exprs(1);
    smin=arg1.graphics.exprs(2);
    m=arg1.graphics.exprs(3);
    F_prop=arg1.graphics.exprs(4);
    F_Coulomb=arg1.graphics.exprs(5);
    F_Stribeck=arg1.graphics.exprs(6);
    fexp=arg1.graphics.exprs(7);
    mode_start=arg1.graphics.exprs(8);
    L=arg1.graphics.exprs(9);
    standard_draw(arg1,%f,_MMTI_Rigid_dp);
   case 'getinputs' then
    [x,y,typ]=_MMTI_Rigid_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MMTI_Rigid_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,smax,smin,m,F_prop,F_Coulomb,F_Stribeck,fexp,mode_start,L,exprs]=...
	  getvalue(['';'CMT_Stop';'';'Sliding mass with hard stop and Stribeck friction';''],...
		   [' smax [m] : right stop for (right end of) sliding mass',' smin [m] : left stop for (left end of) sliding mass',' m [kg] : mass of the sliding mass',' F_prop [N/(m/s)] : velocity dependent friction',' F_Coulomb [N] : constant friction: Coulomb force',' F_Stribeck [N] : Stribeck effect',' fexp [1/(m/s)] : exponential decay of Stribeck effect',' mode_start [-] : Initial sliding mode (-1=Backward,0=Sticking,1=Forward)',' L [m] : length of component from left flange to right flange (= flange_b.s - flange_a.s)'],...
		   list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(smax,smin,m,F_prop,F_Coulomb,F_Stribeck,fexp,mode_start,L)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    smax=25;
    smin=-25;
    m=1;
    F_prop=1;
    F_Coulomb=5;
    F_Stribeck=10;
    fexp=2;
    mode_start=0;
    L=0;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Coselica.Mechanics.Translational.Stop';
    mo.inputs=['flange_a'];
    mo.outputs=['flange_b'];
    mo.parameters=list(['smax','smin','m','F_prop','F_Coulomb','F_Stribeck','fexp','mode_start','L'],...
		       list(smax,smin,m,F_prop,F_Coulomb,F_Stribeck,fexp,mode_start,L),...
		       [0,0,0,0,0,0,0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(smax);sci2exp(smin);sci2exp(m);sci2exp(F_prop);sci2exp(F_Coulomb);sci2exp(F_Stribeck);sci2exp(fexp);sci2exp(mode_start);sci2exp(L)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
