// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MMT_Spring(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    white=xget('color','white');
    black=xget('color','black');
    gray=xget('color','gray');

    
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xpoly(xx+ww*[0.2;0.6],yy+hh*[0.05;0.05],color=black);
    xfpoly(xx+ww*[0.75;0.6;0.6;0.75],yy+hh*[0.05;0.1;0;0.05],color=gray,fill_color=gray);
    
    if orient then
      xstringb(orig(1)+sz(1)*0,orig(2)+sz(2)*0.75,label,sz(1)*1,sz(2)*0.3,"fill");
    else
      xstringb(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*0.75,label,sz(1)*1,sz(2)*0.3,"fill");
    end
    
    xpoly(xx+ww*[0.07;0.2;0.28;0.42;0.57;0.72;0.8;0.92],yy+hh*[0.5;0.5;0.35;0.65;0.35;0.65;0.5;0.5],color=black);
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    s_rel0=arg1.graphics.exprs(1);
    c=arg1.graphics.exprs(2);
    standard_draw(arg1,%f,_MMTI_Rigid_dp);
   case 'getinputs' then
    [x,y,typ]=_MMTI_Rigid_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MMTI_Rigid_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,s_rel0,c,exprs]=...
	  getvalue(['';'MMT_Spring';'';'Linear 1D translational spring';''],...
		   [' s_rel0 [m] : unstretched spring length',' c [N/m] : spring constant '],...
		   list('vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(s_rel0,c)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    s_rel0=0;
    c=1;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Mechanics.Translational.Spring';
    mo.inputs=['flange_a'];
    mo.outputs=['flange_b'];
    mo.parameters=list(['s_rel0','c'],...
		       list(s_rel0,c),...
		       [0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(s_rel0);sci2exp(c)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
