// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MMT_SpringDamper(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    white=xget('color','white');
    black=xget('color','black');
    gray=xget('color','gray');

    
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xpoly(xx+ww*[0.1;0.2;0.275;0.425;0.575;0.725;0.8;0.9],yy+hh*[0.7;0.7;0.55;0.85;0.55;0.85;0.7;0.7],color=black);
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.7;0.15],color=black);
    xpoly(xx+ww*[0.1;0.24],yy+hh*[0.15;0.15],color=black);
    if orient then
      xrect(orig(1)+sz(1)*0.24,orig(2)+sz(2)*0.255,sz(1)*0.45,sz(2)*0.21,color=black,background=gray);
    else
      xrect(orig(1)+sz(1)*(1-0.24-0.45),orig(2)+sz(2)*0.255,sz(1)*0.45,sz(2)*0.21,color=black,background=gray);
    end
    xpoly(xx+ww*[0.24;0.84],yy+hh*[0.255;0.255],color=black);
    xpoly(xx+ww*[0.245;0.845],yy+hh*[0.045;0.045],color=black);
    xpoly(xx+ww*[0.69;0.9],yy+hh*[0.15;0.15],color=black);
    xpoly(xx+ww*[0.9;0.9],yy+hh*[0.7;0.15],color=black);
    xpoly(xx+ww*[0.05;0.1],yy+hh*[0.5;0.5],color=black);
    xpoly(xx+ww*[0.9;0.95],yy+hh*[0.5;0.5],color=black);
    xfpoly(xx+ww*[0.765;0.615;0.615;0.765],yy+hh*[0.41;0.46;0.36;0.41],color=gray,fill_color=gray);
    
    xpoly(xx+ww*[0.215;0.615],yy+hh*[0.41;0.41],color=black);
    if orient then
      xstringb(orig(1)+sz(1)*0.005,orig(2)+sz(2)*0.9,label,sz(1)*1,sz(2)*0.3,"fill");
    else
      xstringb(orig(1)+sz(1)*(1-0.005-1),orig(2)+sz(2)*0.9,label,sz(1)*1,sz(2)*0.3,"fill");
    end

  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    s_rel0=arg1.graphics.exprs(1);
    c=arg1.graphics.exprs(2);
    d=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_MMTI_Rigid_dp);
   case 'getinputs' then
    [x,y,typ]=_MMTI_Rigid_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MMTI_Rigid_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,s_rel0,c,d,exprs]=...
	  getvalue(['';'MMT_SpringDamper';'';'Linear 1D translational spring and damper in parallel';''],...
		   [' s_rel0 [m] : unstretched spring length',' c [N/m] : spring constant',' d [N/(m/s)] : damping constant'],...
		   list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(s_rel0,c,d)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    s_rel0=0;
    c=1;
    d=1;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Mechanics.Translational.SpringDamper';
    mo.inputs=['flange_a'];
    mo.outputs=['flange_b'];
    mo.parameters=list(['s_rel0','c','d'],...
		       list(s_rel0,c,d),...
		       [0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(s_rel0);sci2exp(c);sci2exp(d)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
