// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MMR_TorqueStep(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    black=xget('color','black');
    white=xget('color','white');
    gray=xget('color','gray');
    red=xget('color','red');

    
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.02,orig(2)+sz(2)*0.98,sz(1)*0.96,sz(2)*0.96,color=white,background=white);
    else
      xrect(orig(1)+sz(1)*(1-0.02-0.96),orig(2)+sz(2)*0.98,sz(1)*0.96,sz(2)*0.96,color=white,background=white);
    end
    
    xpoly(xx+ww*[0.35;0.65],yy+hh*[0.15;0.15],color=black);
    xpoly(xx+ww*[0.35;0.45],yy+hh*[0.05;0.15],color=black);
    xpoly(xx+ww*[0.45;0.55],yy+hh*[0.05;0.15],color=black);
    if orient then
      xrect(orig(1)+sz(1)*0.4,orig(2)+sz(2)*0,sz(1)*0.2,sz(2)*0.2,color=gray,background=gray);
    else
      xrect(orig(1)+sz(1)*(1-0.4-0.2),orig(2)+sz(2)*0,sz(1)*0.2,sz(2)*0.2,color=gray,background=gray);
    end
    
    xpoly(xx+ww*[0.55;0.65],yy+hh*[0.05;0.15],color=black);
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.15;-0.05],color=black);
    xpoly(xx+ww*[0.04;0.12;0.23;0.35;0.43;0.55;0.63;0.73;0.82],yy+hh*[0.5;0.68;0.81;0.9;0.94;0.96;0.95;0.9;0.81],color=black);
    if orient then
      xstringb(orig(1)+sz(1)*-0.31,orig(2)+sz(2)*0.99,label,sz(1)*1.62,sz(2)*0.32,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.31-1.62),orig(2)+sz(2)*0.99,label,sz(1)*1.62,sz(2)*0.32,"fill");
    end
    xfpoly(xx+ww*[0.97;0.9;0.75;0.97],yy+hh*[0.58;0.87;0.76;0.58],color=black,fill_color=black)
    
    xpoly(xx+ww*[0.1;0.5;0.5;0.9],yy+hh*[0.2;0.2;0.8;0.8],color=blue);
    if orient then
      xstringb(orig(1)+sz(1)*0.5,orig(2)+sz(2)*0.2,"time",sz(1)*0.5,sz(2)*0.1,"fill");
    else
      xstringb(orig(1)+sz(1)*(1-0.5-0.5),orig(2)+sz(2)*0.2,"time",sz(1)*0.5,sz(2)*0.1,"fill");
    end
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    stepTorque=arg1.graphics.exprs(1);
    offsetTorque=arg1.graphics.exprs(2);
    startTime=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_MMR_ConstantTorque_dp);
   case 'getinputs' then
    [x,y,typ]=_MMR_ConstantTorque_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MMR_ConstantTorque_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,stepTorque,offsetTorque,startTime,exprs]=...
	  getvalue(['';'MMR_TorqueStep';'';'Constant torque, not dependent on speed';''],...
		   [' stepTorque [N.m] : height of torque step (if negative, torque is acting as load)',' offsetTorque [N.m] : offset of torque',' startTime [s] : output = offset for time < startTime'],...
		   list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(stepTorque,offsetTorque,startTime)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    stepTorque=1;
    offsetTorque=0;
    startTime=0;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Mechanics.Rotational.TorqueStep';
    mo.inputs=[];
    mo.outputs=['flange','bearing'];
    mo.parameters=list(['stepTorque','offsetTorque','startTime'],...
		       list(stepTorque,offsetTorque,startTime),...
		       [0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(stepTorque);sci2exp(offsetTorque);sci2exp(startTime)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['I','I'];
  end
endfunction
