// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MTH_BodyRadiation(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    green=xget('color','green')
    black=xget('color','black');
    white=xget('color','white');
    gray=xget('color','gray');
    red=xget('color','red');

    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.75,orig(2)+sz(2)*0.9,sz(1)*0.2,sz(2)*0.8,color=black,background=gray);
    else
      xrect(orig(1)+sz(1)*(1-0.75-0.2),orig(2)+sz(2)*0.9,sz(1)*0.2,sz(2)*0.8,color=black,background=gray);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.05,orig(2)+sz(2)*0.9,sz(1)*0.2,sz(2)*0.8,color=black,background=gray);
    else
      xrect(orig(1)+sz(1)*(1-0.05-0.2),orig(2)+sz(2)*0.9,sz(1)*0.2,sz(2)*0.8,color=black,background=gray);
    end
    xpoly(xx+ww*[0.32;0.68],yy+hh*[0.55;0.55],color=red);
    xpoly(xx+ww*[0.32;0.37],yy+hh*[0.55;0.58],color=red);
    xpoly(xx+ww*[0.32;0.37],yy+hh*[0.55;0.52],color=red);
    xpoly(xx+ww*[0.32;0.68],yy+hh*[0.45;0.45],color=red);
    xpoly(xx+ww*[0.63;0.68],yy+hh*[0.42;0.45],color=red);
    xpoly(xx+ww*[0.63;0.68],yy+hh*[0.48;0.45],color=red);
    xpoly(xx+ww*[0.32;0.68],yy+hh*[0.35;0.35],color=red);
    xpoly(xx+ww*[0.32;0.37],yy+hh*[0.35;0.38],color=red);
    xpoly(xx+ww*[0.32;0.37],yy+hh*[0.35;0.32],color=red);
    xpoly(xx+ww*[0.32;0.68],yy+hh*[0.65;0.65],color=red);
    xpoly(xx+ww*[0.63;0.68],yy+hh*[0.62;0.65],color=red);
    xpoly(xx+ww*[0.63;0.68],yy+hh*[0.68;0.65],color=red);
    if orient then
      xstringb(orig(1)+sz(1)*-0.16,orig(2)+sz(2)*0.92,label,sz(1)*1.2,sz(2)*0.3,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.16-1.2),orig(2)+sz(2)*0.92,label,sz(1)*1.2,sz(2)*0.3,"fill");
    end
    if orient then
      xstringb(orig(1)+sz(1)*-0.095,orig(2)+sz(2)*-0.125,"G=%G",sz(1)*1.18,sz(2)*0.195,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.095-1.18),orig(2)+sz(2)*-0.125,"G=%G",sz(1)*1.18,sz(2)*0.195,"fill");
    end
    if orient then
      xrect(orig(1)+sz(1)*0.25,orig(2)+sz(2)*0.9,sz(1)*0.03,sz(2)*0.8,color=red,background=red);
    else
      xrect(orig(1)+sz(1)*(1-0.25-0.03),orig(2)+sz(2)*0.9,sz(1)*0.03,sz(2)*0.8,color=red,background=red);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.725,orig(2)+sz(2)*0.9,sz(1)*0.025,sz(2)*0.8,color=red,background=red);
    else
      xrect(orig(1)+sz(1)*(1-0.725-0.025),orig(2)+sz(2)*0.9,sz(1)*0.025,sz(2)*0.8,color=red,background=red);
    end
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    Gr=arg1.graphics.exprs(1);
    standard_draw(arg1,%f,_MTH_ThermalConductor_dp);
   case 'getinputs' then
    [x,y,typ]=_MTH_ThermalConductor_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MTH_ThermalConductor_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Gr,exprs]=...
	  getvalue(['';'MTH_BodyRadiation';'';'Lumped thermal element for radiation heat transfer';''],...
		   [' Gr [m2] : Net radiation conductance between two surfaces (see docu)'],...
		   list('vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Gr)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    Gr=1;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Thermal.HeatTransfer.BodyRadiation';
    mo.inputs=['port_a'];
    mo.outputs=['port_b'];
    mo.parameters=list(['Gr'],...
		       list(Gr),...
		       [0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(Gr)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
