// Coselica Toolbox for Scicoslab
// Copyright (C) 2009-2011  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=MTH_FixedHeatFlow(job,arg1,arg2)
  function blk_draw(sz,orig,orient,label)
    blue=xget('color','blue');
    red=xget('color','red');
    white=xget('color','white');
    green=xget('color','green')
    black=xget('color','black');y
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xstringb(orig(1)+sz(1)*-0.17,orig(2)+sz(2)*0.8,label,sz(1)*1.33,sz(2)*0.3,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.17-1.33),orig(2)+sz(2)*0.8,label,sz(1)*1.33,sz(2)*0.3,"fill");
    end
    if orient then
      xstringb(orig(1)+sz(1)*-0.185,orig(2)+sz(2)*0,"Q_flow="+string(Q_flow),sz(1)*1.35,sz(2)*0.24,"fill");
    else
      xstringb(orig(1)+sz(1)*(1--0.185-1.35),orig(2)+sz(2)*0,"Q_flow="+string(Q_flow),sz(1)*1.35,sz(2)*0.24,"fill");
    end
    xpoly(xx+ww*[0;0.74],yy+hh*[0.4;0.4],color=red);
    
    xpoly(xx+ww*[0;0.73],yy+hh*[0.6;0.6],color=red);
    
    xfpoly(xx+ww*[0.7;0.7;0.85;0.7],yy+hh*[0.5;0.7;0.6;0.5],color=red,fill_color=red)
    
    xfpoly(xx+ww*[0.7;0.7;0.85;0.7],yy+hh*[0.3;0.5;0.4;0.3],color=red,fill_color=red)
    
    if orient then
      xrect(orig(1)+sz(1)*0.85,orig(2)+sz(2)*0.7,sz(1)*0.1,sz(2)*0.4,color=red,background=red);
    else
      xrect(orig(1)+sz(1)*(1-0.85-0.1),orig(2)+sz(2)*0.7,sz(1)*0.1,sz(2)*0.4,color=red,background=red);
    end
  endfunction

  
  x=[];y=[];typ=[];
  select job
   case 'plot' then
    Q_flow=arg1.graphics.exprs(1);
    T_ref=arg1.graphics.exprs(2);
    alpha=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_MTH_FixedTemperature_dp);
   case 'getinputs' then
    [x,y,typ]=_MTH_FixedTemperature_ip(arg1);
   case 'getoutputs' then
    [x,y,typ]=_MTH_FixedTemperature_op(arg1);
   case 'getorigin' then
    [x,y]=standard_origin(arg1);
   case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Q_flow,T_ref,alpha,exprs]=...
	  getvalue(['';'MTH_FixedHeatFlow';'';'Fixed heat flow boundary condition';''],...
		   [' Q_flow [W] : Fixed heat flow rate at port',' T_ref [K] : Reference temperature',' alpha [1/K] : Temperature coefficient of heat flow rate'],...
		   list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Q_flow,T_ref,alpha)
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
   case 'define' then
    model=scicos_model();
    Q_flow=1;
    T_ref=293.15;
    alpha=0;
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    mo=modelica();
    mo.model='Modelica.Thermal.HeatTransfer.FixedHeatFlow';
    mo.inputs=[];
    mo.outputs=['port'];
    mo.parameters=list(['Q_flow','T_ref','alpha'],...
		       list(Q_flow,T_ref,alpha),...
		       [0,0,0]);
    model.equations=mo;
    model.in=ones(size(mo.inputs,'*'),1);
    model.out=ones(size(mo.outputs,'*'),1);
    exprs=[sci2exp(Q_flow);sci2exp(T_ref);sci2exp(alpha)];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['I'];
  end
endfunction
