/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* demodgen Scicos Generic symbol demodulator block
 * Type 4 simulation function ver 1.0 - scilab-4.1
 * 25 mars 2007 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: demodgen.c,v $
 * Revision 1.1  2016-07-20 14:00:20  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:51  layec
 * add
 *
 */

#include "modnum_lib.h"
#include <scicos/scicos_block.h>

/* Cette fonction de simulation ralise un
 * dmodulateur symbole gnrique.
 *
 * Les valeurs I/Q sont donnes en paramtres.
 * A chaque instant de dclenchement, on cherche les
 * valeurs donnes en entre dans le tableau de paramtre
 * de taille m et l'on retourne l'indexe associ en sortie
 * (compris entre 0 et m-1)
 *
 * entres rgulires
 *    u1[0..insz[0]-1] : vecteur des composantes I
 *    u2[0..insz[0]-1] : vecteur des composantes Q
 *
 * sorties rgulires
 *    y[0..insz[0]-1]  : vecteur des numros symboles
 *
 * entre vnementielle : instants de dclenchement.
 * sortie vnementielle : nant
 *
 * paramtres entiers
 *    ipar[0] : m nombre d'tats
 *
 * paramtres rels
 *    rpar[0..ipar[0]-1] :
 *       vecteur des parties relles des points de la constellation
 *    rpar[ipar[0]..2*ipar[0]-1] :
 *      vecteur des parties imaginaires des points de la constellation
 */

/*prototype*/
void demodgen(scicos_block *block,int flag)
{
 /*Dclaration des variables*/
 double *u1,*u2;
 double *y;
 int nu; /*taille du vecteur d'entre*/
 double *u_r,*u_i;
 int m; /*nombre d'tats*/
 int i;

 /*Rcupration des adresses des ports rguliers*/
 u1 = (double *)block->inptr[0];
 u2 = (double *)block->inptr[1];
 y  = (double *)block->outptr[0];

 /*Rcupration de la taille du vecteur d'entre*/
 nu=block->insz[0];

 /*Rcupration du nombre d'tats*/
 m=block->ipar[0];

 /*Rcupration des valeurs relles et imaginaires du symbole*/
 u_r=(double *)&block->rpar[0];
 u_i=(double *)&block->rpar[m];

 /*
  * Le flag 1 calcule le numro symbole y[]  partir
  * des valeurs de u1 et u2
  */
 if(flag==1||flag==6) {
  /*pour chaque lments du vecteur d'entre*/
  for (i=0;i<nu;i++) {
     /*Appel routine demodgen*/
     demodgen_c(&nu,&m,&u1[0],&u2[0],&u_r[0],&u_i[0],&y[0]);
  }
 }
}
