/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* demodiq Scicos demodulator I/Q block
 * Type 2 simulation function ver 1.0 - scilab-2.6&2.7
 * 8 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: demodiq.c,v $
 * Revision 1.1  2016-07-20 14:00:20  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:51  layec
 * add
 *
 */

#include <math.h>

#include <modnum_lib.h>

#if WITH_NSPSRC
#include <nsp/machine.h>
#else
#include <machine.h>
#endif

/* Cette fonction calcule la partie relle et imaginaire de l'enveloppe
 * complexe en bande de base  partir de la partie relle et imaginaire
 * de l'enveloppe module autour d'une frquence wo :
 * y1=u1*cos(wot)+u2*sin(wot)
 * y2=u1*sin(wot)-u2*cos(wot)
 * o y1, y2 sont les sorties, u1 et u2 sont la partie relle et imaginaire
 * du signal modul et wo un paramtre rel en rad/s. Les porteuses sont en fait
 * sur-chantillonnes par la variable *t. Le rythme des dclenchements dfini
 * donc la priode de sur-chantillonnage.
 *
 * entres rgulires: u1[0..insz[0]-1] vecteur des parties relles du signal modul
 *                     u2[0..insz[0]-1] vecteur des parties imaginaires du signal modul
 * sorties rgulires : y1[0..insz[0]-1] vecteur des parties relles du signal en bande de base
 *                     y2[0..insz[0]-1] vecteur des parties imaginaires du signal en bande de base
 * entre vnementielles : instants de surchantillonnage
 * sorties vnementielles : nant
 * paramatres rels : rpar[0..insz[0]-1] : vecteur des pulsations porteuses
 */

/*prototype*/
void demodiq(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
             ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
   /*dclaration des variables*/
   double *y1,*y2;
   double *u1,*u2;
   int nu,i;

   /*rcupration des adresses des ports rguliers*/
   y1=(double *)outptr[0];
   y2=(double *)outptr[1];
   u1=(double *)inptr[0];
   u2=(double *)inptr[1];

   /*rcupration de la taille des ports d'entres*/
   nu=insz[0];

   /*Le flag 1 calcule la partie relle et imaginaire de
    * l'enveloppe complexe en bande de base.
    */
   if(*flag==1)
   {
    for(i=0;i<nu;i++)
     {
      /*Calcul du registre y*/
      y1[i]=cos(*t*rpar[i])*u1[i]+sin(*t*rpar[i])*u2[i];
      y2[i]=sin(*t*rpar[i])*u1[i]-cos(*t*rpar[i])*u2[i];
     }
   }
}
