/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* dac Scicos Digital to Analog Converter block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 8 Mars 2006 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: dac.c,v $
 * Revision 1.1  2016-07-20 14:00:20  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:51  layec
 * add
 *
 */

#include <scicos/scicos_block.h>
#include "modnum_lib.h"

/* Cette fonction de calcul ralise la convertion numrique analogique.
 *
 *
 * entre rgulire : u[0..nu-1] vecteur des entres rgulires  convertir
 * sortie rgulire : y[0..ny-1] vecteur des sorties converties
 *
 * entre vnementielle : Dates de dclenchement
 * sortie vnementeille : nant
 *
 * paramtre entier : ipar[0..nu-1]     : Nombre de bits de la valeur entire de sortie
 *                    ipar[nu..2*nu-1]  : flag code complment  2
 *                                        0 : nombre entier non sign
 *                                        1 : nombre entier sign
 *
 * paramtre rel : rpar[0..nu-1]      : pas (ou quantum)
 *                  rpar[nu..2*nu-1]   : valeur minimale ou dbute la conversion
 */

/*prototype*/
void dac(scicos_block *block,int flag)
{
  /*Dclaration des variables*/
  double *u,*y;

  int nu;

  int nbit;
  int cc2;

  double q;
  double vmin;

  /*Rcupration des adresses des ports rguliers*/
  u=(double *)block->inptr[0];
  y=(double *)block->outptr[0];

  /*Rcupration de la taille du port d'entre rgulier*/
  nu=block->insz[0];

  /*Rcupre le nombre de bit*/
  nbit=block->ipar[0];
  /*Rcupre le type de code*/
  cc2=block->ipar[nu];

  /*Rcupre le quantum*/
  q=block->rpar[0];
  /*Rcupre la grandeur minimale d'entre*/
  vmin=block->rpar[nu];

  /*Appel routine dacv_c*/
  dacv_c(&nu,&nbit,&cc2,&q,&vmin,&u[0],&y[0]);
}
