/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2016 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <scicos/scicos_block.h>
#include <nsp/math.h>
#include <nsp/cnumeric.h>
#include <modnum_lib.h>

static integer c__2 = 2;

/*    can.f Analog to Digital Convertor 
 *    IRCOM Group - A. Layec 
 */

#define d_int(x) ((*(x) >0) ? floor(*(x)) : -floor( - *(x)) )
#define d_nint(x) (x)>=0 ? floor(x + .5) : -floor(.5 - x)

int can(int *flag__, int *nevprt, double *t, double *xd, double *x, int *nx, double *z__, int *nz, double *tvec, int *ntvec, double *rpar, int *nrpar, int *ipar, int *nipar, double *u, int *nu, double *y, int *ny)
{
    int i__1, i__2;
    double d__1;
    int i__;
    double fsr;

    /* Parameter adjustments */
    --y;
    --u;
    --ipar;
    --rpar;
    --tvec;
    --z__;
    --x;
    --xd;

    /* Function Body */
    i__1 = *nu;
    for (i__ = 1; i__ <= i__1; ++i__) {
	y[i__] = d_nint(u[i__]);
/*     qztrn.f 
 */
	if (ipar[i__] == 1) {
	    if (u[i__] < 0.) {
		d__1 = u[i__] / rpar[i__] + .5;
		y[i__] = rpar[i__] * d_nint(d__1);
	    } else {
		d__1 = u[i__] / rpar[i__] - .5;
		y[i__] = rpar[i__] * d_nint(d__1);
	    }
	} else if (ipar[i__] == 2) {
/*     qzrnd.f 
 */
	    if (u[i__] < 0.) {
		d__1 = u[i__] / rpar[i__] + .5;
		y[i__] = rpar[i__] * (d_nint(d__1) - .5);
	    } else {
		d__1 = u[i__] / rpar[i__] - .5;
		y[i__] = rpar[i__] * (d_nint(d__1) + .5);
	    }
	} else if (ipar[i__] == 3) {
/*     qzflr.f 
 */
	    d__1 = u[i__] / rpar[i__] + .5;
	    y[i__] = rpar[i__] * d_nint(d__1);
	} else if (ipar[i__] == 4) {
/*     qzcel.f 
 */
	    d__1 = u[i__] / rpar[i__] - .5;
	    y[i__] = rpar[i__] * d_nint(d__1);
	}
/* L15: */
    }
    i__1 = *nu;
    for (i__ = 1; i__ <= i__1; ++i__) {
	fsr = nsp_pow_ii(c__2, ipar[i__ + *nu]) * rpar[i__];
	if (y[i__] < rpar[*nu + i__]) {
	    y[i__] = rpar[*nu + i__];
	} else if (y[i__] > rpar[*nu + i__] + fsr) {
	    y[i__] = rpar[*nu + i__] + fsr;
	}
/*          write(6,'(''F: y(i)='',e10.3)') y(i) 
 *     write(6,'(''F: fsr='',e10.3,'' ipar(i+nu)='',i1)') fsr,ipar(i+nu) 
 *     write(6,'(''F: rpar(nu+i)+fsr='',e10.3)') rpar(nu+i)+fsr 
 */
/* L20: */
    }
    i__1 = *nu;
    for (i__ = 1; i__ <= i__1; ++i__) {
	d__1 = (y[i__] - rpar[*nu + i__]) / rpar[i__];
	y[i__] = d_int(&d__1);
/*         y(i)=(y(i)-rpar(nu+i))/rpar(i) 
 *      write(6,'(''F: s= '',i3)') 2**(ipar(i+nu)-1) 
 */
	if (ipar[(*nu << 1) + i__] == 1) {
	    i__2 = ipar[i__ + *nu] - 1;
	    y[i__] -= nsp_pow_ii(c__2, i__2);
	}
/* L30: */
    }
    return 0;
} /* can_ */

