/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* vco Scicos continuous VCO block with frequency divider
 * Type 4 simulation function ver 1.0 - scilab-4.1
 * 24 Mars 2009 - INRIA - Author : A.Layec
 */

#include <scicos/scicos_block.h>
#include "modnum_lib.h"

/*
 * ys(t)  = cos(wot + phi(t))
 *
 * phi(t) = int(F(u3(t) + w(t))dt)
 *
 */


void vcoz(scicos_block *block,int flag)
{
  int i,k;

  double *phi__n; /* phase noise */

  double *u1 = block->inptr[0];  /* input voltage */
  double *u2 = block->inptr[1];  /* divider value or phase noise*/
  double *u3 = block->inptr[2];  /* phase noise*/
  double *y  = block->outptr[0]; /* cos(theta) */
  int nu = block->insz[0];
  int mu=1;
 
  int with_dyncross = block->ipar[0];
  /* int with_infunc   = block->ipar[1]; */
  int with_2evout   = block->ipar[2];
  int with_wnoise   = block->ipar[3];/* white noise on input */
  int with_pnoise   = block->ipar[4];/* phase noise */

  double omega      = block->rpar[0];
  double cross_val  = block->rpar[1];
  double cross_val2 = block->rpar[2];
  /* double kv    = block->rpar[4];  */
  /* double alpha = block->rpar[5]; */
  /* double beta  = block->rpar[6]; */
  double gamma = block->rpar[7];
  double sigma = block->rpar[8];

  double *w__n   = &block->z[0];/* white noise on input */
 
  switch(flag)
    {
    case 0 : /* compute continuous state derivative */
      for (i=0;i<nu*mu;i++) {
	block->xd[i] = omega + u1[i] + w__n[i];
      }
      if(with_2evout!=0) {
	for (i=0;i<nu*mu;i++) {
	  block->xd[nu*mu+i] = omega + u1[i] + w__n[i];
	}
      }
      break;
    case 1 : /* compute output */
      if (with_pnoise!=0) {
	if(with_dyncross!=0) {
	  phi__n = u3;
	}
	else {
	  phi__n = u2;
	}
	/* test on phase noise */
	for (i=0;i<nu*mu;i++) {
	  if ((phi__n[i] >= (M_PI/4))||(phi__n[i] <= -(M_PI/4))) {
	    set_block_error(-6);
	    return;
	  }
	}
	for (i=0;i<nu*mu;i++) {
	  y[i] = block->x[i] + phi__n[i];
	}
      }
      else {
	for (i=0;i<nu*mu;i++) {
	  y[i] = block->x[i];
	}
      }
      break;
    case 4 : /* compute discrete state */
    case 2 : //fprintf(stderr,"nevprt=%d\n",block->nevprt);
      if(block->nevprt<0) {
	for (i=0;i<nu*mu;i++) {
	  if (block->jroot[i]!=0) {
	    block->x[i]=0.;
	  }
	}
	if(with_2evout!=0) {
	  for (i=nu*mu;i<2*nu*mu;i++) {
	    if (block->jroot[i]!=0) {
	      block->x[i]=0.;
	    }
	  }
	}
      }
      else if(block->nevprt>0)
	{
	  /* to use that, the block must have an input event
	   * because time dependant block inherit of all events
	   * (nevprt=0) and the block is not called with flag 2
	   */
	  if (with_wnoise!=0) {
	    noiseblk_c((i=nu*mu,&i),(k=0,&k),&sigma,&gamma,w__n);
	  }
	}
      break;
    case 3 :/* compute date of crossing */
      for (i=0;i<nu*mu;i++) {
	if (block->jroot[i]!=0) {
	  block->evout[i]=0.;
	}
      }
      if(with_2evout!=0) {
	for (i=nu*mu;i<2*nu*mu;i++) {
	  if (block->jroot[i]!=0) {
	    block->evout[i]=0.;
	  }
	}
      }
      break;
    case 9 : /* compute zero crossing */
      if(with_dyncross!=0) {
	cross_val = cross_val * u2[0];
      }
      if (with_pnoise!=0) {
	if(with_dyncross!=0) {
	  phi__n = u3;
	}
	else {
	  phi__n = u2;
	}
	/* test on phase noise */
	for (i=0;i<nu*mu;i++) {
	  if ((phi__n[i] >= (M_PI/4))||(phi__n[i] <= -(M_PI/4))) {
	    set_block_error(-6);
	    return;
	  }
	}
	for (i=0;i<nu*mu;i++) {
	  block->g[i] = block->x[i] - cross_val  + phi__n[i];
	}
	if(with_2evout!=0) {
	  for (i=0;i<nu*mu;i++) {
	    block->g[nu*mu+i] = block->x[nu*mu+i] - cross_val2  + phi__n[i];
	  }
	}
      }
      else {
	for (i=0;i<nu*mu;i++) {
	  block->g[i] = block->x[i] - cross_val;
	}
	if(with_2evout!=0) {
	  for (i=0;i<nu*mu;i++) {
	    block->g[nu*mu+i] = block->x[nu*mu+i] - cross_val2;
	  }
	}
      }
      break;
    case 5 : /* finish */
      break;
    default : break;
    }
}

