/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* overlaprsr Scicos right shift register with memo word block
 * Type 4 simulation function ver 1.0 - scilab-2.6&2.7
 * 24 Dcembre 2004 Author : - IRCOM GROUP - A.Layec
 */

/* REVISION HISTORY :
 * $Log: overlaprsr.c,v $
 * Revision 1.1  2016-07-20 14:00:21  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:51  layec
 * add
 *
 */

#include "modnum_lib.h"
#include <scicos/scicos_block.h>

/* Cette fonction de simulation dcale le vecteur reu en entre de nz
 * cases vers la droite et le recopie en sortie y. L'execendent du mot
 * dcal est stock dans Z puis plac au dbut du vecteur suivant.
 *
 * Entre rguliere : u[0..nu-1] vecteur d'entre  dcaler
 * Sortie reguliere : y[0..nu-1] vecteur de sortie dcal
 * 
 * Entre vnementielle : nant
 * Sortie vnementielle : nant
 *
 * Paramtres entier : insz[0] : taille du vecteur en entre
 *                     nz : longueur du mot mmoire
 */

/*prototype*/
void overlaprsr(scicos_block *block,int flag)
{
  /*Dclaration des variables*/
  double *y;
  double *u;
  double *z;
  int nu,nd;

  /*Rcupration des adresses des ports rguliers*/
  y=(double *)block->outptr[0];
  u=(double *)block->inptr[0];
  z=block->z;

  /*Rcupration de la taille du port d'entre*/
  nu=block->insz[0];

  /*Rcupration de la longueur du mot mmoire*/
  nd=block->ipar[0];

  /* Le flag1 place le mot dcal dans y*/
  if(flag==1) {
    copyd_c(&nu,z,y);
  }
  /* le flag 2 dcale  droite le vecteur d'entre et
   * mmorise l'excedent du mot vecteur d'entre dans z
   */
  else if(flag==2) {
    copyd_c(&nd,&z[nu],&z[0]);
    copyd_c(&nu,u,&z[nd]);
  }
}
