/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* pspec_scope Scicos Spectrum Analyzer block
 *
 * - Based on auto-correlation method (see
 *   pspect.sci from standard Signal Scilab
 *   library)
 *
 * - Based on Scilab NG routines
 * - Based on Nsp NG routines
 *
 * Type 4 simulation function ver 1.0 - scilab-4.1
 * 23 Mars 2007 - INRIA - Author : A.Layec
 *
 * Type 4 simulation function ver 1.1 - scicos-4.4
 * 18 avril 2013 - ENPC - Author : A.Layec
 *
 */

/* REVISION HISTORY :
 * $Log: pspec_scope_nsp.c,v $
 * Revision 1.2  2016-07-21 07:49:57  jpc
 * warnings
 *
 * Revision 1.1  2016-07-20 14:00:21  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:52  layec
 * add
 *
 */

/*Standarc C include*/
#include <stdio.h>
#include <string.h>

#include <nsp/axes.h>
#include <nsp/qcurve.h>

/*Scicos block st include*/
#include <scicos/scicos_block.h>

#include <modnum_lib.h>

extern BCG *scicos_set_win(int wid, int *oldwid);

/*work structure of that block*/
typedef struct work_st {
  double *z_t;   /*time domain in buffer*/
  double *win;   /*window coefficient*/
  double *r_io;  /*real part of time/freq domain in/out buffer*/
  double *i_io;  /*imag part of time/freq domain in/out buffer*/
  double *r_out; /*real part of freq domain display buffer*/
  double *work;  /*working array for fft*/
  NspAxes *Axes; /*hdl of the NG scope*/
  int    wid;    /*wid of the NG scope*/
  int    minmax_win[2]; /*min/max sample of the x_axis*/
  /*y axis autoscale variables*/
  double y_min;
  double y_max;
  double y_up;
  double y_down;
  double y_delta;
} work_struct;

/*
 * ipar[0]  : fftsiz
 * ipar[1]  : sec_s
 * ipar[2]  : n_sec
 * ipar[3]  : ffttyp 0:fft842/1:dfftmx
 * ipar[4]  : n_work
 * ipar[5]  : xlog 0:no/1:yes
 * ipar[6]  : ylog 0:no/1:yes
 * ipar[7]  : wtyp 1-4
 * ipar[8]  : wid
 * ipar[9]  : yunit 0:U^2 / 1:dB
 * ipar[10] : scaltyp 0:use ymin,ymax / 1:autoscale
 * ipar[11] : view_in
 *
 * rpar[0] : Tse
 * rpar[1] : cfreq -1-0:no
 * rpar[2] : fdenorm  -1:no
 * rpar[3] : ymin
 * rpar[4] : ymax
 *
 * z[0]    : count
 * z[1]    : n_call
 */

/* specview_draw :
 *  Initialization and re-initialization
 *   of the pspec_scope block
 */
void specview_draw(scicos_block *block)
{
 BCG *Xgc;
 NspFigure *F;
 NspAxes *Axe,*Axe1;
 NspQcurve *curve;
 NspMatrix *Pts;
 double *qx;
 double *qy;

 /* counter variables */
 int i,l;

 /* variables to get rpar/ipar ptr */
 double *rpar;
 int    *ipar;

 /* variables to store main paramaters */
 double xmin, xmax;
 double ymin, ymax;
 double Tse;
 double stepf;
 double cfreq;
 double fdenorm;
 int    wtyp;

 int l_win;
 int fftsiz;
 int sec_s;
 int min_win;
 int max_win;
 int xlog;
 int ylog;
 int n_sec;
 int yunit;

 /* graphic ptr variables */
 double frect[4];
 char wnam[100];
 char title[150];
 char x_label[30];
 char y_label[30];

 int win_dim[2] = {-1,-1};

 int wid,oldwid; /*the windows id*/

 /*ptr of the struct of that block*/
 work_struct *work;

 /*get he struct of that block*/
 work     = (work_struct*) *block->work;

 /*get rpar/ipar*/
 rpar = block->rpar;
 ipar = block->ipar;

 fftsiz = ipar[0];
 sec_s  = ipar[1];
 n_sec  = ipar[2];
 xlog   = ipar[5];
 ylog   = ipar[6];
 wtyp   = ipar[7];
 wid    = ipar[8];
 yunit  = ipar[9];

 win_dim[0] = ipar[12];
 win_dim[1] = ipar[13];

 Tse     = rpar[0];
 cfreq   = rpar[1];
 fdenorm = rpar[2];
 ymin    = rpar[3];
 ymax    = rpar[4];

 /* compute freq indices of the scope */
 if (fdenorm<=0.) fdenorm = 1.;

 /* freq step */
 stepf = 1. / ( ((double)fftsiz) * Tse) * fdenorm;
 cfreq = cfreq * fdenorm;

 /* compute xmin/xmax bound of the scope */
 xmin = stepf;
 xmax = ((double)(fftsiz/2)-1)*stepf;

 /* compute xmin/xmax bound of the scope */
 if ( (cfreq<xmax) && (cfreq>xmin) ) {
   /* plot from a center frequency */
   if (xlog == 1) { /*logplot on x_axis*/
     min_win = ((int) (cfreq/stepf)) + 1;
     /*max_win = min_win + ((int)(min((xmax-cfreq)/stepf , 4e6))) - 1;*/
     max_win = min_win + (int)((xmax-cfreq)/stepf) - 1;
     l_win   = max_win - min_win + 1;   /* length of window */
     xmin    = stepf;
     xmax    = (l_win) * stepf;
   }
   else {  /*linearplot on x_axis*/
     l_win   = 2 *((int) (min( (cfreq-xmin)/stepf , (xmax-cfreq)/stepf )));
     /*l_win   = (int) (min(l_win,4e6));*/  /* length of window */
     min_win = ((int) (cfreq/stepf)) - l_win/2;
     max_win = ((int) (cfreq/stepf)) + l_win/2 - 1;
     xmin    = ((double)min_win) * stepf;
     xmax    = ((double)max_win) * stepf;
  }
 }
 else {
   /* plot from DC */
   l_win   = fftsiz / 2;
   /*l_win   = (int) (min(l_win,4e6));*/  /* length of window */
   min_win = 1;
   max_win = l_win -1 ;
   xmin    = ((double)min_win) * stepf;
   xmax    = ((double)max_win) * stepf;
 }
 /*store min/max x_axis indices in my_st*/
 work->minmax_win[0] = min_win;
 work->minmax_win[1] = max_win;

 /***** display part ****/

 /* get wid of the scope
  * and store it in my_st*/
 if (wid<0) {
   work->wid = 20000 + get_block_number();
 }
 else {
   work->wid = wid;
 }

 /* set current window to id number wid */
 Xgc = scicos_set_win(work->wid, &oldwid);

 /* set window dims */
 if (win_dim[0] >= 0) {
   Xgc->graphic_engine->xset_windowdim(Xgc, win_dim[0], win_dim[1]);
 }

 /* set title and axes labels of the scope
  */
 sprintf(wnam,"Spectrum Analyzer (%d)",work->wid);
 sprintf(title,"fft size=%d - overlap size=%d",fftsiz,sec_s);
 sprintf(title,"%s - avrg=%d",title,n_sec);
 switch(wtyp)
 {
  case 1  : sprintf(title,"%s - re win",title);
            break;

  case 2  : sprintf(title,"%s - tr win",title);
            break;

  case 3  : sprintf(title,"%s - hm win",title);
            break;

  case 4  : sprintf(title,"%s - hn win",title);
            break;

  default : sprintf(title,"%s - re win",title);
            break;
 }
 sprintf(title,"%s - step=%-3.3e Hz (%2.1f dB)",title,stepf,10*log10(stepf));
 if (((cfreq<((double)(fftsiz/2)-1)*stepf)) && (cfreq>stepf)) {
  sprintf(title,"%s - c.freq=%#-3.3e Hz (%2.1f dB)",title,cfreq,10*log10(stepf));
 }
 /* axis labels */
 sprintf(x_label,"Freq [Hz]"); /*x axis*/
 if (xlog==1) {
   sprintf(x_label,"%s - (LOG)",x_label);
 }
 else {
   sprintf(x_label,"%s - (LIN)",x_label);
 }
 if (yunit==1) { /*y axis*/
   sprintf(y_label,"PSD [dB]");
 }
 else if (yunit==2) {
   sprintf(y_label,"PSD [dB/Hz]");
 }
 else {
   sprintf(y_label,"PSD [U^2]");
 }
 if ((ylog==1)||(yunit==1)) {
   sprintf(y_label,"%s - (LOG)",y_label);
 }
 else {
   sprintf(y_label,"%s - (LIN)",y_label);
 }

 Xgc->graphic_engine->setpopupname (Xgc, wnam);

 if ((F = nsp_check_for_figure(Xgc,FALSE)) == NULL) {
   scicos_set_block_error(-16);
   return;
 }

 /* clean the figure */
 l = nsp_list_length(F->obj->children);
 for (i=0;i<l;i++) {
   nsp_list_remove_first(F->obj->children);
 }

 /* create a new axe */
 if ((Axe = nsp_check_for_axes(Xgc, NULL)) == NULL) {
   scicos_set_block_error(-16);
   return;
 }

 /* create a qcurve and insert them in axe */
 Pts = nsp_matrix_create("Pts", 'r', (max_win-min_win+1), 2);
 if (Pts == NULL) {
   scicos_set_block_error(-16);
   return;
 }

 qx = Pts->R;
 qy = Pts->R + (max_win-min_win+1);

 /* initial values */
 for(i=0;i<(max_win-min_win+1);i++) {
   qx[i] = xmin + i*stepf;
   qy[i] = 0.;
 }

 curve = nsp_qcurve_create("curve", -1, 0, 0, 1, qcurve_std, Pts, NULL, -1, -1, NULL);
 if (curve == NULL) {
   scicos_set_block_error(-16);
   return;
 }

 curve->obj->start = 0;
 curve->obj->last  = max_win-min_win;

 /* insert the new curve */
 if (nsp_axes_insert_child(Axe, (NspGraphic *)curve, FALSE) == FAIL) {
   scicos_set_block_error(-16);
   return;
 }

 /* updates the axes scale */
 frect[0] = xmin; frect[1] = ymin;
 frect[2] = xmax; frect[3] = ymax;
 nsp_strf_axes(Axe, frect, '5');
 memcpy(Axe->obj->frect->R, frect, 4*sizeof(double));
 memcpy(Axe->obj->rect->R, frect, 4*sizeof(double));

 /* set grid, log scale and etc */
 Axe->obj->grid  = 0;
 Axe->obj->axes  = 1;
 Axe->obj->xlog  = FALSE;
 Axe->obj->ylog  = FALSE;
 if(xlog == 1) { /*set x logflag if needed*/
   Axe->obj->xlog  = TRUE;
 }
 if(ylog == 1) { /*set y logflag if needed*/
   Axe->obj->ylog  = TRUE;
 }
 Axe->obj->fixed = TRUE;

 /* title */
 Axe->obj->title = nsp_string_copy(title);

 /* axis labels */
 Axe->obj->x = nsp_string_copy(x_label);
 Axe->obj->y = nsp_string_copy(y_label);

 /* keep a copy in case Axes is destroyed during simulation
  * axe is a by reference object
  */
 Axe1 = nsp_axes_copy(Axe);
 if (Axe1 == NULL) {
   scicos_set_block_error(-16);
   return;
 }

 /* store Axes1 ptr in work st */
 work->Axes=Axe1;

 /* draw initial axe */
 nsp_axes_invalidate((NspGraphic *)work->Axes);
}

/* main routine */
void pspec_scope(scicos_block *block,int flag)
{
 /* graphic ptr variables */
 Cell *Loc;
 NspList *L;
 NspQcurve *curve;
 NspMatrix *M;
 double *qy;

 /* counter variables */
 int i;

 double zero=0.0;
 void *bidon=NULL; /*a void parameter for calc_win*/

 double eps=1e-40;

 /*variables to get block ptr*/
 int *ipar;
 double *rpar;
 double *u1;

 /*variables of the pspec method*/
 int count;
 int n_call;
 int n_sec;
 int sec_l;
 int sec_s;
 int ptr_u;
 double idsec_l; /* to disable division of constant in loops */

 /*freq step*/
 double stepf;
 double Tse;
 double fdenorm;
 int fftsiz;
 double steplog=0.;

 /*graphic indices variables*/
 int min_win;
 int max_win;

 /*ptr of the struct of that block*/
 work_struct *work;

 /*fft variables*/
 int isn=-1; /*sign of the fft*/
 int ffttyp;
 int wtyp;
 int n_work;
 fft_pr_struct fft_pr;

 /*some scope parameters*/
 int yunit; /*comput log() on y axis*/
 int scaltyp; /*0:ymin/ymax/1:autoscale*/
 /* int view_in; */ /*view input buffer*/

 /*autoscale variables*/
 double min_y;
 double max_y;
 double y_height;
 double y_delta;
 double y_up;
 double y_down;
 double fact=0.4;

 /*get block ptr*/
 ipar = block->ipar;
 rpar = block->rpar;
 u1   = block->inptr[0];

 /*get ipar variables*/
 sec_l  = ipar[0];
 sec_s  = ipar[1];
 n_sec  = ipar[2];

 ffttyp = ipar[3];
 n_work = ipar[4]; /*size of the working array is computed
                     *by the scilab interfacing function
                     *of that block*/
 wtyp    = ipar[7];
 yunit   = ipar[9];
 scaltyp = ipar[10];
 /* view_in = ipar[11]; */

 /*ptr of the struct of that block*/
 work   = (work_struct*) *block->work;

 /*get counters value*/
 count  = (int)block->z[0];
 n_call = (int)block->z[1];

 /*freq step*/
 fftsiz  = ipar[0];
 Tse     = rpar[0];
 fdenorm = rpar[2];
 /* compute freq indices of the scope */
 if (fdenorm<=0.) fdenorm = 1.;
 stepf = 1. / ( ((double)fftsiz) * Tse) * fdenorm;

 idsec_l = 1/((double)sec_l);

 if(flag==4) { /*flag 4*/
  /*get campaign of allocation*/
  if ((work=(work_struct*) /*my_st*/
        scicos_malloc(sizeof(work_struct))) == NULL) {
         set_block_error(-16); return;
  }
  work->z_t   = NULL;
  work->win   = NULL;
  work->r_io  = NULL;
  work->i_io  = NULL;
  work->r_out = NULL;
  work->work  = NULL;

  if ((work->z_t=(double*) /*array of input time domain buffer*/
        scicos_malloc(sizeof(double)*sec_l)) == NULL) {
         set_block_error(-16); return;
  }
  if ((work->win=(double*) /*array of coef of window*/
        scicos_malloc(sizeof(double)*sec_l)) == NULL) {
         set_block_error(-16); return;
  }
  if ((work->r_io=(double*) /*array of real part for fft*/
        scicos_malloc(sizeof(double)*sec_l)) == NULL) {
         set_block_error(-16); return;
  }
  if ((work->i_io=(double*) /*array of imaginary part for fft*/
        scicos_malloc(sizeof(double)*sec_l)) == NULL) {
         set_block_error(-16); return;
  }
  if ((work->r_out=(double*) /*array of output freq buffer*/
        scicos_malloc(sizeof(double)*sec_l)) == NULL) {
         set_block_error(-16); return;
  }
  if (ffttyp==1) { /*if dfftmx is used then set a working array*/
    if ((work->work=(double*)
        scicos_malloc(sizeof(double)*n_work)) == NULL) {
       set_block_error(-16); return;
    }
  }

  /*init. autoscale y axis bound*/
  work->y_up=1;
  work->y_down=eps;
  work->y_delta=0;

  /*store the coefficient of the window */
  calc_win_c(&sec_l,&wtyp,bidon,work->win);

 /*init. vectors with zero*/
  setd_c(&sec_l,&zero,work->r_out);
  setd_c(&sec_l,&zero,work->r_io);
  setd_c(&sec_l,&zero,work->i_io);

  *block->work=(void *)work; /*store my_st in *block->work*/

  specview_draw(block); /*call specview_draw for initialization*/
 }

 else if(flag==2) { /*flag 2*/

  if ((count + block->insz[0])<sec_l) {
   for(i=0;i<block->insz[0];i++) { /*store new samples in the input buffer*/
     work->z_t[count + i] = u1[i];
   }
   count += block->insz[0];/*update input buffer counter*/
  }
  else {

   ptr_u = 0;

   /* compute auto-correlated spectrum */
   while( (count + (block->insz[0]-ptr_u)) >= sec_l) {
    for(i=ptr_u ;i<(sec_l-count) ;i++) { /*store new samples in the input buffer*/
     work->z_t[count+i] = u1[i];
    }
    ptr_u += sec_l-count;
    /*if in buffer is full then compute fft */
    count=sec_l-sec_s; /*update buffer counter*/
    /*copy input buffer in real part of
     * fft computation array
     */
    copyd_c(&sec_l,work->z_t,work->r_io);

    /*set imaginary vector to zero*/
    setd_c(&sec_l,&zero,work->i_io);

    /* initialize fft_pr*/
    fft_pr.lfft   = sec_l;
    fft_pr.isn    = isn;
    fft_pr.ffttyp = 2;
    fft_pr.nwork  = n_work;
    fft_pr.work   = work->work;

    /* call pspect_c */
    pspect_c(sec_l,n_sec,&fft_pr, \
               &n_call,work->win,work->r_io,work->i_io,work->r_out);

    /*if average counter cross user average param
     *then display spectrum
     */
    if (n_call==0) {

      /* get indices of the x_axis */
      min_win = work->minmax_win[0];
      max_win = work->minmax_win[1];

      /* Axes was destroyed during simulation */
      if (work->Axes->obj->ref_count <= 1) {
       return;
      }

      L     = work->Axes->obj->children; /* qcurves list */
      Loc   = L->first;
      curve = (NspQcurve *) Loc->O;
      M     = curve->obj->Pts;
      qy    = M->R + M->m;

      if (scaltyp==1) {
        /***********/
        /*autoscale*/
        /***********/
        if (yunit>=1) { /*plot 10*log10(y)*/
          if (yunit==2) {/*plot 10*log10(y)-10*log10(stepf)*/
            steplog = 10*log10(stepf);
          }
          /*def min max value*/
          min_y=10*log10(work->r_out[min_win+1] * idsec_l) - steplog;
          max_y=min_y;

          /*store value of spectrum sample in NG polyline*/
          for(i=min_win+1;i<max_win;i++) {
            qy[i-min_win] = 10*log10(work->r_out[i] * idsec_l) - steplog;
            min_y=min(min_y,qy[i-min_win]);
            max_y=max(max_y,qy[i-min_win]);
          }
          qy[0] = 10*log10(work->r_out[min_win] * idsec_l) - steplog;
          qy[max_win-min_win] = 10*log10(work->r_out[max_win] * idsec_l) - steplog;

          /*set minimal y axis bound*/
          if (min_y<10*log10(eps)) {
            min_y=10*log10(eps);
          }

          y_height = max_y - min_y;
          y_delta  = min_y + y_height/2; /*also equal to max_y-y_height/2*/
          y_down   = y_delta - (1+fact) * y_height/2;
          y_up     = y_delta + (1+fact) * y_height/2;

         if (  (((1+fact) * y_height/2) > (1+fact) * work->y_delta) || \
               (((1+fact) * y_height/2) < (1-fact) * work->y_delta) ) {
            work->y_delta = ( (1+fact) * y_height/2);
            work->y_up = y_up;
            work->y_down = y_down;
          }

        }
        else { /*plot y*/
          /*def min max value*/
          min_y=work->r_out[min_win+1] * idsec_l;
          max_y=min_y;
          /*store value of spectrum sample in NG polyline*/
          for(i=min_win+1;i<max_win;i++) {
            qy[i-min_win] = work->r_out[i] * idsec_l;
            min_y=min(min_y,qy[i-min_win]);
            max_y=max(max_y,qy[i-min_win]);
          }
          qy[0] = work->r_out[min_win] * idsec_l;
          qy[max_win-min_win] = work->r_out[max_win] * idsec_l;

          if (min_y<eps) min_y=eps;
          max_y = log10(max_y); /*change to log scale to disable error computation*/
          min_y =  log10(min_y);

          y_height = max_y - min_y;
          y_delta  = min_y + y_height/2; /*also equal to max_y-y_height/2*/
          y_down   = y_delta - (1+fact) * y_height/2;
          y_up     = y_delta + (1+fact) * y_height/2;

          if ( (((1+fact) * y_height/2) > (1+fact) * work->y_delta) || \
               (((1+fact) * y_height/2) < (1-fact) * work->y_delta) ) {
            work->y_delta = pow(10,(((1+fact) * y_height/2)));
            work->y_up = pow(10,(y_up));
            work->y_down = pow(10,(y_down));
          }
        }

        work->Axes->obj->rect->R[1] = work->y_down;
        work->Axes->obj->rect->R[3] = work->y_up;
      }
      else { /*scale given by ymin/max*/
        if (yunit>=1) { /*plot 10*log10(y)*/
          if (yunit==2) {/*plot 10*log10(y)-10*log10(stepf)*/
            steplog = 10*log10(stepf);
          }
          /*store value of spectrum sample in NG polyline*/
          for(i=min_win;i<=max_win;i++) {
            qy[i-min_win] = 10*log10(work->r_out[i] * idsec_l) - steplog;
          }
        }
        else { /*plot y*/
          /*store value of spectrum sample in NG polyline*/
          for(i=min_win;i<=max_win;i++) {
            qy[i-min_win] = work->r_out[i] * idsec_l;
          }
        }
      }

      nsp_axes_invalidate((NspGraphic *)work->Axes);
      /***** end of the display part ****/

    }
    /*shift input buffer*/
    copyd_c((i=sec_l-sec_s,&i),&work->z_t[sec_s],&work->z_t[0]);
   }
   /*update input buffer*/
   if(ptr_u!=block->insz[0]) {
     for(i=ptr_u;i<block->insz[0];i++) {
       work->z_t[count+i-ptr_u] = u1[i];
     }
     count += (block->insz[0]-ptr_u);
   }
  }

  /*store counters in my_st*/
  block->z[0]=count;
  block->z[1]=n_call;
 }

 else if (flag==5) { /*flag 5*/
   /* Axes was not destroyed during simulation */
   if (work->Axes->obj->ref_count >= 1) {
     nsp_axes_invalidate((NspGraphic *)work->Axes);
     nsp_axes_destroy(work->Axes);
   }
  /*free fields of the struct of that block*/
  if (work!=NULL) {
    scicos_free(work->z_t);
    scicos_free(work->win);
    scicos_free(work->r_io);
    scicos_free(work->i_io);
    scicos_free(work->r_out);
    scicos_free(work->work);
  }
  scicos_free(work);
 }

}

