/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* z_buft41 Scicos discrete event memory block
 * Type 4 simulation function ver 1.0 - scilab-4.1
 * 31 Mars 2007 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: z_buft41.c,v $
 * Revision 1.1  2016-07-20 14:00:21  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:52  layec
 * add
 *
 */
#include <scicos/scicos_block.h>

#include <modnum_lib.h>

#if WITH_NSPSRC
#include <nsp/machine.h>
#else
#include <machine.h>
#endif

/* Cette fonction de simulation est un buffer qui mmorise
 * les dates de dclenchement dans le vecteur des tats discrets z
 * si le paramtre entier ipar[1] est plac  1 alors la
 * fonction calcul les carts temporels entre chaques activation
 *
 * entre rgulire : nant
 * sortie rgulire : nant
 * entre vnementielle : Date de dclenchement
 * sortie vnementielle : nant
 *
 * tat discret : z[0] : compteur chantillon
 *                z[1] : date d'activation prcdente
 *                z[2..2+n-1] : mot mmoire
 *
 * paramtre entier : ipar[0] : longueur du mot mmoire (n)
 *                    ipar[1]  : option : 1 : calcul du jitter period
 */

/*prototype*/
void z_buft41(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 int j,n;
 double t;

 /*le flag 2 mmorise t dans z[]*/
 if(flag==2)
 {
  /*rcupration de la longeur du mot mmoire*/
  n=block->ipar[0];
  /*rcupre valeur du compteur chantillons*/
  j=(int)block->z[0];
  /*rcupre la date d'activation*/
  t=get_scicos_time();
  /*met en mmoire dans z ou calcule la priode jitter*/
  if (block->ipar[1]==1) {
    block->z[2+j]=t-block->z[1];
  }
  else {
    block->z[2+j]=t;
  }
  /*met en mmoire la date d'activation*/
  block->z[1]=t;
  /*incrmente compteur chantillons*/
  j++;
  /*Test valeur compteur chantillons*/
  if(j==n) j=0;
  /*mmorise compteur dans z[n+1]*/
  block->z[0]=(double)j;
 }
}
