/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* genmsymb Scicos Generic Symbol Generator
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 5 janvier 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: gensymb.c,v $
 * Revision 1.2  2016-07-22 11:56:16  jpc
 * updates
 *
 * Revision 1.1  2016-07-20 14:00:21  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:52  layec
 * add
 *
 */

#include <math.h>

#include <scicos/scicos_block.h>

#include "modnum_lib.h"

#if WITH_NSPSRC
#include <nsp/machine.h>
#else
#include <machine.h>
#endif

/* entres rgulires      : u[0..insz[0]-1]  : vecteur des numros symboles
 * sorties rgulires      : y1[0..nech*insz[0]-1] : vecteur des composantes I
 *                           y2[0..nech*insz[0]-1] : vecteur des composantes Q
 * entre vnementielles  : instants de dclenchement.
 * sortie vnementielle   : nant
 * paramtres entiers      : ipar[0] :nombre de symbole  gnrer
 *                           ipar[1] :longueur en bit des symboles
 *                           ipar[2] :m nombre d'tats
 *                           ipar[3] :nech nombre d'chantillons
 *                           ipar[4] :compteur initial du surchantillonneur
 *                           ipar[5] :nbr de coef du filtre
 *                           ipar[6] :taille du vecteur en puissance de 2 (pour filtre avec fft842)
 *                           ipar[7] :flag gnrateur squentiel(0:scalaire/1:Vectoriel)
 *
 * paramtres rels        : rpar[0..nb_coef-1] : vecteur de la rponse impulsionnelle
 * tat discret            : flag_seq=0
 *                            z[0..nb_coef-1]         : vecteur du mot mmoire voie I
 *                            z[nb_coef..2*nb_coef-1] : vecteur du mot mmoire voie Q
 *                           flag_seq=1
 *                            z[0] : compteur sur-chantillonneur
 *                            z[1..nb_coef]         : vecteur du mot mmoire voie I
 *                            z[nb_coef+1..2*nb_coef] : vecteur du mot mmoire voie Q
 */

/*prototype*/
void gensymb(scicos_block *block,int flag)
{
 /*Dclaration des variables*/
 double *y1,*y2=NULL;
 /*double *u;*/
 double *z;
 int ny;
 int k;
 int nu,n,m,m1,nech,init_c,nb_coef;
 int mu=1;
 int opt;
 int flag_seq;
 double *z__;
 double *zi1_r=NULL, *zi1_i=NULL, *zi2_r=NULL, *zi2_i=NULL, *yi_r=NULL, *yi_i=NULL;
 double *zq1_r=NULL, *zq1_i=NULL, *zq2_r=NULL, *zq2_i=NULL, *yq_r=NULL, *yq_i=NULL;

 /*fft variables*/
 fft_pr_struct fft_pr;

 /*Rcupration des valeurs des variables*/
 ny=block->outsz[0];
 nu=block->ipar[0];
 n=block->ipar[1];
 m=block->ipar[2];
 nech=block->ipar[3];
 init_c=block->ipar[4];
 nb_coef=block->ipar[5];
 m1=block->ipar[6];
 flag_seq=block->ipar[7];

 /*Rcupration des adresses des ports rguliers*/
 y1=(double *)block->outptr[0];
 if (n>1) y2=(double *)block->outptr[1];
 z=block->z;

 /* init : fait l'allocation de work */
 if (flag==4) {
  if(!flag_seq) {
   /*fprintf(stderr,"n=%d\n",n);*/
   /*fait un test sur filtre*/
   if(m1!=0) {
    /*fait un test sur nombre d'tat*/
    if(n>1) {
     /*Allocation de 2*(3*2) vecteurs de taille m1*/
     if ((*block->work=scicos_malloc(sizeof(double)*(m1*2*(2*3))))== NULL) {
      set_block_error(-16);
      return;
     }
    }
    /*Allocation de (3*2) vecteurs de taille m1*/
    else {
     if ((*block->work=scicos_malloc(sizeof(double)*(m1*(2*3))))== NULL) {
      set_block_error(-16);
      return;
     }
    }
   }
  }
 }
 /* Le flag 1 calcule la valeur de I et de Q en fonction
  * du numro symbole u[] et du nombre du nombre d'tat m
  */
 else if (flag==1) {

  /********/
  /*Scalar*/
  /********/
  if(flag_seq)
  {
   /*fprintf(stderr,"flag_seq=%d,n=%d,nu=%d,nb_coef=%d,nz=%d\n",flag_seq,n,nu,nb_coef,block->nz);*/
   if((int)block->z[0]==nech)
   {
    block->z[0]=1; /*RAZ compteur*/
    if (nu>0)
    {
     if (n>1)
     {
      /*Appel genint*/
      genint_c(&nu,&mu,&n,(k=1,&k),&y1[0]);

      /*Appel routine modpsk*/
      modpsk_c(&nu,&mu,&m,&y1[0],&y1[0],&y2[0]);
     }
     else genint_c(&nu,&mu,&n,(k=0,&k),&y1[0]);
    }
    else
    {
     /*dclaration*/
     double *u;
     /*Rcupration de l'adresse du port d'entre*/
     u=(double *)block->inptr[0];
     /*rcupration de la taille du port d'entre*/
     nu=block->insz[0];
     /*Recopie u[] dans y1[]*/
     y1[0]=u[0];
     if (n>1)
      /*Appel routine modpsk*/
      modpsk_c(&nu,&mu,&m,&y1[0],&y1[0],&y2[0]);
    }
   }
   else
   {
    y1[0]=0;
    if(n>1) y2[0]=0;

    block->z[0]++; /*incrmente compteur*/
   }
   if (nu<=0)
    /*rcupration de la taille du port d'entre*/
    nu=block->insz[0];

   if (nb_coef!=0)
   {
    /*Appel nfilter : TODO correction flag1/2*/
    nfilter_c(&nu,&mu,&nb_coef,&y1[0],&block->rpar[0],&y1[0],&block->z[1]);
    if (n>1) {
      nfilter_c(&nu,&mu,&nb_coef,&y2[0],&block->rpar[0],&y2[0],&block->z[nb_coef+1]);
    }
   }
  }
  /********/
  /*Vector*/
  /********/
  else
  {
   /*fprintf(stderr,"n=%d\n",n);*/
   if(m1!=0) /*fait un test sur filtre*/
   {
    if(n>1) /*fait un test sur nombre d'tat*/
    {
     /*Allocation de 2*(3*2) vecteurs de taille m1*/
     if ((*block->work=scicos_malloc(sizeof(double)*(m1*2*(2*3))))== NULL)
     {
      set_block_error(-16);
      return;
     }
    }
    else
    {
     /*Allocation de (3*2) vecteurs de taille m1*/
     if ((*block->work=scicos_malloc(sizeof(double)*(m1*(2*3))))== NULL)
     {
      set_block_error(-16);
      return;
     }
    }
    /*Rcupration de l'adresse de dpart du vecteur allou*/
    z__=*block->work;

    /*Dclaration de pointeurs auxiliaires*/
    zi1_r = &(z__[0])   ; zi1_i = &(z__[m1]);   /*vecteur voie i du complexe 1*/
    zi2_r = &(z__[2*m1]); zi2_i = &(z__[3*m1]); /*vecteur voie i du complexe 2*/
    yi_r  = &(z__[4*m1]); yi_i  = &(z__[5*m1]); /*vecteur voie i du complexe rsultat*/
    if(n>1)
    {
     zq1_r = &(z__[6*m1]); zq1_i = &(z__[7*m1]);   /*vecteur voie q du complexe 1*/
     zq2_r = &(z__[8*m1]); zq2_i = &(z__[9*m1]); /*vecteur voie q du complexe 2*/
     yq_r  = &(z__[10*m1]); yq_i  = &(z__[11*m1]); /*vecteur voie q du complexe rsultat*/
    }
   }
   else
   {
    zi1_r = &(y1[0]); yi_r  = &(z[0]); /*vecteur voie i*/
    if(n>1)
    {
     zq1_r = &(y2[0]); yq_r  = &(z[nb_coef]); /*vecteur voie q*/
    }
   }
   if (nu>0) /*fait un test sur prsence port entre*/
   {
    /*Appel genint_c*/
    if(n>1) genint_c(&nu,&mu,&n,(k=1,&k),&zi1_r[0]);
    else genint_c(&nu,&mu,&n,(k=0,&k),&zi1_r[0]);
   }
   else if(nu==0)
   {
    /*dclaration*/
    double *u;
    /*Rcupration de l'adresse du port d'entre*/
    u=(double *)block->inptr[0];
    /*rcupration de la taille du port d'entre*/
    nu=block->insz[0];
    /*Recopie u[] dans zi1_r[]*/
    copyd_c(&nu,&u[0],&zi1_r[0]);
   }

   if(n>1)
   /*Appel routine modpsk*/
    modpsk_c(&nu,&mu,&m,&zi1_r[0],&yi_r[0],&yq_r[0]);
   else
    /*Recopie zi1_r[] dans yi_r[]*/
    copyd_c(&nu,&zi1_r[0],&yi_r[0]);

   /*Appel routine surecht_c*/
   surecht_c((opt=1,&opt),&nu,&mu,&nech,&init_c,&yi_r[0],&zi1_r[0]);
   if(n>1) surecht_c((opt=1,&opt),&nu,&mu,&nech,&init_c,&yq_r[0],&zq1_r[0]);

   if(m1!=0)
   {
    /*Recopie rpar[] dans zi2_r[] et zq2_r*/
    copyd_c(&nb_coef,&(block->rpar[0]),&zi2_r[0]);
    if(n>1) copyd_c(&nb_coef,&(block->rpar[0]),&zq2_r[0]);

    /* initialize fft_pr*/
    fft_pr.lfft   = m1;
#if WITH_FFTW
    fft_pr.ffttyp = 2;
#else
    fft_pr.ffttyp = 0;
#endif

    /*Appel convolr_c*/
    convolr_c((k=nu*nech,&k),&nb_coef,&m1,&zi1_r[0],&zi1_i[0],&zi2_r[0],&zi2_i[0],&yi_r[0],&yi_i[0],&z[0],&fft_pr);
    if(n>1) convolr_c((k=nu*nech,&k),&nb_coef,&m1,&zq1_r[0],&zq1_i[0],&zq2_r[0],&zq2_i[0],&yq_r[0],&yq_i[0],&z[nb_coef],&fft_pr);

    /*Recopie y_r[] dans y[]*/
    copyd_c(&ny,&yi_r[0],&y1[0]);
    if(n>1) copyd_c(&ny,&yq_r[0],&y2[0]);
   }
  }
 }
 /*Terminaison */
 else if (flag==5) {
  if(!flag_seq) {
   /*fait un test sur filtre*/
   if(m1!=0) {
    /*Libre mmoire alloue*/
    scicos_free(*block->work);
   }
  }
 }
}
