/* Modnumlib Scicos interfacing function
 * Copyright (C) 2009-2015 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/* ndelay Scicos vector delay block
 * Type 2 simulation function ver 1.0 - scilab-2.6&2.7
 * 18 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log: ndelay.c,v $
 * Revision 1.1  2016-07-20 14:00:21  jpc
 * updates
 *
 * Revision 1.1  2015-12-25 15:47:52  layec
 * add
 *
 */

#include <modnum_lib.h>

#if WITH_NSPSRC
#include <nsp/machine.h>
#else
#include <machine.h>
#endif

/* Cette fonction de simulation prsente la valeur du vecteur u[]
 * sur le vecteur de sortie y[] aprs n priode de dclenchements.
 * Elle se compose d'un vecteur de registre  dcalage dont chaque registre
 * a une taille donne par le paramtre entier ipar[0..Nu-1] o Nu reprsente
 * la taille des vecteurs d'entres/sorties.
 *
 * entre rgulire u[0..Nu-1] : vecteur d'entre
 * sortie rgulier y[0..Nu-1] : vecteur de sortie
 * entre vnementielle : priode de dclenchement
 * sortie vnementielle : nant
 *
 * paramtre entier ipar[0..Nu-1] :  vecteur des nombres de retard
 * tats discrets : z[0..ipar[0]-1] : buffer entre u[0]
 *                  z[ipar[0]..ipar[1]-1] : buffer entre u[1]
 *                 ...........
 *                 z[ipar[Nu-2]..ipar[Nu-1] : buffer entre u[Nu-1]
 */

/*prototype*/
void ndelay(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
            ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
 /*Dclaration*/
 double *y;
 double *u;
 int nu,i,j,n;

 /*rcupration des adresses des ports rguliers*/
 y = (double *)outptr[0];
 u = (double *)inptr[0];

 /*rcupration de la taille du port d'entre*/
 nu=insz[0];

 switch (*flag)
 {
  /* Le flag 1 place la premire valeur du buffer
   * dans le registre de sortie y[]
   */
  case 1 :
  {
   n = 0;
   for (i=0;i<nu;i++)
   {
    y[i]=z[n];
    n=n+ipar[i];
   }
   break;
  }

  /* Le flag 2 fait un dcalage  gauche des buffers
   * et place la valeur de u[]  la sortie des buffers
   */
  case 2 :
  {
   /*Dcale buffer*/
   n = 0;
   for (i=0;i<nu;i++)
   {
    for (j=0;j<ipar[i];j++) z[n+j]=z[n+j+1];
    n=n+ipar[i];
   }
   /*Ajoute nouvelle valeur dans buffer*/
   n = 0;
   for(j=0;j<nu;j++)
   {
    n=n+ipar[j];
    z[n-1]=u[j];
   }
   break;
  }
  break;
 }
}
