#######################################################################
#                                                                     #
#                               Simport                               #
#                                                                     #
#                   Pierre Weis, INRIA Rocquencourt                   #
#                                                                     #
#  Copyright 2010-2019,                                               #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved.                                               #
#                                                                     #
#  This file is distributed under the terms of the BSD License.       #
#                                                                     #
#######################################################################

# $Id: External.mk,v 1.1 2019-05-10 13:58:19 jpc Exp $

# All the definitions in this file can be redefined by the user.
#
# Ideally, the configure script shell should handle arguments to set up
# all those bindings at software configuration time.
#
# The minimum required configuration settings to comfortably tune the
# compilation of the software requires settings for
# OCAML_PREFIX_DIR
# (since OCaml packages could install commands in /usr, /usr/local, ...)
# OCAML_BIN_DIR
# (since OCaml commands may be installed anywhere by the user)
# OCAML_LIB_DIR
# (since OCaml libraries could be installed anywhere by the user)
# (we also need to install internal libraries in an alien place,
#  namely a sub directory of the software hierarchy)
#
# For cross-platform compilation we need configuration bindings for
# compiled files extensions
# LIB_EXT with default a
# DLL_EXT with default so
# OBJ_EXT with default o
# EXE_EXT with default ""

# For packaging the software we should have a setting for all the OCaml
# compilers and tools.
# (since when cross compiling the Caml compiler basename is no more ocamlc)
# CAMLC_BYT_COMPILER
# CAMLC_BIN_COMPILER
# CAMLC_YAC_COMPILER
# CAMLC_LEX_COMPILER
# CAMLC_DOC_COMPILER
# CAMLC_DEP_COMPILER
# CAMLC_MKLIB_COMPILER
# CAMLC_MKTOP_COMPILER
#
# Having a binding for all the OCaml compilers and tools is the best way to
# go, since it is a definitive answer.
#
# To ease multi platform compilation, configuration should handle bindings
# for the C compiler
# CC
# and C internal libraries
#
# ZLIB_LIB with default -lz
# ZLIB_LIB_DIR with default /usr/local/lib
# ZLIB_INCLUDE with default /usr/local/include
# FLEX_LINK_DIR with default ""
# PATH=$(FLEX_LINK_DIR):$(PATH)

# Note:
# Using a configuration binding for the additional variable
# OCAML_NAME_PREFIX
# could help packaging, since the configure script could then compute
# all the *_COMPILER bindings mentioned above.
# (since when packaging and cross compiling, the Caml compiler and some of
#  its associated tools gets new basenames with the addition of a prefix;
#  for instance, ocamlc is named ${OCAML_NAME_PREFIX}ocamlc)

# Section 1: Directory definitions
##################################

# Directory definitions for Caml compilers and tools
OCAML_PREFIX_DIR=/usr/local

OCAML_BIN_DIR=$(OCAML_PREFIX_DIR)/bin
OCAML_LIB_DIR=$(OCAML_PREFIX_DIR)/lib

# Directory definitions for Caml source file generators
CAMLC_PPR_BIN_DIR=$(OCAML_BIN_DIR)
CAMLC_PPR_LIB_DIR=$(OCAML_LIB_DIR)
CAMLC_MOCA_BIN_DIR=$(OCAML_BIN_DIR)

# Directory definitions for general purpose file generators
HTMLC_BIN_DIR=$(OCAML_BIN_DIR)
BH_BIN_DIR=$(OCAML_BIN_DIR)

# Section 2: command definitions
################################

# OCaml commands names
OCAML_NAME_PREFIX=

CAMLC_BYT_COMPILER=$(OCAML_BIN_DIR)/$(OCAML_NAME_PREFIX)ocamlc
CAMLC_BIN_COMPILER=$(OCAML_BIN_DIR)/$(OCAML_NAME_PREFIX)ocamlopt
CAMLC_YAC_COMPILER=$(OCAML_BIN_DIR)/ocamlyacc
CAMLC_LEX_COMPILER=$(OCAML_BIN_DIR)/ocamllex
CAMLC_DEP_COMPILER=$(OCAML_BIN_DIR)/$(OCAML_NAME_PREFIX)ocamldep
CAMLC_DOC_COMPILER=$(OCAML_BIN_DIR)/ocamldoc
CAMLC_MKLIB_COMPILER=$(OCAML_BIN_DIR)/$(OCAML_NAME_PREFIX)ocamlmklib
CAMLC_MKTOP_COMPILER=$(OCAML_BIN_DIR)/$(OCAML_NAME_PREFIX)ocamlmktop

# Auxiliary compilers/file generators for Caml source files
CAMLC_PPR_COMPILER=$(CAMLC_PPR_BIN_DIR)/ocamlprintc
CAMLC_MOCA_COMPILER=$(CAMLC_MOCA_BIN_DIR)/mocac

# Auxiliary general file generators
HTMLC_COMPILER=$(HTMLC_BIN_DIR)/htmlc
BH_COMPILER=$(BH_BIN_DIR)/bh

# C compiler names and definitions
CC=gcc

# Zlib directories
# The option to link the C Zlib library. Usually -lz
ZLIB_LIB=-lz

# The directory containing the C Zlib library (libz.a or libz.so)
ZLIB_LIB_DIR=/usr/local/lib

# The directory containing the C Zlib header file (zlib.h)
ZLIB_INCLUDE=/usr/local/include

# Windows specific
# FLEX_LINK_DIR=
# PATH=$(FLEX_LINK_DIR):$(PATH)

# This file has not to be recursively rebuilt by implicit make rules.
$(CONFIG_DIR)/External.mk: ;
