#######################################################################
#                                                                     #
#                               Simport                               #
#                                                                     #
#                   Pierre Weis, INRIA Rocquencourt                   #
#                                                                     #
#  Copyright 2010-2019,                                               #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved.                                               #
#                                                                     #
#  This file is distributed under the terms of the BSD License.       #
#                                                                     #
#######################################################################

# $Id: Simport.mk,v 1.1 2019-05-10 13:58:19 jpc Exp $

# This Makefile should be included at the end of a Makefile that handles a
# set of Simulink files (to translate them to nsp or scicoslab).
# Simply write at the end of your Makefile:
# include <path_to_Simport.mk>/Simport.mk

.SUFFIXES: .mdlo .mdl .mdlio .mdli
.SUFFIXES: .m .oct .sci .nsp .hml .oml .mo .mtlb .slx
.SUFFIXES: .cose .sime

.mdl.mdlo:
	@echo "Compiling Simulink implementation $<" && \
	$(SIMPORT) $<
# Should use option -c of the compiler (no target code generation)
#	$(SIMPORT) -c $<

.mdli.mdlio:
	@echo "Compiling Simulink interface $<" && \
	$(SIMPORT) $<

.mdl.sci:
	@echo "Importing Simulink file $< to ScicosLab" && \
	$(SIMPORT) -tl scicoslab $<

.mdl.nsp:
	@echo "Importing Simulink file $< to Nsp" && \
	$(SIMPORT) -tl nsp $<

.mdl.oct:
	@echo "Importing Simulink file $< to Octave" && \
	$(SIMPORT) -tl octave $<

.mdl.hml:
	@echo "Importing Simulink file $< to Hypermath" && \
	$(SIMPORT_PRO) -tl hypermath $<

.mdl.oml:
	@echo "Importing Simulink file $< to Oml" && \
	$(SIMPORT_PRO) -tl oml $<

.mdl.mo:
	@echo "Importing Simulink file $< to Oml" && \
	$(SIMPORT_MODELICOS) -tl modelica $<

.slx.mdlo:
	@echo "Compiling Simulink Slx file $<" && \
	$(SIMPORT) $<

# Should use option -c of the compiler (no target code generation)
#	$(SIMPORT) -c $<

.slx.sci:
	@echo "Importing Simulink Slx file $< to ScicosLab" && \
	$(SIMPORT) -tl scicoslab $<

.slx.nsp:
	@echo "Importing Simulink Slx file $< to Nsp" && \
	$(SIMPORT) -tl nsp $<

.slx.oct:
	@echo "Importing Simulink Slx file $< to Octave" && \
	$(SIMPORT) -tl octave $<

.slx.hml:
	@echo "Importing Simulink Slx file $< to Hypermath" && \
	$(SIMPORT_PRO) -tl hypermath $<

.slx.oml:
	@echo "Importing Simulink Slx file $< to Oml" && \
	$(SIMPORT_PRO) -tl oml $<

.slx.mo:
	@echo "Importing Simulink Slx file $< to Modelica" && \
	$(SIMPORT_MODELICOS) -tl modelica $<

.mtlb.sci:
	@echo "Importing Matlab file $< to ScicosLab" && \
	$(SIMPORT) -target-language scicoslab $<

.mtlb.nsp:
	@echo "Importing Matlab file $< to Nsp" && \
	$(SIMPORT) -target-language nsp $<

.mtlb.oct:
	@echo "Importing Matlab file $< to Octave" && \
	$(SIMPORT) -target-language octave $<

.mtlb.m:
	@echo "Importing Matlab file $< to Matlab" && \
	$(SIMPORT) -target-language matlab $<

.mtlb.hml:
	echo "Importing Matlab file $< to Hypermath" && \
	$(SIMPORT_PRO) -target-language hypermath $<

.mtlb.oml:
	@echo "Importing Matlab file $< to Oml" && \
	$(SIMPORT_PRO) -target-language oml $<

.mtlb.mo:
	@echo "Importing Matlab file $< to Modelica" && \
	$(SIMPORT_MODELICOS) -target-language modelica $<

.sime.cose:
	@echo "Importing Simulink embedded file $< to Scicos" && \
	$(SIMPORT) -target-language e-scicos $<

# .mdlo.sci:
# 	@echo "Importing Simulink file $< to ScicosLab" && \
# 	$(SIMPORT) -tl scicoslab $<

# .mdlo.nsp:
# 	@echo "Importing Simulink file $< to Nsp" && \
# 	$(SIMPORT) -tl nsp $<

# .mdlo.oct:
# 	@echo "Importing Simulink file $< to Octave" && \
# 	$(SIMPORT) -tl octave $<

# .mdlo.hml:
# 	@echo "Importing Simulink file $< to Hypermath" && \
# 	$(SIMPORT_PRO) -tl hypermath $<

# .mdlo.oml:
# 	@echo "Importing Simulink file $< to Oml" && \
# 	$(SIMPORT_PRO) -tl oml $<

# .mdlo.mo:
# 	@echo "Importing Simulink file $< to Modelica" && \
# 	$(SIMPORT_MODELICOS) -tl modelica $<

# This file has not to be recursively rebuilt by implicit make rules.
$(CONFIG_DIR)/Simport.mk: ;
