#######################################################################
#                                                                     #
#                               Simport                               #
#                                                                     #
#                   Pierre Weis, INRIA Rocquencourt                   #
#                                                                     #
#  Copyright 2010-2019,                                               #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved.                                               #
#                                                                     #
#  This file is distributed under the terms of the BSD License.       #
#                                                                     #
#######################################################################

# $Id: Objs.mk,v 1.1 2019-05-10 13:58:19 jpc Exp $

# The list of all the Caml compiled objects to make for this project.
# Each source directory has its specific make variable associated
# with the relevant list of object and source files.
#
# This file is included in the main make file of the software after the
# definitions of variables MAIN_DIR and CONFIG_DIR as follows:
#
# MAIN_DIR=.
# CONFIG_DIR=$(MAIN_DIR)/config
#
# In some sense, we could say that variables MAIN_DIR and CONFIG_DIR
# are inherited from $(MAIN_DIR)/Makefile (from Objs.mk point of view) !
#
# After the definition of CONFIG_DIR and before loading this file,
# the file $(CONFIG_DIR)/Config.mk is loaded and provides basic
# directory definitions, namely:
# BIN_DIR=$(MAIN_DIR)/bin
# LIB_DIR=$(MAIN_DIR)/lib
# SRC_DIR=$(MAIN_DIR)/src
# DOC_SRC_DIR=$(MAIN_DIR)/doc_src
# AUTO_DOC_DIR=$(MAIN_DIR)/autodoc/html

# Directory definitions

SIMPORT_INTERNAL_LIBS_PREFIX_INSTALL_DIR=$(SRC_DIR)/internal_libs

SIMPORT_INTERNAL_LIBS_INSTALL_DIR=\
$(SIMPORT_INTERNAL_LIBS_PREFIX_INSTALL_DIR)/lib/ocaml

OCAML_XML_SVN_PARENT_DIR=$(SLX_FILE_FORMAT_PARSING_DIR)
OCAML_XML_SVN_DIR_NAME=ocaml-xml.svn
OCAML_XML_SVN_DIR=$(OCAML_XML_SVN_PARENT_DIR)/$(OCAML_XML_SVN_DIR_NAME)

OCAML_XML_LIB_DIR=$(SIMPORT_INTERNAL_LIBS_INSTALL_DIR)/ocaml-xml

CAMLZIP_CVS_PARENT_DIR=$(SLX_FILE_FORMAT_PARSING_DIR)
CAMLZIP_CVS_DIR_NAME=camlzip.cvs
CAMLZIP_CVS_DIR=$(CAMLZIP_CVS_PARENT_DIR)/$(CAMLZIP_CVS_DIR_NAME)

CAMLZIP_LIB_DIR=$(SIMPORT_INTERNAL_LIBS_INSTALL_DIR)/camlzip

SIMPORT_TEST_SVN_PARENT_DIR=$(MAIN_DIR)
SIMPORT_TEST_SVN_DIR_NAME=simport-test.svn
SIMPORT_TEST_SVN_DIR=$(SIMPORT_TEST_SVN_PARENT_DIR)/$(SIMPORT_TEST_SVN_DIR_NAME)

CONFIGURATION_DIR=$(SRC_DIR)/configuration
SCRIPTS_DIR=$(SRC_DIR)/scripts

INFRA_DIR=$(SRC_DIR)/infra

MDL_DIR=$(SRC_DIR)/mdl
MDL_AST_DIR=$(MDL_DIR)/ast
MDL_DAST_DIR=$(MDL_DIR)/dast
MDL_EXPLICIT_DIR=$(MDL_DIR)/explicit
MDL_SPLIT_DIR=$(MDL_DIR)/split
MDL_ACTION_DIR=$(MDL_DIR)/action
MDL_RENAME_DIR=$(MDL_DIR)/rename
MDL_TRANSLATION_DIR=$(MDL_DIR)/translation
MDL_CODE_DIR=$(MDL_DIR)/code

MDL_GENR_BLOCK_DIR=$(MDL_DIR)/genr_block
MDL_GENR_BLOCK_COMMON_DIR=$(MDL_GENR_BLOCK_DIR)/common
MDL_GENR_BLOCK_SCICOS_DIR=$(MDL_GENR_BLOCK_DIR)/scicos
MDL_GENR_BLOCK_SCICOS_PRO_DIR=$(MDL_GENR_BLOCK_DIR)/scicos_pro
MDL_GENR_BLOCK_MODELICOS_DIR=$(MDL_GENR_BLOCK_DIR)/modelicos

MDL_FILE_FORMAT_DIR=$(SRC_DIR)/mdl_file_format
MDL_FILE_FORMAT_PARSING_DIR=$(MDL_FILE_FORMAT_DIR)/parsing

SLX_DIR=$(SRC_DIR)/slx_file_format
SLX_FILE_FORMAT_DIR=$(SLX_DIR)
SLX_FILE_FORMAT_PARSING_DIR=$(SLX_FILE_FORMAT_DIR)/parsing
SLX_FILE_FORMAT_DAST_DIR=$(SLX_FILE_FORMAT_DIR)/dast
SLX_FILE_FORMAT_EXPLICIT_DIR=$(SLX_FILE_FORMAT_DIR)/explicit
SLX_FILE_FORMAT_TRANSLATING_DIR=$(SLX_FILE_FORMAT_DIR)/translating

OCAML_XML_DIR=$(SLX_FILE_FORMAT_PARSING_DIR)/ocaml-xml
OCAML_XML_SRC_XML_PARSING_DIR=$(OCAML_XML_DIR)/src/xml/parsing

CAMLZIP_DIR=$(SLX_FILE_FORMAT_PARSING_DIR)/camlzip

SIMULINK_EMBEDDED_DIR=$(SRC_DIR)/simulink_embedded

MATLAB_DIR=$(SRC_DIR)/matlab
MATLAB_PARSING_DIR=$(MATLAB_DIR)/parsing
MATLAB_REWRITING_DIR=$(MATLAB_DIR)/rewriting
MATLAB_PRINTING_DIR=$(MATLAB_DIR)/printing
MATLAB_TRANSLATING_DIR=$(MATLAB_DIR)/translating

COMPILER_DIR=$(SRC_DIR)/compiler
DRIVER_DIR=$(SRC_DIR)/driver

# Sub directories of the application

SIMPORT_SUB_DIRS=\
 $(OCAML_XML_LIB_DIR)\
 $(CAMLZIP_LIB_DIR)\
 $(INFRA_DIR)\
 $(CONFIGURATION_DIR)\
 $(EMBEDDED_DIR)\
 $(MATLAB_PARSING_DIR)\
 $(MATLAB_REWRITING_DIR)\
 $(MATLAB_PRINTING_DIR)\
 $(MATLAB_TRANSLATING_DIR)\
 $(MDL_AST_DIR)\
 $(MDL_FILE_FORMAT_PARSING_DIR)\
 $(MDL_DAST_DIR)\
 $(MDL_EXPLICIT_DIR)\
 $(MDL_SPLIT_DIR)\
 $(MDL_ACTION_DIR)\
 $(MDL_RENAME_DIR)\
 $(MDL_TRANSLATION_DIR)\
 $(MDL_CODE_DIR)\
 $(MDL_GENR_BLOCK_COMMON_DIR)\
 $(MDL_GENR_BLOCK_SCICOS_DIR)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)\
 $(SLX_FILE_FORMAT_DAST_DIR)\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)\
 $(COMPILER_DIR)\
 $(DRIVER_DIR)\

# Include files for the compiler

SIMPORT_COMMON_CAML_INCLUDES=\
 -I $(OCAML_XML_LIB_DIR)\
 -I $(CAMLZIP_LIB_DIR)\
 -I $(INFRA_DIR)\
 -I $(CONFIGURATION_DIR)\
 -I $(SIMULINK_EMBEDDED_DIR)\
 -I $(MATLAB_PARSING_DIR)\
 -I $(MATLAB_REWRITING_DIR)\
 -I $(MATLAB_PRINTING_DIR)\
 -I $(MATLAB_TRANSLATING_DIR)\
 -I $(MDL_AST_DIR)\
 -I $(MDL_DAST_DIR)\
 -I $(MDL_EXPLICIT_DIR)\
 -I $(MDL_SPLIT_DIR)\
 -I $(MDL_ACTION_DIR)\
 -I $(MDL_RENAME_DIR)\
 -I $(MDL_TRANSLATION_DIR)\
 -I $(MDL_CODE_DIR)\
 -I $(MDL_GENR_BLOCK_COMMON_DIR)\

SIMPORT_TRAILING_CAML_INCLUDES=\
 -I $(MDL_FILE_FORMAT_PARSING_DIR)\
 -I $(SLX_FILE_FORMAT_PARSING_DIR)\
 -I $(SLX_FILE_FORMAT_DAST_DIR)\
 -I $(SLX_FILE_FORMAT_EXPLICIT_DIR)\
 -I $(SLX_FILE_FORMAT_TRANSLATING_DIR)\
 -I $(COMPILER_DIR)\
 -I $(DRIVER_DIR)\

SIMPORT_CAML_INCLUDES=\
 $(SIMPORT_COMMON_CAML_INCLUDES)\
 -I $(MDL_GENR_BLOCK_SCICOS_DIR)\
 -I $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)\
 -I $(MDL_GENR_BLOCK_MODELICOS_DIR)\
 $(SIMPORT_TRAILING_CAML_INCLUDES)\

#
# External libraries
#

# (An external library is a Caml library provided by the Caml compiler
#  distribution; just need to link the provided compiled library)

# Simport needs the unix library
# EXTERNAL_LIBRARIES=unix
SIMPORT_EXTERNAL_LIBRARIES_BYT_OBJS=unix.cma
SIMPORT_EXTERNAL_LIBRARIES_BIN_OBJS=\
 $(SIMPORT_EXTERNAL_LIBRARIES_BYT_OBJS:.cma=.cmxa)

#
# Internal libraries
#

# (An internal library is a library provided in source form by an external
#  source. Simport incorporates a private source version to break dependency
#  w.r.t. unexpected modifications from the original provider.
#  Need to compile the source version and link the relevant compiled library
#  or list of modules with the application)

# To support slx format, Simport needs to handle zip files and XML files
# INTERNAL_LIBRARIES=ocaml-xml camlzip

# Simport uses regular command 'make; make install' to compile internal
# libraries.
# The make invocation should define a lot of Make variables to ensure proper
# compilation since internal library and Simport should be compiled and
# linked with exactly the same compilers and tools.
#
# We encapsulate the corresponding trickery in the MAKE_INTERNAL_LIBRARY
# Makefile variable below

# The make invocation to compile the internal libraries
MAKE_INTERNAL_LIBRARY=\
 $(MAKE)\
  CC=$(CC)\
  OCAML_BIN_DIR=$(OCAML_BIN_DIR)\
  OCAML_LIB_DIR=$(OCAML_LIB_DIR)\
  CAMLC_BYT_COMPILER=$(CAMLC_BYT_COMPILER)\
  CAMLC_BIN_COMPILER=$(CAMLC_BIN_COMPILER)\
  CAMLC_YAC_COMPILER=$(CAMLC_YAC_COMPILER)\
  CAMLC_LEX_COMPILER=$(CAMLC_LEX_COMPILER)\
  CAMLC_DEP_COMPILER=$(CAMLC_DEP_COMPILER)\
  CAMLC_MKLIB_COMPILER=$(CAMLC_MKLIB_COMPILER)\
  CAMLC_MKTOP_COMPILER=$(CAMLC_MKTOP_COMPILER)\
  CAMLC_DOC_COMPILER=$(CAMLC_DOC_COMPILER)\
  CAMLC_PPR_COMPILER=$(CAMLC_PPR_COMPILER)\
  CAMLC_MOCA_COMPILER=$(CAMLC_MOCA_COMPILER)\
  HTMLC_COMPILER=$(HTMLC_COMPILER)\
  BH_COMPILER=$(BH_COMPILER)\
  LIB_EXT=$(LIB_EXT)\
  DLL_EXT=$(DLL_EXT)\
  OBJ_EXT=$(OBJ_EXT)\
  EXE_EXT=$(EXE_EXT)\
  LDCONF_DIR=../../../../$(SIMPORT_INTERNAL_LIBS_PREFIX_INSTALL_DIR)\
  PREFIX_INSTALL_DIR=../../../../$(SIMPORT_INTERNAL_LIBS_PREFIX_INSTALL_DIR)\

# Directory src/internal_libs/ocaml/ocaml-xml
OCAML_XML_BYT_OBJS=\
 $(OCAML_XML_LIB_DIR)/ocamlxml.cma

OCAML_XML_BIN_OBJS=$(OCAML_XML_BYT_OBJS:.cma=.cmxa)

$(OCAML_XML_BYT_OBJS):
	@($(CD) $(OCAML_XML_DIR) &&\
	  $(MAKE_INTERNAL_LIBRARY) byt &&\
	  $(MAKE_INTERNAL_LIBRARY) install)

$(OCAML_XML_BIN_OBJS):
	@($(CD) $(OCAML_XML_DIR) &&\
	  $(MAKE_INTERNAL_LIBRARY) bin &&\
	  $(MAKE_INTERNAL_LIBRARY) install)

# Directory src/internal_libs/ocaml/camlzip
CAMLZIP_BYT_OBJS=\
 $(CAMLZIP_LIB_DIR)/zip.cma

CAMLZIP_BIN_OBJS=$(CAMLZIP_BYT_OBJS:.cma=.cmxa)

CAMLZIP_ADDITIONAL_MAKE_FLAGS=\
 ZLIB_LIB_DIR=$(ZLIB_LIB_DIR)\
 ZLIB_INCLUDE=$(ZLIB_INCLUDE)\
 ZLIB_LIB=$(ZLIB_LIB)\

$(CAMLZIP_BYT_OBJS):
	($(CD) $(CAMLZIP_DIR) &&\
	  $(MAKE_INTERNAL_LIBRARY)\
	    $(CAMLZIP_ADDITIONAL_MAKE_FLAGS)\
	    byt &&\
	  $(RM) $(CAMLZIP_DIR)/*.so &&\
	  $(MAKE_INTERNAL_LIBRARY) install-byt)

$(CAMLZIP_BIN_OBJS):
	($(CD) $(CAMLZIP_DIR) &&\
	  $(MAKE_INTERNAL_LIBRARY)\
	    $(CAMLZIP_ADDITIONAL_MAKE_FLAGS)\
	    bin &&\
	  $(MAKE_INTERNAL_LIBRARY) install-bin)

# All internal libraries
SIMPORT_INTERNAL_LIBRARIES_BYT_OBJS=\
 $(OCAML_XML_BYT_OBJS)\
 $(CAMLZIP_BYT_OBJS)\

SIMPORT_INTERNAL_LIBRARIES_BIN_OBJS=\
 $(SIMPORT_INTERNAL_LIBRARIES_BYT_OBJS:.cma=.cmxa)

SIMPORT_INTERNAL_LIBRARIES=\
 $(SIMPORT_INTERNAL_LIBRARIES_BYT_OBJS)\
 $(SIMPORT_INTERNAL_LIBRARIES_BIN_OBJS)

# Directory src/infra

INFRA_BYT_OBJS=\
 $(INFRA_DIR)/ast_types.cmo\
 $(INFRA_DIR)/ast_types_print.cmo\

INFRA_CAML_PRINTER_SRC_FILES=\
 $(INFRA_DIR)/ast_types.mli\

INFRA_CAML_FILES=\
 $(INFRA_CAML_PRINTER_SRC_FILES)\
 $(INFRA_BYT_OBJS:.cmo=.ml)\

INFRA_CAML_GENERATED_FILES=\
 $(INFRA_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/configuration

CONFIGURATION_BYT_OBJS=\
 $(CONFIGURATION_DIR)/configuration.cmo\
 $(CONFIGURATION_DIR)/simport_configuration_print.cmo\
 $(CONFIGURATION_DIR)/say.cmo\
 $(CONFIGURATION_DIR)/arguments.cmo\

CONFIGURATION_CAML_PRINTER_SRC_FILES=\
 $(CONFIGURATION_DIR)/simport_configuration.mli\

CONFIGURATION_CAML_FILES=\
 $(CONFIGURATION_DIR)/configuration.mli\
 $(CONFIGURATION_CAML_PRINTER_SRC_FILES)\
 $(CONFIGURATION_DIR)/say.mli\
 $(CONFIGURATION_DIR)/arguments.mli\
 $(CONFIGURATION_BYT_OBJS:.cmo=.ml)\

CONFIGURATION_CAML_GENERATED_FILES=\
 $(CONFIGURATION_DIR)/configuration.ml\
 $(CONFIGURATION_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

$(CONFIGURATION_DIR)/configuration.ml:\
 $(CONFIG_DIR)/package.env\

# Directory src/mdl/ast

MDL_AST_BYT_OBJS=\
 $(MDL_AST_DIR)/mdl_block_types_tables.cmo\
 $(MDL_AST_DIR)/mdl_block_types_print.cmo\
 $(MDL_AST_DIR)/mdl_ast_functions.cmo\
 $(MDL_AST_DIR)/mdl_ast_print.cmo\
 $(MDL_AST_DIR)/mdl_parsing_ast_print.cmo\

# $(MDL_AST_DIR)/mdl_ast_pprint.cmo\

MDL_AST_CAML_PRINTER_SRC_FILES=\
 $(MDL_AST_DIR)/mdl_ast.mli\
 $(MDL_AST_DIR)/mdl_parsing_ast.mli\
 $(MDL_AST_DIR)/mdl_block_types.mli\

MDL_AST_CAML_FILES=\
 $(MDL_AST_CAML_PRINTER_SRC_FILES)\
 $(MDL_AST_DIR)/mdl_block_types_tables.mli\
 $(MDL_AST_DIR)/mdl_ast_functions.mli\
 $(MDL_AST_BYT_OBJS:.cmo=.ml)\

MDL_AST_CAML_GENERATED_FILES=\
 $(MDL_AST_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/dast

MDL_DAST_BYT_OBJS=\
 $(MDL_DAST_DIR)/mdl_basics_deep_ast_print.cmo\
 $(MDL_DAST_DIR)/mdl_stateflow_deep_ast_print.cmo\
 $(MDL_DAST_DIR)/mdl_deep_ast_print.cmo\
 $(MDL_DAST_DIR)/mdl_deep_basics.cmo\
 $(MDL_DAST_DIR)/mdl_deep_mask.cmo\
 $(MDL_DAST_DIR)/mdl_stateflow_to_deep_ast.cmo\
 $(MDL_DAST_DIR)/mdl_ast_to_deep_ast.cmo\
 $(MDL_DAST_DIR)/mdl_to_deep_ast.cmo\

MDL_DAST_CAML_PRINTER_SRC_FILES=\
 $(MDL_DAST_DIR)/mdl_basics_deep_ast.mli\
 $(MDL_DAST_DIR)/mdl_stateflow_deep_ast.mli\
 $(MDL_DAST_DIR)/mdl_deep_ast.mli\

MDL_DAST_CAML_FILES=\
 $(MDL_DAST_CAML_PRINTER_SRC_FILES)\
 $(MDL_DAST_DIR)/mdl_deep_basics.mli\
 $(MDL_DAST_DIR)/mdl_deep_mask.mli\
 $(MDL_DAST_DIR)/mdl_stateflow_to_deep_ast.mli\
 $(MDL_DAST_DIR)/mdl_ast_to_deep_ast.mli\
 $(MDL_DAST_DIR)/mdl_to_deep_ast.mli\
 $(MDL_DAST_BYT_OBJS:.cmo=.ml)\

MDL_DAST_CAML_GENERATED_FILES=\
 $(MDL_DAST_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/explicit

MDL_EXPLICIT_BYT_OBJS=\
 $(MDL_EXPLICIT_DIR)/mdl_basics_explicit_ast_print.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_stateflow_explicit_ast_print.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_interface_explicit_ast_print.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_implementation_explicit_ast_print.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_explicit_ast_print.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_basics_to_explicit_ast.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_stateflow_to_explicit_ast.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_interface_to_explicit_ast.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_implementation_to_explicit_ast.cmo\
 $(MDL_EXPLICIT_DIR)/mdl_to_explicit_ast.cmo\

MDL_EXPLICIT_CAML_PRINTER_SRC_FILES=\
 $(MDL_EXPLICIT_DIR)/mdl_basics_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_stateflow_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_interface_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_implementation_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_explicit_ast.mli\

MDL_EXPLICIT_CAML_FILES=\
 $(MDL_EXPLICIT_CAML_PRINTER_SRC_FILES)\
 $(MDL_EXPLICIT_DIR)/mdl_basics_to_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_stateflow_to_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_interface_to_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_implementation_to_explicit_ast.mli\
 $(MDL_EXPLICIT_DIR)/mdl_to_explicit_ast.mli\
 $(MDL_EXPLICIT_BYT_OBJS:.cmo=.ml)\

MDL_EXPLICIT_CAML_GENERATED_FILES=\
 $(MDL_EXPLICIT_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/split

MDL_SPLIT_BYT_OBJS=\
 $(MDL_SPLIT_DIR)/mdl_fake_block_types.cmo\
 $(MDL_SPLIT_DIR)/mdl_make_block.cmo\
 $(MDL_SPLIT_DIR)/mdl_split.cmo\
 $(MDL_SPLIT_DIR)/mdl_split_ast_print.cmo\
 $(MDL_SPLIT_DIR)/mdl_to_split_ast.cmo\

MDL_SPLIT_CAML_PRINTER_SRC_FILES=\
 $(MDL_SPLIT_DIR)/mdl_split_ast.mli\

MDL_SPLIT_CAML_FILES=\
 $(MDL_SPLIT_CAML_PRINTER_SRC_FILES)\
 $(MDL_SPLIT_DIR)/mdl_fake_block_types.mli\
 $(MDL_SPLIT_DIR)/mdl_make_block_types.mli\
 $(MDL_SPLIT_DIR)/mdl_make_block.mli\
 $(MDL_SPLIT_DIR)/mdl_split.mli\
 $(MDL_SPLIT_DIR)/mdl_to_split_ast.mli\
 $(MDL_SPLIT_BYT_OBJS:.cmo=.ml)\

MDL_SPLIT_CAML_GENERATED_FILES=\
 $(MDL_SPLIT_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/action

MDL_ACTION_BYT_OBJS=\
 $(MDL_ACTION_DIR)/mdl_explicit_block_utils.cmo\
 $(MDL_ACTION_DIR)/mdl_event_generator.cmo\
 $(MDL_ACTION_DIR)/mdl_equip_block.cmo\
 $(MDL_ACTION_DIR)/mdl_action.cmo\
 $(MDL_ACTION_DIR)/mdl_action_ast_print.cmo\
 $(MDL_ACTION_DIR)/mdl_to_action_ast.cmo\

MDL_ACTION_CAML_PRINTER_SRC_FILES=\
 $(MDL_ACTION_DIR)/mdl_action_ast.mli\

MDL_ACTION_CAML_FILES=\
 $(MDL_ACTION_CAML_PRINTER_SRC_FILES)\
 $(MDL_ACTION_DIR)/mdl_explicit_block_utils.mli\
 $(MDL_ACTION_DIR)/mdl_action_ast.mli\
 $(MDL_ACTION_DIR)/mdl_equiped_block.mli\
 $(MDL_ACTION_DIR)/mdl_event_generator.mli\
 $(MDL_ACTION_DIR)/mdl_equip_block.mli\
 $(MDL_ACTION_DIR)/mdl_action.mli\
 $(MDL_ACTION_DIR)/mdl_to_action_ast.mli\
 $(MDL_ACTION_BYT_OBJS:.cmo=.ml)\

MDL_ACTION_CAML_GENERATED_FILES=\
$(MDL_ACTION_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/rename

MDL_RENAME_BYT_OBJS=\
 $(MDL_RENAME_DIR)/mdl_rename_ast_print.cmo\
 $(MDL_RENAME_DIR)/mdl_rename.cmo\
 $(MDL_RENAME_DIR)/mdl_to_rename_ast.cmo\

MDL_RENAME_CAML_PRINTER_SRC_FILES=\
 $(MDL_RENAME_DIR)/mdl_rename_ast.mli\

MDL_RENAME_CAML_FILES=\
 $(MDL_RENAME_CAML_PRINTER_SRC_FILES)\
 $(MDL_RENAME_DIR)/mdl_rename.mli\
 $(MDL_RENAME_DIR)/mdl_to_rename_ast.mli\
 $(MDL_RENAME_BYT_OBJS:.cmo=.ml)\

MDL_RENAME_CAML_GENERATED_FILES=\
 $(MDL_RENAME_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl_file_format/parsing

# The lexer only part
MDL_FILE_FORMAT_LEX_BYT_OBJS=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_lexer.cmo\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_lex_pprint.cmo\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_tokens.cmo\

MDL_FILE_FORMAT_LEX_CAML_PRINTER_SRC_FILES=\

MDL_FILE_FORMAT_LEX_CAML_FILES=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_tokens.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_lexer.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_lexer.mll\
 $(MDL_FILE_FORMAT_LEX_BYT_OBJS:.cmo=.ml)\

MDL_FILE_FORMAT_LEX_CAML_GENERATED_FILES=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_lexer.ml\

# The parser only part
MDL_FILE_FORMAT_PARSE_BYT_OBJS=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parse_error.cmo\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parser.cmo\

MDL_FILE_FORMAT_PARSE_CAML_PRINTER_SRC_FILES=\

MDL_FILE_FORMAT_PARSE_CAML_FILES=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parse_error.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parser.mly\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parser.mli\
 $(MDL_FILE_FORMAT_PARSE_BYT_OBJS:.cmo=.ml)\

MDL_FILE_FORMAT_PARSE_CAML_GENERATED_FILES=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parser.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parser.ml\

# The parser phase, complete MDL file parsing

MDL_FILE_FORMAT_PARSER_BYT_OBJS=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parsing_ast_to_mdl_ast.cmo\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_parsing_ast.cmo\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_ast.cmo\

MDL_FILE_FORMAT_PARSER_CAML_PRINTER_SRC_FILES=\

MDL_FILE_FORMAT_PARSER_CAML_GENERATED_FILES=\

MDL_FILE_FORMAT_PARSER_CAML_FILES=\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_parsing_ast_to_mdl_ast.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_parsing_ast.mli\
 $(MDL_FILE_FORMAT_PARSING_DIR)/mdl_to_ast.mli\
 $(MDL_FILE_FORMAT_PARSER_BYT_OBJS:.cmo=.ml)\

# The complete parsing phase
MDL_FILE_FORMAT_PARSING_BYT_OBJS=\
 $(MDL_FILE_FORMAT_PARSE_BYT_OBJS)\
 $(MDL_FILE_FORMAT_LEX_BYT_OBJS)\
 $(MDL_FILE_FORMAT_PARSER_BYT_OBJS)\

MDL_FILE_FORMAT_PARSING_CAML_PRINTER_SRC_FILES=\
 $(MDL_FILE_FORMAT_PARSE_CAML_PRINTER_SRC_FILES)\
 $(MDL_FILE_FORMAT_LEX_CAML_PRINTER_SRC_FILES)\
 $(MDL_FILE_FORMAT_PARSER_CAML_PRINTER_SRC_FILES)\

MDL_FILE_FORMAT_PARSING_CAML_GENERATED_FILES=\
 $(MDL_FILE_FORMAT_PARSE_CAML_GENERATED_FILES)\
 $(MDL_FILE_FORMAT_LEX_CAML_GENERATED_FILES)\
 $(MDL_FILE_FORMAT_PARSER_CAML_GENERATED_FILES)\

MDL_FILE_FORMAT_PARSING_CAML_FILES=\
 $(MDL_FILE_FORMAT_PARSE_CAML_FILES)\
 $(MDL_FILE_FORMAT_LEX_CAML_FILES)\
 $(MDL_FILE_FORMAT_PARSER_CAML_FILES)\

# Directory src/slx_file_format/parsing

SLX_FILE_FORMAT_PARSING_BYT_OBJS=\
 $(SLX_FILE_FORMAT_PARSING_DIR)/slx_to_xml_ast.cmo\

SLX_FILE_FORMAT_PARSING_CAML_PRINTER_SRC_FILES=\
 $(OCAML_XML_SRC_XML_PARSING_DIR)/xml_parser.mli\

SLX_FILE_FORMAT_PARSING_CAML_FILES=\
 $(SLX_FILE_FORMAT_PARSING_DIR)/slx_to_xml_ast.mli\
 $(SLX_FILE_FORMAT_PARSING_BYT_OBJS:.cmo=.ml)\

SLX_FILE_FORMAT_PARSING_CAML_GENERATED_FILES=\
 $(OCAML_XML_SRC_XML_PARSING_DIR)/xml_parser.mli\
 $(OCAML_XML_SRC_XML_PARSING_DIR)/xml_parser.ml\
 $(SLX_FILE_FORMAT_PARSING_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/slx_file_format/dast

SLX_FILE_FORMAT_DAST_BYT_OBJS=\
 $(SLX_FILE_FORMAT_DAST_DIR)/slx_xml_check.cmo\
 $(SLX_FILE_FORMAT_DAST_DIR)/xml_deep_ast_print.cmo\
 $(SLX_FILE_FORMAT_DAST_DIR)/xml_ast_to_xml_deep_ast.cmo\
 $(SLX_FILE_FORMAT_DAST_DIR)/slx_to_xml_deep_ast.cmo\

SLX_FILE_FORMAT_DAST_CAML_PRINTER_SRC_FILES=\
 $(SLX_FILE_FORMAT_DAST_DIR)/xml_deep_ast.mli\

SLX_FILE_FORMAT_DAST_CAML_FILES=\
 $(SLX_FILE_FORMAT_DAST_CAML_PRINTER_SRC_FILES)\
 $(SLX_FILE_FORMAT_DAST_DIR)/slx_xml_check.mli\
 $(SLX_FILE_FORMAT_DAST_DIR)/xml_ast_to_xml_deep_ast.mli\
 $(SLX_FILE_FORMAT_DAST_DIR)/slx_to_xml_deep_ast.mli\
 $(SLX_FILE_FORMAT_DAST_BYT_OBJS:.cmo=.ml)\

SLX_FILE_FORMAT_DAST_CAML_GENERATED_FILES=\
 $(SLX_FILE_FORMAT_DAST_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/slx_file_format/explicit

SLX_FILE_FORMAT_EXPLICIT_BYT_OBJS=\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/mdl_entry_parsers.cmo\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/xml_explicit_ast_print.cmo\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/xml_deep_ast_to_xml_explicit_ast.cmo\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/slx_to_xml_explicit_ast.cmo\

SLX_FILE_FORMAT_EXPLICIT_CAML_PRINTER_SRC_FILES=\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/xml_explicit_ast.mli\

SLX_FILE_FORMAT_EXPLICIT_CAML_FILES=\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/mdl_entry_parsers.mli\
 $(SLX_FILE_FORMAT_EXPLICIT_CAML_PRINTER_SRC_FILES)\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/xml_deep_ast_to_xml_explicit_ast.mli\
 $(SLX_FILE_FORMAT_EXPLICIT_DIR)/slx_to_xml_explicit_ast.mli\
 $(SLX_FILE_FORMAT_EXPLICIT_BYT_OBJS:.cmo=.ml)\

SLX_FILE_FORMAT_EXPLICIT_CAML_GENERATED_FILES=\
 $(SLX_FILE_FORMAT_EXPLICIT_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/slx_file_format/translating

SLX_FILE_FORMAT_TRANSLATING_BYT_OBJS=\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_translating_basics.cmo\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_stateflow_to_mdl_stateflow.cmo\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_explicit_ast_to_mdl_ast.cmo\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/slx_to_mdl_ast.cmo\

SLX_FILE_FORMAT_TRANSLATING_CAML_PRINTER_SRC_FILES=\

SLX_FILE_FORMAT_TRANSLATING_CAML_FILES=\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_translating_basics.mli\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_stateflow_to_mdl_stateflow.mli\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/xml_explicit_ast_to_mdl_ast.mli\
 $(SLX_FILE_FORMAT_TRANSLATING_DIR)/slx_to_mdl_ast.mli\
 $(SLX_FILE_FORMAT_TRANSLATING_BYT_OBJS:.cmo=.ml)\

SLX_FILE_FORMAT_TRANSLATING_CAML_GENERATED_FILES=\

# Directory src/slx_file_format

SLX_FILE_FORMAT_BYT_OBJS=\
 $(SLX_FILE_FORMAT_PARSING_BYT_OBJS)\
 $(SLX_FILE_FORMAT_DAST_BYT_OBJS)\
 $(SLX_FILE_FORMAT_EXPLICIT_BYT_OBJS)\
 $(SLX_FILE_FORMAT_TRANSLATING_BYT_OBJS)\

SLX_FILE_FORMAT_CAML_PRINTER_SRC_FILES=\
 $(SLX_FILE_FORMAT_PARSING_CAML_PRINTER_SRC_FILES)\
 $(SLX_FILE_FORMAT_DAST_CAML_PRINTER_SRC_FILES)\
 $(SLX_FILE_FORMAT_EXPLICIT_CAML_PRINTER_SRC_FILES)\
 $(SLX_FILE_FORMAT_TRANSLATING_CAML_PRINTER_SRC_FILES)\

SLX_FILE_FORMAT_CAML_FILES=\
 $(SLX_FILE_FORMAT_PARSING_CAML_FILES)\
 $(SLX_FILE_FORMAT_DAST_CAML_FILES)\
 $(SLX_FILE_FORMAT_EXPLICIT_CAML_FILES)\
 $(SLX_FILE_FORMAT_TRANSLATING_CAML_FILES)\

SLX_FILE_FORMAT_CAML_GENERATED_FILES=\
 $(SLX_FILE_FORMAT_PARSING_CAML_GENERATED_FILES)\
 $(SLX_FILE_FORMAT_DAST_CAML_GENERATED_FILES)\
 $(SLX_FILE_FORMAT_EXPLICIT_CAML_GENERATED_FILES)\
 $(SLX_FILE_FORMAT_TRANSLATING_CAML_GENERATED_FILES)\

# Directory src/mdl/simulink_embedded

SIMULINK_EMBEDDED_BYT_OBJS=\
 $(SIMULINK_EMBEDDED_DIR)/mbd_ast_pprint.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_parser.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_lexer.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_predefined.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_ast_to_deep_ast.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_transl.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_deep_ast_pprint.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_treat.cmo\
 $(SIMULINK_EMBEDDED_DIR)/mbd_to_target.cmo\

SIMULINK_EMBEDDED_CAML_PRINTER_SRC_FILES=\

SIMULINK_EMBEDDED_CAML_FILES=\
 $(SIMULINK_EMBEDDED_DIR)/mbd_ast.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_ast_pprint.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_parser.mly\
 $(SIMULINK_EMBEDDED_DIR)/mbd_parser.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_lexer.mll\
 $(SIMULINK_EMBEDDED_DIR)/mbd_predef.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_predefined.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_deep_ast.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_ast_to_deep_ast.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_transl.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_deep_ast_pprint.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_treat.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_to_target.mli\
 $(SIMULINK_EMBEDDED_BYT_OBJS:.cmo=.ml)\

SIMULINK_EMBEDDED_CAML_GENERATED_FILES=\
 $(SIMULINK_EMBEDDED_DIR)/mbd_parser.mli\
 $(SIMULINK_EMBEDDED_DIR)/mbd_parser.ml\
 $(SIMULINK_EMBEDDED_DIR)/mbd_lexer.ml\

# Directory src/matlab

# Directory src/matlab/parsing

MATLAB_LEX_BYT_OBJS=\
 $(MATLAB_PARSING_DIR)/mtlb_location.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_location_print.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_lexer.cmo\

MATLAB_LEX_CAML_PRINTER_SRC_FILES=\
 $(MATLAB_PARSING_DIR)/mtlb_location.mli\

MATLAB_LEX_CAML_FILES=\
 $(MATLAB_LEX_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_PARSING_DIR)/mtlb_lexer.mll\
 $(MATLAB_LEX_BYT_OBJS:.cmo=.ml)\

MATLAB_LEX_CAML_GENERATED_FILES=\
 $(MATLAB_LEX_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\
 $(MATLAB_PARSING_DIR)/mtlb_lexer.ml\

MATLAB_PARSE_BYT_OBJS=\
 $(MATLAB_PARSING_DIR)/ast_node_utils.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_syntaxerr.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_ast_funs.cmo\
 $(MATLAB_PARSING_DIR)/ast_node_print.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_ast_print.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_parser.cmo\
 $(MATLAB_PARSING_DIR)/mtlb_to_ast.cmo\

MATLAB_PARSE_CAML_PRINTER_SRC_FILES=\
 $(MATLAB_PARSING_DIR)/ast_node.mli\
 $(MATLAB_PARSING_DIR)/mtlb_ast.mli\

MATLAB_PARSE_CAML_FILES=\
 $(MATLAB_PARSE_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_PARSING_DIR)/ast_node_utils.mli\
 $(MATLAB_PARSING_DIR)/mtlb_syntaxerr.mli\
 $(MATLAB_PARSING_DIR)/mtlb_parser.mly\
 $(MATLAB_PARSING_DIR)/mtlb_parser.mli\
 $(MATLAB_PARSING_DIR)/mtlb_to_ast.mli\
 $(MATLAB_PARSE_BYT_OBJS:.cmo=.ml)\

MATLAB_PARSE_CAML_GENERATED_FILES=\
 $(MATLAB_PARSE_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\
 $(MATLAB_PARSING_DIR)/mtlb_parser.mli\
 $(MATLAB_PARSING_DIR)/mtlb_parser.ml\

# Directory src/matlab/rewriting

MATLAB_REWRITING_BYT_OBJS=\
 $(MATLAB_REWRITING_DIR)/mtlb_rewrite_ast.cmo\
 $(MATLAB_REWRITING_DIR)/mtlb_to_rewritten_ast.cmo\

MATLAB_REWRITING_CAML_PRINTER_SRC_FILES=\

MATLAB_REWRITING_CAML_FILES=\
 $(MATLAB_REWRITING_DIR)/mtlb_rewrite_ast.mli\
 $(MATLAB_REWRITING_DIR)/mtlb_to_rewritten_ast.mli\
 $(MATLAB_REWRITING_BYT_OBJS:.cmo=.ml)\

MATLAB_REWRITING_CAML_GENERATED_FILES=\

# Directory src/matlab/printing

MATLAB_PRINTING_BYT_OBJS=\
 $(MATLAB_PRINTING_DIR)/mtlb_ppf.cmo\
 $(MATLAB_PRINTING_DIR)/mtlb_make_host_print.cmo\

MATLAB_PRINTING_CAML_PRINTER_SRC_FILES=\

MATLAB_PRINTING_CAML_FILES=\
 $(MATLAB_PRINTING_DIR)/mtlb_ppf.mli\
 $(MATLAB_PRINTING_DIR)/mtlb_make_host_print_spec.mli\
 $(MATLAB_PRINTING_DIR)/mtlb_make_host_print.mli\
 $(MATLAB_PRINTING_BYT_OBJS:.cmo=.ml)\

MATLAB_PRINTING_CAML_GENERATED_FILES=\

# Directory src/matlab/translating

MATLAB_TRANSLATING_BYT_OBJS=\
 $(MATLAB_TRANSLATING_DIR)/mtlb_nsp_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_modelica_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_matlab_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_octave_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_hypermath_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_oml_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/mtlb_scicoslab_host.cmo\
 $(MATLAB_TRANSLATING_DIR)/m_to_host.cmo\

MATLAB_TRANSLATING_CAML_PRINTER_SRC_FILES=\

MATLAB_TRANSLATING_CAML_FILES=\
 $(MATLAB_TRANSLATING_DIR)/mtlb_nsp_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_modelica_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_matlab_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_octave_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_hypermath_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_oml_host.mli\
 $(MATLAB_TRANSLATING_DIR)/mtlb_scicoslab_host.mli\
 $(MATLAB_TRANSLATING_DIR)/m_to_host.mli\
 $(MATLAB_TRANSLATING_BYT_OBJS:.cmo=.ml)\

MATLAB_TRANSLATING_CAML_GENERATED_FILES=\

# Directory src/matlab

MATLAB_BYT_OBJS=\
 $(MATLAB_LEX_BYT_OBJS)\
 $(MATLAB_PARSE_BYT_OBJS)\
 $(MATLAB_REWRITING_BYT_OBJS)\
 $(MATLAB_PRINTING_BYT_OBJS)\
 $(MATLAB_TRANSLATING_BYT_OBJS)\

MATLAB_CAML_PRINTER_SRC_FILES=\
 $(MATLAB_LEX_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_PARSE_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_REWRITING_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_PRINTING_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_TRANSLATING_CAML_PRINTER_SRC_FILES)\

MATLAB_CAML_FILES=\
 $(MATLAB_LEX_CAML_FILES)\
 $(MATLAB_PARSE_CAML_FILES)\
 $(MATLAB_REWRITING_CAML_FILES)\
 $(MATLAB_PRINTING_CAML_FILES)\
 $(MATLAB_TRANSLATING_CAML_FILES)\

MATLAB_CAML_GENERATED_FILES=\
 $(MATLAB_LEX_CAML_GENERATED_FILES)\
 $(MATLAB_PARSE_CAML_GENERATED_FILES)\
 $(MATLAB_REWRITING_CAML_GENERATED_FILES)\
 $(MATLAB_PRINTING_CAML_GENERATED_FILES)\
 $(MATLAB_TRANSLATING_CAML_GENERATED_FILES)\

# Directory src/mdl/translation

MDL_TRANSLATION_BYT_OBJS=\
 $(MDL_TRANSLATION_DIR)/concat_list.cmo\
 $(MDL_TRANSLATION_DIR)/concat_list_print.cmo\
 $(MDL_TRANSLATION_DIR)/instructions_print.cmo\
 $(MDL_TRANSLATION_DIR)/genr_utils.cmo\
 $(MDL_TRANSLATION_DIR)/genr_block.cmo\
 $(MDL_TRANSLATION_DIR)/genr_block_types_print.cmo\
 $(MDL_TRANSLATION_DIR)/mdl_translation.cmo\
 $(MDL_TRANSLATION_DIR)/model_to_code.cmo\
 $(MDL_TRANSLATION_DIR)/mdl_to_code.cmo\

MDL_TRANSLATION_CAML_PRINTER_SRC_FILES=\
 $(MDL_TRANSLATION_DIR)/concat_list.mli\
 $(MDL_TRANSLATION_DIR)/instructions.mli\
 $(MDL_TRANSLATION_DIR)/genr_block_types.mli\

MDL_TRANSLATION_CAML_FILES=\
 $(MDL_TRANSLATION_CAML_PRINTER_SRC_FILES)\
 $(MDL_TRANSLATION_DIR)/genr_utils.mli\
 $(MDL_TRANSLATION_DIR)/genr_block.mli\
 $(MDL_TRANSLATION_DIR)/mdl_translation.mli\
 $(MDL_TRANSLATION_DIR)/model_to_code.mli\
 $(MDL_TRANSLATION_DIR)/mdl_to_code.mli\
 $(MDL_TRANSLATION_BYT_OBJS:.cmo=.ml)\

MDL_TRANSLATION_CAML_GENERATED_FILES=\
 $(MDL_TRANSLATION_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/code

MDL_CODE_BYT_OBJS=\
 $(MDL_CODE_DIR)/code_make_host_print.cmo\
 $(MDL_CODE_DIR)/code_nsp_host.cmo\
 $(MDL_CODE_DIR)/code_modelica_host.cmo\
 $(MDL_CODE_DIR)/code_scicoslab_host.cmo\
 $(MDL_CODE_DIR)/code_octave_host.cmo\
 $(MDL_CODE_DIR)/code_hypermath_host.cmo\
 $(MDL_CODE_DIR)/code_oml_host.cmo\
 $(MDL_CODE_DIR)/code_to_host.cmo\

MDL_CODE_CAML_PRINTER_SRC_FILES=\

MDL_CODE_CAML_FILES=\
 $(MDL_CODE_DIR)/code_make_host_print_spec.mli\
 $(MDL_CODE_DIR)/code_make_host_print.mli\
 $(MDL_CODE_DIR)/code_nsp_host.mli\
 $(MDL_CODE_DIR)/code_modelica_host.mli\
 $(MDL_CODE_DIR)/code_scicoslab_host.mli\
 $(MDL_CODE_DIR)/code_octave_host.mli\
 $(MDL_CODE_DIR)/code_hypermath_host.mli\
 $(MDL_CODE_DIR)/code_oml_host.mli\
 $(MDL_CODE_DIR)/code_to_host.mli\
 $(MDL_CODE_BYT_OBJS:.cmo=.ml)\

MDL_CODE_CAML_GENERATED_FILES=\

# Directory src/mdl/genr_block

# Directory src/mdl/genr_block/common

MDL_GENR_BLOCK_COMMON_BYT_OBJS=\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_common.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/make_genr_utilities.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_variant.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_pro_variant.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/modelicos_variant.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_scicos.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_scicos_pro.cmo\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_modelicos.cmo\

## $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_block_types_print.cmo\

MDL_GENR_BLOCK_COMMON_CAML_PRINTER_SRC_FILES=\

## $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_block_types.mli\

MDL_GENR_BLOCK_COMMON_CAML_FILES=\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_types.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_common.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/make_genr_utilities_spec.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/make_genr_utilities.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_variant.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_pro_variant.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/modelicos_variant.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_scicos.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_scicos_pro.mli\
 $(MDL_GENR_BLOCK_COMMON_DIR)/genr_utilities_modelicos.mli\
 $(MDL_GENR_BLOCK_COMMON_BYT_OBJS:.cmo=.ml)\

# $(MDL_GENR_BLOCK_COMMON_DIR)/genr_interface_block.mli\
# $(MDL_GENR_BLOCK_COMMON_DIR)/scicos_block_types.mli\

MDL_GENR_BLOCK_COMMON_CAML_GENERATED_FILES=\
 $(MDL_GENR_BLOCK_COMMON_CAML_PRINTER_SRC_FILES:mli=_print.ml)\

# Directory src/mdl/genr_block/scicos

# Warning: genr_scicos_not_yet_translated.cmo should be linked before other
# block generators, since some block generators uses its code.
# Otherwise, the modules are sorted and linked alphabetically.
MDL_GENR_BLOCK_SCICOS_BYT_OBJS=\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_not_yet_translated.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_abs.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_aliases.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_atomic_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_backlash.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_band_limited_white_noise.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_bias.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_c_m_block.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_celsius_to_fahrenheit.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_compare_to_constant.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_complex_to_coordinates.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_coordinates_to_complex.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_data_type_conversion.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_difference.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_digital_clock.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_discrete_derivative.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_discrete_pulse_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_discrete_state_space.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_discrete_transfer_fcn.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_dot_product.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_elementary_math.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_fahrenheit_to_celsius.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_fcn.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_from_workspace.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_function_call_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_if_action_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_integrator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_logic.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_lookup.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_lookup2d.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_math.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_matlab_function.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_merge.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_min_max.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_mport_switch.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_pmio_port.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_quantizer.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_random_number.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_relational_operator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_repeating_sequence_interpolated.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_repeating_table.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_saturate.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_selector.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_signal_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_signum.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_sin.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_stop.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_switch.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_time_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_to_workspace.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_trigger_and_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_trigger_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_trigonometry.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_uniform_random_number.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_unit_delay.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_xy_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/genr_scicos_zero_order_hold.cmo\

MDL_GENR_BLOCK_SCICOS_CAML_PRINTER_SRC_FILES=\

MDL_GENR_BLOCK_SCICOS_CAML_FILES=\
 $(MDL_GENR_BLOCK_SCICOS_BYT_OBJS:.cmo=.mli)\
 $(MDL_GENR_BLOCK_SCICOS_BYT_OBJS:.cmo=.ml)\

MDL_GENR_BLOCK_SCICOS_CAML_GENERATED_FILES=\

# Directory src/mdl/genr_block/scicos_pro

# Warning: genr_scicos_pro_not_yet_translated.cmo should be linked before other
# block generators, since some block generators uses its code.
# Module Genr_utilities_pro should also be linked early.
# Otherwise, the modules are sorted and linked alphabetically.
MDL_GENR_BLOCK_SCICOS_PRO_BYT_OBJS=\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_utilities_pro.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_not_yet_translated.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_abs.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_aliases.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_atomic_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_backlash.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_band_limited_white_noise.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_bus_assignment.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_bus_creator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_bus_selector.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_c_m_block.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_combinatorial_logic.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_compare_to_constant.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_complex_to_coordinates.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_concatenate.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_conversion.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_coordinates_to_complex.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_coulomb_viscous_friction.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_counter_limited.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_cross_product.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_data_type_conversion.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_dead_zone.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_dead_zone_dynamic.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_delay.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_demux.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_detect.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_detect_rise_and_fall.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_difference.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_digital_clock.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_derivative.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_integral.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_pulse_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_state_space.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_transfer_fcn.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_discrete_zero_pole.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_dot_product.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_elementary_math.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_fcn.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_from_workspace.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_function_call_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_gain.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_hit_cross.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_if.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_if_action_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_initial_condition.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_integer_delay.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_integrator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_interval_test.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_logic.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_lookup.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_lookup2d.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_lookup_nd.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_math.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_matlab_fcn.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_matlab_function.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_matrix_concatenation.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_merge.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_min_max.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_mport_switch.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_multi_plot_graph.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_mux.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_pmio_port.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_quantizer.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_random_number.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_rate_transition.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_relational_operator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_repeating_sequence_interpolated.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_repeating_sequence_stair.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_repeating_table.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_rounding.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_saturate.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_saturation_dynamic.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_selector.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_signal_generator.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_signum.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_sin.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_sqrt.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_stop.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_switch.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_tapped_delay_line.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_time_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_to_workspace.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_trigger_and_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_trigger_event_gen.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_trigonometry.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_unary_minus.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_uniform_random_number.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_unit_delay.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_xy_scope.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_zero_order_hold.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_electrical_inductor.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_electrical_sensors.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_electrical_mechanical_sources.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_mechanical_rotational_inertia.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_mechanical_rotational_reference.cmo\
 $(MDL_GENR_BLOCK_SCICOS_PRO_DIR)/genr_scicos_pro_mechanical_ideal_rotational_sensor.cmo\

MDL_GENR_BLOCK_SCICOS_PRO_CAML_PRINTER_SRC_FILES=\

MDL_GENR_BLOCK_SCICOS_PRO_CAML_FILES=\
 $(MDL_GENR_BLOCK_SCICOS_PRO_BYT_OBJS:.cmo=.mli)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_BYT_OBJS:.cmo=.ml)\

MDL_GENR_BLOCK_SCICOS_PRO_CAML_GENERATED_FILES=\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_PRINTER_SRC_FILES:.mli=_print.ml)\

# Directory src/mdl/genr_block/modelicos

# Warning: genr_modelicos_not_yet_translated.cmo should be linked before other
# block generators, since some block generators uses its code.
# Module Genr_utilities_pro should also be linked early.
# Otherwise, the modules are sorted and linked alphabetically.
MDL_GENR_BLOCK_MODELICOS_BYT_OBJS=\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_not_yet_translated.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_aliases.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_compare_to_constant.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_gain.cmo\

# Not yet implemented \
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_utilities_pro.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_not_yet_translated.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_abs.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_aliases.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_atomic_event_gen.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_backlash.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_band_limited_white_noise.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_bus_assignment.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_bus_creator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_bus_selector.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_c_m_block.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_combinatorial_logic.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_compare_to_constant.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_complex_to_coordinates.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_concatenate.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_conversion.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_coordinates_to_complex.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_coulomb_viscous_friction.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_counter_limited.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_cross_product.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_data_type_conversion.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_dead_zone.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_dead_zone_dynamic.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_delay.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_demux.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_detect.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_detect_rise_and_fall.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_difference.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_digital_clock.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_derivative.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_integral.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_pulse_generator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_state_space.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_transfer_fcn.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_discrete_zero_pole.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_dot_product.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_elementary_math.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_fcn.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_from_workspace.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_function_call_generator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_gain.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_hit_cross.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_if.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_if_action_event_gen.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_initial_condition.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_integer_delay.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_integrator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_interval_test.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_logic.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_lookup.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_lookup2d.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_lookup_nd.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_math.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_matlab_fcn.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_matlab_function.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_matrix_concatenation.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_merge.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_min_max.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_mport_switch.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_multi_plot_graph.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_mux.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_pmio_port.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_quantizer.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_random_number.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_rate_transition.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_relational_operator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_repeating_sequence_interpolated.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_repeating_sequence_stair.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_repeating_table.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_rounding.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_saturate.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_saturation_dynamic.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_scope.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_selector.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_signal_generator.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_signum.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_sin.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_sqrt.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_stop.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_switch.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_tapped_delay_line.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_time_scope.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_to_workspace.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_trigger_and_enable_event_gen.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_trigger_event_gen.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_trigonometry.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_unary_minus.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_uniform_random_number.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_unit_delay.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_xy_scope.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_zero_order_hold.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_electrical_inductor.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_electrical_sensors.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_electrical_mechanical_sources.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_mechanical_rotational_inertia.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_mechanical_rotational_reference.cmo\
 $(MDL_GENR_BLOCK_MODELICOS_DIR)/genr_modelicos_mechanical_ideal_rotational_sensor.cmo\

MDL_GENR_BLOCK_MODELICOS_CAML_PRINTER_SRC_FILES=\

MDL_GENR_BLOCK_MODELICOS_CAML_FILES=\
 $(MDL_GENR_BLOCK_MODELICOS_BYT_OBJS:.cmo=.mli)\
 $(MDL_GENR_BLOCK_MODELICOS_BYT_OBJS:.cmo=.ml)\

MDL_GENR_BLOCK_MODELICOS_CAML_GENERATED_FILES=\

# Directory src/mdl/genr_block

MDL_GENR_BLOCK_BYT_OBJS=\
 $(MDL_GENR_BLOCK_COMMON_BYT_OBJS)\
 $(MDL_GENR_BLOCK_SCICOS_BYT_OBJS)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_BYT_OBJS)\
 $(MDL_GENR_BLOCK_MODELICOS_BYT_OBJS)\

MDL_GENR_BLOCK_CAML_PRINTER_SRC_FILES=\
 $(MDL_GENR_BLOCK_COMMON_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_PRINTER_SRC_FILES)\

MDL_GENR_BLOCK_CAML_FILES=\
 $(MDL_GENR_BLOCK_COMMON_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_FILES)\

MDL_GENR_BLOCK_CAML_GENERATED_FILES=\
 $(MDL_GENR_BLOCK_COMMON_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_GENERATED_FILES)\

# Directory src/compiler

COMPILER_BYT_OBJS=\
 $(COMPILER_DIR)/mdl_compile.cmo\

COMPILER_CAML_PRINTER_SRC_FILES=\

COMPILER_CAML_FILES=\
 $(COMPILER_DIR)/mdl_compile.mli\
 $(COMPILER_BYT_OBJS:.cmo=.ml)\

COMPILER_CAML_GENERATED_FILES=\

# Directory src/driver

DRIVER_BYT_OBJS=\
 $(DRIVER_DIR)/main_gen.cmo\
 $(DRIVER_DIR)/driver.cmo\

DRIVER_CAML_PRINTER_SRC_FILES=\

DRIVER_CAML_FILES=\
 $(DRIVER_DIR)/main_gen.mli\
 $(DRIVER_DIR)/driver.mli\
 $(DRIVER_BYT_OBJS:.cmo=.ml)\

DRIVER_CAML_GENERATED_FILES=\

# Directory src/mdl

MDL_BYT_OBJS=\
 $(MDL_AST_BYT_OBJS)\
 $(MDL_FILE_FORMAT_PARSING_BYT_OBJS)\
 $(MDL_DAST_BYT_OBJS)\
 $(MDL_EXPLICIT_BYT_OBJS)\
 $(MDL_SPLIT_BYT_OBJS)\
 $(MDL_ACTION_BYT_OBJS)\
 $(MDL_RENAME_BYT_OBJS)\
 $(MDL_TRANSLATION_BYT_OBJS)\
 $(MDL_CODE_BYT_OBJS)\

MDL_CAML_PRINTER_SRC_FILES=\
 $(MDL_AST_CAML_PRINTER_SRC_FILES)\
 $(MDL_FILE_FORMAT_PARSING_CAML_PRINTER_SRC_FILES)\
 $(MDL_DAST_CAML_PRINTER_SRC_FILES)\
 $(MDL_EXPLICIT_CAML_PRINTER_SRC_FILES)\
 $(MDL_SPLIT_CAML_PRINTER_SRC_FILES)\
 $(MDL_ACTION_CAML_PRINTER_SRC_FILES)\
 $(MDL_RENAME_CAML_PRINTER_SRC_FILES)\
 $(MDL_TRANSLATION_CAML_PRINTER_SRC_FILES)\
 $(MDL_CODE_CAML_PRINTER_SRC_FILES)\

MDL_CAML_FILES=\
 $(MDL_AST_CAML_FILES)\
 $(MDL_FILE_FORMAT_PARSING_CAML_FILES)\
 $(MDL_DAST_CAML_FILES)\
 $(MDL_EXPLICIT_CAML_FILES)\
 $(MDL_SPLIT_CAML_FILES)\
 $(MDL_ACTION_CAML_FILES)\
 $(MDL_RENAME_CAML_FILES)\
 $(MDL_TRANSLATION_CAML_FILES)\
 $(MDL_CODE_CAML_FILES)\

MDL_CAML_GENERATED_FILES=\
 $(MDL_AST_CAML_GENERATED_FILES)\
 $(MDL_FILE_FORMAT_PARSING_CAML_GENERATED_FILES)\
 $(MDL_DAST_CAML_GENERATED_FILES)\
 $(MDL_EXPLICIT_CAML_GENERATED_FILES)\
 $(MDL_SPLIT_CAML_GENERATED_FILES)\
 $(MDL_ACTION_CAML_GENERATED_FILES)\
 $(MDL_RENAME_CAML_GENERATED_FILES)\
 $(MDL_TRANSLATION_CAML_GENERATED_FILES)\
 $(MDL_CODE_CAML_GENERATED_FILES)\

# Directory src/

SIMPORT_COMMON_BYT_OBJS=\
 $(INFRA_BYT_OBJS)\
 $(CONFIGURATION_BYT_OBJS)\
 $(SIMULINK_EMBEDDED_BYT_OBJS)\
 $(MATLAB_BYT_OBJS)\
 $(MDL_BYT_OBJS)\
 $(MDL_GENR_BLOCK_COMMON_BYT_OBJS)\

SIMPORT_TRAILING_BYT_OBJS=\
 $(SLX_FILE_FORMAT_BYT_OBJS)\
 $(COMPILER_BYT_OBJS)\
 $(DRIVER_BYT_OBJS)\

SIMPORT_BYT_OBJS=\
 $(SIMPORT_COMMON_BYT_OBJS)\
 $(MDL_GENR_BLOCK_SCICOS_BYT_OBJS)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_BYT_OBJS)\
 $(MDL_GENR_BLOCK_MODELICOS_BYT_OBJS)\
 $(SIMPORT_TRAILING_BYT_OBJS)\

SIMPORT_COMMON_CAML_PRINTER_SRC_FILES=\
 $(INFRA_CAML_PRINTER_SRC_FILES)\
 $(CONFIGURATION_CAML_PRINTER_SRC_FILES)\
 $(SIMULINK_EMBEDDED_CAML_PRINTER_SRC_FILES)\
 $(MATLAB_CAML_PRINTER_SRC_FILES)\
 $(MDL_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_COMMON_CAML_PRINTER_SRC_FILES)\

SIMPORT_COMMON_CAML_FILES=\
 $(INFRA_CAML_FILES)\
 $(CONFIGURATION_CAML_FILES)\
 $(SIMULINK_EMBEDDED_CAML_FILES)\
 $(MATLAB_CAML_FILES)\
 $(MDL_CAML_FILES)\
 $(MDL_GENR_BLOCK_COMMON_CAML_FILES)\

SIMPORT_TRAILING_CAML_PRINTER_SRC_FILES=\
 $(SLX_FILE_FORMAT_CAML_PRINTER_SRC_FILES)\
 $(COMPILER_CAML_PRINTER_SRC_FILES)\
 $(DRIVER_CAML_PRINTER_SRC_FILES)\

SIMPORT_TRAILING_CAML_FILES=\
 $(SLX_FILE_FORMAT_CAML_FILES)\
 $(COMPILER_CAML_FILES)\
 $(DRIVER_CAML_FILES)\

SIMPORT_CAML_PRINTER_SRC_FILES=\
 $(SIMPORT_COMMON_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_PRINTER_SRC_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_PRINTER_SRC_FILES)\
 $(SIMPORT_TRAILING_CAML_PRINTER_SRC_FILES)\

SIMPORT_CAML_FILES=\
 $(SIMPORT_COMMON_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_FILES)\
 $(SIMPORT_TRAILING_CAML_FILES)\

SIMPORT_CAML_GENERATED_FILES=\
 $(INFRA_CAML_GENERATED_FILES)\
 $(CONFIGURATION_CAML_GENERATED_FILES)\
 $(SIMULINK_EMBEDDED_CAML_GENERATED_FILES)\
 $(MATLAB_CAML_GENERATED_FILES)\
 $(MDL_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_COMMON_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_GENERATED_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_GENERATED_FILES)\
 $(SLX_FILE_FORMAT_CAML_GENERATED_FILES)\
 $(COMPILER_CAML_GENERATED_FILES)\
 $(DRIVER_CAML_GENERATED_FILES)\

# List of files to bannerize

SIMPORT_BANNER_FILES=\
 $(CONFIG_DIR)/banner\
 $(MATLAB_DIR)/banner\
 $(SLX_DIR)/banner

SIMPORT_FILES_TO_BANNERIZE=\
 $(INFRA_CAML_FILES)\
 $(CONFIGURATION_CAML_FILES)\
 $(SIMULINK_EMBEDDED_CAML_FILES)\
 $(MDL_CAML_FILES)\
 $(MDL_GENR_BLOCK_COMMON_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_CAML_FILES)\
 $(MDL_GENR_BLOCK_SCICOS_PRO_CAML_FILES)\
 $(MDL_GENR_BLOCK_MODELICOS_CAML_FILES)\
 $(COMPILER_CAML_FILES)\
 $(DRIVER_CAML_FILES)\

CONFIG_FILES_TO_BANNERIZE=\
 $(CONFIG_DIR)/Basic_commands.mk\
 $(CONFIG_DIR)/Caml.mk\
 $(CONFIG_DIR)/Caml_commands.mk\
 $(CONFIG_DIR)/Config.mk.indata\
 $(CONFIG_DIR)/Doc.mk\
 $(CONFIG_DIR)/External.mk\
 $(CONFIG_DIR)/Params.mk\
 $(CONFIG_DIR)/Params_win32.mk\
 $(CONFIG_DIR)/Simport.mk\
 $(CONFIG_DIR)/Simport_commands.mk\
 $(CONFIG_DIR)/package.env\

CONFIG_SCRIPTS_TO_BANNERIZE=\
 $(CONFIG_DIR)/configure\

# List of .mlin .mliin, etc. to enter in the list of files to bannerize
CAML_GENERATING_FILES=\
 $(CONFIGURATION_DIR)/configuration.mlin\

LOCAL_TEST_TO_BANNERIZE=\
 $(MDL_FILE_FORMAT_DIR)/test/Makefile\
 $(MDL_FILE_FORMAT_DIR)/test/lex/Makefile\
 $(MDL_FILE_FORMAT_DIR)/test/lex/main.ml\
 $(MDL_FILE_FORMAT_DIR)/test/parse/Makefile\
 $(MDL_FILE_FORMAT_DIR)/test/parse/main.ml\
 $(MDL_DAST_DIR)/test/Makefile\
 $(MDL_DAST_DIR)/test/main.ml\
 $(MDL_EXPLICIT_DIR)/test/Makefile\
 $(MDL_EXPLICIT_DIR)/test/main.ml\
 $(MDL_SPLIT_DIR)/test/Makefile\
 $(MDL_SPLIT_DIR)/test/main.ml\
 $(MDL_ACTION_DIR)/test/Makefile\
 $(MDL_ACTION_DIR)/test/main.ml\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/test/Makefile\
 $(MDL_GENR_BLOCK_SCICOS_DIR)/test/main.ml\

LOCAL_FILES_TO_BANNERIZE=\
 $(CONFIG_FILES_TO_BANNERIZE)\
 $(MAIN_DIR)/INSTALL\
 $(MAIN_DIR)/Makefile\
 $(MAIN_DIR)/Makefile.admin\
 $(MAIN_DIR)/Makefile.distrib\
 $(MAIN_DIR)/Makefile.test\
 $(MAIN_DIR)/README\
 $(MAIN_DIR)/README_FOR_DEVELOPERS\
 $(MAIN_DIR)/CHANGES\
 $(SRC_DIR)/Objs.mk\
 $(CAML_GENERATING_FILES)\
 $(SIMPORT_FILES_TO_BANNERIZE)\
 $(LOCAL_TEST_TO_BANNERIZE)\

LOCAL_SCRIPTS_TO_BANNERIZE=\
 $(MAIN_DIR)/configure\
 $(CONFIG_SCRIPTS_TO_BANNERIZE)\

MATLAB_FILES_TO_BANNERIZE=\
 $(MATLAB_CAML_FILES)\

MATLAB_SCRIPTS_TO_BANNERIZE=\

SLX_FILES_TO_BANNERIZE=\
 $(SLX_FILE_FORMAT_CAML_FILES)\

SLX_SCRIPTS_TO_BANNERIZE=\

# Refrain from using $< in this rule: this is a non portable gnu-ism.
$(CONFIG_DIR)/banner: $(CONFIG_DIR)/banner.in
	$(HTMLC) -f $(CONFIG_DIR)/banner.in -o $@

# Refrain from using $< in this rule: this is a non portable gnu-ism.
$(MATLAB_DIR)/banner: $(MATLAB_DIR)/banner.in
	$(HTMLC) -f $(MATLAB_DIR)/banner.in -o $@

# Refrain from using $< in this rule: this is a non portable gnu-ism.
$(SLX_DIR)/banner: $(SLX_DIR)/banner.in
	$(HTMLC) -f $(SLX_DIR)/banner.in -o $@

SIMPORT_BANNER_GENERATED_FILES=\
 $(CONFIG_DIR)/banner\
 $(MATLAB_DIR)/banner\
 $(SLX_DIR)/banner\

SIMPORT_TEXT_GENERATED_FILES=\
 $(MAIN_DIR)/LICENSE\
 $(CONFIG_DIR)/Config.mk.in\

SIMPORT_GENERATED_FILES=\
 $(SIMPORT_TEXT_GENERATED_FILES)\
 $(SIMPORT_CAML_GENERATED_FILES)\
 $(SIMPORT_BANNER_GENERATED_FILES)\

# This file has not to be recursively rebuilt by implicit make rules.
$(SRC_DIR)/Objs.mk: ;

#
# Local Variables:
#  compile-command: "cd .. && make"
#  End:
#
