(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: say.mli,v 1.1 2019-05-10 14:09:17 jpc Exp $ *)

type message = Configuration.message;;

type error =
  | Fatal_error of message
;;

exception Error of error;;

val report_error : Format.formatter -> error -> unit;;

val fatal_error : message -> 'a;;

val warning : message -> unit;;

val debug : message -> unit;;

val fdebug : (Format.formatter -> unit) -> unit
(** [fdebug f] apply [f] to a formatter only if debug is set. *)
;;

val fdebug_call : string -> ('a -> 'b) -> 'a -> 'b
(** [fdebug_call fname f x] returns [f x] while surrounding the call with
  debug messages indicating that function [f_name] is called. *)
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
