(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: simport_configuration.mli,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

open Ast_types;;

type length = Ast_types.natural;;

type translator =
  | T_simulink
  | T_mtlb
  | T_sime
;;

type simulink_source_language =
  | Mdl
  | Slx
  | Xml
;;

type simulink_target_language =
  | ScicosLab
  | Nsp
  | Modelica
  | Hypermath
  | Oml
  | Octave
;;

type mtlb_source_language =
  | Mtlb
;;

type mtlb_target_language =
  | Matlab
  | Scicos_host of simulink_target_language
;;

type sime_source_language =
  | E_simulink
;;
type sime_target_language =
  | E_scicos
;;

type target_language =
  | Target_simulink of simulink_target_language
  | Target_mtlb of mtlb_target_language
  | Target_sime of sime_target_language
;;

type source_language =
  | Source_simulink of simulink_source_language
  | Source_mtlb of mtlb_source_language
  | Source_sime of sime_source_language
;;

type language =
  | Source_language of source_language
  | Target_language of target_language
;;

type target_modeler =
  | Modeler_scicos
  | Modeler_scicos_pro
  | Modeler_modelicos
;;

type version = string;;
type full_version = string;;

type interface_version = string;;

type interface_source_file_name = Path.explicit_file_name;;
type interface_marshaled_file_name = Path.explicit_file_name;;

type implementation_source_file_name = Path.explicit_file_name;;
type implementation_marshaled_file_name = Path.explicit_file_name;;

type implementation_translated_file_name = Path.explicit_file_name;;

type matlab_source_file_name = Path.explicit_file_name;;
type matlab_translated_file_name = Path.explicit_file_name;;

type mtlb_source_file_name = Path.explicit_file_name;;
type mtlb_translated_file_name = Path.explicit_file_name;;

type sime_source_file_name = Path.explicit_file_name;;
type sime_translated_file_name = Path.explicit_file_name;;

type configuration = {

          software_name : string;
          software_version : version;
          software_full_version : full_version;
          interface_version : interface_version;

  mutable compile_only : bool;

  mutable verbose : bool;
  mutable debug : bool;
  mutable debug_slx : bool;
  mutable warnings : bool;

  mutable search_path : Path.search_path;

  mutable translator : translator option;

  mutable slx_model_ident : Ident.ident option;

  mutable ignore_encoding : bool;

  mutable source_file : Path.file_name option;
  mutable source_encoding : Uencoding_types.character_encoding option;
  mutable companion_source_files : Path.file_name list;

  mutable target_file : Path.file_name option;
  mutable target_encoding : Uencoding_types.character_encoding option;
  mutable companion_target_files : Path.file_name list;

  mutable interfaces : Ident.ident list;

  mutable target_language : target_language option;
  mutable source_language : source_language option;
  mutable target_modeler : target_modeler option;

(* FIXME: should be supressed
  mutable window_width : length;
  mutable window_height : length;

  mutable diagram_left_margin : length;
  mutable diagram_right_margin : length;
  mutable diagram_bottom_margin : length;
 FIXME: end of suppressed*)

  mutable default_font : Ast_types.mdl_font;

          mean_blocks_by_system : natural;
          mean_block_bindings : natural;
          max_block_types : natural;
          mean_block_parameters : natural;

  mutable block_vertical_spacing : length;
  mutable block_port_width : length;
  mutable block_port_height : length;
  mutable block_event_width : length;
  mutable block_event_height : length;
  mutable block_geometry_magnification : natural;

  mutable not_yet_translated_mark : string;

  mutable target_model_ident : Ident.ident;
}
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
