(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main_gen.ml,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

type phase_name = string;;
type treat_file = Path.file_name -> unit;;

let do_phase phase (treat_file : treat_file) =
  let fname =
    try Arguments.parse_and_get_source_file () with
     (* Exceptions from the standard library Arg module. *)
    | Arg.Help s ->
      Format.fprintf Format.std_formatter "@[%s@]@." s;
      exit 0
    | Arg.Bad s ->
      Format.fprintf Format.err_formatter
        "@[command line parsing failure: %s@]@."
        s;
      exit 1

     (* Exceptions from the Simport configuration module. *)
    | Configuration.Error (Configuration.Command_line_error _ as err) ->
      Configuration.report_error Format.err_formatter err;
      exit 2 in

  try

    if Configuration.get_verbose () then
      Format.fprintf Format.err_formatter "%s%,ing file %S@." phase fname;

    treat_file fname;

    if Configuration.get_verbose () then
      Format.fprintf Format.err_formatter
        "File %S successfully %s%,ed@." fname phase;
  with

    (* General OCaml standard library errors. *)
  | Say.Error (Say.Fatal_error _ as err) ->
    Say.report_error Format.err_formatter err;
    exit 3
  | Parsing.Parse_error ->
    Format.fprintf Format.err_formatter "Parsing error: bad file format@.";
    exit 80
  | Failure s ->
    Format.fprintf Format.err_formatter "failure during evaluation: %s@." s;
    exit 85
  | Sys_error err ->
    Format.fprintf Format.err_formatter "system error: %s@." err;
    exit 90

  | x ->
    Format.fprintf Format.err_formatter
      "@[<v>Internal error: spurious exception during evaluation:@ \
       %s@ \
       Please report@]@."
      (Printexc.to_string x);
    exit 100
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
