(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ast_types.mli,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

type natural = Unatural.t;;

type mdl_name = private
  | Mdl_name of string
;;

type mdl_path = private
  | Mdl_path of string
;;

type mdl_bool = private
  | Mdl_bool of string
;;

type mdl_text = private
  | Mdl_text of string
;;

(*
  Shallow code is specific to mask initialisation code.
  In this code, we must substitute variables to get true Matlsb codfe:
  mask variable number i is associated to some matlab ident id
  and must be replaced by id in the script; furthermore,
  if associated ident id is empty, then variable number i should be
  substituted by its associated value field.
*)
type mdl_shallow_code = private
  | Mdl_shallow_code of mdl_text
;;

type mdl_code = private
  | Mdl_code of mdl_text
;;

type mdl_port_number = private
  | Mdl_port_number of natural
;;

type mdl_font = {
  mdl_font_name : mdl_font_name;
  mdl_font_angle : string;
  mdl_font_size : natural;
  mdl_font_weight : string;
}

and mdl_font_name = string
;;

val mdl_bool_of_string : string -> mdl_bool;;
val mdl_text_of_string : string -> mdl_text;;
val mdl_shallow_code_of_string : string -> mdl_shallow_code;;
val mdl_code_of_string : string -> mdl_code;;
val mdl_path_of_string : string -> mdl_path;;

val mdl_port_number_of_natural : natural -> mdl_port_number;;
val mdl_port_number_of_int : int -> mdl_port_number;;

val string_of_ident : Ident.ident -> string;;
val ident_from_string : string -> Ident.ident;;
val ident_from_any_string : string -> Ident.ident;;

val string_of_mdl_name : mdl_name -> string;;
val ident_from_mdl_name : mdl_name -> Ident.ident;;

val string_of_mdl_path : mdl_path -> string;;
val string_of_mdl_bool : mdl_bool -> string;;
val string_of_mdl_code : mdl_code -> string;;
val string_of_mdl_shallow_code : mdl_shallow_code -> string;;
val string_of_mdl_text : mdl_text -> string;;

val natural_of_mdl_port_number : mdl_port_number -> natural;;
val int_of_mdl_port_number : mdl_port_number -> int;;
val string_of_mdl_port_number : mdl_port_number -> string;;

val mdl_text_of_ident : Ident.ident -> mdl_text;;
val mdl_text_of_mdl_name : mdl_name -> mdl_text;;
val mdl_text_of_mdl_code : mdl_code -> mdl_text;;
val mdl_name_of_ident : Ident.ident -> mdl_name;;
val mdl_name_of_string : string -> mdl_name;;

val bool_of_mdl_bool : mdl_bool -> bool;;
val bool_of_mdl_string : string -> bool;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
