(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: intercal.ml,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type ('a, 'b) intercal =
  'a option * ('b * 'a option) list
;;
type ('a, 'b) t = ('a, 'b) intercal;;

type ('a, 'b) sum =
| In_left of 'a
| In_right of 'b
;;

let empty = None, [];;

let second (_, parts) = parts;;
let first (first_alpha_opt, _) = first_alpha_opt;;

let cons_alpha alpha = function
  | None, [] -> Some alpha, []
  | first_alpha_opt, (beta, None) :: parts ->
    first_alpha_opt, (beta, Some alpha) :: parts
  | _, (_, Some _) :: _
  | Some _, [] ->
    failwith "Intercal.cons_alpha: list should start with a 'b element"
;;

let cons_beta beta (first_alpha_opt, parts) =
  first_alpha_opt, (beta, None) :: parts
;;

let singleton_alpha a = cons_alpha a empty;;
let singleton_beta b = cons_beta b empty;;

let is_empty = function
  | None, [] -> true
  | _ -> false
;;

let is_head_alpha = function
  | None, [] -> false
  | Some _, [] -> true
  | _, (_, None) :: _ -> false
  | _, (_, Some _) :: _ -> true
;;

let is_head_beta = function
  | _, [] -> false
  | _, (_, None) :: _ -> true
  | _, (_, Some _) :: _ -> false
;;

let rev (first_alpha_opt, parts) = (first_alpha_opt, List.rev parts);;

let part_fold alpha_folder beta_folder accu = function
  | (beta, None) -> beta_folder accu beta
  | (beta, Some alpha) -> alpha_folder (beta_folder accu beta) alpha
;;

let fold_left alpha_folder beta_folder accu = function
  | None, parts ->
    List.fold_left (part_fold alpha_folder beta_folder) accu parts
  | Some alpha, parts ->
    List.fold_left
      (part_fold alpha_folder beta_folder)
      (alpha_folder accu alpha)
      parts
;;

let iter alpha beta intercal =
  fold_left (fun _ a -> alpha a) (fun _ b -> beta b) () intercal
;;

let part_map alpha_mapper beta_mapper = function
  | beta, None -> beta_mapper beta, None
  | beta, Some alpha -> beta_mapper beta, Some (alpha_mapper alpha)
;;

(*

Strange behaviour of the compiler: pats is not reported as useless

let map alpha_mapper beta_mapper intercal =
  match intercal with
  | (None, parts : (_, _) intercal) ->
    let pats =
      List.map (part_map alpha_mapper beta_mapper) parts in
    (None, parts : (_, _) intercal)
    (*List.map (part_map alpha_mapper beta_mapper) parts*)
  | Some alpha, parts ->
    Some (alpha_mapper alpha), []
(*cons_alpha (alpha_mapper alpha)
      (List.map (part_map alpha_mapper beta_mapper) parts)*)
;;

*)

let map alpha_mapper beta_mapper intercal =
  match intercal with
  | None, parts ->
    None,
    List.map (part_map alpha_mapper beta_mapper) parts
  | Some alpha, parts ->
    Some (alpha_mapper alpha),
    List.map (part_map alpha_mapper beta_mapper) parts
;;

let rev_append intercal accu =
  fold_left
    (fun accu x_a -> In_left x_a :: accu)
    (fun accu x_a -> In_right x_a :: accu)
    accu
    intercal
;;

let to_elements intercal = List.rev (rev_append intercal []);;

let of_elements elements =
  let l =
    List.fold_left
      (fun intercal -> function
      | In_left alpha -> cons_alpha alpha intercal
      | In_right beta -> cons_beta beta intercal)
      empty
      elements in
  rev l
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
