(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 1997-2018 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Clement Franchini <Clement.Franchini@inria.fr>                     *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_ident.mli,v 1.1 2019-05-10 14:02:41 jpc Exp $ *)

(** {3 Type definitions for OCaml identifiers} *)

type name = string;;

type lowercase_name = private name;;
val of_lowercase_name : lowercase_name -> name;;
val to_lowercase_name : name -> lowercase_name;;

type uppercase_name = private name;;
val of_uppercase_name : uppercase_name -> name;;
val to_uppercase_name : name -> uppercase_name;;

type symbolic_name = private name;;
val of_symbolic_name : symbolic_name -> name;;
val to_symbolic_name : name -> symbolic_name;;

type ident =
  | Uppercase of uppercase_name
  | Lowercase of lowercase_name
  | Symbolic of symbolic_name
;;

val name_of_ident : ident -> name;;
val ident_of_name : name -> ident;;

val ident_of_lowercase_name : lowercase_name -> ident;;
val ident_of_uppercase_name : uppercase_name -> ident;;
val ident_of_symbolic_name : symbolic_name -> ident;;

type longident =
  | Lident of ident
  | Ldot of longident * ident
  | Lapply of longident * longident
(** Qualified ident: an ident with the module path corresponding to the
  location of the ident definition. *)
;;

val longident_of_name : name -> longident
(** [longident_of_name name] returns a longident corresponding to [name].

  This function always returns a [longident] which is a simple ident (no
  qualification). Hence, if [name] contains functor application or
  nodule qualification, the result is meaningless.

  Note: more secure verifications could ensure that [name] is
  indeed unqualified, or else more complex scanning of name values may
  analyse qualification and provide a correct projection from [name] to
  [longident]. *)
;;

val longident_of_uppercase_name : uppercase_name -> longident;;
val longident_of_lowercase_name : lowercase_name -> longident
(** Specialized versions of [longident_of_name] for uppercase and lowercase
  names. *)
;;

val name_of_longident : longident -> name
(** [name_of_longident longident] returns the string representation of
  [longident]. *)
;;
val short_name_of_longident : longident -> name
(** [short_name_of_longident longident] returns the string representation of
  the last ident part of [longident]. *)
;;

val split_longident : longident -> longident option * ident
(** [split_longident longident] extracts the path qualifier and the ident part
  of a longident. *)
;;

val unused_var_name : lowercase_name -> lowercase_name
(** [unused_var_name lname] prefixes [lname] with a ['_'] character (the OCaml
  'unused variable' prefix marker). *)
;;
