(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ulist2.ml,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type 'a list2 = 'a * 'a Ulist1.t;;

type 'a t = 'a list2;;

let length (_, l) = succ (Ulist1.length l);;

let to_list (h1, l) = h1 :: Ulist1.to_list l;;

let of_list = function
| [] | [ _ ] -> invalid_arg "List2.of_list"
| h :: l -> h, Ulist1.of_list l
;;

let of_list1 (h1, l1) =
  match l1 with
  | [] | [ _ ] -> invalid_arg "List2.of_list1"
  | h2 :: l2 -> h1, (h2, l2)
;;

let cons x (h, l) = x, Ulist1.cons h l;;

let rev_append (h, l) ll = of_list1 (Ulist1.rev_append l (h :: ll));;

let rev l = rev_append l [];;

let map f (h, l) = (f h, Ulist1.map f l);;

let iter f (h, l) = f h; Ulist1.iter f l;;

let fold_left f accu (h, l) = Ulist1.fold_left f (f accu h) l;;

let fold_right f (h, l) accu = f h (Ulist1.fold_right f l accu);;

let map2 f (h1, l1) (h2, l2) =
  (f h1 h2, Ulist1.map2 f l1 l2)
;;

let fold_map f env (h, l) =
  let env, h = f env h in
  let env, l = Ulist1.fold_map f env l in
  env, (h, l)
;;

let rev_fold_flat_map_end f env (x, l) raccu =
  let env, lres = f env x in
  let raccu = List.rev_append lres raccu in
  Ulist1.rev_fold_flat_map_end f env l raccu
;;

let rev_fold_flat_map f env l = rev_fold_flat_map_end f env l [];;

let fold_flat_map f env l =
  let env, l = rev_fold_flat_map f env l in
  env, List.rev l
;;

let iter2 f (h1, l1) (h2, l2) =
  f h1 h2; Ulist1.iter2 f l1 l2
;;

let find p (h, l) =
  if p h then h else
  Ulist1.find p l
;;

let find_all p (h, l) =
  if p h then h :: Ulist1.find_all p l else Ulist1.find_all p l
;;

let filter = find_all;;

let partition p l = List.partition p (to_list l)
;;

let max_snd_assoc cmp (x, l) =
  Ulist1.max_snd_assoc cmp (Ulist1.cons x l)
;;

let split ((x1, x2), l) =
  let l1, l2 = Ulist1.split l in
  (x1, l1), (x2, l2)
;;

let combine (x1, l1) (x2, l2) = (x1, x2), Ulist1.combine l1 l2;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
