(***********************************************************************)
(*                                                                     *)
(*                           Simport                                   *)
(*                                                                     *)
(*          Pierre Weis, INRIA Rocquencourt                            *)
(*                                                                     *)
(*  Copyright 2010-2013,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_configuration.ml,v 1.1 2019-05-10 14:00:17 jpc Exp $ *)

open Ocamlxml_configuration;;

type error =
  | Fatal_error of string
;;

exception Error of error;;

let print_version v = Format.fprintf Format.std_formatter "%s" v;;

let ocaml_xml_lib_name = "ocaml-xml";;
let ocaml_xml_lib_version = "0.1.0";;

let conf = {
  Ocamlxml_configuration.
  library_name = ocaml_xml_lib_name;
  library_version = ocaml_xml_lib_version;

  verbose = false;
  debug = false;
  warnings = true;
}
;;

let set_verbose, get_verbose =
  (fun () -> conf.verbose <- true),
  (fun () -> conf.verbose)
;;

let set_debug, get_debug =
  (fun () -> conf.debug <- true),
  (fun () -> conf.debug)
;;

let set_warnings, get_warnings =
  (fun () -> conf.warnings <- true),
  (fun () -> conf.warnings)
;;

let set_no_warnings, get_no_warnings =
  (fun () -> conf.warnings <- false),
  (fun () -> not conf.warnings)
;;

let set_parsing_debug () = ignore (Parsing.set_trace true);;

let get_library_name () = conf.library_name;;
let get_library_version () = conf.library_version;;

let report_error ppf = function
  | Fatal_error s ->
    let soft = get_library_name () in
    Format.fprintf ppf
      "@[<v 5>%s: fatal error %s@]@." soft s
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
