(***********************************************************************)
(*                                                                     *)
(*                           Matlab Parser                             *)
(*                                                                     *)
(*          J.Ph Chancelier, Enpc/Cermics                              *)
(*                                                                     *)
(*  Copyright 2012-2019,                                               *)
(*  Ecole Nationale des ponts et chaussées                             *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mtlb_to_ast.ml,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

(** Entry points to the parser *)

let parse lexbuf =
  try
    Mtlb_lexer.init ();
    let ast = Mtlb_parser.mtlb_ast Mtlb_lexer.token lexbuf in
    Parsing.clear_parser();
    ast
  with
  | Mtlb_lexer.Error (e, pos_begin, pos_end) ->
    let loc = Mtlb_location.mk_loc pos_begin pos_end in
    let s =
      Format.asprintf
        "Lexical error: %a@.%a"
        Mtlb_location.print loc
        Mtlb_lexer.report_error e in
    failwith s
  | Mtlb_syntaxerr.Error err ->
    let s =
      Format.asprintf "%a"
        Mtlb_syntaxerr.report_error err in
    failwith s
  | Parsing.Parse_error ->
    let loc =
      { Mtlb_location.loc_beg = lexbuf.Lexing.lex_start_p;
        Mtlb_location.loc_end = lexbuf.Lexing.lex_curr_p} in
    let err = Mtlb_syntaxerr.Other loc in
    let s =
      Format.asprintf "%a"
        Mtlb_syntaxerr.report_error err in
    failwith s
;;

(* Parse a file *)

let parse_ic fname ic =
  let lexbuf = Lexing.from_channel ic in
  Mtlb_location.init lexbuf fname;
  parse lexbuf
;;

let parse_file efname =
  Path.with_explicit_in_file_bin efname (fun ic ->
    parse_ic (Path.of_explicit_file_name efname) ic)
;;

(* Parse a string *)

let parse_string s =
  let lexbuf = Lexing.from_string s in
  Mtlb_location.init lexbuf (Printf.sprintf "<string: %s>" s);
  parse lexbuf
;;

let read_mtlb_file = parse_file;;
let read_mtlb_string = parse_string;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
