(***********************************************************************)
(*                                                                     *)
(*                           Matlab Parser                             *)
(*                                                                     *)
(*          J.Ph Chancelier, Enpc/Cermics                              *)
(*                                                                     *)
(*  Copyright 2012-2019,                                               *)
(*  Ecole Nationale des ponts et chaussées                             *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mtlb_ppf.mli,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

(*
For nsp
let nsp_tnlm = {
  m_tag_new_line = " ...\n";
  n_tag_new_line = "\n";
}

For scicoslab use nsp_tnlm.

For hml

let hml_tnlm = {
  m_tag_new_line = "\n";
  n_tag_new_line = "\n";
}

For octave use nsp_tnlm

For matlab use nsp_tnlm
*)

type tag_new_line_markers = {
  m_tag_new_line : string;
  n_tag_new_line : string;
}
;;

val make_tag_printer : tag_new_line_markers ->
  'a Og_types.top_printer -> 'a Og_types.top_printer
(** A printer transformer: turn a printer (a function with type
  [Format.formatter -> 'a -> unit]) into a printer with two semantics tags
  defined [m] and [n] *)
;;

val make_single_line_printer :
 'a Og_types.top_printer -> 'a Og_types.top_printer
;;

val make_single_line_tag_printer : tag_new_line_markers ->
  'a Og_types.top_printer -> 'a Og_types.top_printer
(** A printer transformer: turn a printer into a printer with two semantics tags
  that prints all its material on a single line. *)
;;

val make_tag_sprinter : tag_new_line_markers ->
  'a Og_types.top_printer -> 'a -> string
(** A printer transformer: turn a printer argument into a printer to a string
  with two semantics tags defined m and n. *)
;;

val make_single_line_sprinter :
  'a Og_types.top_printer -> 'a -> string
(** A printer transformer: turn a printer into a printer to a
  string that prints all its material on a single line. *)
;;

val make_single_line_tag_sprinter : tag_new_line_markers ->
  'a Og_types.top_printer -> 'a -> string
(** A printer transformer: turn a printer into a printer to a
  string with two semantics tags defined m and n,
  that prints all its material on a single line. *)
;;

val matlab_printer :
  'a Og_types.top_printer -> tag_new_line_markers -> 'a Og_types.top_printer
(** A printer transformer: turn a printer (a function with type
  [Format.formatter -> 'a -> unit]) into a matlab end of line aware printer.

  The resulting printer uses semantics tags to handle end of lines: tag [m]
  delimits a formatting area where end of lines are written the matlab way as
  [" ...\n"]; tag [n] delimits a formatting area where end of lines are
  treated the regular way as a simple ["\n"].
  Semantics tags [m] and [n] must be properly delimited with usual module
  [Format] semantics tag markers: open tag [m] (resp. tag [n]) using opening
  marker ["@{<m>"] (resp. marker ["@{<n>"]) and close tags using the closing
  marker ["@}"].
  Areas delimited with tags [m] and [n] could be nested.
  For simplicity, the empty tag is equivalent to tag [m], hence an area
  simply delimited between ["@{"] and ["@}"] will treat end of lines the
  matlab way.
*)
;;

val matlab_sprinter :
  'a Og_types.top_printer -> tag_new_line_markers -> 'a -> string
(** A printer transformer: turn a printer into a printer to string that is
  host end of line aware. *)
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
