(***********************************************************************)
(*                                                                     *)
(*                           Matlab Parser                             *)
(*                                                                     *)
(*          J.Ph Chancelier, Enpc/Cermics                              *)
(*                                                                     *)
(*  Copyright 2012-2019,                                               *)
(*  Ecole Nationale des ponts et chaussées                             *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mtlb_hypermath_host.ml,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

(** The specific format strings and formatters for Hypermath translation. *)

module Host :
       Mtlb_make_host_print_spec.HOST = struct

  let then_delim = " then";;
  let return_list_delim = false;;
  let do_delim = " do";;

  let persistent_keyword = "global";;
  let persistent_declaration_sep = ",";;

  let break_case_keyword = "break";;

  let switch_keyword = "switch";;

  let begin_switch_marker = "{";;

  let switch_condition_open_marker = "(";;
  let switch_condition_close_marker = ")";;

  let case_condition_sep_marker = ":";;

  let otherwise_keyword = "default:";;

  let end_switch_marker = "}";;

  let extract_delim_start = function
    | "{" -> "("
    | s -> s
  ;;

  let extract_delim_end  = function
    | "}" -> ")"
    | s -> s
  ;;

  let tnlm = {
    Mtlb_ppf.m_tag_new_line = "\n";
    Mtlb_ppf.n_tag_new_line = "\n";
  }
  ;;

  let imag s = String.sub s 0 (String.length s - 1) ^ "i";;

  let dict_fun = function
    | "clear" -> "//clear"
    | "disp" -> "print"
    | "interp1" -> "LinInterp"
    | "load" -> "ImportMatFile"
    | "end" -> "_last_index"
    | "do" -> "_do"
    | "~" -> "_v"
    | s -> s
  ;;

  let close_block_dict_fun = function
    | "endfor" -> "end"
    | "endif" -> "end"
    | "endwhile" -> "end"
    | "end_try_catch" -> "end"
    | s -> s
  ;;

  let close_fun_dict_fun = function
    | "endfunction" -> "end"
    | s -> s
  ;;

  let comment_char_dict_fun = function
    | '#' -> " //"
    | '%' -> " //"
    | _s ->  assert false
  ;;

  let postfix_op_dict_fun = function
    | "'" -> "`"
    | ".'" -> ".`"
    | s -> s
  ;;

  let sq_char_dict_fun = function
    | '\'' -> "\\\'"
    | '\"' -> "\\\""
    | '\r' -> ""
    | '\n' ->
      Say.fatal_error "Invalid character \\n in Matlab string."
    | '\\' -> "\\\\"
    | '%' -> "\\%"
    | c -> String.make 1 c
  ;;

  let sq_string_delimiter = '\"';;

  let text_delimiter = String.make 1 sq_string_delimiter;;

  let text_char_dict_fun = function
    | '\n' -> "\\n"
    | c -> sq_char_dict_fun c
  ;;

  let text_delimiters_record = {
    Mtlb_make_host_print_spec.
    open_text = text_delimiter;
    close_text = text_delimiter;
  }
  ;;

  let text_delimiters _txt = text_delimiters_record;;

  let not_op = "!";;

  let ne_op = "~=";;

  let protect_colon_expr = true;;

  let protect_lhs = false;;

end
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
