(***********************************************************************)
(*                                                                     *)
(*                           Matlab Parser                             *)
(*                                                                     *)
(*          J.Ph Chancelier, Enpc/Cermics                              *)
(*                                                                     *)
(*  Copyright 2012-2019,                                               *)
(*  Ecole Nationale des ponts et chaussées                             *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mtlb_matlab_host.ml,v 1.1 2019-05-10 14:09:18 jpc Exp $ *)

(** The specific format strings and formatters for Matlab translation. *)

module Host :
       Mtlb_make_host_print_spec.HOST = struct

  let then_delim = "";;
  let return_list_delim = true;;
  let do_delim = "";;

  let persistent_keyword = "persistent";;
  let persistent_declaration_sep = " ";;

  let break_case_keyword = "";;

  let switch_keyword = "switch";;

  let begin_switch_marker = "";;

  let switch_condition_open_marker = "";;
  let switch_condition_close_marker = "";;

  let case_condition_sep_marker = "";;

  let otherwise_keyword = "otherwise";;

  let end_switch_marker = "end";;

  let extract_delim_start delim = delim;;
  let extract_delim_end delim = delim;;

  let tnlm = {
    Mtlb_ppf.m_tag_new_line = " ...\n";
    Mtlb_ppf.n_tag_new_line = "\n";
  }
  ;;

  let imag s = s;;

  let dict_fun s = s;;

  let close_block_dict_fun = function
    | "endfor" -> "end"
    | "endif" -> "end"
    | "endwhile" -> "end"
    | "endswitch" -> "end"
    | "end_try_catch" -> "end"
    | s -> s
  ;;

  let close_fun_dict_fun = function
    | "endfunction" -> "end"
    | s -> s
  ;;

  let comment_char_dict_fun = function
    | '#'
    | '%' -> " %"
    | _s -> assert false
  ;;

  let postfix_op_dict_fun s = s;;

  (* dealing with escapes in single quoted strings *)

  let sq_char_dict_fun = function
    | '\'' -> "\'\'"
    | '\r' -> ""
    | '\n' ->
      Say.fatal_error "Invalid character \\n in Matlab string."
    | c -> String.make 1 c
  ;;

  let sq_string_delimiter = '\'';;

  let text_delimiter = String.make 1 sq_string_delimiter;;

  let text_char_dict_fun = function
    | '\n' -> "\',char(10),\'"
    | c -> sq_char_dict_fun c
  ;;

  let text_delimiters_record = {
    Mtlb_make_host_print_spec.
    open_text = text_delimiter;
    close_text = text_delimiter;
  }
  ;;

  let concatenated_text_delimiters_record = {
    Mtlb_make_host_print_spec.
    open_text = "[" ^ text_delimiter;
    close_text = text_delimiter ^ "]";
  }
  ;;

  let text_delimiters text =
    let txt = Ast_types.string_of_mdl_text text in
    if String.contains txt '\n'
    then concatenated_text_delimiters_record
    else text_delimiters_record
  ;;

  let not_op = "!";;

  let ne_op = "~=";;

  let protect_colon_expr = false;;

  let protect_lhs = true;;

end
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
