(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.5+dev0 generated code. *)

let printer_explicit_block pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_block pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_system pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_system pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_model pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_model pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_simulink_file pp_a ctx ppf = function
  | a_m_e ->
    Mdl_explicit_ast_print.printer_explicit_simulink_file pp_a ctx ppf a_m_e
;;

let printer_split_link_connectors pp_a pp_b ctx ppf = function
  | a_b_m_s ->
    Mdl_split_ast_print.printer_split_link_connectors pp_a pp_b
      ctx
      ppf
      a_b_m_s
;;

let printer_directed_connector_desc ctx ppf = function
  | m_d -> Mdl_split_ast_print.printer_directed_connector_desc ctx ppf m_d
;;

let printer_split_connector_port pp_a ctx ppf = function
  | a_m_s ->
    Mdl_split_ast_print.printer_split_connector_port pp_a ctx ppf a_m_s
;;

let printer_poly_split_link pp_a ctx ppf = function
  | a_m_p -> Mdl_split_ast_print.printer_poly_split_link pp_a ctx ppf a_m_p
;;

let printer_action_connector_port pp_a ctx ppf = function
  | a_s -> printer_split_connector_port pp_a ctx ppf a_s
;;

let printer_poly_action_link pp_link_desc ctx ppf = function
  | l_p -> printer_poly_split_link pp_link_desc ctx ppf l_p
;;

let printer_action_link_connectors pp_a ctx ppf = function
  | a_a_s -> printer_split_link_connectors pp_a pp_a ctx ppf a_a_s
;;

let printer_action_directed_connector_desc ctx ppf = function
  | d -> printer_directed_connector_desc ctx ppf d
;;

let printer_action_event_connector_desc ctx ppf = function
  | a -> printer_action_directed_connector_desc ctx ppf a

and printer_action_connection_connector_desc ctx ppf = function
  | m_s ->
    Mdl_split_ast_print.printer_split_connection_connector_desc ctx ppf m_s
;;

let printer_action_directed_connector ctx ppf = function
  | a_a ->
    printer_action_connector_port printer_action_directed_connector_desc
      ctx
      ppf
      a_a
;;

let printer_action_connection_connector ctx ppf = function
  | a_a ->
    printer_action_connector_port printer_action_connection_connector_desc
      ctx
      ppf
      a_a
;;

let printer_action_directed_connectors ctx ppf = function
  | a_a ->
    printer_action_link_connectors printer_action_directed_connector_desc
      ctx
      ppf
      a_a
;;

let printer_action_event_connectors ctx ppf = function
  | a_a ->
    printer_action_link_connectors printer_action_event_connector_desc
      ctx
      ppf
      a_a
;;

let printer_action_connection_connectors ctx ppf = function
  | a_a ->
    printer_action_link_connectors printer_action_connection_connector_desc
      ctx
      ppf
      a_a
;;

let printer_action_link_desc ctx ppf = function
  | Mdl_action_ast.Action_directed_link a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_action_ast.Action_directed_link";
    printer_action_directed_connectors Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_action_ast.Action_event_link a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_action_ast.Action_event_link";
    printer_action_event_connectors Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_action_ast.Action_connection_link a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_action_ast.Action_connection_link";
    printer_action_connection_connectors Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_action_links ctx ppf = function
  | a_l -> Og_printer.printer_List printer_action_link ctx ppf a_l

and printer_action_link ctx ppf = function
  | a_p -> printer_poly_action_link printer_action_link_desc ctx ppf a_p
;;

let printer_action_block ctx ppf = function
  | a_e -> printer_explicit_block printer_action_link ctx ppf a_e

and printer_action_system ctx ppf = function
  | a_e -> printer_explicit_system printer_action_link ctx ppf a_e

and printer_action_model ctx ppf = function
  | a_e -> printer_explicit_model printer_action_link ctx ppf a_e

and printer_action_simulink_file ctx ppf = function
  | a_e -> printer_explicit_simulink_file printer_action_link ctx ppf a_e
;;

(* Defining top printers. *)

let print_explicit_block pp_a ppf =
  printer_explicit_block pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_system pp_a ppf =
  printer_explicit_system pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_model pp_a ppf =
  printer_explicit_model pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_simulink_file pp_a ppf =
  printer_explicit_simulink_file pp_a Og_precedence.Ck_toplevel ppf
;;

let print_split_link_connectors pp_a pp_b ppf =
  printer_split_link_connectors pp_a pp_b Og_precedence.Ck_toplevel ppf
;;

let print_directed_connector_desc ppf =
  printer_directed_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_split_connector_port pp_a ppf =
  printer_split_connector_port pp_a Og_precedence.Ck_toplevel ppf
;;

let print_poly_split_link pp_a ppf =
  printer_poly_split_link pp_a Og_precedence.Ck_toplevel ppf
;;

let print_action_connector_port pp_a ppf =
  printer_action_connector_port pp_a Og_precedence.Ck_toplevel ppf
;;

let print_poly_action_link pp_link_desc ppf =
  printer_poly_action_link pp_link_desc Og_precedence.Ck_toplevel ppf
;;

let print_action_link_connectors pp_a ppf =
  printer_action_link_connectors pp_a Og_precedence.Ck_toplevel ppf
;;

let print_action_directed_connector_desc ppf =
  printer_action_directed_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_action_event_connector_desc ppf =
  printer_action_event_connector_desc Og_precedence.Ck_toplevel ppf

and print_action_connection_connector_desc ppf =
  printer_action_connection_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_action_directed_connector ppf =
  printer_action_directed_connector Og_precedence.Ck_toplevel ppf
;;

let print_action_connection_connector ppf =
  printer_action_connection_connector Og_precedence.Ck_toplevel ppf
;;

let print_action_directed_connectors ppf =
  printer_action_directed_connectors Og_precedence.Ck_toplevel ppf
;;

let print_action_event_connectors ppf =
  printer_action_event_connectors Og_precedence.Ck_toplevel ppf
;;

let print_action_connection_connectors ppf =
  printer_action_connection_connectors Og_precedence.Ck_toplevel ppf
;;

let print_action_link_desc ppf =
  printer_action_link_desc Og_precedence.Ck_toplevel ppf
;;

let print_action_links ppf =
  printer_action_links Og_precedence.Ck_toplevel ppf

and print_action_link ppf = printer_action_link Og_precedence.Ck_toplevel ppf
;;

let print_action_block ppf =
  printer_action_block Og_precedence.Ck_toplevel ppf

and print_action_system ppf =
  printer_action_system Og_precedence.Ck_toplevel ppf

and print_action_model ppf =
  printer_action_model Og_precedence.Ck_toplevel ppf

and print_action_simulink_file ppf =
  printer_action_simulink_file Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.5+dev0 generated code. *)
