(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_explicit_block_utils.mli,v 1.1 2019-05-10 14:07:13 jpc Exp $ *)

(** {3 Useful utilities to access and set explicit_block fields} *)

(** {6 Accessing various explicit block fields} *)

val string_of_block_name : Ast_types.mdl_name -> string
;;

val get_string_block_name :
  'a Mdl_implementation_explicit_ast.explicit_block -> string
;;

val get_block_body :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_body
;;

val get_block_all_bindings :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_all_bindings
;;

val add_block_parameters :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  (Ident.ident * Mdl_basics_explicit_ast.explicit_value) list ->
  unit
;;

val get_block_parameters :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_parameters
;;

val get_block_parameter_value :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Ident.ident ->
  Mdl_basics_explicit_ast.explicit_value
;;

val get_block_graphical :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_graphical
;;

val get_block_mirror :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_mirror
;;

val get_block_ports_description :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_ports_description
;;

val get_block_regular_port_nin :
  'a Mdl_implementation_explicit_ast.explicit_block -> Unatural.t
;;

val get_block_regular_port_nout :
  'a Mdl_implementation_explicit_ast.explicit_block -> Unatural.t
;;

val get_block_implicit_port_nin :
  'a Mdl_implementation_explicit_ast.explicit_block -> Unatural.t
;;

val get_block_implicit_port_nout :
  'a Mdl_implementation_explicit_ast.explicit_block -> Unatural.t
;;

val get_block_position :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_position
;;

val get_block_rotation :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_rotation
;;

(** {6 Setting various explicit block fields} *)

val set_block_body :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_body ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

(*
val set_block_type :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_type ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;
*)

(*
val set_block_all_bindings :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_all_bindings ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;
*)

val set_block_graphical :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_implementation_explicit_ast.explicit_block_graphical ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_ports_description :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_ports_description ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_regular_port_nin :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Unatural.t -> 'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_regular_port_nout :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Unatural.t -> 'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_implicit_port_nin :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Unatural.t -> 'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_implicit_port_nout :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Unatural.t -> 'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_position :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_position ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_rotation :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_rotation ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val map_block_rotation :
  (Mdl_basics_explicit_ast.explicit_rotation ->
   Mdl_basics_explicit_ast.explicit_rotation) ->
  'a Mdl_implementation_explicit_ast.explicit_block ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val set_block_mirror :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  Mdl_basics_explicit_ast.explicit_mirror ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val map_block_mirror :
  (Mdl_basics_explicit_ast.explicit_mirror ->
   Mdl_basics_explicit_ast.explicit_mirror) ->
  'a Mdl_implementation_explicit_ast.explicit_block ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

val get_ebss_blocks :
  'a Mdl_implementation_explicit_ast.explicit_block_sub_system_desc ->
  'a Mdl_implementation_explicit_ast.explicit_blocks;;
val set_ebss_blocks :
  'a Mdl_implementation_explicit_ast.explicit_block_sub_system_desc ->
  'a Mdl_implementation_explicit_ast.explicit_blocks ->
  'a Mdl_implementation_explicit_ast.explicit_block_sub_system_desc

(** Add a block to the list of blocks of a sub-system *)
val add_ebss_block :
  'a Mdl_implementation_explicit_ast.explicit_block_sub_system_desc ->
  'a Mdl_implementation_explicit_ast.explicit_block ->
  'a Mdl_implementation_explicit_ast.explicit_block_sub_system_desc
;;

val set_block_ebss :
  'a Mdl_implementation_explicit_ast.explicit_block ->
  'b Mdl_implementation_explicit_ast.explicit_block_sub_system_desc ->
  'b Mdl_implementation_explicit_ast.explicit_block
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
