(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_ast_functions.ml,v 1.1 2019-05-10 14:07:13 jpc Exp $ *)

let make_binding id v =
  { Binding.
    bound_ident = id;
    bound_value = v;
  }
;;

(* We use this function with strings that came from the lexer that classified
   them as IDENT tokens.
   Hence, we do not check again that the string argument is indeed a valid
   ident token using function Ast_types.ident_of_string; we directly inject
   the string as a true ident.
   Be cautious if using this function with a literal string!
*)
let make_ident s = Ident.Ident s;;

let make_configuration_longident s =
  Mdl_ast.Configuration_lident (make_ident s)
;;

(* Upper case identifiers for blocks and other MDL file sections *)
let config_set_longident = make_configuration_longident "ConfigSet"
(*
let config_set_ident = make_ident "ConfigSet"
*)
and data_type_ident = make_ident "DataType"
and dimension_ident = make_ident "Dimension"
and location_ident = make_ident "Location"
and port_ident = make_ident "Port"
and side_ident = make_ident "Side"
and port_dimensions_ident = make_ident "PortDimensions"
and port_number_ident = make_ident "PortNumber"
and string_ident = make_ident "String"
and saved_character_encoding_ident = make_ident "SavedCharacterEncoding"
and system_ident = make_ident "System"
and tag_ident = make_ident "Tag"
and treat_as_atomic_unit_ident = make_ident "TreatAsAtomicUnit"
and sf_block_type_ident = make_ident "SFBlockType"
and system_sample_time_ident = make_ident "SystemSampleTime"
and zoom_factor_ident = make_ident "ZoomFactor"
and sample_time_ident = make_ident "SampleTime"
and action_type_ident = make_ident "ActionType"
and array_ident = make_ident "Array"
and trigger_type_ident = make_ident "TriggerType"
and value_ident = make_ident "Value"
and interface_name_ident = make_ident "InterfaceName"
and text_ident = make_ident "Text"
and type_ident = make_ident "Type"
and version_ident = make_ident "Version"
and sid_ident = make_ident "SID"
;;

(* Lower case identifiers for StateFlow section. *)
let larray_ident = make_ident "array"
and lchart_ident = make_ident "chart"
and ldata_ident = make_ident "data"
and ldst_ident = make_ident "dst"
and leml_ident = make_ident "eml"
and lenable_ident = make_ident "enable"
and levent_ident = make_ident "event"
and lexecution_order_ident = make_ident "executionOrder"
and lfirst_data_ident = make_ident "firstData"
and lfirst_event_ident = make_ident "firstEvent"
and lfirst_index_ident = make_ident "firstIndex"
and lfirst_junction_ident = make_ident "firstJunction"
and lfirst_transition_ident = make_ident "firstTransition"
and lfirst_target_ident = make_ident "firstTarget"
and lid_ident = make_ident "id"
and linstance_ident = make_ident "instance"
and lintersection_ident = make_ident "intersection"
and ljunction_ident = make_ident "junction"
and llabel_string_ident = make_ident "labelString"
and llink_node_ident = make_ident "linkNode"
and lmachine_ident = make_ident "machine"
and lprops_ident = make_ident "props"
and lchart_file_number_ident = make_ident "chartFileNumber"
and lname_ident = make_ident "name"
and lscope_ident = make_ident "scope"
and lsrc_ident = make_ident "src"
and lss_id_number_ident = make_ident "ssIdNumber"
and lstate_ident = make_ident "state"
and lsize_ident = make_ident "size"
and ltarget_ident = make_ident "target"
and ltransition_ident = make_ident "transition"
and ltree_node_ident = make_ident "treeNode"
and ltrigger_ident = make_ident "trigger"
and ltype_ident = make_ident "type"
and lscript_ident = make_ident "script"
and lparamIndexForInitFromWorkspace_ident =
  make_ident "paramIndexForInitFromWorkspace"
;;

let make_dimension_binding = make_binding dimension_ident;;
let make_port_binding = make_binding port_ident;;
let make_port_number_binding = make_binding port_number_ident;;
let make_system_binding = make_binding system_ident;;
let make_treat_as_atomic_unit_binding = make_binding treat_as_atomic_unit_ident;;
let make_sf_block_type_binding = make_binding sf_block_type_ident;;

let make_interface_block_parameter pname ptype =
  make_binding pname ptype
;;

let mdl_name_of_vendor = function
  | Mdl_ast.Simulink -> Ast_types.mdl_name_of_string "Simulink"
  | Mdl_ast.Scicos -> Ast_types.mdl_name_of_string "Scicos"
  | Mdl_ast.Rtw -> Ast_types.mdl_name_of_string "RTW"
  | Mdl_ast.Ui ident -> Ast_types.mdl_name_of_ident ident
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
