(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_block_types_tables.ml,v 1.1 2019-05-10 14:07:13 jpc Exp $ *)

open Mdl_block_types;;

let string_specified_block_type_association_list = [
  Mdl_block_types.
  Interpolation_n_D, "\"Interpolation_n-D\"";
  Lookup_n_D, "\"Lookup_n-D\"";
  M_S_Function, "\"M-S-Function\"";
  S_Function, "\"S-Function\"";
]
;;

let string_specified_block_type_table, xstring_specified_block_type_table =
  let t = Hashtbl.create 501
  and xt = Hashtbl.create 501 in
  List.iter
    (fun (bty, tyname) ->
     Hashtbl.add t bty tyname;
     Hashtbl.add xt tyname bty)
    string_specified_block_type_association_list;
  t, xt
;;

let string_of_string_specified_block_type ty =
  Hashtbl.find string_specified_block_type_table ty
;;

let string_specified_block_type_of_string s =
  Hashtbl.find xstring_specified_block_type_table s
;;

let block_type_association_list = [
  Mdl_block_types.
  Interpolation_n_D, "Interpolation_n-D";
  Lookup_n_D, "Lookup_n-D";
  M_S_Function, "M-S-Function";
  S_Function, "S-Function";
  Abs, "Abs";
  ActionPort, "ActionPort";
  Actuator, "Actuator";
  Assertion, "Assertion";
  AsynchronousTaskSpecification, "AsynchronousTaskSpecification";
  ArithShift, "ArithShift";
  Assignment, "Assignment";
  Backlash, "Backlash";
  Bias, "Bias";
  Buffer, "Buffer";
  BusAssignment, "BusAssignment";
  BusCreator, "BusCreator";
  BusSelector, "BusSelector";
  BusToVector, "BusToVector";
  Clock, "Clock";
  CombinatorialLogic, "CombinatorialLogic";
  ComplexToMagnitudeAngle, "ComplexToMagnitudeAngle";
  ComplexToRealImag, "ComplexToRealImag";
  Concatenate, "Concatenate";
  Constant, "Constant";
  ConstellationDiagram, "ConstellationDiagram";
  DataStoreMemory, "DataStoreMemory";
  DataStoreRead, "DataStoreRead";
  DataStoreWrite, "DataStoreWrite";
  DataTypeConversion, "DataTypeConversion";
  DataTypeDuplicate, "DataTypeDuplicate";
  DeadZone, "DeadZone";
  Delay, "Delay";
  Demux, "Demux";
  Derivative, "Derivative";
  DigitalClock, "DigitalClock";
  DiscreteFilter, "DiscreteFilter";
  DiscreteFir, "DiscreteFir";
  DiscreteIntegrator, "DiscreteIntegrator";
  DiscretePulseGenerator, "DiscretePulseGenerator";
  DiscreteStateSpace, "DiscreteStateSpace";
  DiscreteTransferFcn, "DiscreteTransferFcn";
  DiscreteZeroPole, "DiscreteZeroPole";
  Display, "Display";
  DotProduct, "DotProduct";
  DSPFlip, "DSPFlip";
  ElementaryMath, "ElementaryMath";
  EnablePort, "EnablePort";
  Fcn, "Fcn";
  Find, "Find";
  ForEach, "ForEach";
  ForIterator, "ForIterator";
  FrameConversion, "FrameConversion";
  From, "From";
  FromFile, "FromFile";
  FromWorkspace, "FromWorkspace";
  FunctionCallSplit, "FunctionCallSplit";
  FunctionCallFeedbackLatch, "FunctionCallFeedbackLatch";
  Gain, "Gain";
  Goto, "Goto";
  GotoTagVisibility, "GotoTagVisibility";
  Ground, "Ground";
  HitCross, "HitCross";
  If, "If";
  InitialCondition, "InitialCondition";
  Inport, "Inport";
  InportShadow, "InportShadow";
  Integrator, "Integrator";
  Logic, "Logic";
  Lookup, "Lookup";
  Lookup2D, "Lookup2D";
  LookupNDDirect, "LookupNDDirect";
  MATLABFcn, "MATLABFcn";
  MATLABSystem, "MATLABSystem";
  MagnitudeAngleToComplex, "MagnitudeAngleToComplex";
  ManualSwitch, "ManualSwitch";
  Math, "Math";
  Memory, "Memory";
  Merge, "Merge";
  MinMax, "MinMax";
  ModelReference, "ModelReference";
  MultiPortSwitch, "MultiPortSwitch";
  Mux, "Mux";
  Outport, "Outport";
  PMComponent, "PMComponent";
  PMIOPort, "PMIOPort";
  PermuteDimensions, "PermuteDimensions";
  Polyval, "Polyval";
  PreLookup, "PreLookup";
  Probe, "Probe";
  Product, "Product";
  Quantizer, "Quantizer";
  RandomNumber, "RandomNumber";
  RateLimiter, "RateLimiter";
  RateTransition, "RateTransition";
  RealImagToComplex, "RealImagToComplex";
  Reference, "Reference";
  RelationalOperator, "RelationalOperator";
  Relay, "Relay";
  Reshape, "Reshape";
  Rounding, "Rounding";
  SampleTimeMath, "SampleTimeMath";
  Saturate, "Saturate";
  Scope, "Scope";
  SecondOrderIntegrator, "SecondOrderIntegrator";
  Selector, "Selector";
  Sensor, "Sensor";
  SignalConversion, "SignalConversion";
  SignalFromWorkspace, "SignalFromWorkspace";
  SignalGenerator, "SignalGenerator";
  SignalSpecification, "SignalSpecification";
  SignalToWorkspace, "SignalToWorkspace";
  SignalViewerScope, "SignalViewerScope";
  Signum, "Signum";
  Sin, "Sin";
  SpectrumAnalyzer, "SpectrumAnalyzer";
  Sqrt, "Sqrt";
  Squeeze, "Squeeze";
  StateSpace, "StateSpace";
  Step, "Step";
  Stop, "Stop";
  SubSystem, "SubSystem";
  Sum, "Sum";
  Switch, "Switch";
  SwitchCase, "SwitchCase";
  Terminator, "Terminator";
  TimeScope, "TimeScope";
  ToFile, "ToFile";
  ToWorkspace, "ToWorkspace";
  TransferFcn, "TransferFcn";
  TransportDelay, "TransportDelay";
  TriggerPort, "TriggerPort";
  Trigonometry, "Trigonometry";
  TwoWayConnection, "TwoWayConnection";
  UnaryMinus, "UnaryMinus";
  Unbuffer, "Unbuffer";
  UniformRandomNumber, "UniformRandomNumber";
  UnitConversion, "UnitConversion";
  UnitDelay, "UnitDelay";
  VariableTransportDelay, "VariableTransportDelay";
  VideoViewer, "VideoViewer";
  Width, "Width";
  WhileIterator, "WhileIterator";
  ZeroOrderHold, "ZeroOrderHold";
  ZeroPole, "ZeroPole";
]
;;

let block_type_table, xblock_type_table =
  let t = Hashtbl.create 501
  and xt = Hashtbl.create 501 in
  List.iter
    (fun (bty, tyname) ->
     Hashtbl.add t bty tyname;
     Hashtbl.add xt tyname bty)
    block_type_association_list;
  t, xt
;;

let string_of_block_type ty =
  Hashtbl.find block_type_table ty
;;

let block_type_of_string s =
  Hashtbl.find xblock_type_table s
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
