(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_parsing_ast.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(* The raw output of mdl_parser.mly.
   The stateflow section is flat. *)

open Ident;;
open Ast_types;;
open Mdl_ast;;

(* FIXME: should be
type stateflow_chart_ref =
  | Stateflow_chart_ref of stateflow_ref
and so on...
*)
type stateflow_machine_ref = stateflow_ref
and stateflow_chart_ref = stateflow_ref
and stateflow_data_ref = stateflow_ref
and stateflow_event_ref = stateflow_ref
and stateflow_junction_ref = stateflow_ref
and stateflow_transition_ref = stateflow_ref
and stateflow_target_ref = stateflow_ref
and stateflow_chart_file_number_ref = natural
;;

type stateflow_link_node =
  {
    link_node_parent_ref : natural;
    link_node_prev_ref : natural;
    link_node_next_ref : natural;
  }

 and stateflow_tree_node =
  {
    tree_node_parent_ref : natural;
    tree_node_first_child_ref : natural;
    tree_node_prev_ref : natural;
    tree_node_next_ref : natural;
  }
;;

type mdl_stateflow_items = mdl_stateflow_item list
and mdl_stateflow_section_item =
  | Mdl_stateflow_chart of mdl_stateflow_items
  | Mdl_stateflow_data of mdl_stateflow_items
  | Mdl_stateflow_event of mdl_stateflow_items
  | Mdl_stateflow_instance of mdl_stateflow_items
  | Mdl_stateflow_junction of mdl_stateflow_items
  | Mdl_stateflow_machine of mdl_stateflow_items
  | Mdl_stateflow_state of mdl_stateflow_items
  | Mdl_stateflow_target of mdl_stateflow_items
  | Mdl_stateflow_transition of mdl_stateflow_items

and mdl_stateflow_item =
  | Mdl_stateflow_binding of binding
  | Mdl_stateflow_chart_ref of stateflow_chart_ref
  | Mdl_stateflow_eml of Mdl_ast.stateflow_eml_items
  | Mdl_stateflow_execution_order of natural
  | Mdl_stateflow_first_data of stateflow_data_ref
  | Mdl_stateflow_first_event of stateflow_event_ref
  | Mdl_stateflow_first_junction of stateflow_junction_ref
  | Mdl_stateflow_first_transition of stateflow_transition_ref
  | Mdl_stateflow_first_target of stateflow_target_ref
  | Mdl_stateflow_id of stateflow_id
  | Mdl_stateflow_label_string of mdl_text
  | Mdl_stateflow_link_node of stateflow_link_node
  | Mdl_stateflow_tree_node of stateflow_tree_node
  | Mdl_stateflow_machine_ref of stateflow_machine_ref
  | Mdl_stateflow_chart_file_number of stateflow_chart_file_number_ref
  | Mdl_stateflow_name of mdl_name
  | Mdl_stateflow_scope of ident
  | Mdl_stateflow_props of Mdl_ast.stateflow_props_items
  | Mdl_stateflow_ssid_number of natural
  | Mdl_stateflow_trigger of ident
  | Mdl_stateflow_type of ident
  | Mdl_stateflow_src of Mdl_ast.stateflow_src_dst_items
  | Mdl_stateflow_dst of Mdl_ast.stateflow_src_dst_items
  | Mdl_stateflow_plant_modeling_info
;;

type mdl_stateflow_section_items = mdl_stateflow_section_item list
and mdl_stateflow_section =
  | Mdl_stateflow_section of mdl_stateflow_section_items
;;

type simulink_file_contents =
  mdl_stateflow_section poly_simulink_file_contents
;;

type simulink_file = mdl_stateflow_section poly_simulink_file;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
