(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: code_make_host_print_spec.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(** The specific format strings and formatters for any host translation. *)

module type HOST = sig

  val tnlm : Mtlb_ppf.tag_new_line_markers;;

  val print_host_target_modeler : Format.formatter ->
    Simport_configuration.target_modeler -> unit;;
  val print_host_bool : Format.formatter -> bool -> unit;;
  val print_host_int : Format.formatter -> int -> unit;;
  val print_host_float : Format.formatter -> float -> unit;;

  val comment_format : (string -> unit, Format.formatter, unit) format;;

  val comment_string : string -> string;;

  val binding_format :
    (string ->
     (Format.formatter -> Instructions.modeler_block_parameter_value -> unit) ->
     Instructions.modeler_block_parameter_value -> unit,
     Format.formatter,
     unit) format
  ;;

  val commented_binding_format :
    (string ->
     (Format.formatter -> Instructions.modeler_block_parameter_value -> unit) ->
     Instructions.modeler_block_parameter_value -> unit,
     Format.formatter,
     unit) format
  ;;

  val print_mask_parameter :
    (Format.formatter -> Instructions.mask_parameter_value -> unit) ->
    Format.formatter -> Instructions.mask_parameter -> unit
  ;;

  val evaluate_model_format :
    ((Format.formatter -> Instructions.host_model_ident -> unit) ->
     Instructions.host_model_ident ->
     (Format.formatter -> Instructions.host_model_ident -> unit) ->
     Instructions.host_model_ident ->
     unit,
     Format.formatter,
     unit) format
  ;;

  val set_block_mask_header : string;;

  val set_block_parameters_format :
    ((Format.formatter -> Instructions.modeler_block_parameter_bindings -> unit) ->
     Instructions.modeler_block_parameter_bindings ->
     unit,
     Format.formatter,
     unit) format
  ;;

  val print_function_code_footer :
    (Format.formatter -> Instructions.host_diagram_ident -> unit) ->
    Format.formatter -> Instructions.host_diagram_ident -> unit
  ;;

  val print_define_function_lhs :
    (Format.formatter -> Instructions.host_diagram_ident -> unit) ->
    Format.formatter -> Instructions.host_diagram_ident -> unit
  ;;

  val function_footer : string;;
  val lhs_begin : string;;
  val lhs_end : string;;

  val strs_begin : string;;
  val strs_end : string;;

  val row_separator : string;;

  val struct_item_format :
    (string ->
     (Format.formatter -> Mdl_deep_ast.deep_value list -> unit) ->
     Mdl_deep_ast.deep_value list -> unit,
     Format.formatter,
     unit) format
  ;;

  val struct_create_format :
    ((Format.formatter ->
      Mdl_deep_ast.deep_value list Binding.ident_binding -> unit) ->
     Mdl_deep_ast.deep_value list Binding.ident_binding ->
     (Format.formatter ->
      Mdl_deep_ast.deep_value list Binding.ident_binding list -> unit) ->
     Mdl_deep_ast.deep_value list Binding.ident_binding list -> unit,
     Format.formatter, unit) format
  ;;

end
;;

module type PRINT_TO_HOST = sig
  val print_translated_file :
    Format.formatter -> Instructions.code -> unit
  ;;
end
;;

module type MAKE_PRINT = functor (Host : HOST) -> PRINT_TO_HOST;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
