(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: code_oml_host.ml,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(** The specific format strings and formatters for Matlab translation. *)

module Host : Code_make_host_print_spec.HOST = struct

  let tnlm = Mtlb_oml_host.Host.tnlm;;

  let string_to_host_string text =
    let lim = String.length text - 1 in
    let accu = Buffer.create (lim + 3) in

    let rec loop i =
      if i <= lim then
        begin
          (match text.[i] with
           | '\'' -> Buffer.add_string accu "\'\'"
           | c -> Buffer.add_char accu c
          );
          loop (i + 1);
        end in

    Buffer.add_char accu '\'';
    loop 0;
    Buffer.add_char accu '\'';
    Buffer.contents accu
  ;;

  let comment_format = format_of_string "%%%s";;

  (* Note that this function uses comment_format format string
     and function string_to_host_string that are specific to OML.
     Thus, even if its code is exactly the same as the one of other
     languages, it cannot be shared.
  *)
  let comment_string text =
    let text =
      Format.fprintf Format.str_formatter comment_format text;
      Format.flush_str_formatter () in
    string_to_host_string text
  ;;

  let print_host_target_modeler ppf modeler =
    Format.fprintf ppf "%s"
      (string_to_host_string
        (Configuration.string_of_target_modeler modeler))
  ;;

  let print_host_bool = Code_scicoslab_host.Host.print_host_bool;;

  let print_host_int = Code_nsp_host.Host.print_host_int;;
  let print_host_float = Code_nsp_host.Host.print_host_float;;

  let binding_format = Code_scicoslab_host.Host.binding_format;;

  let commented_binding_format =
    Code_nsp_host.Host.commented_binding_format
  ;;

  let print_mask_parameter = Code_octave_host.Host.print_mask_parameter;;

  let evaluate_model_format =
    Code_nsp_host.Host.evaluate_model_format
  ;;

  let set_block_mask_header =
    Code_scicoslab_host.Host.set_block_mask_header
  ;;

  let set_block_parameters_format =
    Code_scicoslab_host.Host.set_block_parameters_format
  ;;

  let print_function_code_footer =
    Code_nsp_host.Host.print_function_code_footer
  ;;

  let print_define_function_lhs =
    Code_nsp_host.Host.print_define_function_lhs
  ;;

  let function_footer = "end";;

  let lhs_begin = Code_nsp_host.Host.lhs_begin;;
  let lhs_end = Code_nsp_host.Host.lhs_end;;

  let strs_begin = "{";;
  let strs_end = "}";;

  let row_separator = ",";;

  let struct_item_format =
    format_of_string "\'%s\',@ {{%a}}"
  ;;

  let struct_create_format =
    Code_scicoslab_host.Host.struct_create_format
  ;;

end
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
