(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: code_to_host.ml,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(** The procedure to print the translator instructions to a target language
  source file that generates the Scicos translation of an MDL file. *)

module Print_to_nsp =
  Code_make_host_print.Make_print (Code_nsp_host.Host)
;;

module Print_to_modelica =
  Code_make_host_print.Make_print (Code_modelica_host.Host)
;;

module Print_to_hypermath =
  Code_make_host_print.Make_print (Code_hypermath_host.Host)
;;

module Print_to_oml =
  Code_make_host_print.Make_print (Code_oml_host.Host)
;;

module Print_to_scicoslab =
  Code_make_host_print.Make_print (Code_scicoslab_host.Host)
;;

module Print_to_octave =
  Code_make_host_print.Make_print (Code_octave_host.Host)
;;

type 'a translators = {
  nsp : 'a;
  modelica : 'a;
  hypermath : 'a;
  oml : 'a;
  scicoslab : 'a;
  octave : 'a;
}
;;

let get_translator translators lang =
  match lang with
  | Simport_configuration.Target_mtlb _ ->
    Say.fatal_error
      "No point in translating Simulink code to matlab code."

  | Simport_configuration.Target_simulink lang ->
    (match lang with
     | Simport_configuration.ScicosLab -> translators.scicoslab
     | Simport_configuration.Nsp -> translators.nsp
     | Simport_configuration.Modelica -> translators.modelica
     | Simport_configuration.Hypermath -> translators.hypermath
     | Simport_configuration.Oml -> translators.oml
     | Simport_configuration.Octave -> translators.octave)

  | Simport_configuration.Target_sime _ ->
    Say.fatal_error
      "No point in translating Simulink code to embedded expression."
;;

let file_translators = {
  nsp = Print_to_nsp.print_translated_file;
  modelica = Print_to_modelica.print_translated_file;
  hypermath = Print_to_hypermath.print_translated_file;
  oml = Print_to_oml.print_translated_file;
  scicoslab = Print_to_scicoslab.print_translated_file;
  octave = Print_to_octave.print_translated_file;
}
;;

let print_translated_file ppf =
  get_translator file_translators (Configuration.get_target_language ()) ppf
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
