(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_ast_to_deep_ast.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(* From parse tree to deep syntax:
   - we perform various validity checks on the tree obtained from the parser,
   - we build explicit record representations of data instead of simple
     values with associated so-called property lists.
*)

val string_of_block_type : Mdl_block_types.block_type -> string;;
val string_of_block_name : Ast_types.mdl_name -> string;;
val string_of_block_source_type : Ast_types.mdl_name -> string;;
val string_of_block_source_path : Ast_types.mdl_path -> string;;

val deep_font : Ast_types.mdl_font -> Mdl_basics_deep_ast.deep_font
;;

(** Regular ports *)

val make_number_of_regular_ports :
  Mdl_basics_deep_ast.natural -> Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_number_of_regular_ports
;;

val get_number_of_regular_ports :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.deep_number_of_regular_ports
;;

val get_regular_port_nout :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural
;;

val get_regular_port_nin :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural
;;

val set_number_of_regular_ports :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.deep_number_of_regular_ports ->
  Mdl_basics_deep_ast.deep_ports_description
;;

val set_regular_port_nout :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_ports_description
;;

val set_regular_port_nin :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_ports_description
;;

(** Implicit ports for connection lines *)

val make_number_of_implicit_ports :
  Mdl_basics_deep_ast.natural -> Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_number_of_implicit_ports
;;

val get_number_of_implicit_ports :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.deep_number_of_implicit_ports
;;

val get_implicit_port_nout :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural
;;

val get_implicit_port_nin :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural
;;

val set_number_of_implicit_ports :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.deep_number_of_implicit_ports ->
  Mdl_basics_deep_ast.deep_ports_description
;;

val set_implicit_port_nout :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_ports_description
;;

val set_implicit_port_nin :
  Mdl_basics_deep_ast.deep_ports_description ->
  Mdl_basics_deep_ast.natural ->
  Mdl_basics_deep_ast.deep_ports_description
;;

(** General ports description *)
val make_default_deep_ports_description :
  Mdl_basics_deep_ast.deep_number_of_regular_ports ->
  Mdl_basics_deep_ast.deep_number_of_implicit_ports ->
  Mdl_basics_deep_ast.deep_ports_description
;;

val make_default_deep_block_parameter_defaults :
  Mdl_block_types.block_type ->
  Mdl_basics_deep_ast.deep_block_parameter_defaults
;;

val simulink_file_to_deep_ast :
  Mdl_ast.simulink_file -> Mdl_deep_ast.deep_simulink_file
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
