(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_deep_basics.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(* From parse tree to deep syntax: the basics. *)

val deep_error : string -> 'a;;

val deep_warning : string -> unit;;

val list_mapi : (Unatural.t -> 'a -> 'b) -> Unatural.t -> 'a list -> 'b list;;

val deep_mdl_bool : Ast_types.mdl_bool -> Mdl_deep_ast.deep_value;;

val deep_simple_model_value : Mdl_ast.simple_value -> Mdl_deep_ast.deep_value;;

val deep_model_value : Mdl_ast.model_value -> Mdl_deep_ast.deep_value;;

val deep_binding : Mdl_ast.binding -> Mdl_deep_ast.deep_binding;;
val deep_bindings : Mdl_ast.binding list -> Mdl_deep_ast.deep_binding list;;

val bool_of_simple_value : Say.message -> Mdl_ast.simple_value -> bool;;
val bool_of_model_value : Say.message -> Mdl_ast.model_value -> bool;;

val bool_of_deep_value : Say.message -> Mdl_deep_ast.deep_value -> bool;;
(*
val string_of_model_value : Say.message -> Mdl_ast.model_value -> string;;
*)
val mdl_text_of_model_value :
  Say.message -> Mdl_ast.model_value -> Ast_types.mdl_text
;;

val string_of_deep_value : Say.message -> Mdl_deep_ast.deep_value -> string;;

val matrix_of_deep_value :
  Say.message -> Mdl_deep_ast.deep_value ->  Mdl_basics_deep_ast.deep_values
;;

val vector_of_deep_value :
  Say.message -> Mdl_deep_ast.deep_value ->  Mdl_basics_deep_ast.deep_values
;;

val bindings_of_deep_value :
  Say.message -> Mdl_deep_ast.deep_value -> Mdl_deep_ast.deep_bindings
;;

(* FIXME: to be removed
val mdl_text_of_deep_value : Say.message -> Mdl_deep_ast.deep_value -> Ast_types.mdl_text;;
    
*)

val make_deep_block_reference_type :
  Mdl_deep_ast.deep_block_source_type ->
  Mdl_deep_ast.deep_block_source_path ->
  Mdl_deep_ast.deep_block_reference_type
;;

val deep_block_reference_type_of_strings :
  string -> string ->
  Mdl_deep_ast.deep_block_reference_type
;;

val deep_bool_of_natural : Unatural.t -> bool;;

val natural_of_deep_value :
  Say.message -> Mdl_deep_ast.deep_value -> Unatural.t
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
