(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_basics_explicit_ast.mli,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(*open Mdl_basics_deep_ast;;*)
(*open Mdl_deep_ast;;*)

(* Borrowed from basic modules. *)
type natural = Ast_types.natural
and mdl_text = Ast_types.mdl_text
and mdl_font_name = Ast_types.mdl_font_name
;;

(* Explicit version of basic types *)
type explicit_value = Mdl_basics_deep_ast.deep_value
and explicit_binding = Mdl_basics_deep_ast.deep_binding
and explicit_bindings = Mdl_basics_deep_ast.deep_bindings
and explicit_points = Mdl_basics_deep_ast.deep_points
and explicit_color = Mdl_basics_deep_ast.deep_color
and explicit_rotation = natural
and explicit_mirror = Mdl_basics_deep_ast.deep_mirror
and explicit_number_of_regular_ports =
  Mdl_basics_deep_ast.deep_number_of_regular_ports
and explicit_number_of_implicit_ports =
  Mdl_basics_deep_ast.deep_number_of_implicit_ports
and explicit_position = Mdl_basics_deep_ast.deep_position
and explicit_state = Mdl_basics_deep_ast.deep_state
and explicit_tag = Mdl_basics_deep_ast.deep_tag
and explicit_zorder = Mdl_basics_deep_ast.deep_zorder
and explicit_block_name = Mdl_basics_deep_ast.deep_block_name
and explicit_block_port = Mdl_basics_deep_ast.deep_block_port
and explicit_port_number = Mdl_basics_deep_ast.deep_port_number
and explicit_block_ports = Mdl_basics_deep_ast.deep_block_ports
and explicit_ports_description = Mdl_basics_deep_ast.deep_ports_description
and explicit_line_name = Mdl_basics_deep_ast.deep_line_name
and explicit_placement = Mdl_basics_deep_ast.deep_placement
and explicit_drop_shadow = bool
and explicit_show_name = bool
and explicit_hide_automatic_name = bool
and explicit_treat_as_atomic_unit = bool option
and explicit_system_sample_time = Mdl_basics_deep_ast.deep_value option
and explicit_zoom_factor = natural option
and explicit_sid = Mdl_basics_deep_ast.deep_sid
and explicit_font_and_colors = Mdl_basics_deep_ast.deep_font_and_colors
and explicit_font_size = natural
and explicit_font_angle = Mdl_basics_deep_ast.deep_font_angle
and explicit_font_weight = Mdl_basics_deep_ast.deep_font_weight

and explicit_annotation_position =
  Mdl_basics_deep_ast.deep_annotation_position
and explicit_annotation_graphical_defaults =
    explicit_annotation_graphical_default list
and explicit_annotation_graphical_default =
  Mdl_basics_deep_ast.deep_annotation_graphical_default

and explicit_branches = explicit_branch list
and explicit_branch = Mdl_basics_deep_ast.deep_branch
;;

type explicit_mask_parameter_ident =
  Mdl_basics_deep_ast.deep_mask_parameter_ident
and explicit_mask_parameter_style =
  Mdl_basics_deep_ast.deep_mask_parameter_style
and explicit_mask_parameter_prompt =
  Mdl_basics_deep_ast.deep_mask_parameter_prompt
and explicit_mask_parameter_evaluation =
  Mdl_basics_deep_ast.deep_mask_parameter_evaluation
and explicit_mask_parameter_enable =
  Mdl_basics_deep_ast.deep_mask_parameter_enable
and explicit_mask_parameter_visible =
  Mdl_basics_deep_ast.deep_mask_parameter_visible

and explicit_source_code =
  Mdl_basics_deep_ast.deep_source_code

and explicit_stateflow_name =
  Mdl_stateflow_deep_ast.deep_stateflow_name
and explicit_stateflow_chart_file_number =
  Mdl_stateflow_deep_ast.deep_stateflow_chart_file_number
;;

type explicit_font = {
  explicit_font_name : mdl_font_name;
  explicit_font_angle : explicit_font_angle;
  explicit_font_size : explicit_font_size;
  explicit_font_weight : explicit_font_weight;
}
;;

(** {6 Explicit block mask description} *)

type explicit_mask_parameter_rank = natural;;

type explicit_mask_parameter_value =
    (* A parameter that should be written verbatim. *)
  | Explicit_mask_parameter_quote of mdl_text
    (* A parameter that should be translated to the host language. *)
  | Explicit_mask_parameter_eval of mdl_text
;;

type explicit_mask_parameter = {
  explicit_mask_parameter_ident : explicit_mask_parameter_ident;
  explicit_mask_parameter_rank : explicit_mask_parameter_rank;
  explicit_mask_parameter_style : explicit_mask_parameter_style;
  explicit_mask_parameter_prompt : explicit_mask_parameter_prompt;
  explicit_mask_parameter_value : explicit_mask_parameter_value;
  explicit_mask_parameter_evaluation : explicit_mask_parameter_evaluation;
  explicit_mask_parameter_enable : explicit_mask_parameter_enable option;
  explicit_mask_parameter_visible : explicit_mask_parameter_visible option;
}
(** [explicit_mask_parameters]: the block mask parameters as
  specified in the block or inherited, *)

and explicit_mask_parameters = explicit_mask_parameter list
;;

type explicit_mask_type =(*
  | Mask_type_stateflow
  | Mask_type_other of*)
      Mdl_basics_deep_ast.deep_mask_type option
;;

type explicit_mask_description =(*
  | Mask_description_embedded_matlab
  | Mask_description_other of*)
      Mdl_basics_deep_ast.deep_mask_description option
;;

type explicit_mask = {
  explicit_mask_type : explicit_mask_type;
  explicit_mask_description : explicit_mask_description;
  explicit_mask_initialisation : explicit_source_code option;
  explicit_mask_display : explicit_source_code option;
  explicit_mask_parameters : explicit_mask_parameters;
  explicit_mask_bindings : explicit_bindings;
}
;;

type explicit_array = Mdl_deep_ast.deep_array_description
and explicit_object = Mdl_deep_ast.deep_object_description

and explicit_vendor_bindings = Mdl_deep_ast.vendor_bindings
;;

type explicit_block_regular_type = Mdl_deep_ast.deep_block_regular_type;;
type explicit_block_reference_type = Mdl_deep_ast.deep_block_reference_type;;
type explicit_block_type =
  | Explicit_block_regular_type of explicit_block_regular_type
  | Explicit_block_reference_type of explicit_block_reference_type
;;

type explicit_sf_block_type = Mdl_deep_ast.deep_sf_block_type;;

type explicit_model_version = Mdl_deep_ast.deep_model_version
and explicit_model_workspace_data = Mdl_deep_ast.deep_model_workspace_data
and explicit_active_configuration = Mdl_deep_ast.deep_model_configuration
;;

type explicit_model_body_kind =
  | Explicit_model
  | Explicit_library
;;

(*
 Local Variables:
  compile-command: "cd ../../.. && make"
 End:
*)
