(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_basics_to_explicit_ast.mli,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* From deep syntax to completely explicit inherited tree:
   - defaults sections are incorporated into values,
   - no more default sections in the resulting trees: all fields are explicit
     and checked with respect to various type constraints or range constraints.
*)

open Say;;
open Ast_types;;

val explicit_error : Say.message -> 'a
;;

val bool_of_deep_value : message -> Mdl_deep_ast.deep_value -> bool
;;

val string_of_deep_value : message -> Mdl_deep_ast.deep_value -> string
;;

val natural_of_deep_value :
  message -> Mdl_deep_ast.deep_value -> Ast_types.natural
;;

val natural_list_of_deep_value :
  message -> Mdl_deep_ast.deep_value -> natural list
;;

val ports_of_deep_value : Mdl_deep_ast.deep_value -> natural list
;;

val name_of_explicit_block_regular_type :
  Mdl_basics_explicit_ast.explicit_block_regular_type -> string
;;

val name_of_explicit_block_reference_type :
  Mdl_basics_explicit_ast.explicit_block_reference_type -> string
;;

val name_of_explicit_block_type :
  Mdl_basics_explicit_ast.explicit_block_type -> string
;;

val name_of_explicit_block_source_type :
  Mdl_basics_explicit_ast.explicit_block_type -> string
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)

