(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_stateflow_explicit_ast.mli,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(**
   Similar to [Mdl_explicit_ast] syntax definition but specialized to stateflow.

   From deep syntax to explicit syntax.
*)

type explicit_stateflow_section =
  | Explicit_stateflow_section of explicit_stateflow_section_items

and explicit_stateflow_section_items = explicit_stateflow_section_item list

and explicit_stateflow_section_item =
  (* For the time being, just do nothing! *)
  Mdl_stateflow_deep_ast.deep_stateflow_section_item
;;

(*
For the inheritance part should use

and explicit_sf_block_type =
    (* "NONE" *)
  | Sf_block_type_none
    (* MATLAB Function *)
  | Sf_block_type_matlab_function
    (* Chart *)
  | Sf_block_type_chart
    (* Any other stateflow type *)
  | Sf_block_type_string of mdl_text
and sf_block_type = model_value

explicit_block_item =
  ...
  | Block_sf_block_type of sf_block_type

(*
Should use this function in the inheritance phase
let mk_block_sf_type_from_string_spec = function
  | "NONE" -> Sf_block_type_none
  | "Chart" -> Sf_block_type_chart
  | "MATLAB Function" -> Sf_block_type_matlab_function
  | s -> Sf_block_type_string (Ast_types.mdl_text_of_string s)
;;
*)

*)

(*
open Ast_types;;
open Mdl_basics_deep_ast;;

type matlab_code = Ast_types.mdl_text;;

type deep_stateflow_machine =
  {
    deep_machine_id : Ast_types.natural; (* "id" *)
    deep_machine_name : string; (* "name" *)
    deep_machine_bindings : Mdl_basics_deep_ast.deep_bindings;
  }
;;

type deep_stateflow_chart =
  {
    deep_chart_id : natural; (* "id" *)
    deep_chart_machine_ref : natural; (* "machine" *)
    deep_chart_name : string; (* "name" *)
    deep_chart_bindings : deep_bindings;
  }
;;

type deep_stateflow_state_type =
  | Deep_state_or (* AND_STATE *)
  | Deep_state_and (* OR_STATE *)
  | Deep_state_func (* FUNC_STATE *)
  | Deep_state_group (* GROUP_STATE *)
  | Deep_state_string of mdl_text
;;

type deep_stateflow_state =
  {
    deep_state_id : natural; (* "id" *)
    deep_state_ssid_number : natural option; (* "ssIdNumber" *)
    deep_state_type : deep_stateflow_state_type; (* type *)
    deep_state_chart_ref : natural; (* chart *)
    deep_state_label_string : mdl_text; (* labelString *)
    deep_state_bindings : deep_bindings;
  }
;;

type deep_stateflow_data_scope =
  | Scope_input_data (* INPUT_DATA *)
  | Scope_output_data (* OUTPUT_DATA *)
  | Scope_function_input_data (* FUNCTION_INPUT_DATA *)
  | Scope_function_output_data (* FUNCTION_OUTPUT_DATA *)
  | Scope_parameter_data (* PARAMETER_DATA *)
  | Scope_constant_data (* CONSTANT_DATA *)
  | Scope_local_data (* LOCAL_DATA *)
;;

type deep_stateflow_data =
  {
    deep_data_id : natural;
    deep_data_ssid_number : natural option;
    deep_data_name : mdl_text;
    deep_data_scope : deep_stateflow_data_scope; (* scope *)
    deep_data_machine_ref : natural;
    deep_data_bindings : deep_bindings;
  }
;;

type deep_stateflow_junction =
  {
    deep_junction_id : natural;
    deep_junction_ssid_number : natural option;
    deep_junction_chart_ref : natural;
    deep_junction_type : mdl_text;
    deep_junction_bindings : deep_bindings;
  }
;;

type deep_stateflow_transition =
  {
    deep_transition_id : natural;
    deep_transition_ssid_number : natural option;
    deep_transition_label_string : mdl_text; (* labelString *)
    deep_transition_chart_ref : natural;
    deep_transition_src : deep_bindings; (* src *)
    deep_transition_dst : deep_bindings; (* dst *)
    deep_transition_execution_order : natural; (* executionOrder *)
    deep_transition_bindings : deep_bindings;
  }
;;

type deep_stateflow_instance =
  {
    deep_instance_id : natural;
    deep_instance_name : natural;
    deep_instance_machine_ref : natural;
    deep_instance_chart_ref : natural;
    deep_instance_bindings : deep_bindings;
  }
;;

type deep_stateflow_target =
  {
    deep_target_id : natural;
    deep_target_name : natural;
    deep_target_machine_ref : natural;
    deep_target_bindings : deep_bindings;
  }
;;

type deep_stateflow_event =
  {
    deep_event_id : natural;
    deep_event_name : natural;
    deep_event_machine_ref : natural;
    deep_event_scope : mdl_text; (* scope *)
    deep_event_trigger : mdl_text; (* trigger *)
    deep_event_bindings : deep_bindings;
  }
;;

type deep_stateflow_section = deep_stateflow_section_items

and deep_stateflow_section_items = deep_stateflow_section_item list

and deep_stateflow_section_item =
  | Deep_stateflow_machine of deep_stateflow_machine
  | Deep_stateflow_target of deep_stateflow_target
  | Deep_stateflow_chart of deep_stateflow_chart
  | Deep_stateflow_event of deep_stateflow_event
  | Deep_stateflow_state of deep_stateflow_state
  | Deep_stateflow_junction of deep_stateflow_junction
  | Deep_stateflow_transition of deep_stateflow_transition
  | Deep_stateflow_data of deep_stateflow_data
  | Deep_stateflow_instance of deep_stateflow_instance
;;
*)
(* For explicit syntax!

type chart =
  | Chart_EML of chart_eml
  | Chart_other
;;

type state =
  {
    state_id : natural;
    state_chart : natural;
  }
;;

type eml =
  {
    script : matlab_code;
    scope_input_data : ;
    output_data : ;
    parameters : ;
  }
;;

type chart_eml =
  {
    chart_id : natural;
    chart_machine : natural;
    chart_eml_function_name : string;
    chart_sample_time : natural; (* sampleTime *)
  }
;;
*)

(*
let find_eml_charts_in_stateflow_section_item = function
  | Stateflow_machine _stateflow_items -> []
  | Stateflow_target _stateflow_items -> []
  | Stateflow_chart _stateflow_items -> []
  | Stateflow_event _stateflow_items -> []
  | Stateflow_state _stateflow_items -> []
  | Stateflow_junction _stateflow_items -> []
  | Stateflow_transition _stateflow_items -> []
  | Stateflow_data _stateflow_items -> []
  | Stateflow_instance _stateflow_items -> []

and
...


let find_eml_charts stateflow_section =
  List.fold_left
    find_eml_charts_in_stateflow_section_item
    stateflow_section []
;;

  
find is_eml_chart stateflow_section
let 
*)

(*
 Local Variables:
  compile-command: "cd ../../.. && make"
  End:
*)
