(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_to_explicit_ast.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* From deep syntax to completely explicit inherited tree:
   - defaults sections are incorporated into values,
   - no more default sections in the resulting trees: all fields are explicit
     and checked with respect to various type or range constraints.
*)
open Mdl_deep_ast;;

let pass deep_simulink_file =
  Say.fdebug (fun ppf ->
    Mdl_deep_ast_print.print_deep_simulink_file ppf deep_simulink_file
  );
  match deep_simulink_file with
  | Deep_interface dis ->
    Mdl_interface_to_explicit_ast.
    interface_to_explicit_ast dis
  | Deep_implementation ds ->
    let interfaces =
      Mdl_interface_to_explicit_ast.load_simulink_interfaces () in
    Mdl_implementation_to_explicit_ast.
    implementation_to_explicit_line_ast interfaces ds
;;

let treat_simulink_ast ast =
  let dast = Mdl_to_deep_ast.treat_simulink_ast ast in
  pass dast
;;

let read_mdl_file explicit_fname =
  let dast = Mdl_to_deep_ast.read_mdl_file explicit_fname in
  pass dast
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
