(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_types.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

(** {6 Type abbreviations} *)

type explicit_value = Mdl_basics_explicit_ast.explicit_value;;

type explicit_block_type =
  Mdl_basics_explicit_ast.explicit_block_type
;;

type explicit_block_regular_type =
  Mdl_basics_explicit_ast.explicit_block_regular_type
;;

type reference_block_source_type =
  Mdl_deep_ast.deep_block_source_type
and reference_block_source_path =
  Mdl_deep_ast.deep_block_source_path
;;

type natural = Ast_types.natural;;
type genr_info = Genr_block_types.genr_info;;
type ident = Ident.ident;;

type block_name = Mdl_translation.block_name;;
type block_string = string;;
type explicit_port_number = Mdl_basics_explicit_ast.explicit_port_number;;

type length = Simport_configuration.length;;
type origin = length * length;;

(** {6 Type definitions used by modeler block generators modules} *)

(*open Instructions;;*)

type text = string;;

type simulink_out_data_type =
  | Simulink_single
  | Simulink_double
  | Simulink_int8
  | Simulink_uint8
  | Simulink_int16
  | Simulink_uint16
  | Simulink_int32
  | Simulink_uint32
  | Simulink_boolean
  | Simulink_inherit
;;

type simulink_saturation =
  | Saturation_on
  | Saturation_off
;;

open Instructions;;

type modeler_block_value =
  | Simulink_arg_sample_time
  | Simulink_arg_out_data_type
  | Simulink_arg_zero_crossing
  | Simulink_arg_saturation
  | Simulink_arg_saturation_set of simulink_saturation
  | Simulink_arg_expression of ident
  | Simulink_arg_value of ident
  | Simulink_arg_value_aliases of ident list
  | Simulink_arg_expression_aliases of ident list
  | Simulink_arg_with_default of Instructions.code_value * modeler_block_value
  | Simulink_arg_fun1 of
      (code_value -> code_value) *
      modeler_block_value
  | Simulink_arg_fun2 of
      (code_value ->
       code_value -> code_value) *
      modeler_block_value * modeler_block_value
  | Simulink_arg_fun3 of
      (code_value ->
       code_value ->
       code_value -> code_value) *
      modeler_block_value * modeler_block_value * modeler_block_value
  | Code_value of code_value
;;

type modeler_block_parameter =
  | Value_parameter of Instructions.modeler_ident * modeler_block_value
  | Struct_parameter of
      modeler_ident * (modeler_ident * modeler_block_value list) Ulist1.t

and modeler_block_parameters = modeler_block_parameter list
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
