(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_genr_utilities_spec.mli,v 1.1 2019-05-10 14:07:14 jpc Exp $ *)

open Instructions;;
open Genr_types;;

module type MODELER =
  sig

    val target_modeler : Simport_configuration.target_modeler;;

    val code_out_simulink_saturation :
      Genr_types.simulink_saturation -> Instructions.code_value
    ;;

    val modeler_input_port_type : Instructions.modeler_block_type;;
    val modeler_output_port_type : modeler_block_type;;
    val modeler_input_activation_port_type : modeler_block_type;;
    val modeler_output_activation_port_type : modeler_block_type;;
    val modeler_annotation_type : modeler_block_type;;
    val modeler_samphold_type : modeler_block_type;;
    val modeler_split_type : modeler_block_type;;
    val modeler_implicit_split_type : modeler_block_type;;
    val modeler_clk_split_type : modeler_block_type;;
    val modeler_sample_clk_type : modeler_block_type;;
    val modeler_time_type : modeler_block_type;;
    val modeler_virtual_clk0_type : modeler_block_type;;
    val modeler_relational_op_type : modeler_block_type;;
    val modeler_math_expression_type : modeler_block_type;;

    val get_relop_value :
      Genr_block_types.genr_info -> Ident.ident -> code_value
    ;;

    val code_out_simulink_out_data_type :
      Genr_types.simulink_out_data_type -> code_value
    ;;

    val modeler_block_type_for :
      Genr_types.explicit_block_type ->
      Instructions.modeler_block_type
    ;;

    val in_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;
    val out_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;

    val implicit_in_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;
    val implicit_out_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;

    val in_activation_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;
    val out_activation_port_body :
      Instructions.host_block_ident -> Ast_types.mdl_port_number ->
      Instructions.instruction list
    ;;

    val samphold :
      Instructions.host_diagram_ident -> host_block_ident ->
      Genr_types.block_string -> code_value -> code_value ->
      Genr_types.origin -> instruction list
    ;;

    val sampholds :
      host_diagram_ident -> host_block_ident ->
      code_value -> code_value -> Genr_types.natural ->
      block_string list * code
    ;;

    val time_f :
      host_diagram_ident -> host_block_ident ->
      block_string -> code_value ->
      origin -> instruction list
    ;;

    val relational_op :
      host_diagram_ident -> host_block_ident ->
      block_string -> code_value -> code_value -> code_value -> code_value ->
      origin -> instruction list
    ;;

    val convert_in_scsm0_with_saturation :
      block_string ->
      code_value ->
      simulink_saturation ->
      origin -> instruction list
    ;;

  end
;;

module type GENR_UTILITIES = sig

    include module type of Genr_utilities_common;;
    include MODELER;;

    val get_simulink_out_data_type_value :
      Genr_block_types.genr_info -> Instructions.code_value
    ;;

    val get_simulink_saturation_value : genr_info -> code_value;;

    val simulink_block_reference_type :
      Genr_types.reference_block_source_type ->
      Genr_types.reference_block_source_path ->
      Genr_types.explicit_block_type
    ;;
    val simulink_block_type :
      Genr_types.explicit_block_regular_type ->
      Genr_types.explicit_block_type
    ;;

    val genr_instructions_for_modeler_block_type :
      Instructions.modeler_block_type ->
      Genr_types.modeler_block_parameters ->
      Instructions.instruction list ->
      Instructions.host_block_ident ->
      Genr_block_types.genr_info ->
      Instructions.instruction list
    ;;

    val genr_instructions_for :
      Genr_types.explicit_block_type ->
      Genr_types.modeler_block_parameters ->
      Instructions.instruction list ->
      Instructions.host_block_ident ->
      Genr_block_types.genr_info ->
      Instructions.instruction list
    ;;

    val genr_code_for_modeler_block_type :
      Instructions.modeler_block_type ->
      Genr_types.modeler_block_parameters ->
      Instructions.instruction list ->
      Instructions.host_block_ident ->
      Genr_block_types.genr_info ->
      Instructions.code
    ;;

    val genr_code_for :
      Genr_types.explicit_block_type ->
      Genr_types.modeler_block_parameters ->
      Instructions.instruction list ->
      Instructions.host_block_ident ->
      Genr_block_types.genr_info ->
      Instructions.code
    ;;

    val add_translator :
      Genr_types.explicit_block_type ->
      Genr_block_types.block_translator -> unit
    ;;
    val add_modeler_translation :
      Genr_types.explicit_block_type ->
      Genr_types.modeler_block_parameters ->
      Instructions.instruction list -> unit
    ;;
    val add_modeler_equivalent :
      Genr_types.explicit_block_type -> unit
    ;;

    val add_modeler_conditional_translation :
      Genr_types.explicit_block_type ->
      (Genr_types.explicit_value list ->
       Instructions.modeler_block_type *
       Genr_types.modeler_block_parameters) ->
      Genr_types.modeler_block_value list ->
      Instructions.instruction list -> unit
    ;;
    val add_modeler_conditional_translation1 :
      Genr_types.explicit_block_type ->
      (Genr_types.explicit_value ->
       Instructions.modeler_block_type *
       Genr_types.modeler_block_parameters) ->
      Genr_types.modeler_block_value ->
      Instructions.instruction list -> unit
    ;;
    val add_modeler_conditional_translation2 :
      Genr_types.explicit_block_type ->
      (Genr_types.explicit_value ->
       Genr_types.explicit_value ->
       Instructions.modeler_block_type *
       Genr_types.modeler_block_parameters) ->
      Genr_types.modeler_block_value *
      Genr_types.modeler_block_value ->
      Instructions.instruction list -> unit
    ;;
    val add_modeler_conditional_translation3 :
      Genr_types.explicit_block_type ->
      (Genr_types.explicit_value ->
       Genr_types.explicit_value ->
       Genr_types.explicit_value ->
       Instructions.modeler_block_type *
       Genr_types.modeler_block_parameters) ->
      Genr_types.modeler_block_value * Genr_types.modeler_block_value *
        Genr_types.modeler_block_value ->
      Instructions.instruction list -> unit
    ;;

    val default_block_port_size : Instructions.modeler_size;;

    val inherit_mode_type : code_value;;
    val double_type : code_value;;

    val in_port_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val in_ports_in_scsm0 : natural -> block_string list * code;;

    val out_port_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val out_ports_in_scsm0 :
      Genr_types.length -> natural -> block_string list * code
    ;;

    val implicit_in_port_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val implicit_in_ports_in_scsm0 :
      natural ->  natural -> block_string list * code
    ;;

    val implicit_out_port_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val implicit_out_ports_in_scsm0 :
      Genr_types.length -> natural -> natural -> block_string list * code
    ;;

    val out_port_event_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val out_port_events_in_scsm0 : Genr_types.length -> natural -> code
    ;;

    val in_port_event_in_scsm0 :
      block_string -> explicit_port_number -> origin -> instruction list
    ;;
    val in_port_events_in_scsm0 : Genr_types.length -> natural -> code
    ;;

    val samphold_in_scsm0 :
      block_string ->
      code_value ->
      code_value ->
      origin -> instruction list
    ;;

    val sampholds_in_scsm0 :
      code_value ->
      code_value ->
      natural ->
      block_string list * code
    ;;

    val time_f_in_scsm0 :
      block_string ->
      code_value ->
      origin -> instruction list
    ;;

    val relational_op_in_scsm0 :
      block_string ->
      code_value ->
      code_value ->
      code_value ->
      code_value ->
      origin -> instruction list
    ;;

    val convert_in_scsm0 :
      block_string ->
      code_value ->
      Genr_block_types.genr_info ->
      origin -> instruction list
    ;;

    val split_in_scsm0 :
      block_string ->
      origin -> instruction list
    ;;
     val implicit_split_in_scsm0 :
      block_string ->
      origin -> instruction list
    ;;
    val split_event_in_scsm0 :
      block_string ->
      origin -> instruction list
    ;;

    val sample_clock_in_scsm0 :
      block_string ->
      code_value ->
      origin -> instruction list
    ;;

    val virtual_clk0_in_scsm0 :
      block_string ->
      origin -> instruction list
    ;;

    val super_block_in_scsm0 :
      host_block_ident -> instruction list list -> code
    ;;

    val set_diagram_context_in_scsm0 :
      Instructions.context -> Instructions.instruction
    ;;

  end
;;

module type MAKE_GENR_UTILITIES =
  functor (Target_modeler : MODELER) -> GENR_UTILITIES
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
