(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_abs.ml,v 1.1 2019-05-10 14:05:17 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type Abs
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let out_data_type =
    Genr_utilities_scicos.get_simulink_out_data_type_value genr_info in

  let scicos_block_parameters = [
    Value_parameter (Si "zcr", Simulink_arg_zero_crossing);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    list (scicos_basic_block_instructions block_id)

  | _ ->

    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and out_port_name = "out"
    and samphold_name = "samphold"
    and absi_name = "absi"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos.
      in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
    let out_port =
      Genr_utilities_scicos.
      out_port_in_scsm0 out_port_name port_1_ (_3_, _0_) in
    let samphold =
      Genr_utilities_scicos.
      samphold_in_scsm0
        samphold_name out_data_type
        Genr_utilities_scicos.external_activation_mode_dummy
        (_1_, _0_) in
    let absi =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos.
      add_block_body_in_scsm0 absi_name
        (scicos_basic_block_instructions Genr_utilities_scicos.blk0)
        nports
        (_2_, _0_) in
    let sample_clock =
      Genr_utilities_scicos.
      sample_clock_in_scsm0
        sample_clock_name sample_time (_1_, _1_) in

    let links = [
      Genr_utilities_scicos.
      add_directed_link_in_scsm0
        in_port_name port_1_ samphold_name port_1_ [];
      add_directed_link_in_scsm0 samphold_name port_1_ absi_name port_1_ [];
      add_directed_link_in_scsm0 absi_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos.
      add_event_link_in_scsm0
        sample_clock_name port_1_ samphold_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      samphold;
      sample_clock;
      absi;
      out_port;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
