(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_band_limited_white_noise.ml,v 1.1 2019-05-10 14:05:17 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
open Ast_types
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Band-Limited White Noise.")
    (Ast_types.mdl_path_of_string "simulink/Sources/Band-Limited\nWhite Noise")
;;

let ts_ident = Ast_types.ident_from_string "Ts"
and cov_ident = ident_from_string "Cov"
and seed_ident = ident_from_string "seed"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let typ = code_out_int 1
  and flag = code_out_int 1
  and a = code_out_int 0 in
  let ts_string = get_ident_string ts_ident
  and cov_string = get_ident_string cov_ident
  and seed_string = get_ident_string seed_ident in
  let ts = Genr_utilities_scicos.code_out_expression ts_string
  and b =
    code_out_expression (Printf.sprintf "sqrt (%s) ./ sqrt (%s)" cov_string ts_string)
  and seed_c =
    code_out_expression (Printf.sprintf "[%s, 0]" seed_string) in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "typ", Genr_types.Code_value typ);
    Value_parameter (Si "flag", Code_value flag);
    Value_parameter (Si "a", Code_value a);
    Value_parameter (Si "b", Code_value b);
    Value_parameter (Si "seed_c", Code_value seed_c);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let out_port_name = "out"
  and sample_clock_name = "sample_clock"
  and rand_name = "rand" in

  let out_port =
    Genr_utilities_scicos.out_port_in_scsm0 out_port_name port_1_ (_1_, _0_) in
  let rand =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 rand_name
      (scicos_basic_block_instructions blk0)
      nports
      (_0_, _0_) in
   let sample_clock =
      Genr_utilities_scicos.
        sample_clock_in_scsm0
          sample_clock_name ts (_0_, _1_) in

  let links = [
    add_directed_link_in_scsm0  rand_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos.add_event_link_in_scsm0
      sample_clock_name port_1_  rand_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    rand;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
