(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_bias.ml,v 1.1 2019-05-10 14:05:17 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type Bias
;;

let bias_ident = Ast_types.ident_from_string "Bias"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let bias_string = get_ident_string bias_ident in

  let bias = Genr_utilities_scicos.code_out_expression bias_string in

  let out_data_type =
    Genr_utilities_scicos.get_simulink_out_data_type_value genr_info in

  let scicos_block_parameters = [
    Value_parameter (Si "Datatype", Code_value out_data_type);
    Value_parameter (Si "sgn", Code_value (code_out_expression "[1,1]"));
    Value_parameter (Si "satur", Code_value (code_out_bool false));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let in_port_name = "in"
  and out_port_name = "out"
  and sum_name = "sum"
  and constant_name = "c" in

  let in_port =
    Genr_utilities_scicos.
    in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
  let out_port =
    Genr_utilities_scicos.
    out_port_in_scsm0 out_port_name port_1_ (_2_, _0_) in
  let sum =
    let nports = (_2_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 sum_name
      (scicos_basic_block_instructions Genr_utilities_scicos.blk0)
      nports
      (_1_, _0_) in

  let constant =
   let nports = (_0_, _1_) in
   let scicos_block_type = Sbt "CONST_m" in
   Genr_utilities_scicos.
   add_block_body_in_scsm0 constant_name
     (genr_instructions_for_modeler_block_type scicos_block_type
        [
          Value_parameter (Si "C", Code_value bias);
        ]
        []
        Genr_utils.blk0 genr_info)
     nports
     (_0_, _1_) in

  let links = [
    Genr_utilities_scicos.
    add_directed_link_in_scsm0
      in_port_name port_1_ sum_name port_2_ [];
    add_directed_link_in_scsm0 constant_name port_1_ sum_name port_1_ [];
    add_directed_link_in_scsm0 sum_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    constant;
    sum;
    out_port;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
