(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_coordinates_to_complex.ml,v 1.1 2019-05-10 14:05:17 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
;;

(* Computing complex numbers from their representation *)

(* Simulink bloc types MagnitudeAngleToComplex and RealImagToComplex share
   a lot of code.
   Hence the sharing of the generator.
*)

let magnitude_angle_option = "Magnitude and angle";;
let real_imag_option = "Real and imag";;

let mdl_block_type_real_imag_to_complex =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.RealImagToComplex
;;

let real_imag_to_complex_formula = code_out_int 2;;
let real_imag_to_complex_option = real_imag_option;;

let mdl_block_type_magnitude_angle_to_complex =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.MagnitudeAngleToComplex
;;

let magnitude_angle_to_complex_formula = code_out_int 2;;
let magnitude_angle_to_complex_option = magnitude_angle_option;;

(* The generator for all the above blocks. *)

let input_ident = Ast_types.ident_from_string "Input";;

let genr mdl_block_type conversion_formula input_option block_id genr_info =

  let input = get_ident_string genr_info input_ident in

  let scicos_block_parameters = [
    Value_parameter (Si "decomptyp", Code_value conversion_formula);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type
      scicos_block_parameters
      [] blk_id genr_info in

  if input <> input_option then begin
    Say.warning
      (Printf.sprintf
         "%S to complex Simulink block translation: \
          unsupported option %S"
         input_option
         input);
    Genr_scicos_not_yet_translated.genr block_id genr_info
  end else

  Concat_list.list
    (scicos_basic_block_instructions block_id)
;;

let genr_magnitude_angle =
  genr
    mdl_block_type_magnitude_angle_to_complex
    magnitude_angle_to_complex_formula
    magnitude_angle_to_complex_option
;;

add_translator
  mdl_block_type_magnitude_angle_to_complex genr_magnitude_angle
;;

let genr_real_imag =
  genr
    mdl_block_type_real_imag_to_complex
    real_imag_to_complex_formula
    real_imag_to_complex_option
;;

add_translator
  mdl_block_type_real_imag_to_complex genr_real_imag
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
