(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_counter_limited.mli,v 1.1 2019-05-10 14:05:17 jpc Exp $ *)

(** {6 Common interface for all block generator modules} *)

(**

  A block generator module should have an empty interface (not to clutter
  the name space of the application).

  In effect, a block generator do not export anything at all: it just records
  one or more translators for one or more Simulink blocks.

  In order to acknowledge the fact, you must use this file as the module
  interface for any block generator module.

*)

(** {6 Adding a new block generator modules} *)

(**

  The shell script [make_generator_interfaces.sh] in this directory maintains
  the consistency of all the block generator module interfaces (the script
  simply copies the same module interface to all the block generator module
  interfaces).

  To add a new module named <new_block_generator> you must edit file
  make_generator_interfaces.sh and add the following line:

     cp -p $COMMON_INTERFACE genr_scicos_pro_<new_block_generator>.mli

  You then have to launch the script to get your new module interface up and
  running.

     ./make_generator_interfaces.sh

  Note:
  - be careful to add your line in the right place to respect alphabetic
    ordering,
  - do not forget to modify file src/Objs.mk to add the new module to the
    list of files to be compiled and linked in the application.

*)

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
