(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_data_type_conversion.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type DataTypeConversion
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

   (* OutMin = para.OutMin  // not supported
   OutMax = para.OutMax // not supported *)
  let out_data_type = get_simulink_out_data_type_value genr_info in

  (* LockScale = para.LockScale // not supported
   ConvertRealWorld = para.ConvertRealWorld // not supported
   RndMeth = para.RndMeth // not supported *)

  match sample_time with
  | "-1" ->

    let scicos_block_parameters = [
      Genr_types.Value_parameter
        (Instructions.Si "it", Genr_types.Code_value (code_out_int (-1)));
      Value_parameter (Si "ot", Code_value out_data_type);
      Value_parameter (Si "np", Simulink_arg_saturation);
    ] in

    let scicos_basic_block_instructions blk_id =
      genr_instructions_for
        mdl_block_type scicos_block_parameters
        [] blk_id genr_info in

    list (scicos_basic_block_instructions block_id)

  | _ ->

    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and samphold_name = "samphold"
    and sample_clock_name = "sample_clock"
    and convert_name = "convert"
    and out_port_name = "out_port" in

    let in_port =
      Genr_utilities_scicos.in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in

    let samphold =
      Genr_utilities_scicos.samphold_in_scsm0
        samphold_name
        out_data_type
        Genr_utilities_scicos.external_activation_mode_dummy
        (_1_, _0_) in

    let sample_clock =
      Genr_utilities_scicos.sample_clock_in_scsm0
        sample_clock_name sample_time (_1_, _1_) in

    let convert =
      Genr_utilities_scicos.convert_in_scsm0
        convert_name out_data_type genr_info (_2_, _0_) in

    let out_port =
      Genr_utilities_scicos.out_port_in_scsm0 out_port_name port_1_ (_3_, _0_) in

    let links = [
      Genr_utilities_scicos.add_directed_link_in_scsm0
        in_port_name port_1_ samphold_name port_1_ [];
      add_directed_link_in_scsm0 samphold_name port_1_ convert_name port_1_ [];
      add_directed_link_in_scsm0 convert_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos.add_event_link_in_scsm0
        sample_clock_name port_1_ samphold_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      samphold;
      sample_clock;
      convert;
      out_port;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
