(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_difference.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Difference")
    (Ast_types.mdl_path_of_string "simulink/Discrete/Difference")
;;

let ic_prev_input_ident =
  Ast_types.ident_from_string "ICPrevInput"
and initial_condition_ident =
  Ast_types.ident_from_string "InitialCondition"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let ic_prev_input =
    if Genr_utilities_scicos.exists_argument genr_info ic_prev_input_ident
    then
      get_ident_string ic_prev_input_ident
    else
      Genr_utilities_scicos.get_parameter_value_string
        genr_info initial_condition_ident in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "a",
       Genr_types.Code_value (code_out_expression ic_prev_input));
    Genr_types.Value_parameter
      (Instructions.Si "inh",
       Genr_types.Code_value (code_out_bool true));
  ] in

  let in_port_name = "in"
  and out_port_name = "out"
  and dollar_name = "dollar"
  and summation_name = "summation"
  and split_name = "split" in

  let in_port =
    Genr_utilities_scicos.in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in

  let split = Genr_utilities_scicos.split_in_scsm0 split_name (_1_, _0_) in

  let dollar =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos.
      add_block_body_in_scsm0 dollar_name
      (genr_instructions_for mdl_block_type
         scicos_block_parameters
         [] blk0 genr_info) nports (_2_, _0_) in

  let summation =
    let nports = (_2_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 summation_name
      (genr_instructions_for_modeler_block_type
         (Sbt "SUMMATION")
         [
           Value_parameter (Si "Datatype",
              Simulink_arg_out_data_type
           );
           Value_parameter (Si "sgn",
              Code_value (code_out_expression "[1;-1]")
           );
           Value_parameter (Si "satur",
              Simulink_arg_saturation
           );
         ]
         [] blk0 genr_info)
      nports (_3_, _0_) in

  let out_port =
    Genr_utilities_scicos.
    out_port_in_scsm0 out_port_name port_1_ (_4_, _0_) in

  let links = [
    Genr_utilities_scicos.add_directed_link_in_scsm0
      in_port_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_1_ dollar_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_2_ summation_name port_1_ [];
    add_directed_link_in_scsm0
      dollar_name port_1_ summation_name port_2_ [];
    add_directed_link_in_scsm0
      summation_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    split;
    dollar;
    summation;
    out_port;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
