(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_digital_clock.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type DigitalClock
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time_value genr_info in

  let out_data_type =
    Genr_utilities_scicos.get_simulink_out_data_type_value genr_info in

  let out_port_name = "out"
  and time_f_name = "time_f"
  and virtual_clk0_name = "virtual_clk0"
  and sample_clock_name = "sample_clock"
  and convert_name = "convert" in

  let out_port =
    Genr_utilities_scicos.out_port_in_scsm0 out_port_name port_1_ (_2_, _0_) in

  let time_f =
    Genr_utilities_scicos.time_f_in_scsm0
      time_f_name
      Genr_utilities_scicos.external_activation_mode_dummy
      (_0_, _0_) in

  let virtual_clk0 =
    Genr_utilities_scicos.virtual_clk0_in_scsm0 virtual_clk0_name (_1_, _1_) in

  let sample_clock =
    Genr_utilities_scicos.sample_clock_in_scsm0
      sample_clock_name sample_time (_1_, _2_) in

  let convert =
    Genr_utilities_scicos.convert_in_scsm0
      convert_name out_data_type genr_info (_1_, _0_) in

  let links = [
    Genr_utilities_scicos.add_directed_link_in_scsm0
      time_f_name port_1_ convert_name port_1_ [];
    Genr_utilities_scicos.add_directed_link_in_scsm0
      convert_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos.add_event_link_in_scsm0
      sample_clock_name port_1_ virtual_clk0_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    time_f;
    virtual_clk0;
    sample_clock;
    convert;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
