(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_discrete_derivative.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

(*
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Discrete Derivative")
    (Ast_types.mdl_path_of_string
       "simulink/Discrete/Discrete Derivative")
;;

let _initial_condition_ident =
  Ast_types.ident_of_string "ICPrevScaledInput"
and gainval_ident =
  Ast_types.ident_of_string "gainval"
;;

let genr block_id genr_info =

  (*
  let scicos_basic_block_instructions blk_id =
    let scicos_block_parameters = [
      Value_parameter (Si "init_cond",
        Simulink_arg_expression initial_condition_ident);
    ] in

    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in
  *)

  let in_port1_name = "in1"
  and out_port_name = "out"
  and gain_name = "gain"
  and split_name = "split"
  and extract_activation_name = "extract_activation"
  and derivative_name = "derivative" in

  let in_port1 =
    Genr_utilities_scicos.
    in_port_in_scsm0 in_port1_name port_1_ (_0_, _1_) in
  let split =
    Genr_utilities_scicos.
    split_in_scsm0 split_name (_1_, _0_) in

  let extract_activation =
    let nports = (_1_, _0_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 extract_activation_name
      (genr_instructions_for_scicos_block_type
         (Sbt "Extract_Activation")
         [] [] blk0 genr_info)
      nports (_2_, _1_) in

  let gain =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 gain_name
      (genr_instructions_for_scicos_block_type
         (Sbt "GAINBLK")
         [
           Value_parameter (Si "gain",
             Simulink_arg_expression gainval_ident);
           Value_parameter (Si "overflow",
             Code_value (code_out_string "Nothing"));
           Value_parameter (Si "externalActivation",
             Code_value external_activation_mode_off);
         ]
         [] blk0 genr_info)
      nports (_2_, _0_) in

  let out_port =
    Genr_utilities_scicos.
    out_port_in_scsm0
      out_port_name port_1_ (_4_, _0_) in

  let derivative =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 derivative_name
      (genr_instructions_for_scicos_block_type
         (Sbt "DERIV")
         [
           Value_parameter (Si "gain",
             Simulink_arg_expression gainval_ident);
           Value_parameter (Si "overflow",
             Code_value (code_out_string "Nothing"));
           Value_parameter (Si "externalActivation",
             Code_value external_activation_mode_off);
         ]
         [] blk0 genr_info)
      nports (_3_, _0_) in

  let links = [
    Genr_utilities_scicos.
    add_directed_link_in_scsm0
      in_port1_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_1_ gain_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_2_ extract_activation_name port_1_ [];
    add_directed_link_in_scsm0
      gain_name port_1_ derivative_name port_1_ [];
    add_directed_link_in_scsm0
      derivative_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos.
    add_event_link_in_scsm0
       extract_activation_name port_1_ derivative_name port_1_ [];
  ] in

  let super_body = [
    in_port1;
    extract_activation;
    split;
    out_port;
    gain;
    derivative;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

*)

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
