(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_discrete_transfer_fcn.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type_discrete_transfer_fcn =
  Mdl_basics_explicit_ast.Explicit_block_regular_type DiscreteTransferFcn
;;

let mdl_block_type_discrete_filter =
  Mdl_basics_explicit_ast.Explicit_block_regular_type DiscreteFilter
;;

let numerator_ident = Ast_types.ident_from_string "Numerator"
and denominator_ident = Ast_types.ident_from_string "Denominator"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let get_component n =
    Printf.sprintf "poly (fliplr (%s), \'z\', \'coeff\')" n in

  let num =
    code_out_expression
      (get_component (get_ident_string numerator_ident))
  and den =
    code_out_expression
      (get_component (get_ident_string denominator_ident)) in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "num", Genr_types.Code_value num);
    Value_parameter (Si "den", Code_value den);

  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type_discrete_filter scicos_block_parameters
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    list (scicos_basic_block_instructions block_id)

  | _ ->

    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and out_port_name = "out"
    and dlr_name = "dlr"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos.
      in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
    let out_port =
      Genr_utilities_scicos.
      out_port_in_scsm0 out_port_name port_1_ (_2_, _0_) in
    let dlr =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos.
      add_block_body_in_scsm0 dlr_name
        (scicos_basic_block_instructions blk0)
        nports
        (_1_, _0_) in
    let sample_clock =
      Genr_utilities_scicos.
      sample_clock_in_scsm0
        sample_clock_name sample_time (_1_, _1_) in

    let links = [
      Genr_utilities_scicos.
      add_directed_link_in_scsm0
        in_port_name port_1_ dlr_name port_1_ [];
      add_directed_link_in_scsm0 dlr_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos.
      add_event_link_in_scsm0
        sample_clock_name port_1_ dlr_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      dlr;
      sample_clock;
      out_port;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type_discrete_transfer_fcn genr
;;

add_translator mdl_block_type_discrete_filter genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
