(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_elementary_math.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_utilities_scicos
;;

let mdl_block_type = simulink_block_type ElementaryMath;;

let scicos_block_type =
  Genr_utilities_scicos.modeler_block_type_for mdl_block_type
;;

let operator_ident = Ast_types.ident_from_string "Operator";;

let exp_block = Instructions.Sbt "EXPBLK_m"
and log_block = Sbt "LOGBLK_f"
and inv_block = Sbt "INVBLK"
;;

let euler_number = Genr_utilities_scicos.code_out_expression "exp(1)"
and constant_10 = code_out_int 10
;;

let si_base = Si "a"
and si_fun = Si "fun";;

let get_block_type_params operator =
  match operator with
  | "Exp" -> exp_block, [ si_base, euler_number; ]
  | "10^u" -> exp_block, [ si_base, constant_10; ]
  | "Log" -> log_block, [ si_base, euler_number; ]
  | "Log10" -> log_block, [ si_base, constant_10; ]
  | "Reciprocal" -> inv_block, []
  | _ -> scicos_block_type, [ si_fun, code_out_expression operator; ]
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let operator = get_ident_string operator_ident in

  let scicos_block_type, params = get_block_type_params operator in

  let instructions =

    if scicos_block_type = inv_block then [
      Instantiate_block (block_id, inv_block);
    ] else

    let block_parameter_bindings =
      match params with
      | [] -> assert false
      | param :: params ->
        Ulist1.map
          Genr_utilities_scicos.
          block_parameter_binding
          (param, params) in
    [
      Instantiate_block (block_id, scicos_block_type);
      Set_block_parameters (block_id, block_parameter_bindings);
    ] in

  list instructions
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
