(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_function_call_generator.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Function-Call Generator")
    (Ast_types.mdl_path_of_string
      "simulink/Ports &\nSubsystems/Function-Call\nGenerator")
;;

let number_of_iterations_ident =
  Ast_types.ident_from_string "numberOfIterations"
;;

let genr block_id genr_info =

  let get_ident_int =
    Genr_utilities_scicos.get_ident_int genr_info in

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let scicos_block_parameters =
    [
      Value_parameter (Si "C",
        Code_value (code_out_expression "1"));
    ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let number_of_iterations = get_ident_int number_of_iterations_ident in

  match number_of_iterations with
  | 1 ->

    let sample_time = code_out_expression sample_time in

    let out_port_name = "out"
    and const_name = "const"
    and sample_clock_name = "sample_clock"
    and samphold_name = "samphold" in

    let out_port =
      Genr_utilities_scicos.
      out_port_in_scsm0 out_port_name port_1_ (_3_, _0_) in

    let samphold =
      Genr_utilities_scicos.samphold_in_scsm0 samphold_name
        (code_out_int (-1))
        external_activation_mode_dummy
        (_2_, _0_) in

    let const =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos.
      add_block_body_in_scsm0 const_name
        (scicos_basic_block_instructions blk0)
        nports
        (_1_, _0_) in

    let sample_clock =
      Genr_utilities_scicos.
      sample_clock_in_scsm0
        sample_clock_name sample_time (_1_, _1_) in

    let links = [
      Genr_utilities_scicos.
      add_directed_link_in_scsm0 const_name port_1_ samphold_name port_1_ [];
      add_directed_link_in_scsm0 samphold_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos.
      add_event_link_in_scsm0
        sample_clock_name port_1_ samphold_name port_1_ [];
    ] in

    let super_body = [
      sample_clock;
      samphold;
      const;
      out_port;
      links;
    ] in

    super_block_in_scsm0 block_id super_body

  | _ ->
    Say.warning
      (Printf.sprintf
         "Function generator Simulink block translation: \
          Number of iterations other than one not yet supported.");
    Genr_scicos_not_yet_translated.genr block_id genr_info
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
