(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_if_action_event_gen.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

(** Generate a super block for a Simulink sub system block that is actioned,
  meaning that it has an ActionPort block in its sub system. *)

open Mdl_basics_deep_ast
open Concat_list
open Instructions
open Genr_utilities_scicos
;;

let mdl_block_type =
  Mdl_fake_block_types.if_action_event_generator_block_type
;;

let block_type_name_ident = Ast_types.ident_from_string "BlockType";;

let genr block_id genr_info =

  let ports_desc = genr_info.Genr_block_types.block_ports_description in
  let nin, nout =
    Mdl_ast_to_deep_ast.get_regular_port_nin ports_desc,
    Mdl_ast_to_deep_ast.get_regular_port_nout ports_desc
  and ebp_table = genr_info.Genr_block_types.block_arguments
  and explicit_block_type = genr_info.Genr_block_types.block_explicit_type in

  let short_block_type_name =
    Mdl_basics_to_explicit_ast.
    name_of_explicit_block_source_type explicit_block_type in

  let full_block_type_name =
    Mdl_basics_to_explicit_ast.
    name_of_explicit_block_type explicit_block_type in

  let additional_binding =
    Genr_utils.
    make_binding block_type_name_ident
      (Parameter_value (Deep_string full_block_type_name)) in

  let bindings =
    Mdl_translation.modeler_evaluation_commented_context ebp_table in

  let context =
    Context_commented (additional_binding, bindings) in

  let context_code = [
    set_diagram_context_in_scsm0 context;
    Set_block_icon_text (block_id,
      Ast_types.mdl_text_of_string (
        short_block_type_name ^
        Mdl_fake_block_types.not_yet_translated_block_mark))
  ] in

  let _, in_ports = in_ports_in_scsm0 nin in

  let _, out_ports = out_ports_in_scsm0 Unatural._5_ nout in

  let in_ports = list_of_clist in_ports in
  let out_ports = list_of_clist out_ports in

  let super_body = [
    context_code;
    in_ports;
    out_ports;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
