(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_merge.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_types
open Genr_utilities_scicos
;;

let mdl_block_type = simulink_block_type Merge;;

let inputs_ident =
  Ast_types.ident_from_string "Inputs"
and initial_output_ident =
  Ast_types.ident_from_string "InitialOutput"
and allow_unequal_input_port_widths_ident =
  Ast_types.ident_from_string "AllowUnequalInputPortWidths"
and input_port_offsets_ident =
  Ast_types.ident_from_string "InputPortOffsets"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let nin =
    Genr_utilities_scicos.get_ident_int genr_info inputs_ident
  and initial_output =
    get_ident_string initial_output_ident
  and allow_unequal_input_port_widths =
    Genr_utilities_scicos.
    get_ident_bool genr_info allow_unequal_input_port_widths_ident
  and input_port_offsets =
    get_ident_string input_port_offsets_ident in

  if initial_output <> "[]" then
    Say.warning (Printf.sprintf
      "Merge block translation:\
       initial output is not supported");
  if allow_unequal_input_port_widths then
    Say.warning (Printf.sprintf
      "Merge block translation:\
       AllowUnequalInputPortWidths is not supported");
  if input_port_offsets <> "[]" then
    Say.warning (Printf.sprintf
      "Merge block translation:\
       InputPortOffsets is not supported");

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "nin", Code_value (code_out_int nin));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [ Set_block_nin (blk_id, Unatural.of_int nin); ]
      blk_id genr_info in

  list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
