(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pmio_port.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Genr_block_types
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.PMIOPort
;;

let port_ident = Ast_types.ident_from_string "Port"
and side_ident = Ast_types.ident_from_string "Side"
;;

let genr block_id genr_info =


  let get_ident_string =
    Genr_utilities_scicos.get_ident_string genr_info
  and get_ident_int =
    Genr_utilities_scicos.get_ident_int genr_info in

  let port_int = get_ident_int port_ident
  and side_string = get_ident_string side_ident in

  let
    { Genr_block_types.
      system_pmio_port_sides = pmio_port_sides;
      system_inport_number = parent_nin;
      system_outport_number = parent_nout;
    } = genr_info.Genr_block_types.block_parent_system_io_descriptor in

  let scicos_block_parameters index = [
    Value_parameter (Si "portNumber",
      Code_value (code_out_int index));
  ] in

  let lim = Array.length pmio_port_sides in

  let scicos_block_type, index =
    match side_string with
    | "Left" ->
      Sbt "INIMPL_f",
      if port_int > lim then
        Say.fatal_error "incorrect numbering of PMIOPort" else
      let rec loop i accu =
        if i = port_int then accu else
        begin
          match pmio_port_sides.(i) with
          | Some (
              Genr_block_types.
              Pmio_port_left) -> loop (i + 1) (accu + 1)
          | None | Some Pmio_port_right ->
            loop (i + 1) accu
        end in
      loop 0 (Unatural.to_int parent_nin)
    | "Right" ->
      Sbt "OUTIMPL_f",
      if port_int > lim then
        Say.fatal_error "incorrect numbering of PMIOPort" else
      let rec loop i accu =
        if i = port_int then accu else
        begin
          match pmio_port_sides.(i) with
          | None | Some Pmio_port_left ->
            loop (i + 1) accu
          | Some Pmio_port_right ->
            loop (i + 1) (accu + 1)
        end in
      loop 0 (Unatural.to_int parent_nout)
    | s ->
      Say.fatal_error
        (Printf.sprintf "Unknown port connection side '%s'" s) in

  let scicos_basic_block_instructions =
    genr_instructions_for_modeler_block_type
      scicos_block_type (scicos_block_parameters index)
      [] block_id genr_info in
  Concat_list.list scicos_basic_block_instructions
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
