(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_random_number.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type = simulink_block_type RandomNumber;;

let mean_ident =
  Ast_types.ident_from_string "Mean"
and variance_ident =
  Ast_types.ident_from_string "Variance"
and seed_ident =
  Ast_types.ident_from_string "Seed"

;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let mean = get_ident_string mean_ident
  and variance = get_ident_string variance_ident
  and seed =
    Printf.sprintf "[%s, 0]" (get_ident_string seed_ident) in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "typ",
       Genr_types.Code_value (code_out_int 1));
    Value_parameter (Si "flag", Code_value (code_out_int 1));
    Value_parameter (Si "a", Code_value (code_out_expression mean));
    Value_parameter (Si "b", Code_value (code_out_expression variance));
    Value_parameter (Si "seed_c", Code_value (code_out_expression seed));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in


  let sample_time = if sample_time = "-1" then code_out_expression "0"
              (* above is in case of atomic SB *)
                    else code_out_expression sample_time in

  let out_port_name = "out"
  and rand_name = "rand"
  and sample_clock_name = "sample_clock" in

  let out_port =
    Genr_utilities_scicos.
    out_port_in_scsm0 out_port_name port_1_ (_1_, _0_) in
  let rand =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 rand_name
      (scicos_basic_block_instructions blk0)
      nports
      (_0_, _0_) in
  let sample_clock =
    Genr_utilities_scicos.
    sample_clock_in_scsm0
      sample_clock_name sample_time (_0_, _1_) in

  let links = [
    add_directed_link_in_scsm0 rand_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos.
    add_event_link_in_scsm0
      sample_clock_name port_1_ rand_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    rand;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
