(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_repeating_sequence_interpolated.ml,v 1.1 2019-05-10 14:05:18 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Repeating Sequence Interpolated")
    (Ast_types.mdl_path_of_string
       "simulink/Sources/Repeating\nSequence\nInterpolated")
;;

let time_values_ident = Ast_types.ident_from_string "TimeValues"
and out_values_ident = Ast_types.ident_from_string "OutValues"
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let meth = 1
  and xx = get_ident_string time_values_ident
  and yy = get_ident_string out_values_ident
  and periodic_option = "y"
  and graf = "n" in

  let out_port_name = "out"
  and curv_c_name = "curv_c"
  and sample_clock_name = "sample_clock" in

  let scicos_block_parameters = [
    Genr_types.Value_parameter (Instructions.Si "Method",
      Genr_types.Code_value (code_out_int meth));
    Value_parameter (Si "xx",
      Code_value (code_out_expression xx));
    Value_parameter (Si "yy",
      Code_value (code_out_expression yy));
    Value_parameter (Si "PeriodicOption",
      Code_value (code_out_expression periodic_option));
    Value_parameter (Si "graf",
      Code_value (code_out_expression graf));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let sample_time =
    match sample_time with
    | "-1" -> "0"
    | t -> t in

  let sample_time = code_out_expression sample_time in

  let out_port =
    Genr_utilities_scicos.
    out_port_in_scsm0 out_port_name port_1_ (_1_, _0_) in
  let curv_c =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 curv_c_name
      (scicos_basic_block_instructions blk0)
      nports
      (_0_, _0_) in
  let sample_clock =
    Genr_utilities_scicos.
    sample_clock_in_scsm0 sample_clock_name sample_time (_1_, _1_) in
  let links = [
    add_directed_link_in_scsm0 curv_c_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos.
    add_event_link_in_scsm0 sample_clock_name port_1_ curv_c_name port_1_ [];
  ] in

  let super_body = [
    sample_clock;
    curv_c;
    out_port;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
