(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_stop.ml,v 1.1 2019-05-10 14:05:19 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type = simulink_block_type Stop;;

let genr block_id genr_info =

  let in_name = "in"
  and absi_name = "abs_value"
  and if_then_else_name = "if_then_else"
  and end_c_name = "end" in

  let zcr = code_out_int 0 in

  let in_port =
    Genr_utilities_scicos.in_port_in_scsm0 in_name port_1_ (_0_, _1_) in

  let absi =
    let scicos_block_type = Sbt "ABS_VALUEi" in
    let nports = (_1_, _1_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 absi_name
      (genr_instructions_for_modeler_block_type scicos_block_type
         [
           Genr_types.Value_parameter
             (Instructions.Si "zcr",
              Genr_types.Code_value zcr);
         ]
         [] blk0 genr_info)
      nports
      (_1_, _1_) in
  let if_then_else =
    let scicos_block_type = Sbt "IFTHEL_f" in
    let nports = (_1_, _0_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 if_then_else_name
      (genr_instructions_for_modeler_block_type scicos_block_type
         [
           Value_parameter (Si "inh", Code_value (code_out_int 0));
           Value_parameter (Si "nmod", Code_value zcr);
         ]
         [] blk0 genr_info)
      nports
      (_2_, _1_) in
  let end_c =
    let nports = (_0_, _0_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 end_c_name
      (genr_instructions_for mdl_block_type
         [
           Value_parameter (Si "tf", Code_value (code_out_int 0));
         ]
         [] blk0 genr_info)
      nports
      (_2_, _0_) in

  let links = [
    Genr_utilities_scicos.
    add_directed_link_in_scsm0 in_name port_1_ absi_name port_1_ [];
    add_directed_link_in_scsm0 absi_name port_1_ if_then_else_name port_1_ [];
    add_event_link_in_scsm0 if_then_else_name port_1_ end_c_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    absi;
    if_then_else;
    end_c;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
