(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_switch.ml,v 1.1 2019-05-10 14:05:19 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Instructions
open Genr_types
open Genr_utilities_scicos
;;

let mdl_block_type = simulink_block_type Switch;;

let criteria_ident =
  Ast_types.ident_from_string "Criteria"
and threshold_ident =
  Ast_types.ident_from_string "Threshold"
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  match sample_time with
  | "-1" ->

    let zcr = Genr_utilities_scicos.get_zero_crossing_value genr_info
    and out_data_type =
      Genr_utilities_scicos.get_simulink_out_data_type_value genr_info
    and criteria = get_ident_string criteria_ident in

    let default_criteria = "u2 >= Threshold" in
(*    let default_criteria = Matlab_expression default_criteria_string in *)

    let rec get_op criteria =
      match criteria with
      | c when c = default_criteria -> 0
      | "u2 > Threshold" -> 1
      | "u2 ~= 0" -> 2
      | c ->
        Genr_utilities_scicos.
        warning
          (Printf.sprintf
             "Unsupported criterion in Switch block: \
            using %S instead of %S."
             default_criteria c);
        get_op default_criteria in

    let op = get_op criteria in

    let thresh =
      if op = 2
      then code_out_int 0
      else code_out_expression (get_ident_string threshold_ident) in

    let scicos_block_parameters = [
      Genr_types.Value_parameter (Instructions.Si "ot",
        Genr_types.Code_value (out_data_type));
      Value_parameter (Si "rule", Code_value (code_out_int op));
      Value_parameter (Si "thra", Code_value thresh);
      Value_parameter (Si "nzz", Code_value zcr);
    ] in

    let scicos_basic_block_instructions blk_id =
      genr_instructions_for
        mdl_block_type scicos_block_parameters
        [] blk_id genr_info in

    list (scicos_basic_block_instructions block_id)

  | _ ->
    Genr_utilities_scicos.
    warning "Not inherited Switch block is not supported";
    Genr_scicos_not_yet_translated.genr block_id genr_info
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
