(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_trigonometry.ml,v 1.1 2019-05-10 14:05:19 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Concat_list
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type = simulink_block_type Trigonometry;;

let operator_ident =
  Ast_types.ident_from_string "Operator"
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in
  let operator = get_ident_string operator_ident in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "fun",
       Genr_types.Code_value (code_out_expression operator));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    list (scicos_basic_block_instructions block_id)

  | _ ->

    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and out_port_name = "out"
    and samphold_name = "samphold"
    and trigonometry_name = "trigonometry"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos.in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
    let out_port =
      Genr_utilities_scicos.out_port_in_scsm0 out_port_name port_1_ (_3_, _0_) in
    let samphold =
      Genr_utilities_scicos.samphold_in_scsm0 samphold_name
        (code_out_expression "inherit")
        external_activation_mode_dummy
        (_1_, _0_) in
    let trigonometry =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos.
      add_block_body_in_scsm0 trigonometry_name
        (scicos_basic_block_instructions blk0)
        nports
        (_2_, _0_) in
    let sample_clock =
      Genr_utilities_scicos.
      sample_clock_in_scsm0 sample_clock_name sample_time (_1_, _1_) in

    let links = [
      Genr_utilities_scicos.
      add_directed_link_in_scsm0 in_port_name port_1_ samphold_name port_1_ [];
      add_directed_link_in_scsm0 samphold_name port_1_ trigonometry_name port_1_ [];
      add_directed_link_in_scsm0 trigonometry_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos.
      add_event_link_in_scsm0
        sample_clock_name port_1_ samphold_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      samphold;
      trigonometry;
      sample_clock;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
