(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_xy_scope.ml,v 1.1 2019-05-10 14:05:19 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "XY scope.")
    (Ast_types.mdl_path_of_string "simulink/Sinks/XY Graph")
;;

let xmin_ident =
  Ast_types.ident_from_string "xmin"
and xmax_ident =
  Ast_types.ident_from_string "xmax"
and ymin_ident =
  Ast_types.ident_from_string "ymin"
and ymax_ident =
  Ast_types.ident_from_string "ymax"
and st_ident =
  Ast_types.ident_from_string "st"
;;

let genr block_id genr_info =

  let get_ident_string = Genr_utilities_scicos.get_ident_string genr_info in

  let nbr_curves = 1
  and clrs = 5
  and siz = 1
  and win = -1
  and wpos = code_out_expression "[ ]"
  and wdim = code_out_expression "[400, 600]"
  and xmin = code_out_expression (get_ident_string xmin_ident)
  and xmax = code_out_expression (get_ident_string xmax_ident)
  and ymin = code_out_expression (get_ident_string ymin_ident)
  and ymax = code_out_expression (get_ident_string ymax_ident)
  and n = 2
  and st = get_ident_string st_ident in

  let st =
    match st with
    | "-1" ->
      Say.warning
        "XY Graph Simulink block translation: \
         inherited sample time not supported; using 0.01";
      "0.01"
    | st -> st in

  let scicos_block_parameters = [
    Genr_types.Value_parameter
      (Instructions.Si "nbr_curves",
       Genr_types.Code_value (code_out_int nbr_curves));
    Value_parameter (Si "clrs", Code_value (code_out_int clrs));
    Value_parameter (Si "siz", Code_value (code_out_int siz));
    Value_parameter (Si "win", Code_value (code_out_int win));
    Value_parameter (Si "wpos", Code_value wpos);
    Value_parameter (Si "wdim", Code_value wdim);
    Value_parameter (Si "xmin", Code_value xmin);
    Value_parameter (Si "xmax", Code_value xmax);
    Value_parameter (Si "ymin", Code_value ymin);
    Value_parameter (Si "ymax", Code_value ymax);
    Value_parameter (Si "N", Code_value (code_out_int n));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let in_port1_name = "in1"
  and in_port2_name = "in2"
  and sample_clock_name = "sample_clock"
  and cscopxy_name = "cscopxy" in

  let in_port1 =
    Genr_utilities_scicos.
    in_port_in_scsm0 in_port1_name port_1_ (_0_, _1_) in
  let in_port2 =
    Genr_utilities_scicos.
    in_port_in_scsm0 in_port2_name port_2_ (_0_, _0_) in
  let cscopxy =
    let nports = (_2_, _0_) in
    Genr_utilities_scicos.
    add_block_body_in_scsm0 cscopxy_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in
  let sample_clock =
    Genr_utilities_scicos.
    sample_clock_in_scsm0
      sample_clock_name (code_out_expression st) (_1_, _1_) in

  let links = [
    add_directed_link_in_scsm0 in_port1_name port_1_ cscopxy_name port_1_ [];
    add_directed_link_in_scsm0 in_port2_name port_1_ cscopxy_name port_2_ [];
    Genr_utilities_scicos.
    add_event_link_in_scsm0
      sample_clock_name port_1_ cscopxy_name port_1_ [];
  ] in

  let super_body = [
    in_port1;
    in_port2;
    cscopxy;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
