(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_zero_order_hold.ml,v 1.1 2019-05-10 14:05:19 jpc Exp $ *)

(* In scicos we must use module Genr_utilities_scicos and not
   Genr_utilities_scicos_pro. *)
module Genr_utilities_scicos_pro = struct end;;

open Mdl_block_types
open Instructions
open Genr_utilities_scicos
open Unatural
;;

let mdl_block_type = simulink_block_type ZeroOrderHold;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos.get_sample_time genr_info in

  let in_name = "in"
  and out_name = "out"
  and samphold_name = "samphold" in

  let in_port = Genr_utilities_scicos.in_port_in_scsm0 in_name port_1_ (_0_, _0_) in
  let out_port = Genr_utilities_scicos.out_port_in_scsm0 out_name port_1_ (_3_, _0_) in

  let samphold =
    Genr_utilities_scicos.samphold_in_scsm0 samphold_name
      (code_out_int (-1))
      external_activation_mode_dummy
      (_2_, _0_) in

  let inside_super_body =
    match sample_time with
    | "-1" ->

      let extract_activation_name = "extract_activation"
      and split_name = "split" in

      let extract_activation =
        let nports = (_1_, _0_) in
        let block_body = [
          Instantiate_block (Genr_utils.blk0, Sbt "Extract_Activation");
        ] in
        add_block_body_in_scsm0
          extract_activation_name block_body nports (_2_, _1_) in
      let split = split_in_scsm0 split_name (_1_, _0_) in

      let links = [
        Genr_utilities_scicos.
        add_directed_link_in_scsm0 in_name port_1_ split_name port_1_ [];
        add_directed_link_in_scsm0 split_name port_1_ samphold_name port_1_ [];
        add_directed_link_in_scsm0
          split_name port_2_ extract_activation_name port_1_ [];
        add_directed_link_in_scsm0 samphold_name port_1_ out_name port_1_ [];
        add_event_link_in_scsm0
          extract_activation_name port_1_ samphold_name port_1_ [];
      ] in

      [
        split;
        extract_activation;
        links;
      ]

    | _ ->

      let sample_time = code_out_expression sample_time in

      let sample_clock_name = "sample_clock" in

      let sample_clock =
        Genr_utilities_scicos.sample_clock_in_scsm0
          sample_clock_name sample_time (_2_, _1_) in

      let links = [
        Genr_utilities_scicos.
        add_directed_link_in_scsm0 in_name port_1_ samphold_name port_1_ [];
        add_directed_link_in_scsm0 samphold_name port_1_ out_name port_1_ [];
        add_event_link_in_scsm0
          sample_clock_name port_1_ samphold_name port_1_ [];
      ] in

      [
        sample_clock;
        links;
      ] in

  let super_body = (
    in_port ::
    out_port ::
    samphold ::
    inside_super_body
  ) in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
