(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_atomic_event_gen.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

(** Generate a super block for a Simulink sub system block
  that has atomic property and sample time that is not "-1". *)

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_fake_block_types.atomic_event_generator_block_type
;;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_system_sample_time genr_info in

  let sample_time = code_out_expression sample_time in

    let out_port_event_name = "out"
    and sample_clock_name = "sample_clock" in

    let out_port_event =
      Genr_utilities_scicos_pro.
      out_port_event_in_scsm0 out_port_event_name port_1_
        (_1_, _0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ out_port_event_name port_1_ [];
    ] in

    let context_code = [
      Instructions.Set_block_icon_text (block_id,
        Ast_types.mdl_text_of_string "EventGen");
    ] in

    let super_body = [
      context_code;
      sample_clock;
      out_port_event;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
