(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_band_limited_white_noise.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
open Ast_types
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Band-Limited White Noise.")
    (Ast_types.mdl_path_of_string
       "simulink/Sources/Band-Limited\nWhite Noise")
;;

let ts_ident = Ast_types.ident_from_string "Ts"
and cov_ident = ident_from_string "Cov"
and seed_ident = ident_from_string "seed"
;;

let genr block_id genr_info =

  let get_ident = Genr_utilities_scicos_pro.get_ident_string genr_info in

  let typ = Genr_utilities_scicos_pro.code_out_string "Normal"
  and a = Genr_utilities_scicos_pro.code_out_int 0 in
  let ts_string = get_ident ts_ident
  and cov_string = get_ident cov_ident in
  let ts = Genr_utilities_scicos_pro.code_out_expression ts_string
  and b =
    let expr =
      Printf.sprintf "sqrt (%s) ./ sqrt (%s)" cov_string ts_string in
    Genr_utilities_scicos_pro.
    code_out_expression expr
  and seed =
    code_out_expression (get_ident seed_ident) in

  let scicos_block_parameters = [
    Value_parameter (Si "distribution", Code_value typ);
    Value_parameter (Si "A", Code_value a);
    Value_parameter (Si "B", Code_value b);
    Value_parameter (Si "seed", Code_value seed);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let out_port_name = "out"
  and sample_clock_name = "sample_clock"
  and rand_name = "rand" in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_
      (_1_, _0_) in
  let rand =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 rand_name
      (scicos_basic_block_instructions blk0)
      nports
      (_0_, _0_) in
  let sample_clock =
    Genr_utilities_scicos_pro.
    sample_clock_in_scsm0
      sample_clock_name ts
      (_0_, _1_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      rand_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos_pro.
    add_event_link_in_scsm0
      sample_clock_name port_1_ rand_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    rand;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
