(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_bus_assignment.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

let mdl_block_type = simulink_block_type Mdl_block_types.BusAssignment;;

let assigned_signals_ident =
  Ast_types.ident_from_string "AssignedSignals"
;;

let genr block_id genr_info =

  let assigned_signals =
    List.map
      (fun assigned_signal -> Code_value (code_out_string assigned_signal))
      (Lib_strings.split_string_at_comma
         (Genr_utilities_scicos_pro.
          get_ident_string genr_info assigned_signals_ident)) in

  let nin = Unatural.of_int (List.length assigned_signals) in

  let scicos_block_parameters = [
    Value_parameter
      (Si "nin", Code_value (code_out_int (Unatural.to_int nin)));
    Struct_parameter
      (Si "reg_ports", ((Si "sign", assigned_signals), []));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
