(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_c_m_block.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

(** Generate a fake super block for a Simulink block
  that does not have a proper generator yet. *)

open Instructions
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "CMBlock")
    (Ast_types.mdl_path_of_string
       "simulink/Model-Wide\nUtilities/Model Info")
;;

let icon_text_ident =
  Ast_types.ident_from_string "DisplayStringWithTags"
;;

let genr block_id genr_info =

  let icon_text =
    Genr_utilities_scicos_pro.
    get_ident_string genr_info icon_text_ident in

  let icon_code = [
    Set_block_icon_text (block_id,
      Ast_types.mdl_text_of_string icon_text)
  ] in

  let super_body = [
    icon_code;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
