(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_counter_limited.ml,v 1.1 2019-05-10 14:04:56 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Counter Limited")
    (Ast_types.mdl_path_of_string
       "simulink/Sources/Counter\nLimited")
;;

let up_limit_ident = Ast_types.ident_from_string "uplimit";;

let genr block_id genr_info =

  let get_ident_string = get_ident_string genr_info in

  let up_limit = get_ident_string up_limit_ident in
  let base = up_limit ^ "+1" in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_block_parameters = [
    Value_parameter (Si "ini_state", Code_value (code_out_int 0));
    Value_parameter (Si "base", Code_value (code_out_expression base));
    Value_parameter (Si "step", Code_value (code_out_int 1) );
    Value_parameter (Si "typ",
      Code_value (code_out_string "uint32"));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let sample_time =
    match sample_time with
    | "-1" -> "0"
    | _ -> sample_time in

  let out_port_name = "out"
  and counteri_name = "counteri"
  and sample_clock_name = "sample_clock" in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_
      (_2_, _0_) in

  let counteri =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 counteri_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in

  let sample_clock =
    Genr_utilities_scicos_pro.
    sample_clock_in_scsm0
      sample_clock_name
      (code_out_expression sample_time)
      (_1_, _1_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      counteri_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos_pro.
    add_event_link_in_scsm0
      sample_clock_name port_1_ counteri_name port_1_ [];
  ] in

  let super_body = [
    sample_clock;
    counteri;
    out_port;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
