(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_cross_product.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Cross Product")
    (Ast_types.mdl_path_of_string
       "aerospace/Axes\nTransformations/3x3 cross product")
;;

let genr block_id genr_info =

  let nin = code_out_int 2
  and work = code_out_expression "getcurrentenv"
  and exx = code_out_string
    "[u1(2)*u2(3)-u1(3)*u2(2);\
   \n u1(3)*u2(1)-u1(1)*u2(3);\
   \n u1(1)*u2(2)-u1(2)*u2(1)]" in

  let scicos_block_parameters = [
    Value_parameter (Si "nin", Code_value nin);
    Value_parameter (Si "work", Code_value work);
    Value_parameter (Si "exx", Code_value exx);
    Value_parameter (Si "externalActivation",
       Code_value external_activation_mode_off);
  ] in
  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
