(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_data_type_conversion.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.DataTypeConversion
;;

let genr block_id genr_info =

  (* OutMin = para.OutMin  // not supported
     OutMax = para.OutMax // not supported *)
  let out_data_type = get_simulink_out_data_type_value genr_info in

  (* LockScale = para.LockScale // not supported
     ConvertRealWorld = para.ConvertRealWorld // not supported
     RndMeth = para.RndMeth // not supported *)
  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_block_parameters activation = [
    Value_parameter (Si "it", Code_value inherit_mode_type);
    Value_parameter (Si "ot", Code_value out_data_type);
    Value_parameter (Si "satur", Simulink_arg_saturation);
    Value_parameter (Si "rmethod", Code_value floor_round_method);
    Value_parameter (Si "externalActivation", Code_value activation);
  ] in

  let scicos_basic_block_instructions blk_id activation =
    genr_instructions_for
      mdl_block_type (scicos_block_parameters activation)
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list
      (scicos_basic_block_instructions block_id
       Genr_utilities_scicos_pro.external_activation_mode_off)

  | _ ->
    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and sample_clock_name = "sample_clock"
    and convert_name = "convert"
    and out_port_name = "out_port" in

    let in_port =
      Genr_utilities_scicos_pro.
      in_port_in_scsm0 in_port_name port_1_
        (_0_, _0_) in

    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_1_, _1_) in

    let convert =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0  convert_name
        (scicos_basic_block_instructions blk0
         Genr_utilities_scicos_pro.external_activation_mode_on)
        nports
        (_1_, _0_) in

    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0 out_port_name port_1_
        (_2_, _0_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ convert_name port_1_ [];
      add_directed_link_in_scsm0
        convert_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ convert_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      sample_clock;
      convert;
      out_port;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
