(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_dead_zone_dynamic.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type_path =
  Ast_types.
  mdl_path_of_string "simulink/Discontinuities/Dead Zone\nDynamic"
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Dead Zone Dynamic")
    mdl_block_type_path
;;

let genr block_id genr_info =

  let scicos_block_parameters = [ ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let out_port_name = "out"
  and dead_zone_name = "deadzone"
  and in_port_up_name = "up"
  and in_port_name = "u"
  and in_port_low_name = "lo" in

  let in_port_up =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_up_name port_3_
  and in_port =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_name port_2_
  and in_port_low =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_low_name port_1_
  and out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_
  and dead_zone =
    let nports = (_3_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 dead_zone_name
      (scicos_basic_block_instructions blk0)
      nports in

  let blocks = [
    out_port    (_2_, _1_);
    in_port_low (_0_, _2_);
    in_port_up  (_0_, _0_);
    in_port     (_0_, _1_);
    dead_zone   (_1_, _1_);
  ] in

  let links = [
    Genr_utilities_scicos_pro.add_directed_link_in_scsm0
      in_port_low_name port_1_ dead_zone_name port_1_ [];
    Genr_utilities_scicos_pro.add_directed_link_in_scsm0
      in_port_name port_1_ dead_zone_name port_2_ [];
    Genr_utilities_scicos_pro.add_directed_link_in_scsm0
      in_port_up_name port_1_ dead_zone_name port_3_ [];
    Genr_utilities_scicos_pro.add_directed_link_in_scsm0
      dead_zone_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body =
    blocks @ [ links; ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
