(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_demux.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.Demux
;;

let outputs_ident =
  Ast_types.ident_from_string "Outputs"
;;

let genr block_id genr_info =

  let ports_desc = genr_info.Genr_block_types.block_ports_description in

  let nout =
    Mdl_ast_to_deep_ast.get_regular_port_nout ports_desc in
  let nout_int =  Unatural.to_int nout in

  let outputs =
    Genr_utilities_scicos_pro.get_ident_string genr_info outputs_ident in
  let output_sizes = Genr_utilities_pro.get_size_array "Demux" outputs in

  let scicos_block_parameters =
    match output_sizes with
    | Genr_utilities_pro.Size_array_vector output_list ->
      let nop = List.length output_list in
      if nop <> nout_int then begin
        Say.warning (Printf.sprintf
          "Demux parameter ``%d'' does not match port \
           number ``%d''" nop nout_int);
        [
          Value_parameter
            (Si "nout",
             Code_value (code_out_natural nout));
        ] end else
      let sizes = (Si "size", output_list), [] in
      [
        Value_parameter
          (Si "nout",
           Code_value (code_out_natural nout));
        Struct_parameter
          (Si "outports", sizes);
      ]
    | Genr_utilities_pro.Size_array_expression ->
      let output_list = Genr_utilities_pro.size_array_of_int nout_int in
      let sizes = (Si "size", output_list), [] in
      [
        Value_parameter
          (Si "nout",
           Code_value (code_out_natural nout));
        Struct_parameter
          (Si "outports", sizes)
      ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
