(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_detect.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type_detect_change =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Detect Change")
    (Ast_types.mdl_path_of_string
       "simulink/Logic and Bit\nOperations/Detect\nChange")
;;

let mdl_block_type_detect_decrease =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Detect Decrease")
    (Ast_types.mdl_path_of_string
       "simulink/Logic and Bit\nOperations/Detect\nDecrease")
;;

let mdl_block_type_detect_increase =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Detect Increase")
    (Ast_types.mdl_path_of_string
       "simulink/Logic and Bit\nOperations/Detect\nIncrease")
;;

let vinit_ident = Ast_types.ident_from_string "vinit"
;;

let genr_detect_comparison op_name block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info  in

  let out_data_type =
    Genr_utilities_scicos_pro.get_simulink_out_data_type_value genr_info in

  let vinit = get_ident_string vinit_ident in

  let in_port_name = "in"
  and out_port_name = "out"
  and dollar_name = "dollar"
  and relational_name = "relational"
  and split_name = "split" in

  let split =
    Genr_utilities_scicos_pro.split_in_scsm0 split_name (_1_, _1_) in
  let in_port =
    in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_ (_4_, _0_) in
  let dollar =
    let nports = (_1_, _1_) in
    let scicos_block_type = Sbt "system/Dynamical/DiscreteDelay" in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 dollar_name
      (genr_instructions_for_modeler_block_type scicos_block_type
         [
           Value_parameter (Si "init_cond",
             Code_value (code_out_expression vinit));
           Value_parameter (Si "typ",
             Code_value
               Genr_utilities_scicos_pro.inherit_mode_type);
           Value_parameter (Si "externalActivation",
             Code_value
               Genr_utilities_scicos_pro.external_activation_mode_off);
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_2_, _0_) in

  let op = code_out_string op_name in

  let relational =
    let zcr = code_out_bool false in
    Genr_utilities_scicos_pro.
    relational_op_in_scsm0
      relational_name op zcr out_data_type
      Genr_utilities_scicos_pro.external_activation_mode_off
      (_3_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      in_port_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_1_ relational_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_2_ dollar_name port_1_ [];
    add_directed_link_in_scsm0
      dollar_name port_1_ relational_name port_2_ [];
    add_directed_link_in_scsm0
      relational_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    split;
    out_port;
    dollar;
    relational;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

let genr_detect_change = genr_detect_comparison "~=";;

let genr_detect_decrease = genr_detect_comparison "<";;

let genr_detect_increase = genr_detect_comparison ">";;

add_translator mdl_block_type_detect_change genr_detect_change;;

add_translator mdl_block_type_detect_decrease genr_detect_decrease;;

add_translator mdl_block_type_detect_increase genr_detect_increase;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
