(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_difference.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Difference")
    (Ast_types.mdl_path_of_string "simulink/Discrete/Difference")
;;

let ic_prev_input_ident =
  Ast_types.ident_from_string "ICPrevInput"
and initial_condition_ident =
  Ast_types.ident_from_string "InitialCondition"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let ic_prev_input =
    if Genr_utilities_scicos_pro.
       exists_argument genr_info ic_prev_input_ident
    then
      get_ident_string ic_prev_input_ident
    else
      Genr_utilities_scicos_pro.
      get_parameter_value_string
        genr_info initial_condition_ident in

  let scicos_block_parameters = [
    Value_parameter (Si "init_cond",
      Code_value (code_out_expression ic_prev_input));
    Value_parameter (Si "typ", Code_value (inherit_mode_type));
    Value_parameter (Si "externalActivation",
      Code_value external_activation_mode_off);
  ] in

  let in_port_name = "in"
  and out_port_name = "out"
  and dollar_name = "dollar"
  and summation_name = "summation"
  and split_name = "split" in

  let in_port =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in

  let split =
    Genr_utilities_scicos_pro.
    split_in_scsm0 split_name (_1_, _0_) in

  let dollar =
    let nports = (_1_, _1_) in
    let dollar_body =
      genr_instructions_for mdl_block_type
        scicos_block_parameters
        [] blk0 genr_info in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0
      dollar_name
      dollar_body
      nports (_2_, _0_) in

  let summation =
    let nports = (_2_, _1_) in
    let summation_body =
      genr_instructions_for_modeler_block_type
        (Sbt "system/MathOperations/Sum")
        [
          Value_parameter (Si "nin",
            Code_value (code_out_int 2));
          Struct_parameter (Si "in_ports",
            ((Si "sgn",
             [Code_value (code_out_string "+");
              Code_value (code_out_string "-")]),
             []));
          Value_parameter (Si "satur",
            Simulink_arg_saturation);
          Value_parameter (Si "externalActivation",
            Code_value external_activation_mode_off);
        ]
        [] blk0 genr_info in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0
      summation_name
      summation_body
      nports (_3_, _0_) in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_ (_4_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      in_port_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_1_ dollar_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_2_ summation_name port_1_ [];
    add_directed_link_in_scsm0
      dollar_name port_1_ summation_name port_2_ [];
    add_directed_link_in_scsm0
      summation_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    split;
    dollar;
    summation;
    out_port;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
