(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_digital_clock.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.DigitalClock
;;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in
  (* A source block inheriting should be always activated *)
  let sample_time = if sample_time = "-1" then "0" else sample_time in
  let sample_time = code_out_expression sample_time in

  let out_port_name = "out"
  and time_f_name = "time_f"
  and sample_clock_name = "sample_clock" in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_
      (_1_, _0_) in

  let time_f =
    Genr_utilities_scicos_pro.
    time_f_in_scsm0 time_f_name external_activation_mode_on
      (_0_, _0_) in

  let sample_clock =
    Genr_utilities_scicos_pro.
    sample_clock_in_scsm0
      sample_clock_name sample_time
      (_1_, _1_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      time_f_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ time_f_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    time_f;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
