(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_discrete_state_space.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.DiscreteStateSpace
;;

let a_ident = Ast_types.ident_from_string "A"
and b_ident = Ast_types.ident_from_string "B"
and c_ident = Ast_types.ident_from_string "C"
and d_ident = Ast_types.ident_from_string "D"
and x0_ident = Ast_types.ident_from_string "X0"
and initial_condition_ident =
  Ast_types.ident_from_string "InitialCondition"
;;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_block_parameters = [
    Value_parameter (Si "A", Simulink_arg_expression a_ident);
    Value_parameter (Si "B", Simulink_arg_expression b_ident);
    Value_parameter (Si "C", Simulink_arg_expression c_ident);
    Value_parameter (Si "D", Simulink_arg_expression d_ident);
    Value_parameter
      (Si "x0",
       Simulink_arg_expression_aliases
         [x0_ident; initial_condition_ident; ]);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list (scicos_basic_block_instructions block_id)

  | _ ->

    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and out_port_name = "out"
    and dlss_name = "dlss"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.in_port_in_scsm0 in_port_name port_1_
      (_0_, _0_) in
    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0 out_port_name port_1_
        (_2_, _0_) in
    let dlss =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0 dlss_name
        (scicos_basic_block_instructions blk0)
        nports
        (_1_, _0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ dlss_name port_1_ [];
      add_directed_link_in_scsm0 dlss_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ dlss_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      dlss;
      sample_clock;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
