(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_discrete_zero_pole.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.DiscreteZeroPole
;;

let zeros_ident = Ast_types.ident_from_string "Zeros"
and poles_ident = Ast_types.ident_from_string "Poles"
and gain_ident = Ast_types.ident_from_string "Gain"
;;


let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_parameters ext =
    [
    Value_parameter (Si "zer", Simulink_arg_expression zeros_ident);
    Value_parameter (Si "pol", Simulink_arg_expression poles_ident);
    Value_parameter (Si "k_gain", Simulink_arg_expression gain_ident);
    Value_parameter (Si "externalActivation", Code_value ext);
    ] in

  let scicos_basic_block_instructions blk_id ext =
    genr_instructions_for
      mdl_block_type (scicos_parameters ext)
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list (
      scicos_basic_block_instructions block_id external_activation_mode_off)

  | _ ->

    let in_port_name = "in"
    and out_port_name = "out"
    and dlr_name = "dlr"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.
      in_port_in_scsm0 in_port_name port_1_
      (_0_, _0_) in
    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0 out_port_name port_1_
      (_2_, _0_) in
    let dlr =
      let nports = (_1_, _1_) in
       Genr_utilities_scicos_pro.
      add_block_body_in_scsm0 dlr_name
        (scicos_basic_block_instructions blk0 external_activation_mode_on)
        nports
        (_1_, _0_) in
    let sample_clock =
      let sample_time =
        Genr_utilities_scicos_pro.get_sample_time_value genr_info in
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ dlr_name port_1_ [];
      add_directed_link_in_scsm0
        dlr_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ dlr_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      dlr;
      sample_clock;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
