(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_electrical_inductor.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

(** Simscape block [Inductor] *)

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Inductor")
    (Ast_types.mdl_path_of_string
       "fl_lib/Electrical/Electrical Elements/Inductor")
;;

let inductance_ident = Ast_types.ident_from_string "l"
and initial_current_ident = Ast_types.ident_from_string "i0"
;;

let genr block_id genr_info =

  let fixity_value, initial_current_value =
    if Genr_utilities_scicos_pro.
       exists_parameter genr_info initial_current_ident
    then
      Code_value (code_out_int 1),
      Simulink_arg_expression initial_current_ident
    else
      Code_value (code_out_int 0),
      Code_value (code_out_int 0) in

  let scicos_block_parameters = [
    Value_parameter (Si "L",
      Simulink_arg_expression inductance_ident);
    Struct_parameter (Si "i", (
      (Si "fixed", [ fixity_value; ]),
      [ Si "start", [ initial_current_value; ]]));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)

(* To account for internal resistance "r" and "g" (?), we should vreate a
  super-block ...

  let in_port_name = "in"
  and inertia_name = "inertia"
  and fixed_name = "fixed" in

  let in_port =
    Genr_utilities_scicos_pro.
    implicit_in_port_in_scsm0 in_port_name port_1_
    (_0_, _0_) in

  let inertia =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 inertia_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in

   let fixed =
      let nports = (_0_, _1_) in
      add_block_body_in_scsm0 fixed_name
        (genr_instructions_for_scicos_block_type
           (Sbt "Modelica 3.2.1/Mechanics/Rotational/Components/Fixed")
           [] [] blk0 genr_info)
        nports
        (_2_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      in_port_name port_r1_ inertia_name port_l1_ [];
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      inertia_name port_r1_ fixed_name port_r1_ [];
  ] in

  let super_body = [
    in_port;
    inertia;
    fixed;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
*)
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
