(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_electrical_mechanical_sources.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type_voltage_source =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Controlled Voltage\nSource")
    (Ast_types.mdl_path_of_string
       "fl_lib/Electrical/Electrical Sources/Controlled Voltage\nSource")
;;

let mdl_block_type_current_source =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Controlled Current\nSource")
    (Ast_types.mdl_path_of_string
       "fl_lib/Electrical/Electrical Sources/Controlled Current\nSource")
;;

let mdl_block_type_torque_source =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Ideal Torque Source")
    (Ast_types.mdl_path_of_string
       "fl_lib/Mechanical/Mechanical Sources/Ideal Torque Source")
;;

let port_l1_ = Genr_utilities_scicos_pro.port_l1_ Unatural._0_ _0_
and port_r1_ = Genr_utilities_scicos_pro.port_r1_ _0_ _0_
and port_l2_ = Genr_utilities_scicos_pro.port_l2_ _0_ _0_
;;

let genr_source_block mdl_block_type block_id genr_info =

  let scicos_block_parameters = [] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let out_port_name1 = "out1"
  and out_port_name2 = "out2"
  and in_port_name = "in"
  and source_name = "source" in

  let out_port1 =
    Genr_utilities_scicos_pro.
    implicit_out_port_in_scsm0 out_port_name1 port_1_
    (_2_, _0_) in

  let out_port2 =
    Genr_utilities_scicos_pro.
    implicit_out_port_in_scsm0 out_port_name2 port_2_
    (_2_, _1_) in

  let in_port =
    Genr_utilities_scicos_pro.
    implicit_in_port_in_scsm0 in_port_name port_1_
    (_0_, _0_) in

  let source =
    let nports = (_2_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 source_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      in_port_name port_r1_ source_name port_l1_ [];
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      source_name port_r1_ out_port_name2 port_l1_ [];
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      source_name port_l2_ out_port_name1 port_l1_ [];
  ] in

  let super_body = [
    in_port;
    source;
    out_port1;
    out_port2;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

let genr_voltage_source = genr_source_block mdl_block_type_voltage_source
and genr_current_source = genr_source_block mdl_block_type_current_source
and genr_torque_source = genr_source_block mdl_block_type_torque_source
;;

add_translator mdl_block_type_voltage_source genr_voltage_source;
add_translator mdl_block_type_current_source genr_current_source;
add_translator mdl_block_type_torque_source genr_torque_source
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
