(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_elementary_math.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.ElementaryMath
;;

let scicos_block_type =
  Genr_utilities_scicos_pro.modeler_block_type_for mdl_block_type
;;

let operator_ident = Ast_types.ident_from_string "Operator";;

let exp_block = Instructions.Sbt "system/MathOperations/Exponential"
and log_block = Sbt "system/MathOperations/Log"
and inv_block = Sbt "system/MatrixOperations/MatrixInverse"
and atan2_block = Sbt "system/MathOperations/Atan2"
;;

let euler_number = Genr_utilities_scicos_pro.code_out_expression "exp(1)"
and constant_10 = code_out_int 10
;;

let si_base = Si "base"
and si_external_activation = Si "externalActivation"
and si_fun = Si "func"
;;

let get_block_type_params operator =
  let no_activation =
    (si_external_activation, external_activation_mode_off) in
  match operator with
  | "Exp" -> exp_block, [ si_base, euler_number; no_activation; ]
  | "10^u" -> exp_block, [ si_base, constant_10; no_activation; ]
  | "Log" -> log_block, [ si_base, euler_number; no_activation; ]
  | "Log10" -> log_block, [ si_base, constant_10; no_activation; ]
  | "Reciprocal" -> inv_block, [ no_activation; ]
  | "atan2" -> atan2_block, [ no_activation; ]
  | _ -> scicos_block_type, [ si_fun, code_out_string operator]
;;

let genr block_id genr_info =

  let get_ident = Genr_utilities_scicos_pro.get_ident_string genr_info in

  let operator = get_ident operator_ident in

  let scicos_block_type, params = get_block_type_params operator in

  let instructions =

    let block_parameter_bindings =
      match params with
      | [] -> assert false
      | param :: params ->
        Ulist1.map
          Genr_utilities_scicos_pro.
          block_parameter_binding (param, params) in
    [
      Instantiate_block (block_id, scicos_block_type);
      Set_block_parameters (block_id, block_parameter_bindings);
    ] in

  Concat_list.list instructions
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
