(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_enable_event_gen.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

(** Generate a super block for a Simulink sub system block that is enabled,
  meaning that is has an EnablePort block in its sub system. *)

open Instructions
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_fake_block_types.enable_event_generator_block_type
;;

let scicos_block_parameters activation =
  [
    Genr_types.
    Value_parameter (Si "externalActivation",
      Genr_types.Code_value activation);
    Genr_types.
    Value_parameter (Si "nmod",
      Genr_types.Code_value (code_out_expression "0"));
  ]
;;

let genr block_id genr_info =

  let sample_time = Genr_utilities_scicos_pro.get_sample_time genr_info in

  let if_then_else_name = "If"
  and split_name = "split"
  and sample_clock_name = "sample_clock"

  and in_port_name = "in"
  and out_port_name = "out"
  and out_port_event_name = "out_event" in

  let in_port =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_name port_1_
      (Unatural._0_, _1_) in
  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_
      (_3_, _2_) in
  let out_port_event =
    Genr_utilities_scicos_pro.
    out_port_event_in_scsm0 out_port_event_name port_1_ (_2_, _0_) in
  let split =
    Genr_utilities_scicos_pro.split_in_scsm0
      split_name (_1_, _1_) in

  let enable_diagram activation blocks links =

    let scicos_block_instructions block_id =
      genr_instructions_for
        mdl_block_type (scicos_block_parameters activation)
        [] block_id genr_info in

    let if_then_else =
      let nports = (_1_, _0_) in
      Genr_utilities_scicos_pro.add_block_body_in_scsm0
        if_then_else_name
        (scicos_block_instructions blk0)
        nports
        (_2_, _1_) in

    let links =
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0 in_port_name port_1_ split_name port_1_ [] ::
      add_directed_link_in_scsm0 split_name port_1_ if_then_else_name port_1_ [] ::
      add_directed_link_in_scsm0 split_name port_2_ out_port_name port_1_ [] ::
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        if_then_else_name port_1_ out_port_event_name port_1_ [] ::
      links in

    let blocks =
      in_port :: out_port :: out_port_event ::
      split :: if_then_else :: blocks in

    blocks @ [links] in

  let super_body =
    match sample_time with
    | "-1" ->
      enable_diagram
        Genr_utilities_scicos_pro.external_activation_mode_off
        [] []
    | _ ->
      let sample_time = code_out_expression sample_time in
      let sample_clock =
        Genr_utilities_scicos_pro.
        sample_clock_in_scsm0
          sample_clock_name sample_time
          (_1_, _1_) in
      let sampled_link =
        Genr_utilities_scicos_pro.
        add_directed_link_in_scsm0 sample_clock_name port_1_
          if_then_else_name port_1_ [] in
      enable_diagram
        Genr_utilities_scicos_pro.external_activation_mode_on
        [sample_clock] [sampled_link] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
