(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_fcn.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.Fcn
;;

let expr_ident = Ast_types.ident_from_string "Expr";;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let expr =
    Genr_utilities_scicos_pro.
    get_ident_string genr_info expr_ident in

  let exx = Mbd_to_target.translate_string expr in

  let txt = Ast_types.mdl_text_of_string exx in

  let string_token =
    let txt =
       Ast_types.string_of_mdl_text (
         M_to_host.translate_expression txt) in
    Lib_strings.suppress_spaces txt in

  let scicos_block_parameters ext = [
    Value_parameter (Si "nin", Code_value (code_out_int 1));
    Value_parameter (Si "work", Code_value (code_out_expression "getcurrentenv"));
    Value_parameter (Si "exx", Code_value (code_out_string string_token));
    Value_parameter (Si "zc", Code_value (code_out_int 0));
    Value_parameter (Si "externalActivation", Code_value ext);
  ] in

  let scicos_basic_block_instructions blk_id ext =
    genr_instructions_for
      mdl_block_type (scicos_block_parameters ext)
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list (
      scicos_basic_block_instructions block_id external_activation_mode_off)
  | _ ->
    let sample_time = code_out_expression sample_time in

    let in_port_name = "in"
    and out_port_name = "out"
    and expression_name = "expression"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.
      in_port_in_scsm0
        in_port_name port_1_
        (Unatural._0_, Unatural._0_) in
    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0
        out_port_name port_1_
        (Unatural._2_, Unatural._0_) in
    let expression =
      let nports = (Unatural._1_, Unatural._1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0 expression_name
        (scicos_basic_block_instructions blk0 external_activation_mode_on)
        nports
        (Unatural._1_, Unatural._0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (Unatural._1_, Unatural._1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ expression_name port_1_ [];
      add_directed_link_in_scsm0
        expression_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ expression_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      expression;
      sample_clock;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
