(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_from_workspace.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.FromWorkspace
;;

let method_of_output_after_value = function
  | "Setting to zero" -> "Zero"
  | "Holding final value" -> "Hold"
  | "Cyclic repetition" -> "Repeat"
  | "Extrapolation" -> "Extrapolation"
  | s ->
    Say.warning (
      Printf.sprintf
        "genr_from_workspace: output method %s is not supported,\
         holding final value"
        s);
    "Hold"
;;

let variable_name_ident =
  Ast_types.ident_from_string "VariableName"
and interpolate_ident =
  Ast_types.ident_from_string "Interpolate"
and output_after_final_value_ident =
  Ast_types.ident_from_string "OutputAfterFinalValue"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let variable_name =
    code_out_string
      (get_ident_string variable_name_ident)
  and interpolate =
    if Genr_utilities_scicos_pro.
       get_ident_bool genr_info interpolate_ident
    then "Linear"
    else "ZeroOrder"
  and output_after_final_value =
    get_ident_string output_after_final_value_ident in

  let scicos_block_parameters ext = [
    Value_parameter (Si "varnam", Code_value variable_name);
    Value_parameter (Si "nout", Code_value (code_out_int 1));
    Value_parameter (Si "externalActivation", Code_value ext);
    Value_parameter (Si "outevent", Simulink_arg_zero_crossing);
    Value_parameter (Si "Method", Code_value (code_out_string interpolate));
    Value_parameter (Si "Rightside", Code_value
      (code_out_string
        (method_of_output_after_value output_after_final_value)));
  ] in

  let scicos_basic_block_instructions blk_id ext =
    genr_instructions_for
      mdl_block_type (scicos_block_parameters ext)
      [] blk_id genr_info in

  match sample_time with
  | "0" | "-1" ->
    Concat_list.list (
     scicos_basic_block_instructions block_id external_activation_mode_off)
  | _ ->
    let sample_time = code_out_expression sample_time in

    let out_port_name = "out"
    and from_workspace_name = "from_workspace"
    and sample_clock_name = "sample_clock" in

    let out_port =
      Genr_utilities_scicos_pro.out_port_in_scsm0
        out_port_name port_1_
        (_1_, _0_) in
    let from_workspace =
      let nports = (_0_, _1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0 from_workspace_name
        (scicos_basic_block_instructions blk0 external_activation_mode_on)
        nports
        (_0_, _0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_0_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        from_workspace_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ from_workspace_name port_1_ [];
    ] in

    let super_body = [
      out_port;
      from_workspace;
      sample_clock;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
