(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_function_call_generator.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Function-Call Generator")
    (Ast_types.mdl_path_of_string
      "simulink/Ports &\nSubsystems/Function-Call\nGenerator")
;;

let number_of_iterations_ident =
  Ast_types.ident_from_string "numberOfIterations"
;;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_block_parameters = [
    Value_parameter (Si "C",
      Code_value (code_out_expression "[]"));
    Value_parameter (Si "typ",
      Code_value (
        Genr_utilities_scicos_pro.inherit_mode_type));
    Value_parameter (Si "externalActivation",
      Code_value
        Genr_utilities_scicos_pro.external_activation_mode_on);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let number_of_iterations =
    get_ident_string genr_info number_of_iterations_ident in

  match number_of_iterations with
  | "1" ->

    let sample_time = code_out_expression sample_time in

    let out_port_name = "out"
    and const_name = "const"
    and sample_clock_name = "sample_clock" in

    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0 out_port_name port_1_
      (_2_, _0_) in
    let const =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0
        const_name
        (scicos_basic_block_instructions blk0)
        nports
        (_1_, _0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name sample_time
        (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        const_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ const_name port_1_ [];
    ] in

    let super_body = [
      sample_clock;
      const;
      out_port;
      links;
    ] in

    super_block_in_scsm0 block_id super_body

  | n ->
    Say.warning
      (Printf.sprintf
         "Function call generator Simulink block translation:\
       \n%s number of iterations required,\
       \nonly 1 iteration is supported yet."
         n);
    Genr_scicos_pro_not_yet_translated.genr block_id genr_info
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
