(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_if_action_event_gen.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

(** Generate a super block for a Simulink sub system block that is triggered,
  meaning that is has an TriggerPort block in its sub system. *)

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_fake_block_types.if_action_event_generator_block_type
;;

let scicos_block_parameters =
  [
    Genr_types.
    Value_parameter (Si "externalActivation",
    Code_value Genr_utilities_scicos_pro.external_activation_mode_off);
    Genr_types.
    Value_parameter (Si "nmod",
    Code_value (code_out_expression "0"));
  ]
;;

let genr block_id genr_info =

  let if_then_else_name = "If" in

  let in_port_name = "in"
  and out_port_event_name = "out_event" in

  let in_port =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_name port_1_ (Unatural._0_, Unatural._1_) in
  let out_port_event =
    Genr_utilities_scicos_pro.
    out_port_event_in_scsm0 out_port_event_name port_1_ (_1_, _0_) in

  let scicos_block_instructions block_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] block_id genr_info in

  let if_then_else =
    let nports = (_1_, _0_) in
    Genr_utilities_scicos_pro.add_block_body_in_scsm0
      if_then_else_name
      (scicos_block_instructions blk0)
      nports
      (_1_, _1_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      in_port_name port_1_ if_then_else_name port_1_ [];
    Genr_utilities_scicos_pro.
    add_event_link_in_scsm0
      if_then_else_name port_1_ out_port_event_name port_1_ [];
  ] in


  let super_body = [
    in_port;
    out_port_event;
    if_then_else;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
