(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_initial_condition.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

(* Translating Simulink blocs "InitialCondition" *)

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.InitialCondition
;;

let const_ident = Ast_types.ident_from_string "Value"
;;

let genr block_id genr_info =

  let scicos_block_parameters = [
    Value_parameter (Si "typ", Code_value inherit_mode_type);
    Value_parameter (Si "nin", Code_value (code_out_int 2));
    Value_parameter (Si "externalActivation",
      Code_value
        Genr_utilities_scicos_pro.external_activation_mode_off);
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let in_port_name = "in"
  and out_port_name = "out"
  and constant_name = "constant"
  and sample_clock_name = "sample_clock"
  and selector_name = "selector" in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in
  let sample_time = code_out_expression sample_time in

  let in_port =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0
      in_port_name port_1_
      (_0_, _0_) in
  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0
      out_port_name port_1_
      (_3_, _0_) in
  let constant =
    let nports = (_0_, _1_) in
    let scicos_block_type = Sbt "system/SignalGenerators/Constant" in
    let constant_body =
      genr_instructions_for_modeler_block_type scicos_block_type
        [
          Value_parameter (Si "C", Simulink_arg_expression const_ident);
          Value_parameter (Si "typ", Code_value (
            Genr_utilities_scicos_pro.inherit_mode_type));
          Value_parameter (Si "externalActivation", Code_value
            Genr_utilities_scicos_pro.external_activation_mode_on);
        ] [] Genr_utils.blk0 genr_info in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 constant_name
      constant_body
      nports
      (_1_, _1_) in

  let selector =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 selector_name
      (scicos_basic_block_instructions blk0)
      nports
      (_2_, _0_) in
  let sample_clock =
    Genr_utilities_scicos_pro.
    sample_clock_in_scsm0
      sample_clock_name sample_time
      (_1_, _2_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      constant_name port_1_ selector_name port_1_ [];
    add_directed_link_in_scsm0
      in_port_name port_1_ selector_name port_2_ [];
    add_directed_link_in_scsm0
      selector_name port_1_ out_port_name port_1_ [];
    Genr_utilities_scicos_pro.
    add_event_link_in_scsm0
      sample_clock_name port_1_ constant_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    out_port;
    constant;
    selector;
    sample_clock;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
