(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_interval_test.ml,v 1.1 2019-05-10 14:04:57 jpc Exp $ *)

(*open Unatural*)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Interval Test")
    (Ast_types.mdl_path_of_string
       "simulink/Logic and Bit\nOperations/Interval Test")
;;

let lower_limit_ident =
  Ast_types.ident_from_string "lowlimit"
and upper_limit_ident =
  Ast_types.ident_from_string "uplimit"
and interval_closed_left_ident =
  Ast_types.ident_from_string "IntervalClosedLeft"
and interval_closed_right_ident =
  Ast_types.ident_from_string "IntervalClosedRight";;

let genr block_id genr_info =

  let out_data_type =
    Genr_utilities_scicos_pro.
    get_simulink_out_data_type_value genr_info in

  let scicos_block_parameters op lim = [
    Value_parameter (Si "op",
      Code_value (code_out_string op));
    Value_parameter (Si "C", lim);
    Value_parameter (Si "zcr",
      Code_value (code_out_bool false));
  ] in

  let scicos_basic_block_instructions blk_id op lim =
    genr_instructions_for
      mdl_block_type (scicos_block_parameters op lim)
      [] blk_id genr_info in

  let in_port_name = "in"
  and out_port_name = "out"
  and logical_op_name = "and"
  and comp1_name = "comp1"
  and comp2_name = "comp2"
  and split_name = "split" in

  let in_port =
    in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in

  let split_block =
    Genr_utilities_scicos_pro.
    split_in_scsm0 split_name (_1_, _0_) in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_ (_4_, _0_) in

  let comp1_block =
    let op =
      if Genr_utilities_scicos_pro.
         get_ident_bool genr_info interval_closed_left_ident
      then ">=" else ">" in
    let comp1_ports = (_1_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 comp1_name
      (scicos_basic_block_instructions blk0 op
         (Simulink_arg_expression lower_limit_ident))
      comp1_ports (_2_, _1_) in

  let comp2_block =
    let op =
      if Genr_utilities_scicos_pro.
         get_ident_bool genr_info interval_closed_right_ident
      then "<=" else "<" in
    let comp2_ports = (_1_, _1_) in
    let comp2_block_body =
      scicos_basic_block_instructions blk0 op
        (Simulink_arg_expression upper_limit_ident) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 comp2_name
      comp2_block_body
      comp2_ports (_2_, _0_) in

  let logical_op_block =
    let logical_op_type = Sbt "system/LogicalOperations/LogicalOp" in
    let logical_op_parameters = [
      Value_parameter (Si "nin",
        Code_value (code_out_int 2));
      Value_parameter (Si "rule",
        Code_value (code_out_string "AND"));
      Value_parameter (Si "Datatype",
        Code_value out_data_type);
      Value_parameter (Si "tp",
        Code_value (code_out_int 0));
      Value_parameter (Si "externalActivation",
        Code_value
          Genr_utilities_scicos_pro.external_activation_mode_off);
    ] in

    let logical_op_ports = (_2_, _1_) in
    let logical_op_block_body =
      genr_instructions_for_modeler_block_type
        logical_op_type
        logical_op_parameters
        [] Genr_utils.blk0 genr_info in

    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 logical_op_name
      logical_op_block_body
      logical_op_ports
      (_3_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      in_port_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_1_ comp1_name port_1_ [];
    add_directed_link_in_scsm0
      split_name port_2_ comp2_name port_1_ [];
    add_directed_link_in_scsm0
      comp1_name port_1_ logical_op_name port_1_ [];
    add_directed_link_in_scsm0
      comp2_name port_1_ logical_op_name port_2_ [];
    add_directed_link_in_scsm0
      logical_op_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    split_block;
    out_port;
    comp1_block;
    comp2_block;
    logical_op_block;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
