(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_mechanical_rotational_inertia.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Inertia")
    (Ast_types.mdl_path_of_string
       "fl_lib/Mechanical/Rotational Elements/Inertia")
;;

let inertia_ident = Ast_types.ident_from_string "inertia"
and initial_velocity_ident = Ast_types.ident_from_string "initial_velocity"
and port_l1_ = Genr_utilities_scicos_pro.port_l1_ Unatural._0_ _0_
and port_r1_ = Genr_utilities_scicos_pro.port_r1_ _0_ _0_
;;

let genr block_id genr_info =

  let fixity_value, initial_velocity_value =
    if Genr_utilities_scicos_pro.
       exists_parameter genr_info initial_velocity_ident then
      Code_value (code_out_int 1),
      Simulink_arg_expression initial_velocity_ident else
      Code_value (code_out_int 0),
      Code_value (code_out_int 0) in

  let scicos_block_parameters = [
    Value_parameter (Si "J",
      Simulink_arg_expression inertia_ident);
    Struct_parameter (Si "w", (
      (Si "fixed", [ fixity_value; ]),
      [ Si "start", [ initial_velocity_value; ]]));
  ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let in_port_name = "in"
  and inertia_name = "inertia" in

  let in_port =
    Genr_utilities_scicos_pro.
    implicit_in_port_in_scsm0 in_port_name port_1_
    (_0_, _0_) in

  let inertia =
    let nports = (_1_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 inertia_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      in_port_name port_r1_ inertia_name port_l1_ [];
  ] in

  let super_body = [
    in_port;
    inertia;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
