(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_mechanical_rotational_reference.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_path =
  "fl_lib/Mechanical/Rotational Elements/Mechanical\nRotational Reference"
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string
       "Mechanical\nRotational Reference")
    (Ast_types.mdl_path_of_string mdl_block_path)
;;

let port_r1_ = Genr_utilities_scicos_pro.port_r1_ _0_ _0_
;;

let genr block_id genr_info =

  let scicos_block_parameters = [] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let in_port_name = "in"
  and fixed_name = "fixed" in

  let in_port =
    Genr_utilities_scicos_pro.
    implicit_in_port_in_scsm0 in_port_name port_1_
    (_0_, _0_) in

  let fixed =
    let nports = (_0_, _1_) in
    add_block_body_in_scsm0 fixed_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_connection_link_in_scsm0
      in_port_name port_r1_ fixed_name port_r1_ [];
  ] in

  let super_body = [
    in_port;
    fixed;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
