(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_merge.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.Merge
;;

let inputs_ident =
  Ast_types.ident_from_string "Inputs"
and initial_output_ident =
  Ast_types.ident_from_string "InitialOutput"
and allow_unequal_input_port_widths_ident =
  Ast_types.ident_from_string "AllowUnequalInputPortWidths"
and input_port_offsets_ident =
  Ast_types.ident_from_string "InputPortOffsets"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let inputs = get_ident_string inputs_ident
  and initial_output = get_ident_string initial_output_ident
  and allow_unequal_input_port_widths =
    Genr_utilities_scicos_pro.get_ident_bool genr_info
    allow_unequal_input_port_widths_ident
  and input_port_offsets = get_ident_string input_port_offsets_ident in

  if initial_output <> "[]" then
    Say.warning (Printf.sprintf
      "Merge block translation: initial output is not supported");
  if allow_unequal_input_port_widths then
    Say.warning (Printf.sprintf
      "Merge block translation: \
       AllowUnequalInputPortWidths is not supported");
  if input_port_offsets <> "[]" then
    Say.warning (Printf.sprintf
      "Merge block translation: InputPortOffsets is not supported");

  let scicos_block_parameters = [
    Value_parameter (
      Si "typ", Code_value Genr_utilities_scicos_pro.inherit_mode_type);
    Value_parameter (
      Si "nin", Code_value (code_out_expression inputs));
    Value_parameter (
      Si "externalActivation", Code_value external_activation_mode_off)
  ] in
  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
