(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_mux.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Genr_utilities_pro
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.Mux
;;

let inputs_ident =
  Ast_types.ident_from_string "Inputs"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let ports_desc = genr_info.Genr_block_types.block_ports_description in
  let nin =
    Mdl_ast_to_deep_ast.get_regular_port_nin ports_desc in

   let nin_int = Unatural.to_int nin in

  let inputs = get_ident_string inputs_ident in

  let input_sizes = Genr_utilities_pro.get_size_array "Mux" inputs in

  let scicos_block_parameters =
    match input_sizes with
    | Size_array_vector input_list ->
      let nip = List.length input_list in
      if nip <> nin_int then begin
        Say.warning (Printf.sprintf
          "Mux parameter ``%d'' does not match port number ``%d''"
          nip nin_int);
        [
          Value_parameter
            (Si "nin",
             Code_value (code_out_natural nin));
        ] end else
      let sizes = (Si "size", input_list), [] in
      [
        Value_parameter
          (Si "nin",
           Code_value (code_out_natural nin));
        Struct_parameter
          (Si "inports", sizes);
      ]
    | Size_array_expression ->
      let input_list = size_array_of_int nin_int in
      let sizes = (Si "size", input_list), [] in
      [
        Value_parameter
          (Si "nin",
           Code_value (code_out_natural nin));
        Struct_parameter
          (Si "inports", sizes)
      ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  Concat_list.list (scicos_basic_block_instructions block_id)
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
