(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_repeating_sequence_stair.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Repeating Sequence Stair")
    (Ast_types.mdl_path_of_string
       "simulink/Sources/Repeating\nSequence\nStair")
;;

(*
type context =
  | Context_code of matlab_script
  | Context_bindings of scicos_block_parameter_bindings
  | Context_commented of scicos_block_parameter_bindings
;;

Intruction is:

let out_values = get_ident out_values_ident in

let script =
  let script_text =
    Printf.sprintf "OutValues=%s" out_values in
  Instructions.Matlab_script (mdl_text_of_string script_text) in

let context = Context_code script in

Set_diagram_context (host_diagram_ident, context)
*)

let out_values_ident =
  Ast_types.ident_from_string "OutValues"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let out_values = get_ident_string out_values_ident in

  let out_port_name = "out"
  and shift_register_name = "shift_register"
  and sample_clock_name = "sample_clock"
  and split_name = "split" in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let out_data_type = get_simulink_out_data_type_value genr_info in

  let sample_time =
    match sample_time with
    | "-1" ->  "0"
    | sample_time -> sample_time in

  let script =
    let script_text =
      Printf.sprintf "OutValues=%s" out_values in
    Instructions.Matlab_script (Ast_types.mdl_text_of_string script_text) in

  let context = Context_code script in

  let shift_register =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 shift_register_name
      (genr_instructions_for mdl_block_type
         [
           Value_parameter (Si "K",
             Code_value (code_out_expression "length(OutValues)"));
           Value_parameter (Si "z0",
             Code_value (code_out_expression "OutValues(:)\'"));
           Value_parameter (Si "type",
             Code_value out_data_type);
           Value_parameter
             (Si "externalActivation",
              Code_value external_activation_mode_on);
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_1_, _1_) in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0
      out_port_name port_1_ (_3_, _0_) in

  let sample_clock =
    Genr_utilities_scicos_pro.
    sample_clock_in_scsm0
      sample_clock_name
      (code_out_expression sample_time) (_1_, _2_) in

  let split =
    Genr_utilities_scicos_pro.split_in_scsm0
      split_name (_2_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      shift_register_name port_1_ split_name port_1_ [];
    add_directed_link_in_scsm0 split_name port_1_ out_port_name port_1_ [];
    add_directed_link_in_scsm0 split_name port_2_ shift_register_name port_1_ [];
    Genr_utilities_scicos_pro.
    add_event_link_in_scsm0
      sample_clock_name port_1_ shift_register_name port_1_ [];
  ] in

  let super_body = [
    [Set_diagram_context (scsm0, context)];
    out_port;
    sample_clock;
    shift_register;
    split;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
