(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_repeating_table.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Repeating table")
    (Ast_types.mdl_path_of_string "simulink/Sources/Repeating\nSequence")
;;

let rep_seq_t_ident =
  Ast_types.ident_from_string "rep_seq_t"
and rep_seq_y_ident =
  Ast_types.ident_from_string "rep_seq_y"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in
  let meth = code_out_string "Linear"
  and xx = code_out_expression (get_ident_string rep_seq_t_ident)
  and yy = code_out_expression (get_ident_string rep_seq_y_ident)
  and periodic_option = code_out_string "Repeat" in

  let out_port_name = "out"
  and curv_c_name = "curv_c" in

  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_ (_1_, _0_) in
  let curv_c =
    let nports = (_0_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 curv_c_name
      (genr_instructions_for mdl_block_type
         [
           Value_parameter (Si "xx", Code_value xx);
           Value_parameter (Si "yy", Code_value yy);
           Value_parameter (Si "typ",
             Code_value (code_out_string "double"));
           Value_parameter (Si "outevent",
             Code_value (code_out_int 0));
           Value_parameter (Si "externalActivation",
             Code_value external_activation_mode_off);
           Value_parameter (Si "Method",   Code_value meth);
           Value_parameter (Si "Leftside", Code_value periodic_option);
           Value_parameter (Si "Rightside", Code_value periodic_option);
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_0_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0 curv_c_name port_1_ out_port_name port_1_ [];
(*    Genr_utilities_scicos_pro.add_event_link_in_scsm0
      curv_c_name port_1_ curv_c_name port_1_ [];  *)
  ] in

  let super_body = [
    out_port;
    curv_c;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
