(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_saturation_dynamic.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  simulink_block_reference_type
    (Ast_types.
     mdl_name_of_string "Saturation Dynamic")
    (Ast_types.
     mdl_path_of_string "simulink/Discontinuities/Saturation\nDynamic")
;;

let genr block_id genr_info =

  let scicos_block_parameters = [ ] in

  let scicos_basic_block_instructions blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  let out_port_name = "out"
  and satur_name = "satur"
  and in_port_up_name = "up"
  and in_port_name = "u"
  and in_port_low_name = "lo" in

  let in_port_low =
    Genr_utilities_scicos_pro.
    in_port_in_scsm0 in_port_up_name port_3_ (_0_, _0_) in
  let in_port =
    in_port_in_scsm0 in_port_name port_2_ (_0_, _1_) in
  let in_port_up =
    in_port_in_scsm0 in_port_low_name port_1_ (_0_, _2_) in
  let out_port =
    Genr_utilities_scicos_pro.
    out_port_in_scsm0 out_port_name port_1_ (_2_, _1_) in
  let satur_block =
    let nports = (_3_, _1_) in
    Genr_utilities_scicos_pro.
    add_block_body_in_scsm0 satur_name
      (scicos_basic_block_instructions blk0)
      nports
      (_1_, _1_) in

  let links = [
    Genr_utilities_scicos_pro.
    add_directed_link_in_scsm0
      in_port_low_name port_1_ satur_name port_1_ [];
    add_directed_link_in_scsm0
      in_port_name port_1_ satur_name port_2_ [];
    add_directed_link_in_scsm0
      in_port_up_name port_1_ satur_name port_3_ [];
    add_directed_link_in_scsm0
      satur_name port_1_ out_port_name port_1_ [];
  ] in

  let super_body = [
    out_port;
    in_port_low;
    in_port_up;
    in_port;
    satur_block;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
