(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_stop.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.Stop
;;

let genr block_id genr_info =

  let in_name = "in"
  and absi_name = "abs_value"
  and sum_name = "sum"
  and if_then_else_name = "if_then_else"
  and end_c_name = "end" in

  let zcr = code_out_int 0  in

  let in_port =
    Genr_utilities_scicos_pro.in_port_in_scsm0
      in_name port_1_ (_0_, _1_) in
  let absi =
    let scicos_block_type = Sbt "system/MathOperations/Abs" in
    let nports = (_1_, _1_) in
    Genr_utilities_scicos_pro.add_block_body_in_scsm0
      absi_name
      (genr_instructions_for_modeler_block_type scicos_block_type
         [
           Value_parameter (Si "zcross", Code_value zcr);
           Value_parameter
             (Si "overflow", Simulink_arg_saturation_set saturation_mode_off);
           Value_parameter
            (Si "externalActivation",
             Code_value (external_activation_mode_off));
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_1_, _1_) in

  let sum =
    let inports = [ Code_value (code_out_string "+"); ] in
    let inports = (Si "sgn", inports), [] in
    let nports = (_1_, _1_) in
    Genr_utilities_scicos_pro.add_block_body_in_scsm0
      sum_name
      (genr_instructions_for_modeler_block_type
         (Sbt "system/MathOperations/Sum")
         [
           Value_parameter (Si "nin", Code_value (code_out_int 1));
           Struct_parameter (Si "in_ports", inports);
           Value_parameter
             (Si "satur", Simulink_arg_saturation_set saturation_mode_off);
           Value_parameter
             (Si "externalActivation",
              Code_value (external_activation_mode_off));
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_2_, _1_) in

  let if_then_else =
    let scicos_block_type = Sbt "system/ActivationOperations/IfThenElse" in
    let nports = (_1_, _0_) in
    Genr_utilities_scicos_pro.add_block_body_in_scsm0
      if_then_else_name
      (genr_instructions_for_modeler_block_type scicos_block_type
         [
           Value_parameter (Si "externalActivation",
           Code_value (external_activation_mode_off));
           Value_parameter (Si "nmod", Code_value zcr);
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_3_, _1_) in

  let end_c =
    let nports = (_0_, _0_) in
    Genr_utilities_scicos_pro.add_block_body_in_scsm0
      end_c_name
      (genr_instructions_for mdl_block_type
         [
           Value_parameter
             (Si "job", Code_value (code_out_string "Stop"));
           Value_parameter
             (Si "bmessage", Code_value (code_out_string " "));
         ]
         [] Genr_utils.blk0 genr_info)
      nports
      (_2_, _0_) in

  let links = [
    Genr_utilities_scicos_pro.add_directed_link_in_scsm0
      in_name port_1_ absi_name port_1_ [];
    add_directed_link_in_scsm0
      absi_name port_1_ sum_name port_1_ [];
    add_directed_link_in_scsm0
      sum_name port_1_ if_then_else_name port_1_ [];
    add_event_link_in_scsm0
      if_then_else_name port_1_ end_c_name port_1_ [];
  ] in

  let super_body = [
    in_port;
    absi;
    sum;
    if_then_else;
    end_c;
    links;
  ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
