(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_to_workspace.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.Explicit_block_regular_type
    Mdl_block_types.ToWorkspace
;;

let variable_name_ident =
  Ast_types.ident_from_string "VariableName"
and max_data_points_ident =
  Ast_types.ident_from_string "MaxDataPoints"
and decimation_ident =
  Ast_types.ident_from_string "Decimation"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let variable_name =
    get_ident_string variable_name_ident in

  let decimation_string =
    get_ident_string decimation_ident in

  if decimation_string <> "1" then
    Say.warning (Printf.sprintf
      "genr_to_workspace: decimation factor ``%s'' not supported, set to 1"
      decimation_string);

  let scicos_block_parameters ext = [
    Value_parameter (Si "nin",
      Code_value (code_out_int 1));
    Value_parameter (Si "varname",
      Code_value (code_out_string variable_name));
    Value_parameter (Si "externalActivation",
      Code_value ext);
    Value_parameter (Si "buffsize",
      Simulink_arg_expression max_data_points_ident);
  ] in

  let scicos_basic_block_instructions scicos_block_parameters blk_id ext =
    genr_instructions_for
      mdl_block_type (scicos_block_parameters ext)
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list
      (scicos_basic_block_instructions
         scicos_block_parameters
         block_id external_activation_mode_off)

  | sample_time ->

    let in_port_name = "in"
    and to_workspace_name = "to_workspace"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.
      in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in
    let to_workspace =
      let nports = (_1_, _0_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0
        to_workspace_name
        (scicos_basic_block_instructions
           scicos_block_parameters
           blk0 external_activation_mode_on)
        nports
        (_1_, _0_) in
    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name
        (code_out_expression sample_time) (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ to_workspace_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ to_workspace_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      to_workspace;
      sample_clock;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
