(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_unary_minus.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

(* The native name of the Simulink block to translate. *)
let mdl_block_type_native =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.UnaryMinus
;;

(* The alias reference name of the Simulink block to translate. *)
let mdl_block_type_reference =
  simulink_block_reference_type
    (Ast_types.mdl_name_of_string "Unary Minus")
    (Ast_types.mdl_path_of_string "simulink/Math\nOperations/Unary Minus")
;;

let genr block_id genr_info =

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let scicos_block_parameters activation = [
    Value_parameter (Si "overflow",
      Simulink_arg_saturation_set saturation_mode_off);
    Value_parameter (Si "externalActivation",
      Code_value activation);
  ] in

  let scicos_basic_block_instructions blk_id activation =
    genr_instructions_for
      mdl_block_type_native
      (scicos_block_parameters activation)
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.list
      (scicos_basic_block_instructions
         block_id
         Genr_utilities_scicos_pro.external_activation_mode_off)

  | sample_time ->

    let in_port_name = "in"
    and out_port_name = "out"
    and unary_minus_name = "unary_minus"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.in_port_in_scsm0
        in_port_name port_1_
        (_0_, _0_) in

    let out_port =
      Genr_utilities_scicos_pro.out_port_in_scsm0
        out_port_name port_1_
        (_2_, _0_) in

    let unary_minus =
      let nports = (_1_, _1_) in
      Genr_utilities_scicos_pro.add_block_body_in_scsm0
        unary_minus_name
        (scicos_basic_block_instructions
           block_id
           Genr_utilities_scicos_pro.external_activation_mode_on)
        nports
        (_1_, _0_) in

    let sample_clock =
      Genr_utilities_scicos_pro.sample_clock_in_scsm0
        sample_clock_name
        (code_out_expression sample_time)
        (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.add_directed_link_in_scsm0
        in_port_name port_1_ unary_minus_name port_1_ [];
      add_directed_link_in_scsm0
        unary_minus_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.add_event_link_in_scsm0
        sample_clock_name port_1_ unary_minus_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      unary_minus;
      sample_clock;
      links;
    ] in

  super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type_native genr
;;

add_translator mdl_block_type_reference genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
