(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_scicos_pro_unit_delay.ml,v 1.1 2019-05-10 14:04:58 jpc Exp $ *)

(* Defensive programming: defining an empty Genr_utilities_scicos module.
  (See genr_scicos_pro_compare_to_constant.ml for more information.) *)
module Genr_utilities_scicos = struct end;;

open Instructions
open Genr_types
open Genr_utilities_scicos_pro
open Unatural
;;

let mdl_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_regular_type Mdl_block_types.UnitDelay
;;

let x0_ident =
  Ast_types.ident_from_string "X0"
and initial_condition_ident =
  Ast_types.ident_from_string "InitialCondition"
;;

let genr block_id genr_info =

  let get_ident_string =
    Genr_utilities_scicos_pro.get_ident_string genr_info in

  let sample_time =
    Genr_utilities_scicos_pro.get_sample_time genr_info in

  let x0 =
    if Genr_utilities_scicos_pro.
       exists_argument genr_info x0_ident
    then get_ident_string x0_ident
    else Genr_utilities_scicos_pro.
         get_parameter_value_string genr_info initial_condition_ident in

  let scicos_block_parameters ext = [
    Value_parameter (Si "init_cond",
      Code_value (code_out_expression x0));
    Value_parameter (Si "typ",
      Code_value (inherit_mode_type));
    Value_parameter (Si "externalActivation",
      Code_value ext)
  ] in

  let scicos_basic_block_instructions scicos_block_parameters blk_id =
    genr_instructions_for
      mdl_block_type scicos_block_parameters
      [] blk_id genr_info in

  match sample_time with
  | "-1" ->
    Concat_list.
    list
      (scicos_basic_block_instructions
         (scicos_block_parameters external_activation_mode_off)
         block_id)

  | sample_time ->

    let in_port_name = "in"
    and out_port_name = "out"
    and dollar_name = "dollar"
    and sample_clock_name = "sample_clock" in

    let in_port =
      Genr_utilities_scicos_pro.
      in_port_in_scsm0 in_port_name port_1_ (_0_, _0_) in

    let out_port =
      Genr_utilities_scicos_pro.
      out_port_in_scsm0 out_port_name port_1_ (_2_, _0_) in

    let dollar_block =
      let dollar_ports = (_1_, _1_) in
      Genr_utilities_scicos_pro.
      add_block_body_in_scsm0
        dollar_name
        (scicos_basic_block_instructions
           (scicos_block_parameters external_activation_mode_on)
           blk0)
        dollar_ports
        (_1_, _0_) in

    let sample_clock =
      Genr_utilities_scicos_pro.
      sample_clock_in_scsm0
        sample_clock_name
        (code_out_expression sample_time) (_1_, _1_) in

    let links = [
      Genr_utilities_scicos_pro.
      add_directed_link_in_scsm0
        in_port_name port_1_ dollar_name port_1_ [];
      add_directed_link_in_scsm0
        dollar_name port_1_ out_port_name port_1_ [];
      Genr_utilities_scicos_pro.
      add_event_link_in_scsm0
        sample_clock_name port_1_ dollar_name port_1_ [];
    ] in

    let super_body = [
      in_port;
      out_port;
      dollar_block;
      sample_clock;
      links;
    ] in

    super_block_in_scsm0 block_id super_body
;;

add_translator mdl_block_type genr
;;

(*
 Local Variables:
  compile-command: "cd ../../../..; make"
  End:
*)
