(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_fake_block_types.ml,v 1.1 2019-05-10 14:04:01 jpc Exp $ *)

(** Some fake Simulink block explicit types. *)

let split_block_reference_type =
  let split_block_name =
    Ast_types.mdl_name_of_string "Split" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    split_block_name
    (Ast_types.mdl_path_of_string
       "system/Links/Split")
;;

let split_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type split_block_reference_type
;;

let split_connection_block_reference_type =
  let split_connection_block_name =
    Ast_types.mdl_name_of_string "CSplit" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    split_connection_block_name
    (Ast_types.mdl_path_of_string
       "system/Links/CSplit")
;;

let split_connection_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type split_connection_block_reference_type
;;

let s_function_stateflow_block_reference_type =
  let s_function_stateflow_block_name =
    Ast_types.mdl_name_of_string "S_function_stateflow" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    s_function_stateflow_block_name
    (Ast_types.mdl_path_of_string
       "simport/genr_block/scicos/genr_s_function_stateflow")
;;

let s_function_stateflow_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type s_function_stateflow_block_reference_type
;;

let matlab_function_block_reference_type =
  let matlab_function_block_name =
    Ast_types.mdl_name_of_string "Matlab_Function" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    matlab_function_block_name
    (Ast_types.mdl_path_of_string
       "simport/genr_block/scicos/genr_matlab_function")
;;

let matlab_function_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type matlab_function_block_reference_type
;;

let atomic_port_block_reference_type =
  let atomic_port_block_name =
    Ast_types.mdl_name_of_string "Atomic_port" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    atomic_port_block_name
    (Ast_types.mdl_path_of_string
       "system/Ports/Atomic_port")
;;

let atomic_port_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type atomic_port_block_reference_type
;;

let not_yet_translated_block_type =
  let not_yet_translated_block_reference_type =
    let not_yet_translated_block_name =
      Ast_types.mdl_name_of_string "Not Yet Translated Block" in
    Mdl_deep_basics.make_deep_block_reference_type
      not_yet_translated_block_name
      (Ast_types.mdl_path_of_string
        "simport/src/mdl/genr_block/scicos/genr_not_yet_translated") in
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    not_yet_translated_block_reference_type
;;

let not_yet_translated_block_mark =
  Configuration.get_not_yet_translated_mark ()
;;

let atomic_event_generator_block_reference_type =
  let atomic_event_generator_block_name =
    Ast_types.mdl_name_of_string "AtomicEventGen" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    atomic_event_generator_block_name
    (Ast_types.mdl_path_of_string
       "simport/src/mdl/genr_block/scicos/genr_atomic")
;;

let atomic_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type atomic_event_generator_block_reference_type
;;

let enable_event_generator_block_reference_type =
  let enable_event_generator_block_name =
    Ast_types.mdl_name_of_string "EnableEventGen" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    enable_event_generator_block_name
    (Ast_types.mdl_path_of_string
       "simport/src/mdl/genr_block/scicos/genr_enable")
;;

let enable_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    enable_event_generator_block_reference_type
;;

let if_action_event_generator_block_reference_type =
  let if_action_event_generator_block_name =
    Ast_types.mdl_name_of_string "If_actionEventGen" in
  Mdl_deep_basics.
  make_deep_block_reference_type
    if_action_event_generator_block_name
    (Ast_types.mdl_path_of_string
       "simport/src/mdl/genr_block/scicos/genr_if_action")
;;

let if_action_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    if_action_event_generator_block_reference_type
;;

let trigger_event_generator_block_reference_type_gen opt =
  let trigger_event_generator_block_name =
    Ast_types.
    mdl_name_of_string (Printf.sprintf
      "Trigger %s"
      opt) in
  Mdl_deep_basics.
  make_deep_block_reference_type
    trigger_event_generator_block_name
    (Ast_types.mdl_path_of_string
       "simport/src/mdl/genr_block/scicos/genr_trigger")
;;

let trigger_and_enable_event_generator_block_reference_type_gen opt =
  let opt = "and Enable " ^ opt in
  trigger_event_generator_block_reference_type_gen opt
;;

let trigger_either_event_generator_block_reference_type =
  trigger_event_generator_block_reference_type_gen "either"
and trigger_falling_event_generator_block_reference_type =
  trigger_event_generator_block_reference_type_gen "falling"
and trigger_rising_event_generator_block_reference_type =
  trigger_event_generator_block_reference_type_gen "rising"
and trigger_function_call_generator_block_reference_type =
  trigger_event_generator_block_reference_type_gen "function-call"
;;

let trigger_either_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    trigger_either_event_generator_block_reference_type
and trigger_falling_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    trigger_falling_event_generator_block_reference_type
and trigger_rising_event_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    trigger_rising_event_generator_block_reference_type
and trigger_function_call_generator_block_type =
  Mdl_basics_explicit_ast.
  Explicit_block_reference_type
    trigger_function_call_generator_block_reference_type
;;

let trigger_and_enable_either_event_generator_block_reference_type =
  trigger_and_enable_event_generator_block_reference_type_gen "either"
and trigger_and_enable_falling_event_generator_block_reference_type =
  trigger_and_enable_event_generator_block_reference_type_gen "falling"
and trigger_and_enable_rising_event_generator_block_reference_type =
  trigger_and_enable_event_generator_block_reference_type_gen "rising"
and trigger_and_enable_function_call_generator_block_reference_type =
  trigger_and_enable_event_generator_block_reference_type_gen "function-call"
;;

let trigger_and_enable_either_event_generator_block_type =
  Mdl_basics_explicit_ast.Explicit_block_reference_type
    trigger_and_enable_either_event_generator_block_reference_type
and trigger_and_enable_falling_event_generator_block_type =
  Mdl_basics_explicit_ast.Explicit_block_reference_type
    trigger_and_enable_falling_event_generator_block_reference_type
and trigger_and_enable_rising_event_generator_block_type =
  Mdl_basics_explicit_ast.Explicit_block_reference_type
    trigger_and_enable_rising_event_generator_block_reference_type
and trigger_and_enable_function_call_generator_block_type =
  Mdl_basics_explicit_ast.Explicit_block_reference_type
    trigger_and_enable_function_call_generator_block_reference_type
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
