(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_make_block_types.mli,v 1.1 2019-05-10 14:04:01 jpc Exp $ *)

(** {3 Building blocks in explicit syntax} *)

(**
  Auxiliary type definitions for module [Mdl_make_block].
*)

type 'a block_generator =
  Mdl_basics_deep_ast.deep_port_name option ->
  Mdl_ast.block_name ->
  Mdl_basics_explicit_ast.explicit_number_of_regular_ports ->
  Mdl_basics_explicit_ast.explicit_number_of_implicit_ports ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

type 'a regular_block_generator =
  Mdl_basics_deep_ast.deep_port_name option ->
  Mdl_ast.block_name ->
  Mdl_basics_explicit_ast.explicit_number_of_regular_ports ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

type 'a implicit_block_generator =
  Mdl_basics_deep_ast.deep_port_name option ->
  Mdl_ast.block_name ->
  Mdl_basics_explicit_ast.explicit_number_of_implicit_ports ->
  'a Mdl_implementation_explicit_ast.explicit_block
;;

type 'a event_generator_without_parameter_block_generator =
  Mdl_basics_explicit_ast.explicit_mirror ->
  Mdl_basics_explicit_ast.explicit_position ->
  'a regular_block_generator
;;

(**
  [event_generator_block_generator] needs in this  order

    a binding_table (parameters)
    a position
    a mirror property
    a port name option
    a block name
    a block number of input
    a block number of output ->

  then returns a block: the event generator block
*)
type 'a event_generator_block_generator =
  Mdl_implementation_explicit_ast.explicit_binding_table ->
  'a event_generator_without_parameter_block_generator
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
