(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.5+dev0 generated code. *)

let printer_block_name ctx ppf = function
  | m_b -> Mdl_ast_print.printer_block_name ctx ppf m_b

and printer_line_name ctx ppf = function
  | m_l -> Mdl_ast_print.printer_line_name ctx ppf m_l
;;

let printer_port_number ctx ppf = function
  | m_p -> Mdl_ast_print.printer_port_number ctx ppf m_p
;;

let printer_labels_option ctx ppf = function
  | m_l_o ->
    Og_printer.printer_Option Mdl_ast_print.printer_labels ctx ppf m_l_o
;;

let printer_explicit_font ctx ppf = function
  | m_e -> Mdl_basics_explicit_ast_print.printer_explicit_font ctx ppf m_e

and printer_explicit_relative_points ctx ppf = function
  | m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_relative_points
      ctx
      ppf
      m_e
;;

let printer_explicit_block pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_block pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_system pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_system pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_model pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_model pp_a
      ctx
      ppf
      a_m_e

and printer_explicit_simulink_file pp_a ctx ppf = function
  | a_m_e ->
    Mdl_explicit_ast_print.printer_explicit_simulink_file pp_a ctx ppf a_m_e
;;

let printer_split_connector_port pp_a ctx ppf = function
  | a_m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_connector_port
      pp_a
      ctx
      ppf
      a_m_e
;;

let printer_split_connection_connector_desc ctx ppf = function
  | m_e ->
    Mdl_implementation_explicit_ast_print.printer_explicit_connection_connector_desc
      ctx
      ppf
      m_e
;;

let printer_directed_connector_desc ctx ppf = function
  | p -> printer_port_number ctx ppf p
;;

let printer_activation_connector_desc _ctx ppf = function
  | Mdl_split_ast.Enable_port ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_split_ast.Enable_port"
  | Mdl_split_ast.Trigger_port ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_split_ast.Trigger_port"
  | Mdl_split_ast.Enable_trigger_port ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_split_ast.Enable_trigger_port"
  | Mdl_split_ast.Trigger_enable_port ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_split_ast.Trigger_enable_port"
  | Mdl_split_ast.If_action_port ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_split_ast.If_action_port"
;;

let printer_activation_connector ctx ppf = function
  | a_s ->
    printer_split_connector_port printer_activation_connector_desc
      ctx
      ppf
      a_s
;;

let printer_split_link_connectors pp_src pp_dst ctx ppf = function
  | { Mdl_split_ast.split_link_src = s_s; Mdl_split_ast.split_link_dst = d_s;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_split_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>split_link_src =";
    printer_split_connector_port pp_src Og_precedence.Ck_record_field ppf s_s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>split_link_dst =";
    printer_split_connector_port pp_dst Og_precedence.Ck_record_field ppf d_s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_split_directed_link_connectors ctx ppf = function
  | d_d_s ->
    printer_split_link_connectors
      printer_directed_connector_desc
      printer_dst_directed_connector_desc
      ctx
      ppf
      d_d_s

and printer_dst_directed_connector_desc ctx ppf = function
  | Mdl_split_ast.Split_directed_connector_desc d ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Mdl_split_ast.Split_directed_connector_desc";
    printer_directed_connector_desc Og_precedence.Ck_argument ppf d;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_split_ast.Split_activation_connector_desc a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Mdl_split_ast.Split_activation_connector_desc";
    printer_activation_connector_desc Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_split_connection_link_connectors ctx ppf = function
  | s_s_s ->
    printer_split_link_connectors
      printer_split_connection_connector_desc
      printer_split_connection_connector_desc
      ctx
      ppf
      s_s_s
;;

let printer_poly_split_link pp_link_desc ctx ppf = function
  | {
      Mdl_split_ast.split_link_name = l_o;
      Mdl_split_ast.split_link_font = e;
      Mdl_split_ast.split_link_labels = l;
      Mdl_split_ast.split_link_points = e_o;
      Mdl_split_ast.split_link_desc = l0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_split_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>split_link_name =";
    Og_printer.printer_Option printer_line_name
      Og_precedence.Ck_record_field
      ppf
      l_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>split_link_font =";
    printer_explicit_font Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>split_link_labels =";
    printer_labels_option Og_precedence.Ck_record_field ppf l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>split_link_points =";
    Og_printer.printer_Option printer_explicit_relative_points
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>split_link_desc =";
    pp_link_desc Og_precedence.Ck_record_field ppf l0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_split_link_desc ctx ppf = function
  | Mdl_split_ast.Split_directed_link_desc s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_split_ast.Split_directed_link_desc";
    printer_split_directed_link_connectors Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_split_ast.Split_connection_link_desc s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_split_ast.Split_connection_link_desc";
    printer_split_connection_link_connectors Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_split_links ctx ppf = function
  | s_l -> Og_printer.printer_List printer_split_link ctx ppf s_l

and printer_split_link ctx ppf = function
  | s_p -> printer_poly_split_link printer_split_link_desc ctx ppf s_p
;;

let printer_split_block ctx ppf = function
  | s_e -> printer_explicit_block printer_split_link ctx ppf s_e

and printer_split_system ctx ppf = function
  | s_e -> printer_explicit_system printer_split_link ctx ppf s_e

and printer_split_model ctx ppf = function
  | s_e -> printer_explicit_model printer_split_link ctx ppf s_e

and printer_split_simulink_file ctx ppf = function
  | s_e -> printer_explicit_simulink_file printer_split_link ctx ppf s_e
;;

(* Defining top printers. *)

let print_block_name ppf = printer_block_name Og_precedence.Ck_toplevel ppf

and print_line_name ppf = printer_line_name Og_precedence.Ck_toplevel ppf
;;

let print_port_number ppf = printer_port_number Og_precedence.Ck_toplevel ppf
;;

let print_labels_option ppf =
  printer_labels_option Og_precedence.Ck_toplevel ppf
;;

let print_explicit_font ppf =
  printer_explicit_font Og_precedence.Ck_toplevel ppf

and print_explicit_relative_points ppf =
  printer_explicit_relative_points Og_precedence.Ck_toplevel ppf
;;

let print_explicit_block pp_a ppf =
  printer_explicit_block pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_system pp_a ppf =
  printer_explicit_system pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_model pp_a ppf =
  printer_explicit_model pp_a Og_precedence.Ck_toplevel ppf

and print_explicit_simulink_file pp_a ppf =
  printer_explicit_simulink_file pp_a Og_precedence.Ck_toplevel ppf
;;

let print_split_connector_port pp_a ppf =
  printer_split_connector_port pp_a Og_precedence.Ck_toplevel ppf
;;

let print_split_connection_connector_desc ppf =
  printer_split_connection_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_directed_connector_desc ppf =
  printer_directed_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_activation_connector_desc ppf =
  printer_activation_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_activation_connector ppf =
  printer_activation_connector Og_precedence.Ck_toplevel ppf
;;

let print_split_link_connectors pp_src pp_dst ppf =
  printer_split_link_connectors pp_src pp_dst Og_precedence.Ck_toplevel ppf
;;

let print_split_directed_link_connectors ppf =
  printer_split_directed_link_connectors Og_precedence.Ck_toplevel ppf

and print_dst_directed_connector_desc ppf =
  printer_dst_directed_connector_desc Og_precedence.Ck_toplevel ppf
;;

let print_split_connection_link_connectors ppf =
  printer_split_connection_link_connectors Og_precedence.Ck_toplevel ppf
;;

let print_poly_split_link pp_link_desc ppf =
  printer_poly_split_link pp_link_desc Og_precedence.Ck_toplevel ppf

and print_split_link_desc ppf =
  printer_split_link_desc Og_precedence.Ck_toplevel ppf
;;

let print_split_links ppf = printer_split_links Og_precedence.Ck_toplevel ppf

and print_split_link ppf = printer_split_link Og_precedence.Ck_toplevel ppf
;;

let print_split_block ppf = printer_split_block Og_precedence.Ck_toplevel ppf

and print_split_system ppf =
  printer_split_system Og_precedence.Ck_toplevel ppf

and print_split_model ppf = printer_split_model Og_precedence.Ck_toplevel ppf

and print_split_simulink_file ppf =
  printer_split_simulink_file Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.5+dev0 generated code. *)
