(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: concat_list.ml,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

type 'a clist =
  | List of 'a list
  | Concat of 'a clist * 'a clist
;;

let rec length = function
  | List l -> List.length l
  | Concat (c1, c2) ->
    length c1 + length c2
;;

let list l = List l;;

let concat l1 l2 =
  match l1, l2 with
  | List [], l2 -> l2
  | l1, List [] -> l1
  | List [ x ], List l -> List (x :: l)
  | List _, _ | Concat (_, _), _ ->
    Concat (l1, l2)
;;

let rec cons x l =
  match l with
  | List l -> list (x :: l)
  | Concat (l1, l2) ->
    concat (cons x l1) l2
;;

let cons_list l c =
  match l with
  | [] -> c
  | l -> concat (list l) c
;;

let rec map f = function
  | List l -> list (List.map f l)
  | Concat (l1, l2) -> concat (map f l1) (map f l2)
;;

let rec iter f = function
  | List l -> List.iter f l
  | Concat (l1, l2) -> iter f l1; iter f l2
;;

let list_of_clist cl =
  let rec loop accu = function
  | List l ->
    begin match accu with
    | [] -> l
    | accu -> l @ accu end
  | Concat (l1, l2) ->
    let l = loop accu l2 in
    loop l l1 in
  loop [] cl
;;

let clist_of_lists ls =
  List.fold_left (fun accu l -> concat accu (list l)) (list []) ls
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
