(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_block_types.mli,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

open Mdl_basics_explicit_ast
open Mdl_implementation_explicit_ast
open Instructions
;;

(* Borrowed form external modules. *)
type natural = Ast_types.natural;;

(* The sides of PMIOPort blocks *)
type pmio_port_side =
  | Pmio_port_left
  | Pmio_port_right
;;

(* The exhaustive ordered list of PMIOPort blocks in a system. *)
type pmio_port_sides = pmio_port_side option array;;

type system_io_descriptor =
  {
    system_pmio_port_sides : pmio_port_sides;
    system_inport_number : natural;
    system_outport_number : natural;
  }
;;

type genr_info =
  {
    block_arguments : explicit_binding_table;
    block_bindings : explicit_binding_table;
    block_inport_names :
      (explicit_port_number, explicit_line_name) Hash_table.t;

    block_ports : explicit_block_ports;
    block_ports_description : explicit_ports_description;
    block_parent_system_io_descriptor : system_io_descriptor;
    block_port_activation_needed : bool;
    block_explicit_type : explicit_block_type;
    block_explicit_name : Mdl_basics_explicit_ast.explicit_block_name;
    block_explicit_instance_data : explicit_binding_table option;
  }
;;

type block_translator = host_block_ident -> genr_info -> code;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
