(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: genr_utils.mli,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

val init : Simport_configuration.target_modeler -> unit;;

val get_modeler_input_port_type : unit -> Instructions.modeler_block_type;;
val set_modeler_input_port_type :
  Simport_configuration.target_modeler ->
  Instructions.modeler_block_type -> unit
;;

val get_modeler_output_port_type : unit -> Instructions.modeler_block_type;;
val set_modeler_output_port_type :
  Simport_configuration.target_modeler ->
  Instructions.modeler_block_type -> unit
;;

val get_modeler_annotation_type : unit -> Instructions.modeler_block_type;;
val set_modeler_annotation_type :
  Simport_configuration.target_modeler ->
  Instructions.modeler_block_type -> unit
;;

val modeler_ident : string -> Instructions.modeler_ident;;
val host_block_ident : string -> Instructions.host_block_ident;;
val host_diagram_ident : string -> Instructions.host_diagram_ident;;
val host_block_tag_ident :
  string -> Instructions.host_block_tag_ident;;
val host_function_ident :
  string -> Instructions.host_function_ident;;
val host_block : int -> Instructions.host_block_ident;;
val host_diagram : int -> Instructions.host_diagram_ident;;
val next_block_level : int -> int * Instructions.host_block_ident;;

val blk : Instructions.host_block_ident;;
val blk0 : Instructions.host_block_ident;;
val scsm : Instructions.host_diagram_ident;;
val scsm0 : Instructions.host_diagram_ident;;

val block_parameter_binding :
  Instructions.modeler_ident * Instructions.code_value ->
  Instructions.modeler_block_parameter_binding
;;

val struct_parameter_binding :
  Instructions.modeler_ident *
  (Instructions.modeler_ident * Instructions.code_value list) Ulist1.t ->
  Instructions.modeler_block_parameter_binding
;;

val matlab_string_of_mdl_string : string -> string
(** Turns a Caml string contents (as given by the Mdl parser from a mdl
  file) to a ``valid'' Matlab string.
  It escapes all \' characters in the text as two \'
  characters and enclose this result into \' characters to get a valid
  matlab (string) lexem.
  Raises [Fatal_error], if the string argument is not a valid
  Matlab string contents. *)
;;

val matlab_string_of_file_name : Path.file_name -> string
;;

val simulink_block_reference_type :
  Mdl_deep_ast.deep_block_source_type ->
  Mdl_deep_ast.deep_block_source_path ->
  Mdl_basics_explicit_ast.explicit_block_type
;;

val simulink_block_type :
  Mdl_basics_explicit_ast.explicit_block_regular_type ->
  Mdl_basics_explicit_ast.explicit_block_type
;;

val make_binding : Ident.ident -> 'a -> 'a Binding.ident_binding;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
