(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_translation.mli,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

(** From explicit syntax with splitting blocks and Scicos-style lines,
  to instructions (c)lists (a.k.a. simport true target block system API
  code).
*)

type block_name = Ast_types.mdl_name
and block_tag = Instructions.host_block_tag_ident
and translation_table = {
  mutable tag_count : int;
  mutable diagram_count : int;
  mutable block_count : int;
  mutable diagram_ident : Instructions.host_diagram_ident;
  mutable block_ident : Instructions.host_block_ident;
  table : (block_name, block_tag) Hashtbl.t;
}
;;

(* FIXME: Should be suppressed.
val diagram_left_margin : float;;
val diagram_right_margin : float;;
val diagram_bottom_margin : float;;

val block_vertical_spacing : float;;
val block_port_size : Instructions.scicos_size;;

val window_size : Instructions.scicos_size;;

End of should be suppressed *)

val modeler_font_value : Mdl_basics_explicit_ast.explicit_font ->
  Instructions.modeler_font_value
;;
val modeler_font_size :  Mdl_basics_explicit_ast.explicit_font ->
  Instructions.modeler_font_size
;;
val modeler_rgb : int -> int -> int -> Instructions.modeler_color;;
val modeler_color :
  Mdl_basics_deep_ast.deep_color -> Instructions.modeler_color
;;

(* FIXME: should be suppressed
val host_string_of_deep_value : Mdl_basics_deep_ast.deep_value -> string;;
val matlab_expression_of_deep_value :
  Mdl_basics_deep_ast.deep_value -> Instructions.matlab_expression
;;
val matlab_script_of_name : string -> string;;
val text_of_deep_binding : Mdl_basics_deep_ast.deep_binding -> string;;
*)

val modeler_evaluation_commented_context :
  ('a, Instructions.code_value Binding.ident_binding) Hash_table.t ->
  Instructions.modeler_block_parameter_binding list
;;

val new_translation_table : unit -> translation_table
;;

val get_diagram_ident :
  translation_table -> Instructions.host_diagram_ident
;;
val get_block_ident : translation_table -> Instructions.host_block_ident
;;
val get_block_tag : translation_table -> block_name -> block_tag
;;

(* FIXME: should be removed!
val set_diagram_ident :
  translation_table -> Instructions.host_diagram_ident
val set_block_ident : translation_table -> Instructions.host_block_ident
val set_block_tag : translation_table -> block_name -> block_tag -> unit
val new_diagram_ident :
  translation_table -> Instructions.host_diagram_ident
val new_block_ident : translation_table -> Instructions.host_block_ident
val new_block_tag :
  translation_table -> Instructions.host_block_tag_ident
*)

val genr_table : translation_table
;;

val reset_genr_table : translation_table -> unit
;;

val new_genr_diagram_ident : unit -> Instructions.host_diagram_ident
;;

val new_genr_block_ident : unit -> Instructions.host_block_ident
;;

val add_block :
  translation_table ->
  Instructions.host_diagram_ident ->
  Instructions.host_block_ident ->
  block_name -> Instructions.instruction
;;

val add_annotation_block :
  translation_table ->
  Instructions.host_diagram_ident ->
  Instructions.host_block_ident -> Instructions.instruction
;;

(* FIXME: should be removed
val evaluation_mode_of_deep_value :
  Mdl_basics_deep_ast.deep_value -> Instructions.evaluation_mode
;;
*)
val bool_of_evaluation_mode : Instructions.evaluation_mode -> bool;;

val bool_of_visible_mode_opt : Instructions.visible_mode option -> bool;;

val bool_of_enable_mode_opt : Instructions.enable_mode option -> bool;;

val target_style_of_parameter_style :
  Mdl_basics_deep_ast.deep_mask_parameter_style -> Ast_types.mdl_text
;;

val print_text_as_raw_string_token :
  Format.formatter -> Ast_types.mdl_text -> unit
(** Print a MDL text as a valid string token of the target host language.
  Note; no character set encoding transposition occurs.
*)
;;

val print_text_as_encoded_string_token :
  Format.formatter -> Ast_types.mdl_text -> unit
(** Print a MDL text as a valid string token of the target host language.
  Note; source encoding is taken into account to translate the text in the
  target host language encoding.
*)
;;

(*
val print_text :
  Format.formatter -> Ast_types.mdl_text -> unit
(** Print a MDL text as a is.
  Note; no transcoding nor text translation occurs.
*)
;;
*)

val print_parameter_style_items :
  Format.formatter -> Mdl_basics_deep_ast.deep_mask_parameter_style -> unit
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
