(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_lex_pprint.ml,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

open Mdl_parser;;

let print_keyword ppf kwd =
  Format.fprintf ppf "KEYWORD %s" kwd
;;

let print_symbol ppf kwd =
  Format.fprintf ppf "SYMBOL '%s'" kwd
;;

let print_lexem ppf = function
  | EOF -> Format.fprintf ppf "%,EOF@."; raise End_of_file
  | IDENT s -> Format.fprintf ppf "IDENT %s" s
  | BLOCKTYPENAME bt ->
    Format.fprintf ppf "BLOCKTYPENAME %s"
      (Mdl_block_types_tables.string_of_block_type bt)
  | FLOAT s -> Format.fprintf ppf "FLOAT %s" s
  | INT s -> Format.fprintf ppf "INT %s" s
  | STRING s -> Format.fprintf ppf "STRING %S" s
  | BOOL s -> Format.fprintf ppf "BOOL %s" s
  | OPEN_PAREN -> print_symbol ppf "("
  | CLICKFCN -> print_symbol ppf "ClickFcn"
  | CLOSE_PAREN -> print_symbol ppf ")"
  | CONFIGSET -> print_keyword ppf "ConfigSet"
  | OPEN_BRACKET -> print_symbol ppf "["
  | CLOSE_BRACKET -> print_symbol ppf "]"
  | OPEN_BRACE -> print_symbol ppf "{"
  | CLOSE_BRACE -> print_symbol ppf "}"
  | COMMA -> print_symbol ppf ","
  | SEMICOLON -> print_symbol ppf ";"
  | CONNECTTYPE -> print_keyword ppf "ConnectType"
  | CUSTOMPROPERTY -> print_keyword ppf "CustomProperty"
  | DATA -> print_keyword ppf "Data"
  | DATARECORD -> print_keyword ppf "DataRecord"
  | DATATYPE -> print_keyword ppf "DataType"
  | DOLLAR_CLASSNAME -> print_keyword ppf "$ClassName"
  | DOLLAR_BACKUPCLASS -> print_keyword ppf "$BackupClass"
  | DOLLAR_OBJECTID -> print_keyword ppf "$ObjectId"
  | DOLLAR_PROPNAME -> print_keyword ppf "$PropName"
  | ANNOTATION -> print_keyword ppf "Annotation"
  | ANNOTATIONDEFAULTS -> print_keyword ppf "AnnotationDefaults"
  | ARRAY -> print_keyword ppf "Array"
  | AUTOSIZE -> print_keyword ppf "AutoSize"
  | BACKGROUNDCOLOR -> print_keyword ppf "BackgroundColor"
  | BLOCK -> print_keyword ppf "Block"
  | BLOCKDEFAULTS -> print_keyword ppf "BlockDefaults"
  | BLOCKINTERFACES -> print_keyword ppf "BlockInterfaces"
  | BLOCKMIRROR -> print_keyword ppf "BlockMirror"
  | BLOCKPARAMETERDEFAULTS -> print_keyword ppf "BlockParameterDefaults"
  | BLOCKROTATION -> print_keyword ppf "BlockRotation"
  | BLOCKTYPE -> print_keyword ppf "BlockType"
  | BRANCH -> print_keyword ppf "Branch"
  | COLON -> print_keyword ppf ":"
  | DIMENSION -> print_keyword ppf "Dimension"
  | DOT -> print_symbol ppf "."
  | DROPSHADOW -> print_symbol ppf "DropShadow"
  | DSTBLOCK -> print_keyword ppf "DstBlock"
  | DSTPORT -> print_keyword ppf "DstPort"
  | ENABLEPORT -> print_keyword ppf "Enableport"
  | ENUMSTRPAIRS -> print_keyword ppf "EnumStrPairs"
  | FONTANGLE -> print_keyword ppf "FontAngle"
  | FONTNAME -> print_keyword ppf "FontName"
  | FONTSIZE -> print_keyword ppf "FontSize"
  | FONTWEIGHT -> print_keyword ppf "FontWeight"
  | FOREGROUNDCOLOR -> print_keyword ppf "ForegroundColor"
  | GRAPHICALINTERFACE -> print_keyword ppf "GraphicalInterface"
  | HIDEAUTOMATICNAME -> print_keyword ppf "HideAutomaticName"
  | HORIZONTALALIGNMENT -> print_keyword ppf "HorizontalAlignment"
  | INPORT -> print_keyword ppf "Inport"
  | INTERFACENAME -> print_keyword ppf "InterfaceName"
  | INTERPRETER -> print_keyword ppf "Interpreter"
  | LABELS -> print_keyword ppf "Labels"
  | LIBRARY -> print_keyword ppf "Library"
  | LINE -> print_keyword ppf "Line"
  | LINETYPE -> print_keyword ppf "LineType"
  | LINEDEFAULTS -> print_keyword ppf "LineDefaults"
  | LIST -> print_keyword ppf "List"
  | LISTTYPE -> print_keyword ppf "ListType"

  | LARRAY -> print_keyword ppf "array"
  | LCHART -> print_keyword ppf "chart"
  | LCHARTFILENUMBER -> print_keyword ppf "chartFileNumber"
  | LDATA -> print_keyword ppf "data"
  | LDST -> print_keyword ppf "dst"
  | LEML -> print_keyword ppf "eml";
  | LENABLE -> print_keyword ppf "enable"
  | LEVENT -> print_keyword ppf "event"
  | LEXECUTIONORDER -> print_keyword ppf "executionOrder";
  | LFIRSTDATA -> print_keyword ppf "firstData";
  | LFIRSTEVENT -> print_keyword ppf "firstEvent";
  | LFIRSTINDEX -> print_keyword ppf "firstIndex";
  | LFIRSTJUNCTION -> print_keyword ppf "firstJunction";
  | LFIRSTTARGET -> print_keyword ppf "firstTarget";
  | LFIRSTTRANSITION -> print_keyword ppf "firstTransition";
  | LID -> print_keyword ppf "id";
  | LIFACTION -> print_keyword ppf "ifaction"
  | LINSTANCE -> print_keyword ppf "instance"
  | LINTERSECTION -> print_keyword ppf "intersection"
  | LJUNCTION -> print_keyword ppf "junction"
  | LLABELSTRING -> print_keyword ppf "labelString";
  | LLINKNODE -> print_keyword ppf "linkNode";
  | LMACHINE -> print_keyword ppf "machine"
  | LNAME -> print_keyword ppf "name";
  | LOCATION -> print_keyword ppf "Location"
  | LPLANTMODELINGINFO -> print_keyword ppf "plantModelingInfo"
  | LPROPS -> print_keyword ppf "props"
  | LSCOPE -> print_keyword ppf "scope";
  | LSCRIPT -> print_keyword ppf "script";
  | LSIZE -> print_keyword ppf "size";
  | LSRC -> print_keyword ppf "src";
  | LSSIDNUMBER -> print_keyword ppf "ssidNumber";
  | LSTATE -> print_keyword ppf "state"
  | LTARGET -> print_keyword ppf "target"
  | LTRANSITION -> print_keyword ppf "transition"
  | LTREENODE -> print_keyword ppf "treeNode"
  | LTRIGGER -> print_keyword ppf "trigger"
  | LTYPE -> print_keyword ppf "type";

  | MASKCALLBACKSTRING -> print_keyword ppf "MaskCallbackString"
  | MASKCAPABILITIES -> print_keyword ppf "MaskCapabilities"
  | MASKDEFAULTS -> print_keyword ppf "MaskDefaults"
  | MASKDESCRIPTION -> print_keyword ppf "MaskDescription"
  | MASKDISPLAY -> print_keyword ppf "MaskDisplay"
  | MASKDISPLAYSTRING -> print_keyword ppf "MaskDisplayString"
  | MASKENABLESTRING -> print_keyword ppf "MaskEnableString"
  | MASKHELP -> print_keyword ppf "MaskHelp"
  (* | MASKHIDECONTENTS -> print_keyword ppf "MaskHideContents" *)
  | MASKICONFRAME -> print_keyword ppf "MaskIconFrame"
  | MASKICONOPAQUE -> print_keyword ppf "MaskIconOpaque"
  | MASKICONROTATE -> print_keyword ppf "MaskIconRotate"
  | MASKICONUNITS -> print_keyword ppf "MaskIconUnits"
  | MASKINITIALIZATION -> print_keyword ppf "MaskInitialization"
  | MASKPARAMETERDEFAULTS -> print_keyword ppf "MaskParameterDefaults"
  | MASKPORTROTATE -> print_keyword ppf "MaskPortRotate"
  | MASKPROMPTSTRING -> print_keyword ppf "MaskPromptString"
  | MASKRUNINITFORICONREDRAW -> print_keyword ppf "MaskRunInitForIconRedraw"
  | MASKSELFMODIFIABLE -> print_keyword ppf "MaskSelfModifiable"
  | MASKSTYLESTRING -> print_keyword ppf "MaskStyleString"
  | MASKTABNAMESTRING -> print_keyword ppf "MaskTabNameString"
  | MASKTOOLTIPSTRING -> print_keyword ppf "MaskToolTipString"
  | MASKTUNABLEVALUESTRING -> print_keyword ppf "MaskTunableValueString"
  | MASKTYPE -> print_keyword ppf "MaskType"
  | MASKVALUESTRING -> print_keyword ppf "MaskValueString"
  | MASKVARALIASSTRING -> print_keyword ppf "MaskVarAliasString"
  | MASKVARIABLES -> print_keyword ppf "MaskVariables"
  | MASKVISIBILITYSTRING -> print_keyword ppf "MaskVisibilityString"

  | MATDATA -> print_keyword ppf "MatData"
  | MATSTRUCT -> print_keyword ppf "MATStruct"
  | MODEL -> print_keyword ppf "Model"
  | NAME -> print_keyword ppf "Name"
  | NAMEPLACEMENT -> print_keyword ppf "NamePlacement"
  | NUMRECORDS -> print_keyword ppf "NumRecords"
  | NUMROOTINPORTS -> print_keyword ppf "NumRootInports"
  | NUMROOTOUTPORTS -> print_keyword ppf "NumRootOutports"
  | NUMROOTTRIGPORTS -> print_keyword ppf "NumRootTrigports"
  | ORIENTATION -> print_keyword ppf "Orientation"
  | OBJECT -> print_keyword ppf "Object"
  | OUTPORT -> print_keyword ppf "Outport"
  | BLOCKINTERFACEPARAMETERS ->
    print_keyword ppf "BlockInterfaceParameters"
  | POINTS -> print_keyword ppf "Points"
  | PORT -> print_keyword ppf "Port"
  | PORTNUMBER -> print_keyword ppf "PortNumber"
  | PORTS -> print_keyword ppf "Ports"
  | POSITION -> print_keyword ppf "Position"
  | PROPNAME -> print_keyword ppf "PropName"
  | SAVEDCHARACTERENCODING -> print_keyword ppf "SavedCharacterEncoding"
  | SHARP -> print_keyword ppf "#"
  | SHOWNAME -> print_keyword ppf "ShowName"
  | SID -> print_keyword ppf "SID"
  | SFBLOCKTYPE -> print_keyword ppf "SFBlockType"
  | SOURCEBLOCK -> print_keyword ppf "SourceBlock"
  | SOURCETYPE -> print_keyword ppf "SourceType"
  | SRCBLOCK -> print_keyword ppf "SrcBlock"
  | SRCPORT -> print_keyword ppf "SrcPort"
  | STATEFLOW -> print_keyword ppf "Stateflow"
  | STR -> print_keyword ppf "String"
  | SYSTEM -> print_keyword ppf "System"
  | TAG -> print_keyword ppf "Tag"
  | TEXT -> print_keyword ppf "Text"
  | TEXMODE -> print_keyword ppf "TeXMode"
  | TILDA -> print_symbol ppf "~"
  | TREATASATOMICUNIT -> print_keyword ppf "TreatAsAtomicUnit"
  | TRIGPORT -> print_keyword ppf "Trigport"
  | TYPE -> print_keyword ppf "Type"
  | USEDISPLAYTEXTASCLICKCALLBACK ->
    print_keyword ppf "UseDisplayTextAsClickCallBack"
  | VALUE -> print_keyword ppf "VALUE"
  | VENDOR id ->
    Format.fprintf ppf "VENDOR %a" Mdl_ast_print.print_vendor_ident id
  | VERSION -> print_keyword ppf "Version"
  | VERTICALALIGNMENT -> print_keyword ppf "VerticalAlignment"
  | WORDWRAP -> print_keyword ppf "WordWrap"
  | WSMATLABCODE -> print_keyword ppf "WSMATLABCode"
  | WSMCODE -> print_keyword ppf "WSMCode"
  | WSMATFILENAME -> print_keyword ppf "WSMATFileName"
  | WSSOURCEFILENAME -> print_keyword ppf "WSSOURCEFileName"
  | WSDATASOURCE -> print_keyword ppf "WSDataSource"
  | WSMDLFILEDATA -> print_keyword ppf "WSMdlFileData"
  | ZORDER -> print_keyword ppf "ZOrder"
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
