(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_lexer.mli,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

open Mdl_parser;;

type error =
   | Illegal_character of char
   | Illegal_escape of string
   | Unterminated_string
(** The various errors when lexing. *)
;;

exception Error of error * Lexing.position * Lexing.position
(** The exception raised when encountering an error when lexing. *)
;;

val report_error : Format.formatter -> error -> unit
(** The error reporting function for lexing. *)
;;

val keyword_table : (string, token) Hashtbl.t
(** The keyword table: gives the keyword corresponding to the string
  representation of an ident. *)
;;
val x_keyword_table : (token, string) Hashtbl.t
(** The keyword inverse table: gives the string representation of a keyword. *)
;;

val token : Lexing.lexbuf -> Mdl_parser.token
(** The lexer automaton *)
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
