(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_parse_error.ml,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

type reason = string;;

type error =
  | Wrong_integer of string * reason
  | Not_an_integer of string * reason
  | Not_a_boolean of string * reason
  | Duplicated_defaults_section of string
  | Unknown_connection_type of string
  | Unknown_line_type of string
  | Unknown_alignment of string
  | Unknown_array_type of string
  | Unknown_block_type of string
  | Unknown_block_sf_block_type of string
  | Unknown_workspace_data_source_type of string
  | Unknown_port_kind of string
  | Invalid_string_specification of string * string
  | Invalid_tag_ident of string
  | Wrong_connection_port of string
  | Parsing_error
;;

let report_error ppf = function
  | Wrong_integer (i, reason) ->
    Format.fprintf ppf "wrong integer '%s', %s" i reason

  | Not_an_integer (v, reason) ->
    Format.fprintf ppf "the value '%s' is not an integer, %s" v reason

  | Not_a_boolean (v, reason) ->
    Format.fprintf ppf "the value '%s' is not a boolean, %s" v reason

  | Duplicated_defaults_section s ->
    Format.fprintf ppf "the defaults section '%s' is duplicated" s

  | Unknown_connection_type s ->
    Format.fprintf ppf "unknown connection type '%s'" s

  | Unknown_line_type s ->
    Format.fprintf ppf "unknown line type '%s'" s

  | Unknown_alignment s ->
    Format.fprintf ppf "unknown alignment '%s'" s

  | Unknown_array_type s ->
    Format.fprintf ppf "unknown array type '%s'" s

  | Unknown_block_type s ->
    Format.fprintf ppf "unknown block type '%s'" s

  | Unknown_block_sf_block_type s ->
    Format.fprintf ppf "unknown sf block type '%s'" s

  | Unknown_workspace_data_source_type s ->
    Format.fprintf ppf "unknown workspace data source type '%s'" s

  | Unknown_port_kind s ->
    Format.fprintf ppf "unknown port kind '%s'" s

  | Invalid_string_specification (s, reason) ->
    Format.fprintf ppf "invalid string specification '%s' %s" s reason

  | Invalid_tag_ident s ->
    Format.fprintf ppf "invalid tag ident '%s'" s

  | Wrong_connection_port s ->
    Format.fprintf ppf "invalid connection port '%s'" s

  | Parsing_error ->
    Format.fprintf ppf "invalid file format"
;;

exception Error of Location.t * error;;

let parsing_error loc error =
  raise (Error (loc, error))
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
