(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_to_parsing_ast.ml,v 1.1 2019-05-10 14:04:02 jpc Exp $ *)

let pass_ic ic =
  let lexbuf = Lexing.from_channel ic in
  let sfc =
    try
      Mdl_parser.mdl_file Mdl_lexer.token lexbuf with
    | Parsing.Parse_error ->
      let loc = Location.get_lexbuf_loc lexbuf in
      Mdl_parse_error.parsing_error loc
        Mdl_parse_error.Parsing_error in
  Say.fdebug (fun epf ->
    Format.fprintf epf "Simulink file contents is: %a"
      Mdl_parsing_ast_print.print_simulink_file_contents sfc);
  sfc
;;

let pass efname =
  try efname, Path.with_explicit_in_file_bin efname pass_ic with
    (* Mdl parsing and lexing errors. *)
  | Mdl_lexer.Error (e, pos_begin, pos_end) ->
    Format.fprintf Format.err_formatter
      "Lexical error: file %s, %a, %a@."
      pos_begin.Lexing.pos_fname
      Location.print_token_location (pos_begin, pos_end)
      Mdl_lexer.report_error e;
    exit 4
  | Mdl_parse_error.Error (loc, e) ->
    Format.fprintf Format.err_formatter
      "@[Parse error: file %s, %a, %a@]@."
      loc.Location.loc_start.Lexing.pos_fname
      Location.print_token_location
        (loc.Location.loc_start, loc.Location.loc_end)
      Mdl_parse_error.report_error e;
    Format.fprintf Format.err_formatter
      "@[Perhaps unsupported MDL version: only versions >= 6.0 supported@]@.";
    exit 5
;;

let read_mdl_file = Say.fdebug_call "mdl file parsing" pass;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
