(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mbd_ast_pprint.ml,v 1.1 2019-05-10 13:58:20 jpc Exp $ *)

open Mbd_ast;;
open Format;;

let rec pprint ppf = function
  | E_number n -> fprintf ppf "%s" n
  | E_string n -> fprintf ppf "%S" n
  | E_variable v -> fprintf ppf "%s" v
  | E_uparameter_paren e ->
    fprintf ppf "u (%a)" pprint e
  | E_uparameter_bracket e ->
    fprintf ppf "u [%a]" pprint e
  | E_uparameter -> fprintf ppf "%s" "u"
  | E_parameter v -> fprintf ppf "%s" v
  | E_unary_op (op, x) ->
    fprintf ppf "%s (%a)"
      op
      pprint x
  | E_binary_op (op, x, y) ->
    fprintf ppf "%a %s %a"
      pprint x
      op
      pprint y
  | E_apply (e_f, e_args) ->
    fprintf ppf "%a (%a)"
    pprint e_f print_args e_args
  | E_record_access (e1, label) ->
    fprintf ppf "%a.%s"
      pprint e1
      label
  | E_paren e ->
    fprintf ppf "(%a)" pprint e

and print_args ppf = function
  | [] -> ()
  | arg :: args ->
    pprint ppf arg;
    List.iter (function arg -> fprintf ppf ", %a" pprint arg) args
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
