(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mbd_ast_to_deep_ast.ml,v 1.1 2019-05-10 13:58:20 jpc Exp $ *)

open Mbd_ast;;
open Mbd_deep_ast;;

(*
let deep_unary_operator op =
  match op with
  | "+" | "-"
  | "!"
(*
  | "~" | "\'" | ".\'"
*)
    -> ()
  | s -> failwith (Printf.sprintf "unknown unary operator %S" s)
;;

let deep_binary_operator op =
  match op with
  | "||" | "&&"

  | "==" | "<" | ">" | "<=" | ">=" | "!="

  | "+" | "-"
  | "*" | "/"

  | "^"
(*
  | "**"
  | ":="
  | "~=" | "<>"
  | ".*"
  | "./" | ".\\" | ".^"
  | "," | ";" | "\\"
  | ":"
*)
    -> ()
  | s -> failwith (Printf.sprintf "unknown binary operator %S" s)
;;

let deep_intrinsic_function_name name =
  name,
  match name with
    (* Unary primitives *)
  | "abs"
  | "acos" | "acosh"
  | "asin" | "asinh"
  | "atan"  | "atanh"

  | "ceil"
  | "cos" | "cosh"
  | "exp"

  | "floor" | "round"
(*
  | "fabs"
*)
  | "int"
  | "inv"

  | "ln" | "log" | "log10"

  | "sgn"
  | "sin" | "sinh"
  | "sqrt"

  | "tan" | "tanh"

    -> Some 1

    (* Unary/Binary functions *)
  | "and" | "or" -> None

    (* Binary primitives *)
  | "pow" | "power"

  | "atan2"
  | "hypot"

  | "rem"

    -> Some 2

    (* Unary/Binary functions *)
  | "min" | "max" -> None
  | _ -> raise Not_found
;;
*)

let deep_parameter = function
  | "u" -> 1
  | s ->
    let l = String.length s in
    assert (l > 1);
    int_of_string (String.sub s 1 (l - 1))
;;

let rec deep e =
  match e with
  | E_number n -> D_number n
  | E_string s -> D_string s
  | E_variable v -> D_variable (Mbd_predefined.global_of_string v)
  | E_uparameter_paren e ->
    D_uparameter_paren (deep e)
  | E_uparameter_bracket e ->
    D_uparameter_bracket (deep e)
  | E_uparameter ->
    D_parameter 1
  | E_unary_op (op, e1) ->
    D_unary_op (Mbd_predefined.unary_op_of_string op, deep e1)
  | E_binary_op (op, e1, e2) ->
    D_binary_op (Mbd_predefined.binary_op_of_string op, deep e1, deep e2)
  | E_parameter p -> D_parameter (deep_parameter p)
  | E_apply (f, args) ->
    D_apply (deep f, List.map deep args)
  | E_record_access (e1, label) ->
    D_record_access (deep e1, label)
  | E_paren (E_paren _ as e) -> deep e
  | E_paren e -> D_paren (deep e)
;;

(*
    let e_args = deep_args e_args in
    let e_arity = List.length e_args in
    begin
      try
        let f_name, f_arity = deep_function e_f in
        match f_arity with
        | None ->
          E_apply (E_variable f_name, e_args)
(*
 deep_application f_name e_args
*)
        | Some f_arity ->
          if f_arity <> e_arity then
            failwith (
              Printf.sprintf
                "function %s has arity %d, but is used with arity %d"
                f_name f_arity e_arity)
(*
          else deep_application f_name e_args
*)
          else E_apply (E_variable f_name, e_args)
      with
      | Not_found ->
        E_apply (deep e_f, e_args)
    end
(*
and deep_application f_name e_args =
  match e_args with
  | [ e_arg; ] -> E_unary_op (f_name, e_arg)
  | [ e_arg1; e_arg2; ] -> E_binary_op (f_name, e_arg1, e_arg2)
  | e_args ->
    E_apply (E_variable f_name, e_args)
*)
and deep_args args = List.map deep args

and deep_function e =
  match e with
  | E_number s ->
    failwith (Printf.sprintf "Number %s is not a function" s)
  | E_string s ->
    failwith (Printf.sprintf "String %S is not a function" s)
  | E_variable v -> deep_intrinsic_function_name v
  | E_uparameter_paren _
  | E_uparameter_bracket _
  | E_uparameter ->
    failwith (Printf.sprintf
      "Anonymous parameter u is not a function, it cannot be applied")
  | E_parameter p ->
    failwith (Printf.sprintf
      "Parameter %s is not a function, it cannot be applied" p)
  | E_unary_op (_, _) | E_binary_op (_, _, _) | E_apply (_, _)
  | E_record_access (_, _) -> raise Not_found
  | E_paren f -> deep_function f
;;
*)

let ast_to_deep_ast = deep;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
