(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mbd_predef.mli,v 1.1 2019-05-10 13:58:20 jpc Exp $ *)

type unary_prim =
   | Uabs

   | Uacos | Uacosh
   | Uasin | Uasinh
   | Uatan | Uatanh

   | Uceil
   | Ucos | Ucosh
   | Uexp

   | Ufabs
   | Ufloor

(*
  Note: The Fcn block does not support round and fix.
  Use the Rounding Function block to apply these rounding modes.
*)
   | Uround

(* Those two are not in the official list. *)
(* Is Uint the explicitely put out Ufix ? *)
   | Uint
   | Uinv

   | Uln | Ulog | Ulog10
(*
   | Ulogm
*)

   | Usgn
   | Usin | Usinh
   | Usqrt

   | Utan | Utanh

(* Those two are not in the official list.
   | Umin | Umax
*)

and unary_op =

   | Uplus | Uminus
   | Ubang

(* Those two are not in the official list.
   | Uand | Uor
*)
;;

type binary_op =
   | Bor | Band

   | Beq | Blt | Bgt | Ble | Bge | Bne

   | Badd | Bsub
   | Bmul | Bdiv

   | Bhat

and binary_prim =
   | Bpow | Bpower

   | Batan2
   | Bhypot

(* Those two are not in the official list. *)
   | Bmin | Bmax

   | Brem
;;

type global =
  | Gop1 of unary_op
  | Gop2 of binary_op
  | Gprim1 of unary_prim
  | Gprim2 of binary_prim
  | Gident of Mbd_ast.variable_name
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
