(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mbd_predefined.ml,v 1.1 2019-05-10 13:58:20 jpc Exp $ *)

open Mbd_predef;;

let string_of_unary_op = function
  | Uminus -> "-"
  | Uplus -> "+"
  | Ubang -> "!"

and unary_op_of_string = function
  | "-" -> Uminus
  | "+" -> Uplus
  | "!" -> Ubang
  | _ -> raise Not_found

and string_of_unary_prim = function
  | Uabs -> "abs"
  | Uacos -> "acos"
  | Uacosh -> "acosh"
  | Uasin -> "asin"
  | Uasinh -> "asinh"
  | Uatan -> "atan"
  | Uatanh -> "atanh"

  | Uceil -> "ceil"
  | Ucos -> "cos"
  | Ucosh -> "cosh"
  | Uexp -> "exp"
  | Ufabs -> "fabs"
  | Ufloor -> "floor"
  | Uround -> "round"

  | Uint -> "int"
  | Uinv -> "inv"

  | Uln -> "ln"
  | Ulog -> "log"
  | Ulog10 -> "log10"

  | Usgn -> "sgn"
  | Usin -> "sin"
  | Usinh -> "sinh"
  | Usqrt -> "sqrt"

  | Utan -> "tan"
  | Utanh -> "tanh"

and unary_prim_of_string = function
  | "abs" -> Uabs
  | "acos" -> Uacos
  | "acosh" -> Uacosh
  | "asin" -> Uasin
  | "asinh" -> Uasin
  | "atan" -> Uatan
  | "atanh" -> Uatanh

  | "ceil" -> Uceil
  | "cos" -> Ucos
  | "cosh" -> Ucosh
  | "exp" -> Uexp
  | "fabs" -> Ufabs
  | "floor" -> Ufloor
  | "round" -> Uround

  | "int" -> Uint
  | "inv" -> Uinv

  | "ln" -> Uln
  | "log" -> Ulog
  | "log10" -> Ulog10

  | "sgn" -> Usgn
  | "sin" -> Usin
  | "sinh" -> Usinh
  | "sqrt" -> Usqrt

  | "tan" -> Utan
  | "tanh" -> Utanh

  | "sign" -> Usgn
  | _ -> raise Not_found
;;

let string_of_binary_op = function
  | Bor -> "|"
  | Band -> "&"

  | Beq -> "=="
  | Blt -> "<"
  | Bgt -> ">"
  | Ble -> "<="
  | Bge -> ">="
  | Bne -> "~="

  | Badd -> "+"
  | Bsub -> "-"
  | Bmul -> "*"
  | Bdiv -> "/"

  | Bhat -> "^"

and binary_op_of_string = function
  | "||" -> Bor
  | "&&" -> Band

  | "==" -> Beq
  | "<" -> Blt
  | ">" -> Bgt
  | "<=" -> Ble
  | ">=" -> Bge
  | "!=" -> Bne

  | "+" -> Badd
  | "-" -> Bsub
  | "*" -> Bmul
  | "/" -> Bdiv

  | "^" -> Bhat

  | _ -> raise Not_found

and string_of_binary_prim = function
  | Bpow -> "pow"
  | Bpower -> "power"

  | Batan2 -> "atan2"
  | Bhypot -> "hypot"

  | Bmin -> "min"
  | Bmax -> "max"
  | Brem -> "rem"

and binary_prim_of_string = function
  | "pow" -> Bpow
  | "power" -> Bpower

  | "atan2" -> Batan2
  | "hypot" -> Bhypot

  | "min" -> Bmin
  | "max" -> Bmax
  | "rem" -> Brem

  | _ -> raise Not_found
;;

let global_of_string v =
  try
    Gop1 (unary_op_of_string v) with
  | Not_found ->
  try
    Gop2 (binary_op_of_string v) with
  | Not_found ->
  try
    Gprim1 (unary_prim_of_string v) with
  | Not_found ->
  try
    Gprim2 (binary_prim_of_string v) with
  | Not_found ->
    Gident v
;;

let string_of_global = function
  | Gop1 v -> string_of_unary_op v
  | Gop2 v -> string_of_binary_op v
  | Gprim1 v -> string_of_unary_prim v
  | Gprim2 v -> string_of_binary_prim v
  | Gident v -> v
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
