(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mbd_to_target.ml,v 1.1 2019-05-10 13:58:20 jpc Exp $ *)

(** The procedures to translate Simulink embedded expressions to Scicos expressions. *)

let translate_file ppf explicit_fname =
  let dast = Mbd_treat.treat_file explicit_fname in
  Format.fprintf ppf "%a@." Mbd_deep_ast_pprint.pprint dast
;;

let translate_string s =
  let dast = Mbd_treat.treat_string s in
  let b = Buffer.create 1024 in
  let ppf = Format.formatter_of_buffer b in
  Format.fprintf ppf "%a@?" Mbd_deep_ast_pprint.pprint dast;
  Buffer.contents b
(** Reads a string that contains a Simulink embedded expression and
  returns its translation to the target_language as a string. *)
;;


let print = translate_file
(** Reads a Simulink embedded expression file, translates it to the
  target language, and prints the translation onto the formatter argument. *)
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
