(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, Clément Franchini                    *)
(*                   INRIA Rocquencourt                                *)
(*                                                                     *)
(*  Copyright 2013-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: slx_xml_check.ml,v 1.1 2019-05-10 14:00:16 jpc Exp $ *)

open Xml_ast;;
(* open Xml_deep_ast;; *)

let check_error s = failwith s;;

let check_misc = function
  | Misc_comment _comment -> ()
  | Misc_pi _pi -> check_error "Processing instructions not supported"
  | Misc_space _space -> ()
;;

let rec check_misc_star misc_star =
  match misc_star with
  | [] -> ()
  | misc :: misc_star ->
    check_misc misc;
    check_misc_star misc_star
;;

let get_encoding_from_xml_decl_option = function
  | None -> check_error "Missing XML declaration"
  | Some {
      xml_version_info = _version_number;
      xml_encoding_decl = encoding_name_option;
      xml_standalone_decl = _standalone_decl_option;
    } ->
    encoding_name_option
;;

let check_xml_decl_option xml_decl_option =
  ignore (get_encoding_from_xml_decl_option xml_decl_option)
;;

let check_doc_type_decl_option doc_type_decl_option =
  match doc_type_decl_option with
  | None -> ()
  | Some _ -> check_error "Document type declaration not supported"
;;

let check_prolog prolog =
  match prolog with
  {
    xml_decl = xml_decl_option;
    xml_misc_star = misc_star;
    xml_doc_type_decl = doc_type_decl_option;
    xml_misc_star_trailing = misc_star_trailing;
  } ->
  check_xml_decl_option xml_decl_option;
  check_misc_star misc_star;
  check_doc_type_decl_option doc_type_decl_option;
  check_misc_star misc_star_trailing
;;

let get_encoding prolog = get_encoding_from_xml_decl_option prolog.xml_decl;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
