(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, Clément Franchini                    *)
(*                   INRIA Rocquencourt                                *)
(*                                                                     *)
(*  Copyright 2013-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_deep_ast.mli,v 1.1 2019-05-10 14:00:17 jpc Exp $ *)

(** Xml_deep_ast is a specialized version of XML trees specialized to MDL
    specification and translation. To facilitate pattern matching and
    safety, all (or as much as possible) string values in the XML tree should
    be translated to constructors similar to MDL constructors and values.
    The xml_deep_ast trees will facilitate translation to MDL files (or
    preferably translation to MDL AST values).
*)

type text = string;;
type encoding = string;;

type attribute_name = string
and element_name = string;;

type char_data = text
and attribute_value = string;;

type deep_attribute = {
  da_name : attribute_name;
  da_value : attribute_value;
}
and deep_attributes = deep_attribute list
;;

type deep_child = (char_data, deep_element) Intercal.sum
and deep_children = deep_child list

and deep_element = {
  de_name : element_name;
  de_attributes : deep_attributes;
  de_children : deep_children;
}
;;

type deep_document = {
  deep_prolog : encoding option;
  deep_element : deep_element;
}
;;

type deep_annex_name =
  | Deep_block_diagram_defaults
  | Deep_config_set_info
  | Deep_graphical_interface
  | Deep_stateflow
  | Deep_annex_file_name of Path.file_name
;;

type deep_annex = deep_annex_name * deep_document;;

type deep_annexes = deep_annex list;;

type xml_deep_model = {
  deep_model_ident : Mdl_ast.model_ident;
  deep_blockdiagram : deep_document;
  deep_annexes : deep_annexes;
}
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
