(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, Clément Franchini                    *)
(*                   INRIA Rocquencourt                                *)
(*                                                                     *)
(*  Copyright 2013-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_entry_parsers.ml,v 1.1 2019-05-10 14:00:17 jpc Exp $ *)

(*
type _p_tag = {
  name : string;
  parameter_class : Xml_explicit_ast.parameter_class option;
  parameter_reference : string option;
}
;;

let not_yet _ =
  failwith "Mdl_entry_parsers: not yet"
;;
*)

let unfailing_text_parser parser_name entry_parser string =
  Say.debug (Printf.sprintf
    "trying to use parser %S with %S" parser_name string);
  let lexbuf = Lexing.from_string string in
  try
    let r = entry_parser Mdl_lexer.token lexbuf in
    Say.fdebug (fun epf ->
      Format.fprintf epf
        "parser %S succeed with %a"
        parser_name
        Mdl_ast_print.print_model_value r);
    r
  with
  | Mdl_lexer.Error _
  | Parsing.Parse_error ->
    Say.fdebug (fun epf -> Format.fprintf epf
      "parser %S called with input %S failed"
      parser_name string);
    Mdl_ast.Value_simple (Mdl_ast.String string)
;;

let text_parser parser_name entry_parser string =
  Say.debug (Printf.sprintf
    "trying to use parser %S with %S" parser_name string);
  let lexbuf = Lexing.from_string string in
  try entry_parser Mdl_lexer.token lexbuf with
  | Parsing.Parse_error ->
    Say.fatal_error (Printf.sprintf
      "parser %S called with input %S failed"
      parser_name string)
;;

let string_parser parser_name entry_parser text =
  let string = Printf.sprintf "%S" text in
  text_parser parser_name entry_parser string
;;

let position_parser = text_parser "position" Mdl_parser.position_entry;;

let simple_value_parser str =
  let simple_value =
    text_parser "simple_value" Mdl_parser.simple_value_entry str in
  Say.fdebug (fun epf ->
    Format.fprintf epf "simple_value_parser returns %a"
    Mdl_ast_print.print_simple_value simple_value);
  simple_value
;;

let string_simple_value_parser =
  string_parser "string_simple_value" Mdl_parser.simple_value_entry
;;

let slx_model_value_parser =
  unfailing_text_parser "slx_model_value" Mdl_parser.slx_model_value_entry
;;

let slx_simple_value_parser string =
  let simple_value =
    match slx_model_value_parser string with
    | Mdl_ast.Value_simple simple_value -> simple_value
    | Mdl_ast.Value_vector _
    | Mdl_ast.Value_matrix _
    | Mdl_ast.Value_bindings _ -> Mdl_ast.String string in
  Say.fdebug (fun epf ->
    Format.fprintf epf "slx_simple_value_parser %S -> %a"
      string
      Mdl_ast_print.print_simple_value simple_value);
  simple_value
;;

let block_type_value_parser =
  text_parser "block_type" Mdl_parser.block_type_value_entry
;;

let array_type_parser =
  text_parser "array_type" Mdl_parser.array_type_entry
;;

let line_type_parser =
  string_parser "line_type" Mdl_parser.line_type_entry
;;

let connection_type_parser =
  string_parser "connection_type" Mdl_parser.connection_type_entry
;;

let workspace_data_source_type_parser =
  string_parser
    "workspace_data_source_type"
    Mdl_parser.workspace_data_source_type_entry
;;

let custom_property_type_parser =
  string_parser
    "custom_property_type"
    Mdl_parser.custom_property_type_entry
;;

let naturals_parser = text_parser "naturals" Mdl_parser.naturals_entry;;

let natural_vector_parser =
  text_parser "natural_vector" Mdl_parser.natural_vector_entry
;;

let horizontal_alignment_parser =
  string_parser "horizontal_alignment" Mdl_parser.horizontal_alignment_entry
;;

let vertical_alignment_parser =
  string_parser "vertical_alignment" Mdl_parser.vertical_alignment_entry
;;

let integer_matrix_parser =
  text_parser "integer_matrix" Mdl_parser.integer_matrix_entry
;;

let block_sid_value_parser =
  text_parser "block_sid_value" Mdl_parser.block_sid_value_entry
;;

let slx_port_connector_parser =
  text_parser "slx_port_connector" Mdl_parser.slx_port_connector_entry
;;

let vendor_configuration_longident_parser =
  text_parser
    "vendor_configuration_longident"
    Mdl_parser.vendor_configuration_longident_entry
;;

let intersection_parser =
  text_parser "intersection" Mdl_parser.intersection_entry
;;

let parameter_class_of_string = function
  | "char" -> Xml_explicit_ast.Char
  | "double" -> Xml_explicit_ast.Double
  | "int32" -> Xml_explicit_ast.Int32
  | "logical" -> Xml_explicit_ast.Logical
  | "uint32" -> Xml_explicit_ast.Uint32
  | parameter_class ->
    failwith (Printf.sprintf "unknown parameter class: %S" parameter_class)
;;

let intersection_of_string = intersection_parser;;

(*
  Local Variables:
   compile-command: "cd ../../..; make"
  End:
*)
