(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, Clément Franchini                    *)
(*                   INRIA Rocquencourt                                *)
(*                                                                     *)
(*  Copyright 2013-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mdl_entry_parsers.mli,v 1.1 2019-05-10 14:00:17 jpc Exp $ *)

(** Tools to help analyses of a parsed XML file. *)

(** {6 General functions} *)

val string_parser :
  string ->
  ((Lexing.lexbuf -> Mdl_parser.token) -> Lexing.lexbuf -> 'a) ->
  string ->
  'a
(** [string_parser parser_name parser text] parses [text] using [parser].
    [parser_name] is used for debugging. *)
;;

(** {6 Specific functions} *)

val position_parser : string -> Mdl_ast.position
(** [position_parser text] parses a [position] from [text]. *)
;;

val simple_value_parser : string -> Mdl_ast.simple_value
(** [simple_value_parser text] parses [text] as a [simple_value] value
    (boolean, integer or float). *)
;;

val string_simple_value_parser : string -> Mdl_ast.simple_value
(** [string_simple_value_parser text] parses [text] as a string [simple_value]
    value. *)
;;

val slx_model_value_parser : string -> Mdl_ast.model_value
(** [slx_model_value_parser text] parses [text] as a [model_value].
    In case of error, returns [text] as a [model_value]. *)
;;

val slx_simple_value_parser : string -> Mdl_ast.simple_value
(** [slx_simple_value_parser text] parses [text] as a [simple_value].
    In case of error, returns [text] as a [simple_value]. *)
;;

val block_type_value_parser : string -> Mdl_block_types.block_type;;

val array_type_parser : string -> Mdl_ast.array_type;;

val line_type_parser : string -> Mdl_ast.line_type;;

val connection_type_parser : string -> Mdl_ast.connection_type;;

val naturals_parser : string -> Unatural.t list;;

val intersection_parser : string -> Mdl_ast.stateflow_numbers;;

val natural_vector_parser : string -> Unatural.t list;;

val horizontal_alignment_parser : string -> Mdl_ast.horizontal_alignment;;

val vertical_alignment_parser : string -> Mdl_ast.vertical_alignment;;

val integer_matrix_parser : string -> Mdl_ast.integer_matrix;;

val block_sid_value_parser : string -> Mdl_ast.block_sid;;

val slx_port_connector_parser :
  string -> Mdl_ast.block_sid * Mdl_ast.port_value
;;

val vendor_configuration_longident_parser :
  string -> Mdl_ast.vendor_ident * Mdl_ast.configuration_longident
;;

val workspace_data_source_type_parser :
  string -> Mdl_ast.workspace_data_source_type
;;

val custom_property_type_parser : string -> Mdl_ast.custom_property_type;;

val parameter_class_of_string : string -> Xml_explicit_ast.parameter_class;;

val intersection_of_string : string -> Mdl_ast.stateflow_numbers;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
