(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, Clément Franchini                    *)
(*                   INRIA Rocquencourt                                *)
(*                                                                     *)
(*  Copyright 2013-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.5+dev0 generated code. *)

open Ast_types_print;;

let printer_explicit_version ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_explicit_model_version ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_explicit_reference ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_explicit_list_type ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let printer_explicit_dimension ctx ppf = function
  | (a_n, n) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Ast_types_print.printer_natural Og_precedence.Ck_tuple_item ppf a_n;
    Format.fprintf ppf "%,,@ ";
    printer_natural Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_sid ctx ppf = function
  | m_b -> Mdl_ast_print.printer_block_sid ctx ppf m_b
;;

let printer_explicit_object_id ctx ppf = function
  | (p_f, n) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Path_print.printer_file_name Og_precedence.Ck_tuple_item ppf p_f;
    Format.fprintf ppf "%,,@ ";
    printer_natural Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_explicit_encoding ctx ppf = function
  | Xml_explicit_ast.ASCII ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.ASCII"
  | Xml_explicit_ast.Latin1 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Latin1"
  | Xml_explicit_ast.Latin9 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Latin9"
  | Xml_explicit_ast.UTF16 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.UTF16"
  | Xml_explicit_ast.UTF8 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.UTF8"
  | Xml_explicit_ast.Encoding s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Encoding";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_source_port_description ctx ppf = function
  | { Xml_explicit_ast.src_port = n; Xml_explicit_ast.src_block_name = m_b; } ->
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%, {@ " else
     Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
     Format.fprintf ppf "%,Xml_explicit_ast.@,";
     Format.fprintf ppf "%,@[<hv 2>src_port =";
     printer_natural Og_precedence.Ck_record_field ppf n;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
     Format.fprintf ppf "%,@[<hv 2>src_block_name =";
     Mdl_ast_print.printer_block_name Og_precedence.Ck_record_field ppf m_b;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%,@;<1 -2>}" else
     Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_destination_port_description ctx ppf = function
  | { Xml_explicit_ast.dst_port = n; Xml_explicit_ast.dst_block_name = m_b; } ->
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%, {@ " else
     Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
     Format.fprintf ppf "%,Xml_explicit_ast.@,";
     Format.fprintf ppf "%,@[<hv 2>dst_port =";
     printer_natural Og_precedence.Ck_record_field ppf n;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
     Format.fprintf ppf "%,@[<hv 2>dst_block_name =";
     Mdl_ast_print.printer_block_name Og_precedence.Ck_record_field ppf m_b;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%,@;<1 -2>}" else
     Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_parameter_class _ctx ppf = function
  | Xml_explicit_ast.Char ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Char"
  | Xml_explicit_ast.Double ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Double"
  | Xml_explicit_ast.Int32 ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Int32"
  | Xml_explicit_ast.Uint32 ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Uint32"
  | Xml_explicit_ast.Logical ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Xml_explicit_ast.Logical"
;;

let printer_explicit_cell_class ctx ppf = function
  | p -> printer_parameter_class ctx ppf p

and printer_explicit_field_class ctx ppf = function
  | p -> printer_parameter_class ctx ppf p
;;

let printer_text ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let printer_explicit_position ctx ppf = function
  | t -> printer_text ctx ppf t
;;

let printer_explicit_document_value ctx ppf = function
  | Xml_explicit_ast.Ddv t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Ddv";
    printer_text Og_precedence.Ck_argument ppf t; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Ddv_classified (p, t) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Ddv_classified";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_parameter_class Og_precedence.Ck_tuple_item ppf p;
    Format.fprintf ppf "%,,@ ";
    printer_text Og_precedence.Ck_tuple_item ppf t;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_explicit_binding ctx ppf = function
  | e_b_i ->
    Binding_print.printer_ident_binding printer_explicit_document_value
      ctx
      ppf
      e_b_i

and printer_explicit_bindings ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_binding ctx ppf e_l
;;

let rec printer_explicit_block_parameter_default ctx ppf = function
  | {
      Xml_explicit_ast.explicit_block_parameter_default_type = m_b;
      Xml_explicit_ast.explicit_block_parameter_default_bindings = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_parameter_default_type =";
    Mdl_block_types_print.printer_block_type
      Og_precedence.Ck_record_field
      ppf
      m_b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>explicit_block_parameter_default_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_block_parameter_defaults ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_block_parameter_default
      ctx
      ppf
      e_l
;;

let rec printer_explicit_annotation ctx ppf = function
  | {
      Xml_explicit_ast.explicit_annotation_sid = s;
      Xml_explicit_ast.explicit_annotation_bindings = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_annotation_sid =";
    printer_sid Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_annotation_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_annotations ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_annotation ctx ppf e_l
;;

let rec printer_explicit_connectors ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_connector ctx ppf e_l

and printer_explicit_connector ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_custom_property_item ctx ppf = function
  | Xml_explicit_ast.Explicit_custom_property_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_custom_property_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_custom_property_item_enum_str_pairs e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_custom_property_item_enum_str_pairs";
    printer_explicit_bindings Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_custom_property_items ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_custom_property_item ctx ppf e_l
;;

let printer_explicit_custom_property ctx ppf = function
  | {
      Xml_explicit_ast.explicit_custom_property_name = m_p;
      Xml_explicit_ast.explicit_custom_property_data_type = m_c;
      Xml_explicit_ast.explicit_custom_property_items = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_custom_property_name =";
    Mdl_ast_print.printer_prop_name Og_precedence.Ck_record_field ppf m_p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_custom_property_data_type =";
    Mdl_ast_print.printer_custom_property_type
      Og_precedence.Ck_record_field
      ppf
      m_c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_custom_property_items =";
    printer_explicit_custom_property_items
      Og_precedence.Ck_record_field
      ppf
      e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_dialog_parameters ctx ppf = function
  | {
      Xml_explicit_ast.explicit_dialog_parameters_block_name = m_b;
      Xml_explicit_ast.explicit_dialog_parameters_bindings = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_dialog_parameters_block_name =";
    Mdl_ast_print.printer_block_name Og_precedence.Ck_record_field ppf m_b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_dialog_parameters_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_link_data ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_dialog_parameters ctx ppf e_l
;;

let printer_explicit_mask_display_icon_frame ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_display_icon_units ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_display_icon_opaque ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_display_icon_rotate ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_display_run_init_for_icon_redraw ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_display_script ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_initialisation ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_help ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_explicit_mask_parameter_type ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_prompt ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_value ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_type ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_description ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_evaluate ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_enabled ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_visible ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_callback ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_tunable ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_internal ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_never_save ctx ppf = function
  | t -> printer_text ctx ppf t

and printer_explicit_mask_parameter_read_only ctx ppf = function
  | t -> printer_text ctx ppf t
;;

let printer_explicit_mask_parameter ctx ppf = function
  | {
      Xml_explicit_ast.explicit_mask_parameter_name = e;
      Xml_explicit_ast.explicit_mask_parameter_type = e0;
      Xml_explicit_ast.explicit_mask_parameter_prompt = e1;
      Xml_explicit_ast.explicit_mask_parameter_value = e2;
      Xml_explicit_ast.explicit_mask_parameter_callback = e_o;
      Xml_explicit_ast.explicit_mask_parameter_tunable = e_o0;
      Xml_explicit_ast.explicit_mask_parameter_evaluate = e_o1;
      Xml_explicit_ast.explicit_mask_parameter_enabled = e_o2;
      Xml_explicit_ast.explicit_mask_parameter_visible = e_o3;
      Xml_explicit_ast.explicit_mask_parameter_internal = e_o4;
      Xml_explicit_ast.explicit_mask_parameter_never_save = e_o5;
      Xml_explicit_ast.explicit_mask_parameter_read_only = e_o6;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_name =";
    printer_explicit_mask_parameter_name Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_type =";
    printer_explicit_mask_parameter_type Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_prompt =";
    printer_explicit_mask_parameter_prompt
      Og_precedence.Ck_record_field
      ppf
      e1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_value =";
    printer_explicit_mask_parameter_value
      Og_precedence.Ck_record_field
      ppf
      e2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_callback =";
    Og_printer.printer_Option printer_explicit_mask_parameter_callback
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_tunable =";
    Og_printer.printer_Option printer_explicit_mask_parameter_tunable
      Og_precedence.Ck_record_field
      ppf
      e_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_evaluate =";
    Og_printer.printer_Option printer_explicit_mask_parameter_evaluate
      Og_precedence.Ck_record_field
      ppf
      e_o1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_enabled =";
    Og_printer.printer_Option printer_explicit_mask_parameter_enabled
      Og_precedence.Ck_record_field
      ppf
      e_o2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_visible =";
    Og_printer.printer_Option printer_explicit_mask_parameter_visible
      Og_precedence.Ck_record_field
      ppf
      e_o3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_internal =";
    Og_printer.printer_Option printer_explicit_mask_parameter_internal
      Og_precedence.Ck_record_field
      ppf
      e_o4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_never_save =";
    Og_printer.printer_Option printer_explicit_mask_parameter_never_save
      Og_precedence.Ck_record_field
      ppf
      e_o5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_read_only =";
    Og_printer.printer_Option printer_explicit_mask_parameter_read_only
      Og_precedence.Ck_record_field
      ppf
      e_o6;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_mask_display ctx ppf = function
  | {
      Xml_explicit_ast.explicit_mask_display_icon_frame = e_o;
      Xml_explicit_ast.explicit_mask_display_icon_units = e_o0;
      Xml_explicit_ast.explicit_mask_display_icon_opaque = e_o1;
      Xml_explicit_ast.explicit_mask_display_run_init_for_icon_redraw = e_o2;
      Xml_explicit_ast.explicit_mask_display_icon_rotate = e_o3;
      Xml_explicit_ast.explicit_mask_display_script = e_o4;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display_icon_frame =";
    Og_printer.printer_Option printer_explicit_mask_display_icon_frame
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display_icon_units =";
    Og_printer.printer_Option printer_explicit_mask_display_icon_units
      Og_precedence.Ck_record_field
      ppf
      e_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display_icon_opaque =";
    Og_printer.printer_Option printer_explicit_mask_display_icon_opaque
      Og_precedence.Ck_record_field
      ppf
      e_o1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>explicit_mask_display_run_init_for_icon_redraw =";
    Og_printer.printer_Option
      printer_explicit_mask_display_run_init_for_icon_redraw
      Og_precedence.Ck_record_field
      ppf
      e_o2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display_icon_rotate =";
    Og_printer.printer_Option printer_explicit_mask_display_icon_rotate
      Og_precedence.Ck_record_field
      ppf
      e_o3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display_script =";
    Og_printer.printer_Option printer_explicit_mask_display_script
      Og_precedence.Ck_record_field
      ppf
      e_o4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_mask ctx ppf = function
  | {
      Xml_explicit_ast.explicit_mask_type = e_o;
      Xml_explicit_ast.explicit_mask_display = e;
      Xml_explicit_ast.explicit_mask_description = e_o0;
      Xml_explicit_ast.explicit_mask_initialisation = e_o1;
      Xml_explicit_ast.explicit_mask_help = e_o2;
      Xml_explicit_ast.explicit_mask_parameters = e_l;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_type =";
    Og_printer.printer_Option printer_explicit_mask_type
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_display =";
    printer_explicit_mask_display Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_description =";
    Og_printer.printer_Option printer_explicit_mask_description
      Og_precedence.Ck_record_field
      ppf
      e_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_initialisation =";
    Og_printer.printer_Option printer_explicit_mask_initialisation
      Og_precedence.Ck_record_field
      ppf
      e_o1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_help =";
    Og_printer.printer_Option printer_explicit_mask_help
      Og_precedence.Ck_record_field
      ppf
      e_o2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameters =";
    Og_printer.printer_List printer_explicit_mask_parameter
      Og_precedence.Ck_record_field
      ppf
      e_l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_explicit_system ctx ppf = function
  | {
      Xml_explicit_ast.explicit_system_position = e;
      Xml_explicit_ast.explicit_system_bindings = e0;
      Xml_explicit_ast.explicit_system_blocks = e1;
      Xml_explicit_ast.explicit_system_connections = e2;
      Xml_explicit_ast.explicit_system_annotations = e3;
      Xml_explicit_ast.explicit_system_connectors = e4;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_position =";
    printer_explicit_position Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_blocks =";
    printer_explicit_blocks Og_precedence.Ck_record_field ppf e1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_connections =";
    printer_explicit_lines Og_precedence.Ck_record_field ppf e2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_annotations =";
    printer_explicit_annotations Og_precedence.Ck_record_field ppf e3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_connectors =";
    printer_explicit_connectors Og_precedence.Ck_record_field ppf e4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_block ctx ppf = function
  | {
      Xml_explicit_ast.explicit_block_type = m_b;
      Xml_explicit_ast.explicit_block_name = m_b0;
      Xml_explicit_ast.explicit_block_sid = s;
      Xml_explicit_ast.explicit_block_items = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_type =";
    Mdl_block_types_print.printer_block_type
      Og_precedence.Ck_record_field
      ppf
      m_b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_name =";
    Mdl_ast_print.printer_block_name Og_precedence.Ck_record_field ppf m_b0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_sid =";
    printer_sid Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_items =";
    printer_explicit_block_items Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_blocks ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_block ctx ppf e_l

and printer_explicit_block_item ctx ppf = function
  | Xml_explicit_ast.Explicit_block_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_object e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_object";
    printer_explicit_object Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_system e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_system";
    printer_explicit_system Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_list e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_list";
    printer_explicit_list Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_port e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_port";
    printer_explicit_port Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_instance_data e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_block_instance_data";
    printer_explicit_bindings Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_link_data e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_link_data";
    printer_explicit_link_data Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_block_mask e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_block_mask";
    printer_explicit_mask Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_block_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_block_item ctx ppf e_l

and printer_explicit_port ctx ppf = function
  | e -> printer_explicit_port_items ctx ppf e

and printer_explicit_port_item ctx ppf = function
  | Xml_explicit_ast.Explicit_port_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_port_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_port_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_port_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_port_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_port_item ctx ppf e_l

and printer_explicit_list ctx ppf = function
  | {
      Xml_explicit_ast.explicit_list_type = e;
      Xml_explicit_ast.explicit_list_bindings = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_list_type =";
    printer_explicit_list_type Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_list_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_object ctx ppf = function
  | {
      Xml_explicit_ast.explicit_object_id = e;
      Xml_explicit_ast.explicit_object_class_name = m_c_o;
      Xml_explicit_ast.explicit_object_prop_name = m_p_o;
      Xml_explicit_ast.explicit_object_backup_class = m_b_o;
      Xml_explicit_ast.explicit_object_version = e_o;
      Xml_explicit_ast.explicit_object_reference = e_o0;
      Xml_explicit_ast.explicit_object_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_id =";
    printer_explicit_object_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_class_name =";
    Og_printer.printer_Option Mdl_ast_print.printer_class_name
      Og_precedence.Ck_record_field
      ppf
      m_c_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_prop_name =";
    Og_printer.printer_Option Mdl_ast_print.printer_prop_name
      Og_precedence.Ck_record_field
      ppf
      m_p_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_backup_class =";
    Og_printer.printer_Option Mdl_ast_print.printer_backup_class_name
      Og_precedence.Ck_record_field
      ppf
      m_b_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_version =";
    Og_printer.printer_Option printer_explicit_version
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_reference =";
    Og_printer.printer_Option printer_explicit_reference
      Og_precedence.Ck_record_field
      ppf
      e_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_object_items =";
    printer_explicit_object_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_object_item ctx ppf = function
  | Xml_explicit_ast.Explicit_object_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_object_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_object_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_object_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_object_object e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_object_object";
    printer_explicit_object Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_object_custom_property e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_object_custom_property";
    printer_explicit_custom_property Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_object_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_object_item ctx ppf e_l

and printer_explicit_array ctx ppf = function
  | {
      Xml_explicit_ast.explicit_array_type = m_a;
      Xml_explicit_ast.explicit_array_dimension = e;
      Xml_explicit_ast.explicit_array_prop_name = m_p;
      Xml_explicit_ast.explicit_array_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_array_type =";
    Mdl_ast_print.printer_array_type Og_precedence.Ck_record_field ppf m_a;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_array_dimension =";
    printer_explicit_dimension Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_array_prop_name =";
    Mdl_ast_print.printer_prop_name Og_precedence.Ck_record_field ppf m_p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_array_items =";
    printer_explicit_array_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_array_item ctx ppf = function
  | Xml_explicit_ast.Explicit_array_item_object e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_array_item_object";
    printer_explicit_object Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_array_item_cell e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_array_item_cell";
    printer_explicit_cell Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_array_matstruct e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_array_matstruct";
    printer_explicit_matstruct_items Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_array_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_array_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_array_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_array_item ctx ppf e_l

and printer_explicit_matstruct_item ctx ppf = function
  | Xml_explicit_ast.Explicit_matstruct_field e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_matstruct_field";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_matstruct_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_matstruct_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_matstruct_items ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_matstruct_item ctx ppf e_l

and printer_explicit_cell ctx ppf = function
  | {
      Xml_explicit_ast.explicit_cell_class = e_o;
      Xml_explicit_ast.explicit_cell_value = s;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_cell_class =";
    Og_printer.printer_Option printer_explicit_cell_class
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_cell_value =";
    Og_printer.printer_String Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_line ctx ppf = function
  | {
      Xml_explicit_ast.explicit_line_type = m_l_o;
      Xml_explicit_ast.explicit_line_items = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_line_type =";
    Og_printer.printer_Option Mdl_ast_print.printer_line_type
      Og_precedence.Ck_record_field
      ppf
      m_l_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_line_items =";
    printer_explicit_line_items Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_lines ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_line ctx ppf e_l

and printer_explicit_line_item ctx ppf = function
  | Xml_explicit_ast.Explicit_line_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_line_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_line_branch e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_line_branch";
    printer_explicit_branch Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_line_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_line_item ctx ppf e_l

and printer_explicit_branch ctx ppf = function
  | {
      Xml_explicit_ast.explicit_branch_connection_type = m_c_o;
      Xml_explicit_ast.explicit_branch_items = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_branch_connection_type =";
    Og_printer.printer_Option Mdl_ast_print.printer_connection_type
      Og_precedence.Ck_record_field
      ppf
      m_c_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_branch_items =";
    printer_explicit_branch_items Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_branches ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_branch ctx ppf e_l

and printer_explicit_branch_item ctx ppf = function
  | Xml_explicit_ast.Explicit_branch_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_branch_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_branch_branch e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_branch_branch";
    printer_explicit_branch Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_branch_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_branch_item ctx ppf e_l
;;

let printer_config_manager_settings ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_editor_settings ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_simulation_settings ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_verification ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_external_mode ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_model_reference_settings ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_concurrent_execution_settings ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_explicit_model_workspace ctx ppf = function
  | e_o -> Og_printer.printer_Option printer_explicit_bindings ctx ppf e_o

and printer_explicit_configuration_set ctx ppf = function
  | {
      Xml_explicit_ast.explicit_configuration_set_object = e_o;
      Xml_explicit_ast.explicit_configuration_set_bindings = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_configuration_set_object =";
    Og_printer.printer_Option printer_explicit_object
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_configuration_set_bindings =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_model_configuration ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_array ctx ppf e_l

and printer_explicit_system_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e

and printer_explicit_block_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e

and printer_explicit_annotation_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e

and printer_explicit_line_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e

and printer_explicit_mask_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e

and printer_explicit_mask_parameter_defaults ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_section ctx ppf = function
  | {
      Xml_explicit_ast.explicit_model_workspace = e;
      Xml_explicit_ast.explicit_model_configuration = e0;
      Xml_explicit_ast.explicit_configuration_set = e1;
      Xml_explicit_ast.explicit_system_defaults = e2;
      Xml_explicit_ast.explicit_block_defaults = e3;
      Xml_explicit_ast.explicit_annotation_defaults = e4;
      Xml_explicit_ast.explicit_line_defaults = e5;
      Xml_explicit_ast.explicit_mask_defaults = e6;
      Xml_explicit_ast.explicit_mask_parameter_defaults = e7;
      Xml_explicit_ast.explicit_block_parameter_defaults = e8;
      Xml_explicit_ast.explicit_system = e9;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_model_workspace =";
    printer_explicit_model_workspace Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_model_configuration =";
    printer_explicit_model_configuration Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_configuration_set =";
    printer_explicit_configuration_set Og_precedence.Ck_record_field ppf e1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system_defaults =";
    printer_explicit_system_defaults Og_precedence.Ck_record_field ppf e2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_defaults =";
    printer_explicit_block_defaults Og_precedence.Ck_record_field ppf e3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_annotation_defaults =";
    printer_explicit_annotation_defaults Og_precedence.Ck_record_field ppf e4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_line_defaults =";
    printer_explicit_line_defaults Og_precedence.Ck_record_field ppf e5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_defaults =";
    printer_explicit_mask_defaults Og_precedence.Ck_record_field ppf e6;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_mask_parameter_defaults =";
    printer_explicit_mask_parameter_defaults
      Og_precedence.Ck_record_field
      ppf
      e7;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_block_parameter_defaults =";
    printer_explicit_block_parameter_defaults
      Og_precedence.Ck_record_field
      ppf
      e8;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_system =";
    printer_explicit_system Og_precedence.Ck_record_field ppf e9;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_id ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_explicit_ssid_number ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_explicit_debug ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_target_items ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_target ctx ppf = function
  | {
      Xml_explicit_ast.explicit_target_id = e;
      Xml_explicit_ast.explicit_target_name = s_o;
      Xml_explicit_ast.explicit_target_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_target_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_target_name =";
    Og_printer.printer_Option Og_printer.printer_String
      Og_precedence.Ck_record_field
      ppf
      s_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_target_items =";
    printer_explicit_target_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_active_state_output ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_src ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_dst ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_slide ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_autogen_item ctx ppf = function
  | Xml_explicit_ast.Explicit_autogen_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_autogen_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_autogen_item_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_autogen_item_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_autogen_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_autogen_item ctx ppf e_l
;;

let printer_explicit_autogen ctx ppf = function
  | e -> printer_explicit_autogen_items ctx ppf e
;;

let printer_explicit_transition_item ctx ppf = function
  | Xml_explicit_ast.Explicit_transition_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_transition_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_transition_item_src e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_transition_item_src";
    printer_explicit_src Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_transition_item_dst e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_transition_item_dst";
    printer_explicit_dst Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_transition_item_slide e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_transition_item_slide";
    printer_explicit_slide Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_transition_item_autogen e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_transition_item_autogen";
    printer_explicit_autogen Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_transition_items ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_transition_item ctx ppf e_l
;;

let printer_explicit_transition ctx ppf = function
  | {
      Xml_explicit_ast.explicit_transition_id = e;
      Xml_explicit_ast.explicit_transition_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_transition_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_transition_items =";
    printer_explicit_transition_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_sub_view_s ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_fixpt ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_type_item ctx ppf = function
  | Xml_explicit_ast.Explicit_type_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_type_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_type_item_fixpt e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_type_item_fixpt";
    printer_explicit_fixpt Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_type_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_type_item ctx ppf e_l
;;

let printer_explicit_type ctx ppf = function
  | e -> printer_explicit_type_items ctx ppf e
;;

let printer_explicit_stateflow_array ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_range ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_props_item ctx ppf = function
  | Xml_explicit_ast.Explicit_props_type e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_props_type";
    printer_explicit_type Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_props_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_props_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_props_stateflow_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_props_stateflow_array";
    printer_explicit_stateflow_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_props_range e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_props_range";
    printer_explicit_range Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_props_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_props_item ctx ppf e_l
;;

let printer_explicit_props ctx ppf = function
  | e -> printer_explicit_props_items ctx ppf e
;;

let printer_explicit_data_item ctx ppf = function
  | Xml_explicit_ast.Explicit_data_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_data_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_data_props e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_data_props";
    printer_explicit_props Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_data_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_data_item ctx ppf e_l
;;

let printer_explicit_data_id ctx ppf = function
  | Xml_explicit_ast.Explicit_data_id e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_data_id";
    printer_explicit_id Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_data_ssid ctx ppf = function
  | Xml_explicit_ast.Explicit_data_ssid e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_data_ssid";
    printer_explicit_ssid_number Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_data ctx ppf = function
  | {
      Xml_explicit_ast.explicit_data_id = e_o;
      Xml_explicit_ast.explicit_data_ssid = e_o0;
      Xml_explicit_ast.explicit_data_name = s;
      Xml_explicit_ast.explicit_data_items = e_l;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_data_id =";
    Og_printer.printer_Option printer_explicit_data_id
      Og_precedence.Ck_record_field
      ppf
      e_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_data_ssid =";
    Og_printer.printer_Option printer_explicit_data_ssid
      Og_precedence.Ck_record_field
      ppf
      e_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_data_name =";
    Og_printer.printer_String Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_data_items =";
    Og_printer.printer_List printer_explicit_data_item
      Og_precedence.Ck_record_field
      ppf
      e_l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_junction_item ctx ppf = function
  | Xml_explicit_ast.Explicit_junction_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_junction_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_junction_item_autogen e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_junction_item_autogen";
    printer_explicit_autogen Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_junction_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_junction_item ctx ppf e_l
;;

let printer_explicit_junction ctx ppf = function
  | {
      Xml_explicit_ast.explicit_junction_id = e;
      Xml_explicit_ast.explicit_junction_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_junction_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_junction_items =";
    printer_explicit_junction_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_event_item ctx ppf = function
  | e -> printer_explicit_binding ctx ppf e
;;

let printer_explicit_event_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_event_item ctx ppf e_l
;;

let printer_explicit_event ctx ppf = function
  | {
      Xml_explicit_ast.explicit_event_id = e;
      Xml_explicit_ast.explicit_event_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_event_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_event_items =";
    printer_explicit_event_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_eml ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_simulink ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_layout_info ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_table_ui_prefs ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_state_cell ctx ppf = function
  | {
      Xml_explicit_ast.explicit_state_cell_id = e;
      Xml_explicit_ast.explicit_state_cell_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_state_cell_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_state_cell_items =";
    printer_explicit_bindings Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_transition_cell_items ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_transition_cell ctx ppf = function
  | {
      Xml_explicit_ast.explicit_transition_cell_id = e;
      Xml_explicit_ast.explicit_transition_cell_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_transition_cell_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_transition_cell_items =";
    printer_explicit_transition_cell_items
      Og_precedence.Ck_record_field
      ppf
      e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_truth_table_item ctx ppf = function
  | Xml_explicit_ast.Explicit_truth_table_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_truth_table_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_truth_table_item_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_truth_table_item_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_truth_table_items ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_truth_table_item ctx ppf e_l
;;

let printer_explicit_truth_table ctx ppf = function
  | e -> printer_explicit_truth_table_items ctx ppf e
;;

let rec printer_explicit_chart_item ctx ppf = function
  | Xml_explicit_ast.Explicit_chart_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_chart_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_chart_item_children e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_chart_item_children";
    printer_explicit_children Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_chart_item_sub_view_s e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_chart_item_sub_view_s";
    printer_explicit_sub_view_s Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_chart_item_eml e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_chart_item_eml";
    printer_explicit_eml Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_chart_item_active_state_output e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_chart_item_active_state_output";
    printer_explicit_active_state_output Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_chart_item_plant_modeling_info ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,Xml_explicit_ast.Explicit_chart_item_plant_modeling_info"

and printer_explicit_chart_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_chart_item ctx ppf e_l

and printer_explicit_chart ctx ppf = function
  | {
      Xml_explicit_ast.explicit_chart_id = e;
      Xml_explicit_ast.explicit_chart_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_chart_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_chart_items =";
    printer_explicit_chart_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_table_item ctx ppf = function
  | Xml_explicit_ast.Explicit_table_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_table_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_table_item_layout_info e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_table_item_layout_info";
    printer_explicit_layout_info Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_table_item_array e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_table_item_array";
    printer_explicit_array Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_table_item_table_ui_prefs e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_table_item_table_ui_prefs";
    printer_explicit_table_ui_prefs Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_table_item_children e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_table_item_children";
    printer_explicit_children Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_table_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_table_item ctx ppf e_l

and printer_explicit_table ctx ppf = function
  | {
      Xml_explicit_ast.explicit_table_id = e;
      Xml_explicit_ast.explicit_table_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_table_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_table_items =";
    printer_explicit_table_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_explicit_child ctx ppf = function
  | Xml_explicit_ast.Explicit_child_target e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_target";
    printer_explicit_target Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_chart e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_chart";
    printer_explicit_chart Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_state e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_state";
    printer_explicit_state Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_transition e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_transition";
    printer_explicit_transition Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_data e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_data";
    printer_explicit_data Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_junction e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_junction";
    printer_explicit_junction Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_event e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_event";
    printer_explicit_event Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_table e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_table";
    printer_explicit_table Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_state_cell e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_child_state_cell";
    printer_explicit_state_cell Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_child_transition_cell e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_child_transition_cell";
    printer_explicit_transition_cell Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_children ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_child ctx ppf e_l

and printer_explicit_state_item ctx ppf = function
  | Xml_explicit_ast.Explicit_state_item_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_children e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_children";
    printer_explicit_children Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_active_state_output e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_active_state_output";
    printer_explicit_active_state_output Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_sub_view_s e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_sub_view_s";
    printer_explicit_sub_view_s Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_eml e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_eml";
    printer_explicit_eml Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_simulink e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_simulink";
    printer_explicit_simulink Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_autogen e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_autogen";
    printer_explicit_autogen Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_state_item_truth_table e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_state_item_truth_table";
    printer_explicit_truth_table Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_explicit_state_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_state_item ctx ppf e_l

and printer_explicit_state ctx ppf = function
  | {
      Xml_explicit_ast.explicit_state_id = e;
      Xml_explicit_ast.explicit_state_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_state_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_state_items =";
    printer_explicit_state_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_machine_item ctx ppf = function
  | Xml_explicit_ast.Explicit_machine_binding e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_machine_binding";
    printer_explicit_binding Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_machine_debug e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_machine_debug";
    printer_explicit_debug Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_machine_children e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Xml_explicit_ast.Explicit_machine_children";
    printer_explicit_children Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_machine_items ctx ppf = function
  | e_l -> Og_printer.printer_List printer_explicit_machine_item ctx ppf e_l
;;

let printer_explicit_machine ctx ppf = function
  | {
      Xml_explicit_ast.explicit_machine_id = e;
      Xml_explicit_ast.explicit_machine_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_machine_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_machine_items =";
    printer_explicit_machine_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_instance_items ctx ppf = function
  | e -> printer_explicit_bindings ctx ppf e
;;

let printer_explicit_instance ctx ppf = function
  | {
      Xml_explicit_ast.explicit_instance_id = e;
      Xml_explicit_ast.explicit_instance_items = e0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_instance_id =";
    printer_explicit_id Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_instance_items =";
    printer_explicit_instance_items Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_explicit_stateflow_section_items ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_stateflow_section_item
      ctx
      ppf
      e_l

and printer_explicit_stateflow_section_item ctx ppf = function
  | Xml_explicit_ast.Explicit_stateflow_section_machine e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_stateflow_section_machine";
    printer_explicit_machine Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_stateflow_section_instance e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_stateflow_section_instance";
    printer_explicit_instance Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_stateflow_section ctx ppf = function
  | Xml_explicit_ast.Explicit_stateflow_section e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_stateflow_section";
    printer_explicit_stateflow_section_items Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_implementation_section_item ctx ppf = function
  | Xml_explicit_ast.Explicit_implementation_library e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_implementation_library";
    printer_explicit_section Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_implementation_model e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_implementation_model";
    printer_explicit_section Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Xml_explicit_ast.Explicit_implementation_stateflow e ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Xml_explicit_ast.Explicit_implementation_stateflow";
    printer_explicit_stateflow_section Og_precedence.Ck_argument ppf e;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_explicit_model_sections ctx ppf = function
  | e_l ->
    Og_printer.printer_List printer_explicit_implementation_section_item
      ctx
      ppf
      e_l
;;

let printer_explicit_blockdiagram ctx ppf = function
  | {
      Xml_explicit_ast.explicit_prolog = e;
      Xml_explicit_ast.explicit_model_sections = e0;
      Xml_explicit_ast.explicit_model_version = e1;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_prolog =";
    printer_explicit_encoding Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_model_sections =";
    printer_explicit_model_sections Og_precedence.Ck_record_field ppf e0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_model_version =";
    printer_explicit_model_version Og_precedence.Ck_record_field ppf e1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_explicit_xml_model ctx ppf = function
  | {
      Xml_explicit_ast.explicit_model_ident = m_m;
      Xml_explicit_ast.explicit_blockdiagram = e;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Xml_explicit_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>explicit_model_ident =";
    Mdl_ast_print.printer_model_ident Og_precedence.Ck_record_field ppf m_m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>explicit_blockdiagram =";
    printer_explicit_blockdiagram Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

(* Defining top printers. *)

let print_explicit_version ppf =
  printer_explicit_version Og_precedence.Ck_toplevel ppf

and print_explicit_model_version ppf =
  printer_explicit_model_version Og_precedence.Ck_toplevel ppf

and print_explicit_reference ppf =
  printer_explicit_reference Og_precedence.Ck_toplevel ppf

and print_explicit_list_type ppf =
  printer_explicit_list_type Og_precedence.Ck_toplevel ppf
;;

let print_explicit_dimension ppf =
  printer_explicit_dimension Og_precedence.Ck_toplevel ppf
;;

let print_sid ppf = printer_sid Og_precedence.Ck_toplevel ppf
;;

let print_explicit_object_id ppf =
  printer_explicit_object_id Og_precedence.Ck_toplevel ppf
;;

let print_explicit_encoding ppf =
  printer_explicit_encoding Og_precedence.Ck_toplevel ppf
;;

let print_source_port_description ppf =
  printer_source_port_description Og_precedence.Ck_toplevel ppf

and print_destination_port_description ppf =
  printer_destination_port_description Og_precedence.Ck_toplevel ppf
;;

let print_parameter_class ppf =
  printer_parameter_class Og_precedence.Ck_toplevel ppf
;;

let print_explicit_cell_class ppf =
  printer_explicit_cell_class Og_precedence.Ck_toplevel ppf

and print_explicit_field_class ppf =
  printer_explicit_field_class Og_precedence.Ck_toplevel ppf
;;

let print_text ppf = printer_text Og_precedence.Ck_toplevel ppf
;;

let print_explicit_position ppf =
  printer_explicit_position Og_precedence.Ck_toplevel ppf
;;

let print_explicit_document_value ppf =
  printer_explicit_document_value Og_precedence.Ck_toplevel ppf
;;

let print_explicit_binding ppf =
  printer_explicit_binding Og_precedence.Ck_toplevel ppf

and print_explicit_bindings ppf =
  printer_explicit_bindings Og_precedence.Ck_toplevel ppf
;;

let print_explicit_block_parameter_default ppf =
  printer_explicit_block_parameter_default Og_precedence.Ck_toplevel ppf

and print_explicit_block_parameter_defaults ppf =
  printer_explicit_block_parameter_defaults Og_precedence.Ck_toplevel ppf
;;

let print_explicit_annotation ppf =
  printer_explicit_annotation Og_precedence.Ck_toplevel ppf

and print_explicit_annotations ppf =
  printer_explicit_annotations Og_precedence.Ck_toplevel ppf
;;

let print_explicit_connectors ppf =
  printer_explicit_connectors Og_precedence.Ck_toplevel ppf

and print_explicit_connector ppf =
  printer_explicit_connector Og_precedence.Ck_toplevel ppf
;;

let print_explicit_custom_property_item ppf =
  printer_explicit_custom_property_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_custom_property_items ppf =
  printer_explicit_custom_property_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_custom_property ppf =
  printer_explicit_custom_property Og_precedence.Ck_toplevel ppf
;;

let print_explicit_dialog_parameters ppf =
  printer_explicit_dialog_parameters Og_precedence.Ck_toplevel ppf
;;

let print_explicit_link_data ppf =
  printer_explicit_link_data Og_precedence.Ck_toplevel ppf
;;

let print_explicit_mask_display_icon_frame ppf =
  printer_explicit_mask_display_icon_frame Og_precedence.Ck_toplevel ppf

and print_explicit_mask_display_icon_units ppf =
  printer_explicit_mask_display_icon_units Og_precedence.Ck_toplevel ppf

and print_explicit_mask_display_icon_opaque ppf =
  printer_explicit_mask_display_icon_opaque Og_precedence.Ck_toplevel ppf

and print_explicit_mask_display_icon_rotate ppf =
  printer_explicit_mask_display_icon_rotate Og_precedence.Ck_toplevel ppf

and print_explicit_mask_display_run_init_for_icon_redraw ppf =
  printer_explicit_mask_display_run_init_for_icon_redraw
    Og_precedence.Ck_toplevel
    ppf

and print_explicit_mask_display_script ppf =
  printer_explicit_mask_display_script Og_precedence.Ck_toplevel ppf

and print_explicit_mask_initialisation ppf =
  printer_explicit_mask_initialisation Og_precedence.Ck_toplevel ppf

and print_explicit_mask_help ppf =
  printer_explicit_mask_help Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_name ppf =
  printer_explicit_mask_parameter_name Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_type ppf =
  printer_explicit_mask_parameter_type Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_prompt ppf =
  printer_explicit_mask_parameter_prompt Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_value ppf =
  printer_explicit_mask_parameter_value Og_precedence.Ck_toplevel ppf

and print_explicit_mask_type ppf =
  printer_explicit_mask_type Og_precedence.Ck_toplevel ppf

and print_explicit_mask_description ppf =
  printer_explicit_mask_description Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_evaluate ppf =
  printer_explicit_mask_parameter_evaluate Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_enabled ppf =
  printer_explicit_mask_parameter_enabled Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_visible ppf =
  printer_explicit_mask_parameter_visible Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_callback ppf =
  printer_explicit_mask_parameter_callback Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_tunable ppf =
  printer_explicit_mask_parameter_tunable Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_internal ppf =
  printer_explicit_mask_parameter_internal Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_never_save ppf =
  printer_explicit_mask_parameter_never_save Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_read_only ppf =
  printer_explicit_mask_parameter_read_only Og_precedence.Ck_toplevel ppf
;;

let print_explicit_mask_parameter ppf =
  printer_explicit_mask_parameter Og_precedence.Ck_toplevel ppf
;;

let print_explicit_mask_display ppf =
  printer_explicit_mask_display Og_precedence.Ck_toplevel ppf
;;

let print_explicit_mask ppf =
  printer_explicit_mask Og_precedence.Ck_toplevel ppf
;;

let print_explicit_system ppf =
  printer_explicit_system Og_precedence.Ck_toplevel ppf

and print_explicit_block ppf =
  printer_explicit_block Og_precedence.Ck_toplevel ppf

and print_explicit_blocks ppf =
  printer_explicit_blocks Og_precedence.Ck_toplevel ppf

and print_explicit_block_item ppf =
  printer_explicit_block_item Og_precedence.Ck_toplevel ppf

and print_explicit_block_items ppf =
  printer_explicit_block_items Og_precedence.Ck_toplevel ppf

and print_explicit_port ppf =
  printer_explicit_port Og_precedence.Ck_toplevel ppf

and print_explicit_port_item ppf =
  printer_explicit_port_item Og_precedence.Ck_toplevel ppf

and print_explicit_port_items ppf =
  printer_explicit_port_items Og_precedence.Ck_toplevel ppf

and print_explicit_list ppf =
  printer_explicit_list Og_precedence.Ck_toplevel ppf

and print_explicit_object ppf =
  printer_explicit_object Og_precedence.Ck_toplevel ppf

and print_explicit_object_item ppf =
  printer_explicit_object_item Og_precedence.Ck_toplevel ppf

and print_explicit_object_items ppf =
  printer_explicit_object_items Og_precedence.Ck_toplevel ppf

and print_explicit_array ppf =
  printer_explicit_array Og_precedence.Ck_toplevel ppf

and print_explicit_array_item ppf =
  printer_explicit_array_item Og_precedence.Ck_toplevel ppf

and print_explicit_array_items ppf =
  printer_explicit_array_items Og_precedence.Ck_toplevel ppf

and print_explicit_matstruct_item ppf =
  printer_explicit_matstruct_item Og_precedence.Ck_toplevel ppf

and print_explicit_matstruct_items ppf =
  printer_explicit_matstruct_items Og_precedence.Ck_toplevel ppf

and print_explicit_cell ppf =
  printer_explicit_cell Og_precedence.Ck_toplevel ppf

and print_explicit_line ppf =
  printer_explicit_line Og_precedence.Ck_toplevel ppf

and print_explicit_lines ppf =
  printer_explicit_lines Og_precedence.Ck_toplevel ppf

and print_explicit_line_item ppf =
  printer_explicit_line_item Og_precedence.Ck_toplevel ppf

and print_explicit_line_items ppf =
  printer_explicit_line_items Og_precedence.Ck_toplevel ppf

and print_explicit_branch ppf =
  printer_explicit_branch Og_precedence.Ck_toplevel ppf

and print_explicit_branches ppf =
  printer_explicit_branches Og_precedence.Ck_toplevel ppf

and print_explicit_branch_item ppf =
  printer_explicit_branch_item Og_precedence.Ck_toplevel ppf

and print_explicit_branch_items ppf =
  printer_explicit_branch_items Og_precedence.Ck_toplevel ppf
;;

let print_config_manager_settings ppf =
  printer_config_manager_settings Og_precedence.Ck_toplevel ppf

and print_editor_settings ppf =
  printer_editor_settings Og_precedence.Ck_toplevel ppf

and print_simulation_settings ppf =
  printer_simulation_settings Og_precedence.Ck_toplevel ppf

and print_verification ppf =
  printer_verification Og_precedence.Ck_toplevel ppf

and print_external_mode ppf =
  printer_external_mode Og_precedence.Ck_toplevel ppf

and print_model_reference_settings ppf =
  printer_model_reference_settings Og_precedence.Ck_toplevel ppf

and print_concurrent_execution_settings ppf =
  printer_concurrent_execution_settings Og_precedence.Ck_toplevel ppf

and print_explicit_model_workspace ppf =
  printer_explicit_model_workspace Og_precedence.Ck_toplevel ppf

and print_explicit_configuration_set ppf =
  printer_explicit_configuration_set Og_precedence.Ck_toplevel ppf

and print_explicit_model_configuration ppf =
  printer_explicit_model_configuration Og_precedence.Ck_toplevel ppf

and print_explicit_system_defaults ppf =
  printer_explicit_system_defaults Og_precedence.Ck_toplevel ppf

and print_explicit_block_defaults ppf =
  printer_explicit_block_defaults Og_precedence.Ck_toplevel ppf

and print_explicit_annotation_defaults ppf =
  printer_explicit_annotation_defaults Og_precedence.Ck_toplevel ppf

and print_explicit_line_defaults ppf =
  printer_explicit_line_defaults Og_precedence.Ck_toplevel ppf

and print_explicit_mask_defaults ppf =
  printer_explicit_mask_defaults Og_precedence.Ck_toplevel ppf

and print_explicit_mask_parameter_defaults ppf =
  printer_explicit_mask_parameter_defaults Og_precedence.Ck_toplevel ppf
;;

let print_explicit_section ppf =
  printer_explicit_section Og_precedence.Ck_toplevel ppf
;;

let print_explicit_id ppf = printer_explicit_id Og_precedence.Ck_toplevel ppf
;;

let print_explicit_ssid_number ppf =
  printer_explicit_ssid_number Og_precedence.Ck_toplevel ppf
;;

let print_explicit_debug ppf =
  printer_explicit_debug Og_precedence.Ck_toplevel ppf
;;

let print_explicit_target_items ppf =
  printer_explicit_target_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_target ppf =
  printer_explicit_target Og_precedence.Ck_toplevel ppf
;;

let print_explicit_active_state_output ppf =
  printer_explicit_active_state_output Og_precedence.Ck_toplevel ppf
;;

let print_explicit_src ppf =
  printer_explicit_src Og_precedence.Ck_toplevel ppf
;;

let print_explicit_dst ppf =
  printer_explicit_dst Og_precedence.Ck_toplevel ppf
;;

let print_explicit_slide ppf =
  printer_explicit_slide Og_precedence.Ck_toplevel ppf
;;

let print_explicit_autogen_item ppf =
  printer_explicit_autogen_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_autogen_items ppf =
  printer_explicit_autogen_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_autogen ppf =
  printer_explicit_autogen Og_precedence.Ck_toplevel ppf
;;

let print_explicit_transition_item ppf =
  printer_explicit_transition_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_transition_items ppf =
  printer_explicit_transition_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_transition ppf =
  printer_explicit_transition Og_precedence.Ck_toplevel ppf
;;

let print_explicit_sub_view_s ppf =
  printer_explicit_sub_view_s Og_precedence.Ck_toplevel ppf
;;

let print_explicit_fixpt ppf =
  printer_explicit_fixpt Og_precedence.Ck_toplevel ppf
;;

let print_explicit_type_item ppf =
  printer_explicit_type_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_type_items ppf =
  printer_explicit_type_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_type ppf =
  printer_explicit_type Og_precedence.Ck_toplevel ppf
;;

let print_explicit_stateflow_array ppf =
  printer_explicit_stateflow_array Og_precedence.Ck_toplevel ppf
;;

let print_explicit_range ppf =
  printer_explicit_range Og_precedence.Ck_toplevel ppf
;;

let print_explicit_props_item ppf =
  printer_explicit_props_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_props_items ppf =
  printer_explicit_props_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_props ppf =
  printer_explicit_props Og_precedence.Ck_toplevel ppf
;;

let print_explicit_data_item ppf =
  printer_explicit_data_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_data_items ppf =
  printer_explicit_data_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_data_id ppf =
  printer_explicit_data_id Og_precedence.Ck_toplevel ppf
;;

let print_explicit_data_ssid ppf =
  printer_explicit_data_ssid Og_precedence.Ck_toplevel ppf
;;

let print_explicit_data ppf =
  printer_explicit_data Og_precedence.Ck_toplevel ppf
;;

let print_explicit_junction_item ppf =
  printer_explicit_junction_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_junction_items ppf =
  printer_explicit_junction_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_junction ppf =
  printer_explicit_junction Og_precedence.Ck_toplevel ppf
;;

let print_explicit_event_item ppf =
  printer_explicit_event_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_event_items ppf =
  printer_explicit_event_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_event ppf =
  printer_explicit_event Og_precedence.Ck_toplevel ppf
;;

let print_explicit_eml ppf =
  printer_explicit_eml Og_precedence.Ck_toplevel ppf
;;

let print_explicit_simulink ppf =
  printer_explicit_simulink Og_precedence.Ck_toplevel ppf
;;

let print_explicit_layout_info ppf =
  printer_explicit_layout_info Og_precedence.Ck_toplevel ppf
;;

let print_explicit_table_ui_prefs ppf =
  printer_explicit_table_ui_prefs Og_precedence.Ck_toplevel ppf
;;

let print_explicit_state_cell ppf =
  printer_explicit_state_cell Og_precedence.Ck_toplevel ppf
;;

let print_explicit_transition_cell_items ppf =
  printer_explicit_transition_cell_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_transition_cell ppf =
  printer_explicit_transition_cell Og_precedence.Ck_toplevel ppf
;;

let print_explicit_truth_table_item ppf =
  printer_explicit_truth_table_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_truth_table_items ppf =
  printer_explicit_truth_table_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_truth_table ppf =
  printer_explicit_truth_table Og_precedence.Ck_toplevel ppf
;;

let print_explicit_chart_item ppf =
  printer_explicit_chart_item Og_precedence.Ck_toplevel ppf

and print_explicit_chart_items ppf =
  printer_explicit_chart_items Og_precedence.Ck_toplevel ppf

and print_explicit_chart ppf =
  printer_explicit_chart Og_precedence.Ck_toplevel ppf

and print_explicit_table_item ppf =
  printer_explicit_table_item Og_precedence.Ck_toplevel ppf

and print_explicit_table_items ppf =
  printer_explicit_table_items Og_precedence.Ck_toplevel ppf

and print_explicit_table ppf =
  printer_explicit_table Og_precedence.Ck_toplevel ppf

and print_explicit_child ppf =
  printer_explicit_child Og_precedence.Ck_toplevel ppf

and print_explicit_children ppf =
  printer_explicit_children Og_precedence.Ck_toplevel ppf

and print_explicit_state_item ppf =
  printer_explicit_state_item Og_precedence.Ck_toplevel ppf

and print_explicit_state_items ppf =
  printer_explicit_state_items Og_precedence.Ck_toplevel ppf

and print_explicit_state ppf =
  printer_explicit_state Og_precedence.Ck_toplevel ppf
;;

let print_explicit_machine_item ppf =
  printer_explicit_machine_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_machine_items ppf =
  printer_explicit_machine_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_machine ppf =
  printer_explicit_machine Og_precedence.Ck_toplevel ppf
;;

let print_explicit_instance_items ppf =
  printer_explicit_instance_items Og_precedence.Ck_toplevel ppf
;;

let print_explicit_instance ppf =
  printer_explicit_instance Og_precedence.Ck_toplevel ppf
;;

let print_explicit_stateflow_section_items ppf =
  printer_explicit_stateflow_section_items Og_precedence.Ck_toplevel ppf

and print_explicit_stateflow_section_item ppf =
  printer_explicit_stateflow_section_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_stateflow_section ppf =
  printer_explicit_stateflow_section Og_precedence.Ck_toplevel ppf
;;

let print_explicit_implementation_section_item ppf =
  printer_explicit_implementation_section_item Og_precedence.Ck_toplevel ppf
;;

let print_explicit_model_sections ppf =
  printer_explicit_model_sections Og_precedence.Ck_toplevel ppf
;;

let print_explicit_blockdiagram ppf =
  printer_explicit_blockdiagram Og_precedence.Ck_toplevel ppf
;;

let print_explicit_xml_model ppf =
  printer_explicit_xml_model Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.5+dev0 generated code. *)
