#######################################################################
#                                                                     #
#                          Ocaml-xml Library                          #
#                                                                     #
#                   Pierre Weis, INRIA Rocquencourt                   #
#                                                                     #
#  Copyright 2013-2018,                                               #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved.                                               #
#                                                                     #
#  This file is distributed under the terms of the BSD License.       #
#                                                                     #
#######################################################################

# $Id: Caml_commands.mk,v 1.1 2019-05-10 14:02:41 jpc Exp $

# This Makefile is a generic configuration Makefile for Caml applications.
# This Makefile is independent of the software to be built and
# defines basic Unix commands and tools for Caml compilation.

# This Makefile should be included at the beginning of the Makefile
# that handles a set of Caml files (to build a library or an application).
# Simply write at the beginning of your Makefile:
# include <path_to_Caml_commands.mk>/Caml_commands.mk
# For instance, assuming that Caml_commands.mk is in sub directory
# config of the main directory for your project, write the line
# include $(MAIN_DIR)/config/Caml_commands.mk

# $(OCAML_PREFIX_DIR) is inherited from External.mk

# The Caml compilers default configurations.
OCAML_BIN_DIR=$(OCAML_PREFIX_DIR)/bin
OCAML_LIB_DIR=$(OCAML_PREFIX_DIR)/lib

CAML_INCLUDES=-I $(OCAML_LIB_DIR)

CAMLC_BYT_INCLUDES=$(CAML_INCLUDES)
CAMLC_BIN_INCLUDES=$(CAML_INCLUDES)
CAMLC_DEP_INCLUDES=$(CAML_INCLUDES)

# For safe 4.03 compilation
#CAMLC_COMP_FLAGS_403=-strict-sequence -strict-formats -safe-string

# For pre 4.03 compilation
CAMLC_COMP_FLAGS=-strict-sequence

# Should be
#CAMLC_BYT_FLAGS=$(CAMLC_COMP_FLAGS) -w A -warn-error A -annot -g
# We use more lenient warn-error Ae due to numerous
# fragile matches in the matlab directory.
CAMLC_BYT_FLAGS=$(CAMLC_COMP_FLAGS) -w A -warn-error Ae -annot -g
CAMLC_BIN_FLAGS=$(CAMLC_COMP_FLAGS)\
 -w Ae -warn-error A -inline 20

CAMLC_BYT_CONFIG=$(CAMLC_BYT_FLAGS) $(CAMLC_BYT_INCLUDES)
CAMLC_BIN_CONFIG=$(CAMLC_BIN_FLAGS) $(CAMLC_BIN_INCLUDES)

CAMLC_BYT_LIBS=
CAMLC_BIN_LIBS=$(CAMLC_BYT_LIBS:.cma=.cmxa)
CAMLC_BYT_LIBR_CONFIG=$(CAMLC_BYT_INCLUDES) -a $(CAMLC_BYT_LIBS)
CAMLC_BIN_LIBR_CONFIG=-verbose $(CAMLC_BIN_INCLUDES) -a $(CAMLC_BIN_LIBS)

CAMLC_YAC_FLAGS=-v
CAMLC_YAC_CONFIG=$(CAMLC_YAC_FLAGS)

CAMLC_LEX_FLAGS=
CAMLC_LEX_CONFIG=$(CAMLC_LEX_FLAGS)

CAMLC_DEP_FLAGS=-slash
CAMLC_DEP_CONFIG=$(CAMLC_DEP_FLAGS) $(CAMLC_DEP_INCLUDES)

# For Caml C compilation
CAMLC_LIB_DIR=$(OCAML_LIB_DIR)
CAMLC_SYS_INCLUDES=-ccopt -I$(CAMLC_LIB_DIR)/caml

# For instance
# CAMLC_C_INCLUDES=-ccopt -I$(X11_INCLUDE)
CAMLC_C_INCLUDES=
CAMLC_CC_INCLUDES=$(CAMLC_SYS_INCLUDES) $(CAMLC_C_INCLUDES)

CAMLC_CC_FLAGS=
CAMLC_CC_CONFIG=$(CAMLC_CC_FLAGS) $(CAMLC_CC_INCLUDES)

# The Caml compilers and auxilliaries commands.
CAMLC_BYT_COMPILER=$(OCAML_BIN_DIR)/ocamlc
CAMLC_BIN_COMPILER=$(OCAML_BIN_DIR)/ocamlopt
CAMLC_YAC_COMPILER=$(OCAML_BIN_DIR)/ocamlyacc
CAMLC_LEX_COMPILER=$(OCAML_BIN_DIR)/ocamllex
CAMLC_DEP_COMPILER=$(OCAML_BIN_DIR)/ocamldep

CAMLC_BYT=$(CAMLC_BYT_COMPILER) $(CAMLC_BYT_CONFIG)
CAMLC_BIN=$(CAMLC_BIN_COMPILER) $(CAMLC_BIN_CONFIG)
CAMLC_DEP=$(CAMLC_DEP_COMPILER) $(CAMLC_DEP_CONFIG)

CAMLC_BYT_LIBR=$(CAMLC_BYT_COMPILER) $(CAMLC_BYT_LIBR_CONFIG)
CAMLC_BIN_LIBR=$(CAMLC_BIN_COMPILER) $(CAMLC_BIN_LIBR_CONFIG)

CAMLC_YAC=$(CAMLC_YAC_COMPILER) $(CAMLC_YAC_CONFIG)
CAMLC_LEX=$(CAMLC_LEX_COMPILER) $(CAMLC_LEX_CONFIG)
CAMLC_DEP=$(CAMLC_DEP_COMPILER) $(CAMLC_DEP_CONFIG)

# The Caml compilation of C files
CAMLC_CC_COMPILER=$(CAMLC_BYT_COMPILER)
CAMLC_CC=$(CAMLC_CC_COMPILER) $(CAMLC_CC_CONFIG)

# Tools: the pretty printer program generator
CAMLC_PPR_COMPILER=$(OCAML_BIN_DIR)/ocamlprintc

CAMLC_PPR_FLAGS=
CAMLC_PPR_INCLUDES=$(CAML_INCLUDES)
CAMLC_PPR_CONFIG=$(CAMLC_PPR_FLAGS) $(CAMLC_PPR_INCLUDES)

CAMLC_PPR=$(CAMLC_PPR_COMPILER) $(CAMLC_PPR_CONFIG)

# Tools: the moca program generator
CAMLC_MOCA_INCLUDES=

CAMLC_MOCA_COMPILER=$(OCAML_BIN_DIR)/mocac

CAMLC_MOCA_FLAGS=
CAMLC_MOCA_CONFIG=$(CAMLC_MOCA_FLAGS) $(CAMLC_MOCA_INCLUDES)
CAMLC_MOCA=$(CAMLC_MOCA_COMPILER) $(CAMLC_MOCA_CONFIG)

# Tools: defining some make macros for linking.
WITH_BYT_UNIX=unix.cma -cclib -lunix
WITH_BYT_STR=str.cma -cclib -lstr

WITH_BIN_UNIX=$(WITH_BYT_UNIX:.cma=.cmxa)
WITH_BIN_STR=$(WITH_BYT_STR:.cma=.cmxa)

# Tools: documentation generation.
CAMLC_DOC_INCLUDES=

CAMLC_DOC_COMPILER=$(OCAML_BIN_DIR)/ocamldoc

CAMLC_DOC_FLAGS=-d doc -v
CAMLC_DOC_HTML_FLAGS=-html -colorize-code $(CAMLC_DOC_FLAGS)
CAMLC_DOC_LATEX_FLAGS=-latex $(CAMLC_DOC_FLAGS)

CAMLC_DOC_HTML_CONFIG=$(CAMLC_DOC_HTML_FLAGS) $(CAMLC_DOC_INCLUDES)
CAMLC_DOC_LATEX_CONFIG=$(CAMLC_DOC_LATEX_FLAGS) $(CAMLC_DOC_INCLUDES)

CAMLC_DOC_HTML=$(CAMLC_DOC_COMPILER) $(CAMLC_DOC_HTML_CONFIG)
CAMLC_DOC_LATEX=$(CAMLC_DOC_COMPILER) $(CAMLC_DOC_LATEX_CONFIG)

# Tools: Htmlc file generator.
# Configuration of Htmlc to generate Web pages:

HTMLC_COMPILER=$(HTMLC_BIN_DIR)/htmlc

# where are configuration include files
#HTMLC_CONF_INCLUDES=-I $(MAIN_DIR)/Data/Includes
HTMLC_CONF_INCLUDES=-I $(CONFIG_DIR)

# Where are the configuration environment files for Htmlc.
#HTMLC_CONF_ENV=-env $(CONFIG_DIR)/htmlc_config.ml
HTMLC_CONF_ENV=-env ocamlxml.env

HTMLC_INCLUDES=$(HTMLC_CONF_ENV) $(HTMLC_CONF_INCLUDES)

# Additional flags to set in Makefiles
HTMLC_FLAGS=-lang en -honor_line_continuation true
HTMLC_CONFIG=$(HTMLC_FLAGS) $(HTMLC_INCLUDES)

HTMLC=$(HTMLC_COMPILER) $(HTMLC_CONFIG)

HTMLC_CONDITIONAL=$(HTMLC) -env conditional.env
HTMLC_LT_4_02=$(HTMLC_CONDITIONAL) -D OCAML_VERSION 4.01
HTMLC_GE_4_02=$(HTMLC_CONDITIONAL) -D OCAML_VERSION 4.02

# To handle banners
#BH_INCLUDES=-I $(CONFIG_DIR)
BH_INCLUDES=

BH_COMPILER=$(BH_BIN_DIR)/bh

BH_FLAGS=
BH_CONFIG=$(BH_FLAGS) $(BH_INCLUDES)
BH=$(BH_COMPILER) $(BH_CONFIG)
