(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 1997-2018 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Clement Franchini <Clement.Franchini@inria.fr>                     *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_outputer.mli,v 1.1 2019-05-10 14:02:41 jpc Exp $ *)

(** {3 Runtime library for pretty-outputers generated with OCamlGen} *)

open Og_types;;

(** {6 Printers for basic monomorphic values} *)

val outputer_Unit : unit printer;;
val outputer_Bool : bool printer;;
val outputer_Char : char printer;;
val outputer_String : string printer;;
val outputer_Bytes : Og_types.bytes printer;;

val outputer_Int : int printer;;
val outputer_Int32 : int32 printer;;
val outputer_Int64 : int64 printer;;
val outputer_Nativeint : nativeint printer;;
val outputer_Float : float printer;;

(** {6 Printers for basic polymorphic values} *)

val outputer_Format :
  ('a, 'b, 'c, ('a, 'b, 'c) format) printer_lifter3
;;
val outputer_Format4 :
  ('a, 'b, 'c, 'd, ('a, 'b, 'c, 'd) format4) printer_lifter4
;;
val outputer_Format6 :
  ('a, 'b, 'c, 'd, 'e, 'f, ('a, 'b, 'c, 'd, 'e, 'f) format6) printer_lifter6
;;

val outputer_List : ('a, 'a list) printer_lifter;;
val outputer_Array : ('a, 'a array) printer_lifter;;

val outputer_Option : ('a, 'a option) printer_lifter;;
val outputer_Ref : ('a, 'a ref) printer_lifter;;

(** {6 Printers for functional, polymorphic, abstract and external values} *)

val outputer_Fun : ('a, 'b, 'a -> 'b) printer_lifter2;;
val outputer_Poly : 'a printer;;
val outputer_Abstract : 'a printer;;
val outputer_External : 'a printer;;
