(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 1997-2018 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Clement Franchini <Clement.Franchini@inria.fr>                     *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_printer_xml.mli,v 1.1 2019-05-10 14:02:41 jpc Exp $ *)

(** {3 Runtime library for XML pretty-printers generated with OCamlGen} *)

open Og_types;;

(** {6 Basic xml functions} *)

val xml_escape_string : string -> string;;

(** {6 Basic printers} *)

val printer_xml_Unit : unit printer;;
val printer_xml_Bool : bool printer;;
val printer_xml_Char : char printer;;
val printer_xml_String : string printer;;
val printer_xml_Bytes : Og_types.bytes printer;;

val printer_xml_Int : int printer;;
val printer_xml_Int32 : int32 printer;;
val printer_xml_Int64 : int64 printer;;
val printer_xml_Nativeint : nativeint printer;;
val printer_xml_Float : float printer;;

(** {6 Printers for basic polymorphic values} *)

val printer_xml_Format :
  ('a, 'b, 'c, ('a, 'b, 'c) format) printer_lifter3
;;
val printer_xml_Format4 :
  ('a, 'b, 'c, 'd, ('a, 'b, 'c, 'd) format4) printer_lifter4
;;
val printer_xml_Format6 :
  ('a, 'b, 'c, 'd, 'e, 'f, ('a, 'b, 'c, 'd, 'e, 'f) format6) printer_lifter6
;;

val printer_xml_List : ('a, 'a list) printer_lifter;;
val printer_xml_Array : ('a, 'a array) printer_lifter;;

val printer_xml_Option : ('a, 'a option) printer_lifter;;
val printer_xml_Ref : ('a, 'a ref) printer_lifter;;

(** {6 Printers for functional, polymorphic, abstract and external values} *)

val printer_xml_Fun : ('a, 'b, 'a -> 'b) printer_lifter2;;
val printer_xml_Poly : 'a printer;;
val printer_xml_Abstract : 'a printer;;
val printer_xml_Lazy : unit printer;;
val printer_xml_External : 'a printer;;
