(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uint8.mli,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

(** Implementing one byte unsigned natural numbers. *)

type uint8 = char;;

type saturation = Bounds | Ring | Error;;

val get_saturation : unit -> saturation;;
val set_saturation : saturation -> unit;;

val size : int;;

val int_max_int : int;;
val int_min_int : int;;

val max_int : char;;
val min_int : char;;

val zero : char;;
val one : char;;

val check_range : int -> unit;;

val saturate : int -> char;;

val nth : char -> int -> int;;
val nth_bit : char -> int -> int;;

val bits_of : char -> int list;;

val add : char -> char -> char;;

val sub : char -> char -> char;;

val mul : char -> char -> char;;

val div : char -> char -> char;;

val rem : char -> char -> char;;

val succ : char -> char;;

val pred : char -> char;;

val logand : char -> char -> char;;

val logor : char -> char -> char;;

val lognot : char -> char;;

val shift_left : char -> int -> char;;

val shift_right_logical : char -> int -> char;;

val of_int : int -> char;;

val to_int : char -> int;;

val of_float : float -> char;;

val to_float : char -> float;;

val of_string : string -> char;;

val to_string : char -> string;;

val bits_of_float : float -> char;;

val float_of_bits : char -> float;;

type t = uint8;;

val compare : char -> char -> int;;

