(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ulist1.ml,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type 'a list1 = 'a * 'a list;;

type 'a t = 'a list1;;

let length (_, l) = 1 + List.length l;;

let to_list (h, l) = h :: l;;

let of_list = function
  | [] -> invalid_arg "List1.of_list"
  | h :: l -> h, l
;;

let cons x l = x, to_list l;;

let rev_append (h, l) ll = of_list (List.rev_append l (h :: ll));;

let rev l = rev_append l [];;

let map f (h, l) = (f h, List.map f l);;

let iter f (h, l) = f h; List.iter f l;;

let fold_left f accu (h, l) = List.fold_left f (f accu h) l;;
let foldl f (a, (h, l)) = List.fold_left f (f a h) l;;

let fold_right f (h, l) accu = f h (List.fold_right f l accu);;

let rec foldr f (a, (h, l)) =
  match l with
  | [] -> f a h
  | a1 :: l1 -> f a (foldr f (h, (a1, l1)))
;;

let fold_map f env (h, l) =
  let env, h = f env h in
  let env, l =
    Ulist.fold_map f env l in
  env, (h, l)
;;

let rev_fold_flat_map_end f env (x, l) raccu =
  let env, lres = f env x in
  let raccu = List.rev_append lres raccu in
  Ulist.rev_fold_flat_map_end f env l raccu
;;

let rev_fold_flat_map f env l = rev_fold_flat_map_end f env l [];;

let fold_flat_map f env l =
  let env, l = rev_fold_flat_map f env l in
  env, List.rev l
;;

let map2 f (h1, l1) (h2, l2) = (f h1 h2, List.map2 f l1 l2);;

let iter2 f (h1, l1) (h2, l2) = f h1 h2; List.iter2 f l1 l2;;

let find p (h, l) = if p h then h else List.find p l;;

let find_all p (h, l) =
  if p h then h :: List.find_all p l else List.find_all p l
;;

let filter = find_all;;

let partition p l = List.partition p (to_list l)
;;

let max_snd_assoc cmp (x0, l) =
  List.fold_left
    (fun (_, y0 as x0) (_, y as x) ->
      if cmp y0 y < 0 then x else x0)
    x0 l
;;

let split ((x1, x2), l) =
  let l1, l2 = List.split l in
  (x1, l1), (x2, l2)
;;

let combine (x1, l1) (x2, l2) = (x1, x2), List.combine l1 l2;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
