(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uname.ml,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type t = {
  name : Name.t;
  uid : Uid.t;
}
;;

let fresh name = {
  name = name;
  uid = Uid.fresh ();
}
;;

let name uname = uname.name;;
let uid uname = uname.uid;;

let dummy name = {
  name = name;
  uid = Uid.dummy;
}
;;

let is_dummy uname = uid uname = Uid.dummy;;

let make_dummy s = dummy (Name.to_t s);;

let compare uname1 uname2 =
  Pervasives.compare (uid uname1) (uid uname2)
;;

let same uname1 uname2 = compare uname1 uname2 = 0;;

let same_name uname1 uname2 =
  Name.same_name (name uname1) (name uname2)
;;

let to_string uname =
  Printf.sprintf "%s%s"
    (Name.to_string (name uname))
    (Uid.to_string (uid uname))
;;

let to_delimited_string prefix suffix uname =
  Printf.sprintf "%s%s%s%s"
    (Name.to_string (name uname))
    prefix
    (Uid.to_string (uid uname))
    suffix
;;

let string_of_name uname = Name.to_string (name uname);;

(******************************************
Old implementation using an alias to Name.t.

type t = Name.t;;

exception Not_a_unique_name of Name.t;;

let table = Hashtbl.create 1024;;

let make name =
  if Hashtbl.mem table name then raise (Not_a_unique_name name) else
  begin
    Hashtbl.add table name ();
    name
  end
;;

external from : t -> Name.t = "%identity";;

let fresh name =
  let id = Uid.fresh () in
  let new_name = Name.make (Name.from name ^ Uid.to_string id) in
  make new_name
;;

let to_string uname = Name.to_string (from uname);;

external from : t -> Name.t = "%identity";;
val to_string : Name.t -> string;;
********************************************)

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
