(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_parser_error.ml,v 1.1 2019-05-10 14:00:50 jpc Exp $ *)

type error =
   | Invalid_xml_name of string
   | Invalid_closing_tag of Xml_basics_ast.tag_name * Xml_basics_ast.tag_name
   | Invalid_att_type of Xml_basics_ast.nm_token
   | Invalid_content_spec of Xml_basics_ast.nm_token
   | Invalid_entity_value of Xml_basics_ast.att_value * string
(** The various errors when parsing. *)
;;

exception Error of error * Lexing.position * Lexing.position;;

(** {6 Explaining parsing errors} *)

let report_error ppf = function
  | Invalid_xml_name s ->
    Format.fprintf ppf "Invalid XML name %S." s
  | Invalid_closing_tag (btag, etag) ->
    Format.fprintf ppf
      "Invalid markup: opening (%S) and closing (%S) tag names do not match."
      (Xml_basics_ast.of_xml_name btag)
      (Xml_basics_ast.of_xml_name etag)
  | Invalid_att_type nmtoken ->
    Format.fprintf ppf "Invalid attribute type declaration %S." nmtoken
  | Invalid_content_spec nmtoken ->
    Format.fprintf ppf "Invalid content specification declaration %S." nmtoken
  | Invalid_entity_value (att_value, message) ->
    Format.fprintf ppf
      "Invalid entity_value %a (%s)"
      Xml_basics_ast_print.print_att_value att_value
      message
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
