(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_to_tokens.ml,v 1.1 2019-05-10 14:00:50 jpc Exp $ *)

let read_file fname =
  let ic, lexbuf = Location.lexing_from_file fname in
  let start_token =
    try
      let start_token = Xml_lexer.start lexbuf in
      close_in ic;
      start_token
    with
    | x ->
      close_in ic;
      raise x in

  let ic, lexbuf = Location.lexing_from_file fname in

  let lexer =
    match start_token with
    | Xml_lexer_types.Start_prolog -> (fun () -> Xml_lexer.prolog lexbuf)
    | Xml_lexer_types.Start_element -> (fun () -> Xml_lexer.element lexbuf) in
  let rec loop i =
    let token = lexer () in
    Format.printf "Token %d: %a@."
      i
      Xml_parser_print.print_token token;
    if token <> Xml_parser.Eof then
      loop (succ i) in
  try
    loop 0;
    close_in ic
  with
  | x ->
    close_in ic;
    raise x
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
  End:
*)
