#######################################################################
#                                                                     #
#                          Ocaml-xml Library                          #
#                                                                     #
#                   Pierre Weis, INRIA Rocquencourt                   #
#                                                                     #
#  Copyright 2013-2018,                                               #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved.                                               #
#                                                                     #
#  This file is distributed under the terms of the BSD License.       #
#                                                                     #
#######################################################################

# $Id: Objs.mk,v 1.1 2019-05-10 14:00:17 jpc Exp $

# The list of all the Caml compiled objects to make in this project.
# Each source directory has its specific make variable associated
# with the relevant list of object files.
#
# This file is shared and supposed to be included by
# each make file that needs to link and build a Caml executable.
# See below for a template of make file.

# Inherited from Main_DIR/Makefile
#SRC_DIR=$(MAIN_DIR)/src
#CONFIG_DIR=$(MAIN_DIR)/config

BASICS_DIR=$(SRC_DIR)/basics
CONFIGURATION_DIR=$(SRC_DIR)/configuration

OCAMLGEN_CVS_PARENT_DIR=$(SRC_DIR)
OCAMLGEN_CVS_DIR_NAME=ocamlgen.cvs
OCAMLGEN_CVS_DIR=$(OCAMLGEN_CVS_PARENT_DIR)/$(OCAMLGEN_CVS_DIR_NAME)
OCAMLGEN_CVS_LIB_DIR=$(OCAMLGEN_CVS_DIR)/src/lib

OCAMLGEN_DIR=$(SRC_DIR)/ocamlgen
OCAMLGEN_LIB_DIR=$(OCAMLGEN_DIR)/src/lib

XML_DIR=$(SRC_DIR)/xml
XML_PARSING_DIR=$(XML_DIR)/parsing
XML_EXPAND_DIR=$(XML_DIR)/expand
XML_LIBRARY_DIR=$(SRC_DIR)/library

OCAML_XML_SUB_DIRS=\
 $(OCAMLGEN_LIB_DIR)\
 $(BASICS_DIR)\
 $(CONFIGURATION_DIR)\
 $(XML_PARSING_DIR)\
 $(XML_EXPAND_DIR)\
 $(XML_LIBRARY_DIR)\

OCAML_XML_CAML_INCLUDES=\
 -I $(OCAMLGEN_LIB_DIR)\
 -I $(BASICS_DIR)\
 -I $(CONFIGURATION_DIR)\
 -I $(XML_PARSING_DIR)\
 -I $(XML_EXPAND_DIR)\
 -I $(XML_LIBRARY_DIR)\

# Directory XXX template
# All .ml files without interface (corresponding .mli file).
# XXX_ML_ONLY_FILES=\
#  $(XXX_DIR)/yyy_lexer.ml\
#  $(XXX_DIR)/yyy_print.ml\

# All .mli files in the directory.
# XXX_MLI_FILES=\
#  $(XXX_DIR)/yyy_parser.mli\
#  $(XXX_DIR)/yyy.mli\

# All .ml files in the directory.
# XXX_ML_FILES=\
#  $(XXX_DIR)/yyy_parser.ml\
#  $(XXX_DIR)/yyy_lexer.ml\
#  $(XXX_ML_ONLY_FILES)\

# All generated Caml source files in the directory.
# XXX_CAML_GENERATED_FILES=\
#  $(XXX_DIR)/yyy_print.ml\
#  $(XXX_DIR)/yyy_parser.mli\
#  $(XXX_DIR)/yyy_parser.ml\
#  $(XXX_DIR)/yyy_lexer.ml\

# All Caml source files in the directory,
# (including Yacc and Lex files).
# XXX_CAML_FILES=\
#  $(XXX_DIR)/yyy_parser.mly\
#  $(XXX_DIR)/yyy_lexer.mll\
#  $(XXX_MLI_FILES)\
#  $(XXX_ML_FILES)\

# Generic
# All bytcode compiler objects.
# XXX_BYT_OBJS=\
#  $(XXX_ML_FILES:.ml=.cmo)\

# Generic
# All interfaces to install
# source files and/or byte compiled objects.
# XXX_BYT_FILES_TO_INSTALL=\
#  $(XXX_MLI_FILES)\
#  $(XXX_ML_FILES)\
#  $(XXX_MLI_FILES:.mli=.cmi)\
#  $(XXX_ML_ONLY_FILES:.ml=.cmi)\
#  $(XXX_BYT_OBJS)\

# Generic
# All native compiler objects to install.
# XXX_BIN_FILES_TO_INSTALL=\
#  $(XXX_ML_FILES:.ml=.cmx)\

# Directory src/ocamlgen

OCAMLGEN_LIB_ML_ONLY_FILES=\
 $(OCAMLGEN_LIB_DIR)/og_ident_print.ml\
 $(OCAMLGEN_LIB_DIR)/og_precedence_print.ml\
 $(OCAMLGEN_LIB_DIR)/format_print.ml\
 $(OCAMLGEN_LIB_DIR)/format_print_xml.ml\
 $(OCAMLGEN_LIB_DIR)/format_output.ml\
 $(OCAMLGEN_LIB_DIR)/lazy_print.ml\
 $(OCAMLGEN_LIB_DIR)/lazy_print_xml.ml\
 $(OCAMLGEN_LIB_DIR)/lazy_output.ml\
 $(OCAMLGEN_LIB_DIR)/lexing_print.ml\
 $(OCAMLGEN_LIB_DIR)/lexing_print_xml.ml\

OCAMLGEN_LIB_ML_FILES=\
 $(OCAMLGEN_LIB_DIR)/og_ident.ml\
 $(OCAMLGEN_LIB_DIR)/og_ident_pprint.ml\
 $(OCAMLGEN_LIB_DIR)/og_precedence.ml\
 $(OCAMLGEN_LIB_DIR)/og_printer.ml\
 $(OCAMLGEN_LIB_DIR)/og_printer_xml.ml\
 $(OCAMLGEN_LIB_DIR)/og_outputer.ml\
 $(OCAMLGEN_LIB_DIR)/og_lib_runtime.ml\
 $(OCAMLGEN_LIB_ML_ONLY_FILES)\

OCAMLGEN_LIB_BYT_OBJS=\
 $(OCAMLGEN_LIB_ML_FILES:.ml=.cmo)\

OCAMLGEN_GENERATED_FILES=\
 $(OCAMLGEN_LIB_DIR)/og_types.mli-lt-4.02\
 $(OCAMLGEN_LIB_DIR)/og_types.mli-ge-4.02\
 $(OCAMLGEN_LIB_DIR)/og_printer.ml-lt-4.02\
 $(OCAMLGEN_LIB_DIR)/og_printer.ml-ge-4.02\

OCAMLGEN_LIB_MLI_FILES=\
 $(OCAMLGEN_LIB_DIR)/og_ident.mli\
 $(OCAMLGEN_LIB_DIR)/og_ident_pprint.mli\
 $(OCAMLGEN_LIB_DIR)/og_lib_runtime.mli\
 $(OCAMLGEN_LIB_DIR)/og_outputer.mli\
 $(OCAMLGEN_LIB_DIR)/og_precedence.mli\
 $(OCAMLGEN_LIB_DIR)/og_printer.mli\
 $(OCAMLGEN_LIB_DIR)/og_printer_xml.mli\
 $(OCAMLGEN_LIB_DIR)/og_types.mli\

OCAMLGEN_LIB_CAML_FILES=\
 $(OCAMLGEN_LIB_MLI_FILES)\
 $(OCAMLGEN_LIB_ML_FILES)\

OCAMLGEN_LIB_CAML_GENERATED_FILES=\

OCAMLGEN_LIB_BYT_FILES_TO_INSTALL=\
 $(OCAMLGEN_LIB_MLI_FILES)\
 $(OCAMLGEN_LIB_MLI_FILES:.mli=.cmi)\
 $(OCAMLGEN_LIB_ML_ONLY_FILES:.ml=.cmi)\
 $(OCAMLGEN_LIB_ML_FILES)\
 $(OCAMLGEN_LIB_ML_FILES:.ml=.cmo)\

OCAMLGEN_LIB_BIN_FILES_TO_INSTALL=\
 $(OCAMLGEN_LIB_ML_FILES:.ml=.cmx)\

# Og_ident_print and Og_precedence_print are part of ocamlgen library
# there is no case to generate them since it should be done by ocamlgen
# developer team.
#$(OCAMLGEN_LIB_DIR)/og_ident_print.ml: $(OCAMLGEN_LIB_DIR)/og_ident.mli
#	$(CAML_GEN) $(OCAMLGEN_LIB_DIR)/og_ident.mli
#$(OCAMLGEN_LIB_DIR)/og_precedence_print.ml: $(OCAMLGEN_LIB_DIR)/og_precedence.mli
#	$(CAML_GEN) $(OCAMLGEN_LIB_DIR)/og_precedence.mli

# Directory src/basics

BASICS_ML_ONLY_FILES=\
 $(BASICS_DIR)/unatural_types_print.ml\
 $(BASICS_DIR)/unatural_print.ml\
 $(BASICS_DIR)/ident_print.ml\
 $(BASICS_DIR)/ident_pprint.ml\
 $(BASICS_DIR)/path_print.ml\
 $(BASICS_DIR)/location_print.ml\
 $(BASICS_DIR)/ulist_print.ml\
 $(BASICS_DIR)/ulist1_print.ml\
 $(BASICS_DIR)/ulist2_print.ml\
 $(BASICS_DIR)/binding_print.ml\
 $(BASICS_DIR)/uencoding_types_print.ml\
 $(BASICS_DIR)/wstring_print.ml\
 $(BASICS_DIR)/name_print.ml\
 $(BASICS_DIR)/uid_print.ml\
 $(BASICS_DIR)/uname_print.ml\
 $(BASICS_DIR)/uname_pprint.ml\

BASICS_ML_FILES=\
 $(BASICS_DIR)/lib_strings.ml\
 $(BASICS_DIR)/unatural_types.ml\
 $(BASICS_DIR)/unatural.ml\
 $(BASICS_DIR)/path.ml\
 $(BASICS_DIR)/location.ml\
 $(BASICS_DIR)/hash_table.ml\
 $(BASICS_DIR)/hash_table_print.ml\
 $(BASICS_DIR)/uoption.ml\
 $(BASICS_DIR)/ulist.ml\
 $(BASICS_DIR)/ulist1.ml\
 $(BASICS_DIR)/ulist2.ml\
 $(BASICS_DIR)/umarshal.ml\
 $(BASICS_DIR)/wstring.ml\
 $(BASICS_DIR)/int8.ml\
 $(BASICS_DIR)/name.ml\
 $(BASICS_DIR)/uarray.ml\
 $(BASICS_DIR)/uid.ml\
 $(BASICS_DIR)/uint8.ml\
 $(BASICS_DIR)/uname.ml\
 $(BASICS_ML_ONLY_FILES)\

BASICS_BYT_OBJS=\
 $(BASICS_ML_FILES:.ml=.cmo)\

BASICS_MLI_FILES=\
 $(BASICS_DIR)/unatural_types.mli\
 $(BASICS_DIR)/lib_strings.mli\
 $(BASICS_DIR)/unatural.mli\
 $(BASICS_DIR)/path.mli\
 $(BASICS_DIR)/location.mli\
 $(BASICS_DIR)/ident.mli\
 $(BASICS_DIR)/hash_table.mli\
 $(BASICS_DIR)/hash_table_print.mli\
 $(BASICS_DIR)/uoption.mli\
 $(BASICS_DIR)/ulist.mli\
 $(BASICS_DIR)/ulist1.mli\
 $(BASICS_DIR)/ulist2.mli\
 $(BASICS_DIR)/binding.mli\
 $(BASICS_DIR)/umarshal.mli\
 $(BASICS_DIR)/uencoding_types.mli\
 $(BASICS_DIR)/wstring.mli\
 $(BASICS_DIR)/int8.mli\
 $(BASICS_DIR)/name.mli\
 $(BASICS_DIR)/uarray.mli\
 $(BASICS_DIR)/uid.mli\
 $(BASICS_DIR)/uint8.mli\
 $(BASICS_DIR)/uname.mli\

BASICS_CAML_FILES=\
 $(BASICS_MLI_FILES)\
 $(BASICS_ML_FILES)\

BASICS_CAML_GENERATED_FILES=\
 $(BASICS_DIR)/unatural_types_print.ml\
 $(BASICS_DIR)/unatural_print.ml\
 $(BASICS_DIR)/ident_print.ml\
 $(BASICS_DIR)/location_print.ml\
 $(BASICS_DIR)/path_print.ml\
 $(BASICS_DIR)/uid_print.ml\
 $(BASICS_DIR)/ulist_print.ml\
 $(BASICS_DIR)/ulist1_print.ml\
 $(BASICS_DIR)/ulist2_print.ml\
 $(BASICS_DIR)/uname_print.ml\
 $(BASICS_DIR)/binding_print.ml\
 $(BASICS_DIR)/uencoding_types_print.ml\
 $(BASICS_DIR)/wstring_print.ml\

BASICS_BYT_FILES_TO_INSTALL=\
 $(BASICS_MLI_FILES)\
 $(BASICS_ML_FILES)\
 $(BASICS_ML_FILES:.ml=.cmo)\
 $(BASICS_MLI_FILES:.mli=.cmi)\
 $(BASICS_ML_ONLY_FILES:.ml=.cmi)\

BASICS_BIN_FILES_TO_INSTALL=\
 $(BASICS_ML_FILES:.ml=.cmx)\

$(BASICS_DIR)/unatural_types_print.ml: $(BASICS_DIR)/unatural_types.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/unatural_types.mli

$(BASICS_DIR)/unatural_print.ml: $(BASICS_DIR)/unatural.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/unatural.mli

$(BASICS_DIR)/ulist_print.ml: $(BASICS_DIR)/ulist.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/ulist.mli

$(BASICS_DIR)/ulist1_print.ml: $(BASICS_DIR)/ulist1.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/ulist1.mli

$(BASICS_DIR)/ulist2_print.ml: $(BASICS_DIR)/ulist2.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/ulist2.mli

$(BASICS_DIR)/ident_print.ml: $(BASICS_DIR)/ident.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/ident.mli

$(BASICS_DIR)/path_print.ml: $(BASICS_DIR)/path.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/path.mli

$(BASICS_DIR)/location_print.ml: $(BASICS_DIR)/location.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/location.mli

$(BASICS_DIR)/binding_print.ml: $(BASICS_DIR)/binding.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/binding.mli

$(BASICS_DIR)/uencoding_types_print.ml:\
 $(BASICS_DIR)/uencoding_types.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/uencoding_types.mli

$(BASICS_DIR)/uname_print.ml:\
 $(BASICS_DIR)/uname.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/uname.mli

$(BASICS_DIR)/uid_print.ml:\
 $(BASICS_DIR)/uid.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/uid.mli

$(BASICS_DIR)/wstring_print.ml:\
 $(BASICS_DIR)/wstring.mli
	$(CAMLC_PPR_COMPILER) $(BASICS_DIR)/wstring.mli

# Directory src/configuration
CONFIGURATION_ML_ONLY_FILES=\
 $(CONFIGURATION_DIR)/ocamlxml_configuration_print.ml\

CONFIGURATION_ML_FILES=\
 $(CONFIGURATION_DIR)/xml_configuration.ml\
 $(CONFIGURATION_DIR)/xml_say.ml\
 $(CONFIGURATION_ML_ONLY_FILES)\

CONFIGURATION_BYT_OBJS=\
 $(CONFIGURATION_ML_FILES:.ml=.cmo)\

CONFIGURATION_MLI_FILES=\
 $(CONFIGURATION_DIR)/xml_configuration.mli\
 $(CONFIGURATION_DIR)/ocamlxml_configuration.mli\
 $(CONFIGURATION_DIR)/xml_say.mli\

CONFIGURATION_CAML_FILES=\
 $(CONFIGURATION_MLI_FILES)\
 $(CONFIGURATION_ML_FILES)\

CONFIGURATION_CAML_GENERATED_FILES=\
 $(CONFIGURATION_DIR)/xml_configuration.ml\
 $(CONFIGURATION_DIR)/ocamlxml_configuration_print.ml\

CONFIGURATION_BYT_FILES_TO_INSTALL=\
 $(CONFIGURATION_MLI_FILES)\
 $(CONFIGURATION_ML_FILES)\
 $(CONFIGURATION_ML_FILES:.ml=.cmo)\
 $(CONFIGURATION_MLI_FILES:.mli=.cmi)\
 $(CONFIGURATION_ML_ONLY_FILES:.ml=.cmi)\

CONFIGURATION_BIN_FILES_TO_INSTALL=\
 $(CONFIGURATION_ML_FILES:.ml=.cmx)\

$(CONFIGURATION_DIR)/xml_configuration.ml:\
 $(CONFIG_DIR)/ocamlxml.env\

$(CONFIGURATION_DIR)/ocamlxml_configuration_print.ml:\
 $(CONFIGURATION_DIR)/ocamlxml_configuration.mli
	$(CAMLC_PPR_COMPILER) $(CONFIGURATION_DIR)/ocamlxml_configuration.mli

# Directory src/xml/parsing
XML_LEX_ML_ONLY_FILES=\
 $(XML_PARSING_DIR)/intercal_print.ml\
 $(XML_PARSING_DIR)/xml_basics_ast_print.ml\
 $(XML_PARSING_DIR)/xml_lexer_types_print.ml\
 $(XML_PARSING_DIR)/xml_lexer.ml\

XML_LEX_ML_FILES=\
 $(XML_PARSING_DIR)/intercal.ml\
 $(XML_PARSING_DIR)/xml_basics_ast.ml\
 $(XML_PARSING_DIR)/xml_to_tokens.ml\
 $(XML_LEX_ML_ONLY_FILES)\

XML_LEX_BYT_OBJS=\
 $(XML_LEX_ML_FILES:.ml=.cmo)\

XML_LEX_BIN_OBJS=\
 $(XML_LEX_BYT_OBJS:.cmo=.cmx)\

XML_LEX_MLI_FILES=\
 $(XML_PARSING_DIR)/intercal.mli\
 $(XML_PARSING_DIR)/xml_basics_ast.mli\
 $(XML_PARSING_DIR)/xml_lexer_types.mli\
 $(XML_PARSING_DIR)/xml_to_tokens.mli\

XML_LEX_CAML_FILES=\
 $(XML_LEX_MLI_FILES)\
 $(XML_LEX_ML_FILES)\
 $(XML_PARSING_DIR)/xml_lexer.mll\

XML_LEX_CAML_GENERATED_FILES=\
 $(XML_PARSING_DIR)/intercal_print.ml\
 $(XML_PARSING_DIR)/xml_basics_ast_print.ml\
 $(XML_PARSING_DIR)/xml_lexer_types_print.ml\
 $(XML_PARSING_DIR)/xml_lexer.ml\

XML_LEX_BYT_FILES_TO_INSTALL=\
 $(XML_LEX_MLI_FILES)\
 $(XML_LEX_ML_FILES)\
 $(XML_LEX_BYT_OBJS)\
 $(XML_LEX_MLI_FILES:.mli=.cmi)\
 $(XML_LEX_ML_ONLY_FILES:.ml=.cmi)\

XML_LEX_BIN_FILES_TO_INSTALL=\
 $(XML_LEX_BIN_OBJS)\

$(XML_PARSING_DIR)/intercal_print.ml:\
 $(XML_PARSING_DIR)/intercal.mli
	$(CAMLC_PPR_COMPILER) $(XML_PARSING_DIR)/intercal.mli

$(XML_PARSING_DIR)/xml_basics_ast_print.ml:\
 $(XML_PARSING_DIR)/xml_basics_ast.mli
	$(CAMLC_PPR_COMPILER) $(XML_PARSING_DIR)/xml_basics_ast.mli

$(XML_PARSING_DIR)/xml_lexer_types_print.ml:\
 $(XML_PARSING_DIR)/xml_basics_ast.mli\
 $(XML_PARSING_DIR)/xml_lexer_types.mli
	$(CAMLC_PPR_COMPILER) $(XML_PARSING_DIR)/xml_lexer_types.mli

$(XML_PARSING_DIR)/xml_parser_print.ml:\
 $(XML_PARSING_DIR)/xml_lexer_types.mli\
 $(XML_PARSING_DIR)/xml_basics_ast.mli\
 $(XML_PARSING_DIR)/xml_parser.mli
	$(CAMLC_PPR_COMPILER) $(XML_PARSING_DIR)/xml_parser.mli

# Parse files
XML_PARSE_ML_ONLY_FILES=\
 $(XML_PARSING_DIR)/xml_ast_print.ml\
 $(XML_PARSING_DIR)/xml_parser_print.ml\

XML_PARSE_ML_FILES=\
 $(XML_PARSE_ML_ONLY_FILES)\
 $(XML_PARSING_DIR)/xml_parser_error.ml\
 $(XML_PARSING_DIR)/xml_parser.ml\
 $(XML_PARSING_DIR)/xml_to_ast.ml\

XML_PARSE_BYT_OBJS=\
 $(XML_PARSE_ML_FILES:.ml=.cmo)\

XML_PARSE_BIN_OBJS=\
 $(XML_PARSE_BYT_OBJS:.cmo=.cmx)\

XML_PARSE_MLI_FILES=\
 $(XML_PARSING_DIR)/xml_ast.mli\
 $(XML_PARSING_DIR)/xml_parser_error.mli\
 $(XML_PARSING_DIR)/xml_parser.mli\
 $(XML_PARSING_DIR)/xml_to_ast.mli\

XML_PARSE_CAML_FILES=\
 $(XML_PARSE_MLI_FILES)\
 $(XML_PARSE_ML_FILES)\
 $(XML_PARSING_DIR)/xml_parser.mly\

XML_PARSE_CAML_GENERATED_FILES=\
 $(XML_PARSING_DIR)/xml_ast_print.ml\
 $(XML_PARSING_DIR)/xml_parser.ml\
 $(XML_PARSING_DIR)/xml_parser.mli\
 $(XML_PARSING_DIR)/xml_parser_print.ml\

XML_PARSE_BYT_FILES_TO_INSTALL=\
 $(XML_PARSE_MLI_FILES)\
 $(XML_PARSE_ML_FILES)\
 $(XML_PARSE_BYT_OBJS)\
 $(XML_PARSE_MLI_FILES:.mli=.cmi)\
 $(XML_PARSE_ML_ONLY_FILES:.ml=.cmi)\

XML_PARSE_BIN_FILES_TO_INSTALL=\
 $(XML_PARSE_BIN_OBJS)\

$(XML_PARSING_DIR)/xml_ast_print.ml: $(XML_PARSING_DIR)/xml_ast.mli
	$(CAMLC_PPR_COMPILER) $(XML_PARSING_DIR)/xml_ast.mli

# All parsing files
XML_PARSING_ML_ONLY_FILES=\
 $(XML_LEX_ML_ONLY_FILES)\
 $(XML_PARSE_ML_ONLY_FILES)\

XML_PARSING_ML_FILES=\
 $(XML_LEX_ML_FILES)\
 $(XML_PARSE_ML_FILES)\

XML_PARSING_BYT_OBJS=\
 $(XML_LEX_BYT_OBJS)\
 $(XML_PARSE_BYT_OBJS)\

XML_PARSING_MLI_FILES=\
 $(XML_LEX_MLI_FILES)\
 $(XML_PARSE_MLI_FILES)\

XML_PARSING_CAML_FILES=\
 $(XML_LEX_CAML_FILES)\
 $(XML_PARSE_CAML_FILES)\

XML_PARSING_CAML_GENERATED_FILES=\
 $(XML_LEX_CAML_GENERATED_FILES)\
 $(XML_PARSE_CAML_GENERATED_FILES)\

XML_PARSING_BYT_FILES_TO_INSTALL=\
 $(XML_LEX_BYT_FILES_TO_INSTALL)\
 $(XML_PARSE_BYT_FILES_TO_INSTALL)\

XML_PARSING_BIN_FILES_TO_INSTALL=\
 $(XML_LEX_BIN_FILES_TO_INSTALL)\
 $(XML_PARSE_BIN_FILES_TO_INSTALL)\

# Directory src/xml/expand
XML_EXPAND_ML_ONLY_FILES=\

XML_EXPAND_ML_FILES=\
 $(XML_EXPAND_DIR)/xml_env.ml\
 $(XML_EXPAND_DIR)/xml_expand.ml\
 $(XML_EXPAND_ML_ONLY_FILES)\

XML_EXPAND_BYT_OBJS=\
 $(XML_EXPAND_ML_FILES:.ml=.cmo)\

XML_EXPAND_MLI_FILES=\
 $(XML_EXPAND_DIR)/xml_env.mli\
 $(XML_EXPAND_DIR)/xml_expand.mli\

XML_EXPAND_CAML_FILES=\
 $(XML_EXPAND_MLI_FILES)\
 $(XML_EXPAND_ML_FILES)\

XML_EXPAND_CAML_GENERATED_FILES=\

XML_EXPAND_BYT_FILES_TO_INSTALL=\
 $(XML_EXPAND_MLI_FILES)\
 $(XML_EXPAND_ML_FILES)\
 $(XML_EXPAND_ML_FILES:.ml=.cmo)\
 $(XML_EXPAND_MLI_FILES:.mli=.cmi)\
 $(XML_EXPAND_ML_ONLY_FILES:.ml=.cmi)\

XML_EXPAND_BIN_FILES_TO_INSTALL=\
 $(XML_EXPAND_ML_FILES:.ml=.cmx)\

# Directory src/library
XML_LIBRARY_ML_ONLY_FILES=\

XML_LIBRARY_ML_FILES=\
 $(XML_LIBRARY_DIR)/xml.ml\
 $(XML_LIBRARY_ML_ONLY_FILES)\

XML_LIBRARY_BYT_OBJS=\
 $(XML_LIBRARY_ML_FILES:.ml=.cmo)\

XML_LIBRARY_MLI_FILES=\
 $(XML_LIBRARY_DIR)/xml.mli\

XML_LIBRARY_CAML_FILES=\
 $(XML_LIBRARY_MLI_FILES)\
 $(XML_LIBRARY_ML_FILES)\

XML_LIBRARY_CAML_GENERATED_FILES=\

XML_LIBRARY_BYT_FILES_TO_INSTALL=\
 $(XML_LIBRARY_MLI_FILES)\
 $(XML_LIBRARY_ML_FILES)\
 $(XML_LIBRARY_ML_FILES:.ml=.cmo)\
 $(XML_LIBRARY_MLI_FILES:.mli=.cmi)\
 $(XML_LIBRARY_ML_ONLY_FILES:.ml=.cmi)\

XML_LIBRARY_BIN_FILES_TO_INSTALL=\
 $(XML_LIBRARY_ML_FILES:.ml=.cmx)\

# Directory src/
OCAML_XML_ML_ONLY_FILES=\
 $(OCAMLGEN_LIB_ML_ONLY_FILES)\
 $(BASICS_ML_ONLY_FILES)\
 $(CONFIGURATION_ML_ONLY_FILES)\
 $(XML_PARSING_ML_ONLY_FILES)\
 $(XML_EXPAND_ML_ONLY_FILES)\
 $(XML_LIBRARY_ML_ONLY_FILES)\

OCAML_XML_ML_FILES=\
 $(OCAMLGEN_LIB_ML_FILES)\
 $(BASICS_ML_FILES)\
 $(CONFIGURATION_ML_FILES)\
 $(XML_PARSING_ML_FILES)\
 $(XML_EXPAND_ML_FILES)\
 $(XML_LIBRARY_ML_FILES)\

OCAML_XML_BYT_OBJS=\
 $(OCAMLGEN_LIB_BYT_OBJS)\
 $(BASICS_BYT_OBJS)\
 $(CONFIGURATION_BYT_OBJS)\
 $(XML_PARSING_BYT_OBJS)\
 $(XML_EXPAND_BYT_OBJS)\
 $(XML_LIBRARY_BYT_OBJS)\

OCAML_XML_MLI_FILES=\
 $(OCAMLGEN_LIB_MLI_FILES)\
 $(BASICS_MLI_FILES)\
 $(CONFIGURATION_MLI_FILES)\
 $(XML_PARSING_MLI_FILES)\
 $(XML_EXPAND_MLI_FILES)\
 $(XML_LIBRARY_MLI_FILES)\

OCAML_XML_CAML_FILES=\
 $(OCAMLGEN_LIB_CAML_FILES)\
 $(BASICS_CAML_FILES)\
 $(CONFIGURATION_CAML_FILES)\
 $(XML_PARSING_CAML_FILES)\
 $(XML_EXPAND_CAML_FILES)\
 $(XML_LIBRARY_CAML_FILES)\

OCAML_XML_CAML_GENERATED_FILES=\
 $(OCAMLGEN_LIB_CAML_GENERATED_FILES)\
 $(BASICS_CAML_GENERATED_FILES)\
 $(CONFIGURATION_CAML_GENERATED_FILES)\
 $(XML_PARSING_CAML_GENERATED_FILES)\
 $(XML_EXPAND_CAML_GENERATED_FILES)\
 $(XML_LIBRARY_CAML_GENERATED_FILES)\

OCAML_XML_BYT_FILES_TO_INSTALL=\
 $(OCAMLGEN_LIB_BYT_FILES_TO_INSTALL)\
 $(BASICS_BYT_FILES_TO_INSTALL)\
 $(CONFIGURATION_BYT_FILES_TO_INSTALL)\
 $(XML_PARSING_BYT_FILES_TO_INSTALL)\
 $(XML_EXPAND_BYT_FILES_TO_INSTALL)\
 $(XML_LIBRARY_BYT_FILES_TO_INSTALL)\

OCAML_XML_BIN_FILES_TO_INSTALL=\
 $(OCAMLGEN_LIB_BIN_FILES_TO_INSTALL)\
 $(BASICS_BIN_FILES_TO_INSTALL)\
 $(CONFIGURATION_BIN_FILES_TO_INSTALL)\
 $(XML_PARSING_BIN_FILES_TO_INSTALL)\
 $(XML_EXPAND_BIN_FILES_TO_INSTALL)\
 $(XML_LIBRARY_BIN_FILES_TO_INSTALL)\

# List of files to bannerize

OCAML_XML_FILES_TO_BANNERIZE=\
 $(OCAML_XML_CAML_FILES)\

CONFIG_FILES_TO_BANNERIZE=\
 $(CONFIG_DIR)/Basic_commands.mk\
 $(CONFIG_DIR)/Caml.mk\
 $(CONFIG_DIR)/Caml_commands.mk\
 $(CONFIG_DIR)/Config.mk.in\
 $(CONFIG_DIR)/Doc.mk\
 $(CONFIG_DIR)/Params.mk\
 $(CONFIG_DIR)/Ocamlxml.mk\
 $(CONFIG_DIR)/ocamlxml.env\

CONFIG_SCRIPTS_TO_BANNERIZE=\
 $(CONFIG_DIR)/configure\

# Directory $(MAIN_DIR)

LOCAL_FILES_TO_BANNERIZE=\
 $(CONFIG_FILES_TO_BANNERIZE)\
 $(MAIN_DIR)/Makefile\
 $(SRC_DIR)/Objs.mk\
 $(CAML_GENERATING_FILES)\
 $(OCAML_XML_FILES_TO_BANNERIZE)\
 $(LOCAL_TEST_TO_BANNERIZE)\

LOCAL_SCRIPTS_TO_BANNERIZE=\
 $(MAIN_DIR)/configure\

$(CONFIG_DIR)/banner: $(CONFIG_DIR)/banner.in
	$(HTMLC) -f $< -o $@

LOCAL_FILES_TO_INSTALL=\

# The exhaustive list of files to install

ALL_BYT_FILES_TO_INSTALL=\
 $(OCAML_XML_BYT_FILES_TO_INSTALL)\
 $(LOCAL_FILES_TO_INSTALL)\

ALL_BIN_FILES_TO_INSTALL=\
 $(OCAML_XML_BIN_FILES_TO_INSTALL)\

# The exhaustive list of files to compute dependencies
CAML_FILES=\
 $(OCAML_XML_CAML_FILES)

# The exhaustive list of mandatory includes to compile this set of sources
CAML_INCLUDES=\
 $(OCAML_XML_CAML_INCLUDES)

# The exhaustive list of automatically generated files
CAML_GENERATED_FILES=\
 $(OCAML_XML_CAML_GENERATED_FILES)

CAML_SUB_DIRS=\
 $(OCAML_XML_SUB_DIRS)\
