(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: hash_table.mli,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type ('a, 'b) t = ('a, 'b) Hashtbl.t;;

val create : Unatural_types.natural -> ('a, 'b) t;;

val add_binding : ('a, 'b) t -> 'a * 'b -> unit
(** [add_binding t (key, v)] add to table [t] the entry [key],
  associated to value [v]. *)
;;

val add_bindings : ('a, 'b) t -> ('a * 'b) list -> unit
(** [add_bindings t bdgs] add to table [t] all the bindings in list [bdgs].
  Bindings are entered in order of presentation in [bdgs].
*)
;;

val bindings : ('a, 'b) t -> ('a * 'b) list
(** [bindings t] Return the list of bindings registered in table [t]. *)
;;

val values : ('a, 'b) t -> 'b list
(** [values t] Return the list of values registered in table [t]. *)
;;

val of_bindings : ('a * 'b) list -> ('a, 'b) t
(** [of_bindings bdgs] return the hash table made of all the bindings in list
  of bindings [bdgs]. *)
;;

val t_xt_of_bindings : ('a * 'b) list -> ('a, 'b) t * ('b, 'a) t
(** [t_xt_of_bindings bdgs] return a pair of hash tables [(t, xt)], where
  [t] contains the bindings in list [bdgs], and [xt] is the invert table of
  table [t]: if [key] is bound to [v] in table [t] then [v] is bound to [key]
  in table [xt].
  Note: character [x] in [xt] is a memonic for 'crossed'. *)
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
