(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: name.mli,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

(** {3 The type of names} *)

type prefix = string;;
type suffix = string
(** To create names with given prefix and/or suffix. *)
;;

type t = private string
(** A private type to implement names. Not to confuse a name with any other
  string. *)
;;

external to_t : string -> t = "%identity"
(** [to_t s] Return the name corresponding to string [s].

  [to_t] is the canonical projection from type [string] to type [Name.t].
  [to_t] is a surjective application. *)
;;

external of_t : t -> string = "%identity"
(** [of_t nm] Return the string value representing name [nm].

  [of_t] is the canonical injection from type [Name.t] to type [string].
  [of_t] is a total and injective application. *)
;;

external to_string : t -> string = "%identity"
(** [to_string nm] Return a string representation of name [nm]. *)
;;

external compare : t -> t -> int = "%compare";;
external same_name : t -> t -> bool = "%equal";;

val fresh : prefix -> t
(** [fresh pref] Generate a new name having [pref] as prefix and a fresh uniquely
  identifying number as suffix.
  (See {!Uid.fresh} for more information about uniquely identifying numbers.)
*)
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
