(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uid.mli,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

(** {3 Uniquely identifying numbers} *)

type t = private int64;;

external of_t : t -> int64 = "%identity"
(** [of_t uid] Return the 64 bits long integer representation of uniquely
  identifying number [uid].

  [of_t] is the canonical injection from type [Uid.t] to type [int64].
  [of_t] is a total and injective application. *)
;;

val dummy : t
(** [Uid.dummy] is a uniquely identifying number that is guaranteed
  impossible to obtain via function [fresh] below. *)
;;

val fresh : unit -> t
(** Fresh allocates a new uniquely identifying number at each invocation.
  [fresh] is the only available function to create a [Uid.t] value; as a
  consequence, each uniquely identifying number is different from any
  other uniquely identifying number allocated by module [Uid]. *)
;;

val to_string : t -> string
(** Return a string representation of a uniquely identifying number. *)
;;

(*
 Local Variables:
  compile-command: "cd ../..; make"
  End:
*)
