(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 1997-2018 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Clement Franchini <Clement.Franchini@inria.fr>                     *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 3.6+dev0 generated code. *)

let printer_name ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let printer_lowercase_name ctx ppf = function
  | l -> (fun n -> printer_name ctx ppf n) (Og_ident.of_lowercase_name l)
;;

let printer_uppercase_name ctx ppf = function
  | u -> (fun n -> printer_name ctx ppf n) (Og_ident.of_uppercase_name u)
;;

let printer_symbolic_name ctx ppf = function
  | s -> (fun n -> printer_name ctx ppf n) (Og_ident.of_symbolic_name s)
;;

let printer_ident ctx ppf = function
  | Og_ident.Uppercase u ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Uppercase";
    printer_uppercase_name Og_precedence.Ck_argument ppf u;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Og_ident.Lowercase l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Lowercase";
    printer_lowercase_name Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Og_ident.Symbolic s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Symbolic";
    printer_symbolic_name Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_longident ctx ppf = function
  | Og_ident.Lident i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Lident";
    printer_ident Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Og_ident.Ldot (l, i) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Ldot";
    if Og_precedence.Ck_argument = Og_precedence.Ck_record_field then
      Format.fprintf ppf "%,@ " else
    if Og_precedence.Ck_argument = Og_precedence.Ck_argument then
      Format.fprintf ppf "%, (@," else
    if Og_precedence.Ck_argument = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_longident Og_precedence.Ck_tuple_item ppf l;
    Format.fprintf ppf "%,,@ ";
    printer_ident Og_precedence.Ck_tuple_item ppf i;
    Format.fprintf ppf "%,@]";
    if Og_precedence.Ck_argument = Og_precedence.Ck_argument then
      Format.fprintf ppf "%,@;<0 -2>)" else
    if Og_precedence.Ck_argument = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Og_ident.Lapply (l, l0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Lapply";
    if Og_precedence.Ck_argument = Og_precedence.Ck_record_field then
      Format.fprintf ppf "%,@ " else
    if Og_precedence.Ck_argument = Og_precedence.Ck_argument then
      Format.fprintf ppf "%, (@," else
    if Og_precedence.Ck_argument = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_longident Og_precedence.Ck_tuple_item ppf l;
    Format.fprintf ppf "%,,@ ";
    printer_longident Og_precedence.Ck_tuple_item ppf l0;
    Format.fprintf ppf "%,@]";
    if Og_precedence.Ck_argument = Og_precedence.Ck_argument then
      Format.fprintf ppf "%,@;<0 -2>)" else
    if Og_precedence.Ck_argument = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

(* Defining top printers. *)

let print_name ppf = function
  | n -> printer_name Og_precedence.Ck_toplevel ppf n
;;

let print_lowercase_name ppf = function
  | l -> printer_lowercase_name Og_precedence.Ck_toplevel ppf l
;;

let print_uppercase_name ppf = function
  | u -> printer_uppercase_name Og_precedence.Ck_toplevel ppf u
;;

let print_symbolic_name ppf = function
  | s -> printer_symbolic_name Og_precedence.Ck_toplevel ppf s
;;

let print_ident ppf = function
  | i -> printer_ident Og_precedence.Ck_toplevel ppf i
;;

let print_longident ppf = function
  | l -> printer_longident Og_precedence.Ck_toplevel ppf l
;;

(* End of OCamlGen, 3.6+dev0 generated code. *)
