(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 1997-2018 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Clement Franchini <Clement.Franchini@inria.fr>                     *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_scanner.mli,v 1.1 2019-05-10 14:02:41 jpc Exp $ *)

(** {3 Runtime library for pretty-scanners generated with OCamlGen} *)

open Og_types;;

(** {6 Scanners for basic monomorphic values} *)

val scanner_Unit : unit scanner;;
val scanner_Bool : bool scanner;;
val scanner_Char : char scanner;;
val scanner_String : string scanner;;
val scanner_Bytes : Og_types.bytes scanner;;

val scanner_Int : int scanner;;
val scanner_Int32 : int32 scanner;;
val scanner_Int64 : int64 scanner;;
val scanner_Nativeint : nativeint scanner;;
val scanner_Float : float scanner;;

(** {6 Scanners for basic polymorphic values} *)

(*
val scanner_format : ;;
val scanner_format4 : ;;
val scanner_format6 : ;;
*)

type ('a, 'b, 'c, 'd) delimiter_format =
  ('a, 'b, 'c, 'd, 'd, 'a) format6
;;

type ('a, 'b) scanf_delimiter_format =
  ('a, Scanf.Scanning.in_channel, 'b, 'a -> 'a) delimiter_format
;;

type 'a scanner_delimiter_format = (unit, 'a) scanf_delimiter_format;;

(* We have:
let fmt = format_of_string "";;

Scanf.scanf fmt;;

fmt;;
- : ('_a, Scanf.Scanning.in_channel, '_b, '_a -> '_a, '_a -> '_a, '_a) format6

Scanf.scanf fmt ();;
- : unit = ()

fmt;;
- : (unit, Scanf.Scanning.in_channel, '_a, unit -> unit, unit -> unit, unit)
    format6
*)

val scanner_list :
  'a scanner_delimiter_format ->
  ('c, 'c list) scanner_lifter
;;
val scanner_List : ('a, 'a list) scanner_lifter;;

val scanner_array :
  'a scanner_delimiter_format ->
  ('d, 'd array) scanner_lifter
;;
val scanner_Array : ('a, 'a array) scanner_lifter;;

val scanner_option :
  'a scanner_delimiter_format ->
  ('d, 'd option) scanner_lifter
;;
val scanner_Option : ('a, 'a option) scanner_lifter;;

val scanner_ref :
  'a scanner_delimiter_format ->
  ('d, 'd ref) scanner_lifter
;;
val scanner_Ref : ('a, 'a ref) scanner_lifter;;

val scanner_lazy :
  'a scanner_delimiter_format ->
 ('a, 'a Lazy.t) scanner_lifter
;;
val scanner_Lazy : ('a, 'a Lazy.t) scanner_lifter;;

(** {6 Scanners for fully unknown values} *)

val scanner_fun :
  'c scanner_delimiter_format -> ('a, 'b, 'a -> 'b) scanner_lifter2
;;

val scanner_Fun : ('a, 'b, 'a -> 'b) scanner_lifter2;;

val scanner_poly : 'a scanner_delimiter_format -> 'd scanner;;
val scanner_Poly : 'a scanner;;

val scanner_abstract : 'a scanner_delimiter_format -> 'd scanner;;
val scanner_Abstract : 'a scanner;;

val scanner_external : 'a scanner_delimiter_format -> 'd scanner;;
val scanner_External : 'a scanner;;
