(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_basics_ast.ml,v 1.1 2019-05-10 14:00:49 jpc Exp $ *)

type text = string;;
type char_data_text = text;;

type nm_token = string;;

type xml_name = nm_token;;

let not_x c = c <> 'x';;
let not_m c = c <> 'm';;
let not_l c = c <> 'l';;
let not_dash c = c <> '-';;
let not_colon c = c <> ':';;

let to_xml_name nm_token =
  let len = String.length nm_token in
  let nm =
    match len with
    | 0 -> assert false
    | 1 | 2 -> nm_token
    | _len ->
      if not_x nm_token.[0] then nm_token else
      if not_m nm_token.[1] then nm_token else
      if not_l nm_token.[2] then nm_token else
      if len >= 4 && not_colon nm_token.[3] then nm_token else
      failwith (Format.sprintf "%S is not a valid xml name" nm_token) in
  let valid_start_char = function
    | '-' | '.' | '0' .. '9' | '\183' -> false
    | _ -> true in
  if valid_start_char nm.[0] then nm else
    failwith (Format.sprintf "%S is not a valid xml name" nm)
;;

external of_xml_name :
  xml_name -> nm_token = "%identity";;

type pi_target = xml_name;;

let to_pi_target xml_nm =
  let nm = of_xml_name xml_nm in
  let len = String.length nm in
  match len with
  | 0 -> assert false
  | 1 | 2 -> xml_nm
  | _len ->
    if not_x nm.[0] then xml_nm else
    if not_m nm.[1] then xml_nm else
    if not_l nm.[2] then xml_nm else
    if len >= 4 && not_dash nm.[3] then xml_nm else
    failwith (Format.sprintf "%S is not a valid pi target" nm)
;;

external of_pi_target :
  pi_target -> xml_name = "%identity";;

type pi_instructions = text;;

type tag_name = xml_name;;
type tag_attribute_name = xml_name;;
type entity_name = xml_name;;
type notation_name = xml_name;;

type att_value_text = string;;
type entity_value_text = text;;

type reference =
  | Char_ref of int32
  | Entity_ref of entity_name
;;

type att_value = att_value_elem list
and att_value_elem =
  | Att_value_text of att_value_text
  | Att_value_reference of reference
;;

type entity_value = entity_value_elem list
and entity_value_elem =
  | Entity_value_text of entity_value_text
  | Entity_value_reference of reference
  | Entity_value_pe_reference of entity_name
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
