type token =
  | Eof
  | OpenConditionalSect
  | CloseConditionalSect
  | OpenBracket
  | CloseBracket
  | OpenParen
  | CloseParen
  | CloseParenQuestion
  | CloseParenStar
  | CloseParenPlus
  | GT
  | Question
  | Star
  | Plus
  | Eq
  | Comma
  | Bar
  | XmlDeclStart
  | XmlDeclEnd
  | ExternalIDSystem of (string)
  | ExternalIDPublic of (string * string)
  | PublicID of (string)
  | PCDATA
  | REQUIRED
  | IMPLIED
  | FIXED
  | INCLUDE
  | Ignore of (string)
  | VersionInfo of (string)
  | EncodingDecl of (string)
  | SDDecl of (bool)
  | NDataDecl of (Xml_basics_ast.xml_name)
  | AttlistName of (Xml_basics_ast.xml_name)
  | NotationTypeName of (Xml_basics_ast.xml_name)
  | Space of (string)
  | Comment of (string)
  | NotationName of (Xml_basics_ast.xml_name)
  | GEDeclName of (Xml_basics_ast.xml_name)
  | PEDeclName of (Xml_basics_ast.xml_name)
  | AttValue of (Xml_basics_ast.att_value)
  | PEReference of (Xml_basics_ast.xml_name)
  | CharRef of (int32)
  | EntityRef of (Xml_basics_ast.xml_name)
  | CData of (string)
  | CharDataIntercal of (Xml_lexer_types.char_data_intercal)
  | FirstSTag of (Xml_lexer_types.token_tag)
  | FirstEmptyElemTag of (Xml_lexer_types.token_tag)
  | STag of (Xml_lexer_types.token_tag)
  | EmptyElemTag of (Xml_lexer_types.token_tag)
  | ETag of (Xml_basics_ast.tag_name)
  | Nmtoken of (Xml_basics_ast.nm_token)
  | DocTypeName of (Xml_basics_ast.tag_attribute_name)
  | ElementName of (Xml_basics_ast.tag_attribute_name)
  | PI of (Xml_basics_ast.pi_target * Xml_basics_ast.pi_instructions)

open Parsing;;
let _ = parse_error;;
# 18 "src/xml/parsing/xml_parser.mly"

(* Prelude part: this is pure Caml. *)

(* The parser *)

open Xml_ast;;

(* FIXME: validity checks. *)
let make_enc_name s = s;;

let make_xml_name start_pos end_pos nm_token =
  try
    Xml_basics_ast.to_xml_name nm_token
  with
  | Failure _ ->
    raise (Xml_parser_error.Error
      (Xml_parser_error.Invalid_xml_name nm_token, start_pos, end_pos))
;;

(* FIXME: validity checks. *)
let make_att_value_literal s =
  (* FIXME: Should att_value_text to produce a list of text | Reference. *)
  s
;;

(* FIXME: validity checks. *)
let make_entity_value_of_att_value start_pos end_pos att_value =
  let add_text, add_reference, add_pe_reference, get_result =
    let result = ref [] in
    let add_item i = result := i :: !result in
    let add_text t =
      if t <> "" then
        add_item (Xml_basics_ast.Entity_value_text t)
    and add_reference r = add_item (Xml_basics_ast.Entity_value_reference r)
    and add_pe_reference r =
      let xml_name = Xml_basics_ast.to_xml_name r in
      add_item (Xml_basics_ast.Entity_value_pe_reference xml_name)
    and get_result () = List.rev !result in
    add_text, add_reference, add_pe_reference, get_result in
  let cut s =
    let ib = Scanf.Scanning.from_string s in
    let rec loop () =
      if Scanf.Scanning.end_of_input ib then
        ()
      else
        try
          Scanf.bscanf ib "%[^%%]%%%[^;];%[^%%]" (fun t r s ->
            add_text t;
            add_pe_reference r;
            add_text s);
          loop ()
        with
          | Scanf.Scan_failure message ->
            raise (Xml_parser_error.Error
              (Xml_parser_error.Invalid_entity_value (att_value, message),
               start_pos,
               end_pos))
          | Failure _ -> assert false
          | End_of_file ->
          (* That case happen when there is no parameter entity reference.
             That is why we append all the string. *)
            add_text s
          | Invalid_argument _ -> assert false in
    loop () in
  List.iter
    (function
    | Xml_basics_ast.Att_value_text t ->  cut t
    | Xml_basics_ast.Att_value_reference r -> add_reference r)
    att_value;
  get_result ()
;;

let make_tag (tag_name, tag_attributes) =
  let make_attribute (attr_name, attr_val) =
    let attr_val = make_att_value_literal attr_val in
    {
      tag_attribute_name = attr_name;
      tag_attribute_value = attr_val;
    } in
  let attrs = List.map make_attribute tag_attributes in
  {
    tag_name = tag_name;
    tag_attributes = attrs;
  }
;;

let make_content_element_tag start_pos end_pos tag children etag =
  let tag = make_tag tag in
  if tag.tag_name <> etag then
    raise (Xml_parser_error.Error
      (Xml_parser_error.Invalid_closing_tag (tag.tag_name, etag), start_pos, end_pos))
  else
    Content_element_tag (tag, children)
;;

let make_empty_element_tag (tag_name, attributes) =
  let attributes =
    List.map (fun (att_name, att_value) ->
      {
        tag_attribute_name = att_name;
        tag_attribute_value = att_value;
      }) attributes in
  let tag = {
    tag_name = tag_name;
    tag_attributes = attributes;
  } in
  Xml_ast.Empty_element_tag tag
;;

(*
let check_is_empty_CharData s =
  let ls = String.length s in
  let rec loop i =
    i >= ls ||
    let c = s.[i] in
    (c = ' ' || c = '\t' || c = '\r' || c = '\n') &&
    loop (succ i) in
  if not (loop 0) then
    not_implemented "check_is_empty_CharData error case"
;;
*)

let make_sdd_decl = function
  | true -> Standalone
  | false -> Not_standalone
;;

let make_xml_decl vi enc_opt sd_opt =
  {
    xml_version_info = vi;
    xml_encoding_decl = enc_opt;
    xml_standalone_decl = sd_opt;
  }
;;

let make_att_def start_pos end_pos nm_token att_type att_default =
  let name = make_xml_name start_pos end_pos nm_token in
  {
    att_def_name = name;
    att_def_type = att_type;
    att_def_default = att_default;
  }
;;

let make_att_type start_pos end_pos = function
  | "CDATA" -> String_type
  | "ID" -> Tokenized_type Id_type
  | "IDREF" -> Tokenized_type Idref_type
  | "IDREFS" -> Tokenized_type Idrefs_type
  | "ENTITY" -> Tokenized_type Entity_type
  | "ENTITIES" -> Tokenized_type Entities_type
  | "NMTOKEN" -> Tokenized_type Nmtoken_type
  | "NMTOKENS" -> Tokenized_type Nmtokens_type
  | nm_token ->
    raise (Xml_parser_error.Error
      (Xml_parser_error.Invalid_att_type nm_token, start_pos, end_pos))
;;

let make_pi (tgt, instructions) =
  {
    pi_target = tgt;
    pi_instructions = instructions;
  }
;;

let make_content_spec_mod_choice choice spec_mod =
  Content_spec_choice (spec_mod, choice)
;;

let make_content_spec_mod_seq seq spec_mod =
  Content_spec_seq (spec_mod, seq)
;;

let make_content_spec_particle_mod_choice choice spec_mod =
  Content_particle_choice (spec_mod, choice)
;;

let make_content_spec_particle_mod_seq seq spec_mod =
  Content_particle_seq (spec_mod, seq)
;;

let make_content_spec_particle_mod_name start_pos end_pos nm spec_mod =
  Content_particle_name (spec_mod, make_xml_name start_pos end_pos nm)
;;

let make_content_spec start_pos end_pos = function
  | "EMPTY" -> Content_spec_empty
  | "ANY" -> Content_spec_any
  | nm_token ->
    raise (Xml_parser_error.Error
      (Xml_parser_error.Invalid_content_spec nm_token, start_pos, end_pos))
;;

let make_element_content char_data_intercal_child_intercal =
  let alpha_folder accu alpha =
    let char_data_folder accu char_data_text =
      Intercal.cons_alpha (Char_data char_data_text) accu in
    let reference_folder accu reference =
      Intercal.cons_beta (Child_reference reference) accu in
    Intercal.fold_left char_data_folder reference_folder accu alpha in
  let beta_folder accu beta =
    Intercal.cons_beta beta accu in
  let element_content =
    Intercal.fold_left alpha_folder beta_folder
      Intercal.empty char_data_intercal_child_intercal in
  let element_content = Intercal.rev element_content in
  Element_content element_content
;;

# 270 "src/xml/parsing/xml_parser.ml"
let yytransl_const = [|
  257 (* Eof *);
  258 (* OpenConditionalSect *);
  259 (* CloseConditionalSect *);
  260 (* OpenBracket *);
  261 (* CloseBracket *);
  262 (* OpenParen *);
  263 (* CloseParen *);
  264 (* CloseParenQuestion *);
  265 (* CloseParenStar *);
  266 (* CloseParenPlus *);
  267 (* GT *);
  268 (* Question *);
  269 (* Star *);
  270 (* Plus *);
  271 (* Eq *);
  272 (* Comma *);
  273 (* Bar *);
  274 (* XmlDeclStart *);
  275 (* XmlDeclEnd *);
  279 (* PCDATA *);
  280 (* REQUIRED *);
  281 (* IMPLIED *);
  282 (* FIXED *);
  283 (* INCLUDE *);
    0|]

let yytransl_block = [|
  276 (* ExternalIDSystem *);
  277 (* ExternalIDPublic *);
  278 (* PublicID *);
  284 (* Ignore *);
  285 (* VersionInfo *);
  286 (* EncodingDecl *);
  287 (* SDDecl *);
  288 (* NDataDecl *);
  289 (* AttlistName *);
  290 (* NotationTypeName *);
  291 (* Space *);
  292 (* Comment *);
  293 (* NotationName *);
  294 (* GEDeclName *);
  295 (* PEDeclName *);
  296 (* AttValue *);
  297 (* PEReference *);
  298 (* CharRef *);
  299 (* EntityRef *);
  300 (* CData *);
  301 (* CharDataIntercal *);
  302 (* FirstSTag *);
  303 (* FirstEmptyElemTag *);
  304 (* STag *);
  305 (* EmptyElemTag *);
  306 (* ETag *);
  307 (* Nmtoken *);
  308 (* DocTypeName *);
  309 (* ElementName *);
  310 (* PI *);
    0|]

let yylhs = "\255\255\
\001\000\005\000\005\000\008\000\009\000\009\000\010\000\010\000\
\007\000\007\000\011\000\012\000\012\000\014\000\014\000\013\000\
\013\000\015\000\016\000\016\000\017\000\017\000\019\000\019\000\
\018\000\018\000\018\000\018\000\018\000\018\000\020\000\024\000\
\024\000\024\000\025\000\025\000\027\000\029\000\028\000\028\000\
\028\000\028\000\033\000\033\000\033\000\033\000\030\000\030\000\
\030\000\031\000\034\000\034\000\032\000\032\000\026\000\026\000\
\026\000\035\000\037\000\037\000\036\000\021\000\038\000\038\000\
\039\000\040\000\040\000\042\000\042\000\043\000\045\000\045\000\
\046\000\046\000\044\000\041\000\041\000\041\000\041\000\047\000\
\022\000\022\000\048\000\048\000\051\000\051\000\052\000\049\000\
\049\000\050\000\023\000\023\000\004\000\004\000\054\000\055\000\
\055\000\053\000\053\000\056\000\056\000\056\000\057\000\057\000\
\058\000\059\000\002\000\002\000\003\000\003\000\060\000\061\000\
\061\000\062\000\062\000\063\000\064\000\064\000\064\000\064\000\
\006\000\006\000\065\000\065\000\065\000\000\000\000\000\000\000\
\000\000"

let yylen = "\002\000\
\004\000\000\000\001\000\005\000\000\000\001\000\000\000\001\000\
\000\000\002\000\004\000\000\000\001\000\001\000\001\000\000\000\
\001\000\003\000\000\000\002\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\003\000\001\000\
\001\000\001\000\003\000\003\000\002\000\002\000\001\000\001\000\
\001\000\001\000\000\000\001\000\001\000\001\000\003\000\003\000\
\002\000\003\000\000\000\003\000\000\000\003\000\003\000\004\000\
\003\000\002\000\000\000\002\000\002\000\003\000\000\000\002\000\
\006\000\001\000\001\000\001\000\001\000\003\000\000\000\003\000\
\000\000\003\000\004\000\001\000\001\000\002\000\001\000\001\000\
\003\000\003\000\001\000\002\000\000\000\001\000\001\000\001\000\
\001\000\001\000\003\000\003\000\001\000\002\000\004\000\000\000\
\001\000\000\000\002\000\001\000\001\000\001\000\001\000\001\000\
\003\000\001\000\001\000\003\000\001\000\003\000\002\000\000\000\
\001\000\000\000\002\000\002\000\001\000\001\000\001\000\001\000\
\000\000\002\000\001\000\001\000\001\000\002\000\002\000\002\000\
\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\126\000\000\000\
\003\000\000\000\107\000\127\000\000\000\109\000\128\000\000\000\
\000\000\106\000\000\000\024\000\030\000\000\000\000\000\000\000\
\023\000\000\000\029\000\129\000\100\000\102\000\025\000\026\000\
\027\000\028\000\093\000\000\000\000\000\101\000\103\000\104\000\
\000\000\125\000\123\000\124\000\000\000\000\000\113\000\000\000\
\000\000\000\000\097\000\000\000\000\000\000\000\000\000\000\000\
\014\000\015\000\000\000\000\000\090\000\000\000\000\000\083\000\
\089\000\000\000\088\000\000\000\032\000\000\000\033\000\034\000\
\094\000\099\000\006\000\000\000\000\000\000\000\000\000\122\000\
\108\000\120\000\118\000\119\000\117\000\111\000\000\000\000\000\
\110\000\000\000\105\000\000\000\062\000\064\000\092\000\091\000\
\087\000\084\000\086\000\081\000\082\000\000\000\000\000\000\000\
\000\000\000\000\000\000\031\000\008\000\000\000\000\000\013\000\
\001\000\010\000\115\000\116\000\095\000\000\000\000\000\000\000\
\057\000\055\000\000\000\000\000\000\000\046\000\045\000\044\000\
\049\000\039\000\042\000\041\000\040\000\035\000\036\000\000\000\
\000\000\037\000\038\000\004\000\000\000\000\000\017\000\000\000\
\000\000\066\000\000\000\067\000\068\000\069\000\047\000\048\000\
\061\000\056\000\000\000\058\000\000\000\000\000\000\000\000\000\
\021\000\022\000\011\000\000\000\000\000\000\000\000\000\060\000\
\054\000\000\000\050\000\018\000\020\000\000\000\000\000\000\000\
\070\000\076\000\077\000\000\000\080\000\065\000\079\000\000\000\
\000\000\075\000\072\000\078\000\052\000\074\000"

let yydgoto = "\005\000\
\007\000\085\000\015\000\028\000\008\000\045\000\078\000\009\000\
\076\000\110\000\079\000\111\000\142\000\060\000\143\000\159\000\
\160\000\029\000\030\000\031\000\032\000\033\000\034\000\070\000\
\071\000\072\000\105\000\134\000\106\000\107\000\138\000\139\000\
\129\000\171\000\124\000\155\000\156\000\055\000\056\000\147\000\
\182\000\148\000\149\000\150\000\166\000\175\000\183\000\063\000\
\066\000\064\000\098\000\099\000\035\000\036\000\052\000\037\000\
\038\000\039\000\040\000\048\000\049\000\086\000\087\000\088\000\
\046\000"

let yysindex = "\124\000\
\014\255\218\254\225\254\246\254\000\000\051\255\000\000\240\254\
\000\000\055\255\000\000\000\000\055\255\000\000\000\000\056\255\
\033\255\000\000\083\255\000\000\000\000\042\255\025\255\025\255\
\000\000\007\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\033\255\033\255\000\000\000\000\000\000\
\077\255\000\000\000\000\000\000\067\255\240\254\000\000\071\255\
\054\255\091\255\000\000\092\255\139\255\093\255\132\255\083\255\
\000\000\000\000\134\255\135\255\000\000\115\255\138\255\000\000\
\000\000\140\255\000\000\001\255\000\000\141\255\000\000\000\000\
\000\000\000\000\000\000\117\255\016\255\149\255\240\254\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\054\255\055\255\
\000\000\136\255\000\000\118\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\008\255\075\255\097\255\
\122\255\122\255\072\255\000\000\000\000\142\255\150\255\000\000\
\000\000\000\000\000\000\000\000\000\000\006\255\122\255\122\255\
\000\000\000\000\105\255\148\255\143\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\008\255\
\008\255\000\000\000\000\000\000\040\255\147\255\000\000\108\255\
\145\255\000\000\128\255\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\143\255\000\000\151\255\152\255\159\255\040\255\
\000\000\000\000\000\000\153\255\114\255\161\255\080\255\000\000\
\000\000\008\255\000\000\000\000\000\000\120\255\165\255\145\255\
\000\000\000\000\000\000\133\255\000\000\000\000\000\000\152\255\
\153\255\000\000\000\000\000\000\000\000\000\000"

let yyrindex = "\000\000\
\255\254\000\000\000\000\166\000\000\000\000\000\000\000\003\255\
\000\000\047\255\000\000\000\000\047\255\000\000\000\000\144\255\
\172\255\000\000\166\255\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\166\000\001\000\000\000\000\000\000\000\
\002\255\000\000\000\000\000\000\175\255\003\255\000\000\000\000\
\129\255\000\000\000\000\000\000\000\000\000\000\000\000\166\255\
\000\000\000\000\000\000\000\000\000\000\167\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\162\255\043\255\000\000\179\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\129\255\047\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\107\255\
\000\000\000\000\126\255\000\000\000\000\000\000\171\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\174\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\180\255\000\000\000\000\000\000\
\177\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\174\255\000\000\126\255\130\255\000\000\180\255\
\000\000\000\000\000\000\181\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\177\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\130\255\
\181\255\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\184\000\000\000\000\000\000\000\219\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\235\255\000\000\027\000\
\000\000\137\255\163\255\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\087\000\249\255\088\000\125\255\000\000\034\000\
\000\000\008\000\000\000\090\000\039\000\139\000\000\000\000\000\
\000\000\000\000\000\000\000\000\020\000\012\000\018\000\000\000\
\000\000\175\000\000\000\000\000\013\000\000\000\000\000\000\000\
\000\000\000\000\000\000\187\000\113\000\115\000\000\000\000\000\
\000\000"

let yytablesize = 260
let yytable = "\002\000\
\098\000\062\000\065\000\121\000\157\000\158\000\102\000\016\000\
\080\000\010\000\011\000\144\000\068\000\102\000\013\000\014\000\
\017\000\018\000\042\000\043\000\005\000\161\000\019\000\103\000\
\020\000\021\000\022\000\023\000\024\000\053\000\025\000\006\000\
\005\000\002\000\002\000\057\000\058\000\044\000\184\000\145\000\
\161\000\114\000\026\000\027\000\057\000\058\000\012\000\162\000\
\073\000\074\000\002\000\104\000\002\000\012\000\121\000\112\000\
\146\000\069\000\104\000\017\000\018\000\057\000\058\000\059\000\
\061\000\019\000\162\000\020\000\021\000\022\000\023\000\024\000\
\019\000\025\000\020\000\021\000\022\000\023\000\024\000\041\000\
\025\000\121\000\112\000\122\000\051\000\026\000\027\000\136\000\
\137\000\082\000\112\000\123\000\026\000\027\000\112\000\112\000\
\112\000\083\000\135\000\047\000\112\000\010\000\011\000\178\000\
\179\000\180\000\075\000\084\000\126\000\127\000\128\000\151\000\
\152\000\043\000\043\000\043\000\043\000\054\000\077\000\181\000\
\081\000\090\000\043\000\043\000\001\000\002\000\003\000\004\000\
\130\000\131\000\132\000\133\000\053\000\053\000\053\000\053\000\
\051\000\051\000\051\000\051\000\089\000\091\000\093\000\092\000\
\095\000\096\000\097\000\109\000\100\000\113\000\101\000\108\000\
\118\000\141\000\117\000\153\000\154\000\163\000\164\000\123\000\
\140\000\165\000\167\000\172\000\176\000\098\000\136\000\177\000\
\170\000\174\000\185\000\186\000\181\000\096\000\098\000\009\000\
\063\000\085\000\114\000\121\000\007\000\016\000\059\000\071\000\
\019\000\012\000\173\000\073\000\119\000\120\000\169\000\189\000\
\125\000\168\000\094\000\187\000\190\000\188\000\067\000\050\000\
\116\000\115\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\098\000"

let yycheck = "\001\001\
\000\000\023\000\024\000\001\001\136\000\137\000\006\001\018\001\
\046\000\048\001\049\001\006\001\006\001\006\001\046\001\047\001\
\027\001\028\001\035\001\036\001\019\001\141\000\033\001\023\001\
\035\001\036\001\037\001\038\001\039\001\017\000\041\001\018\001\
\031\001\035\001\036\001\020\001\021\001\054\001\170\000\034\001\
\160\000\079\000\053\001\054\001\020\001\021\001\004\001\141\000\
\036\000\037\000\052\001\051\001\054\001\011\001\052\001\077\000\
\051\001\051\001\051\001\027\001\028\001\020\001\021\001\022\001\
\040\001\033\001\160\000\035\001\036\001\037\001\038\001\039\001\
\033\001\041\001\035\001\036\001\037\001\038\001\039\001\029\001\
\041\001\007\001\036\001\009\001\029\001\053\001\054\001\016\001\
\017\001\036\001\044\001\017\001\053\001\054\001\048\001\049\001\
\050\001\044\001\106\000\045\001\054\001\048\001\049\001\024\001\
\025\001\026\001\030\001\054\001\012\001\013\001\014\001\119\000\
\120\000\007\001\008\001\009\001\010\001\035\001\052\001\040\001\
\050\001\030\001\016\001\017\001\001\000\002\000\003\000\004\000\
\007\001\008\001\009\001\010\001\007\001\008\001\009\001\010\001\
\007\001\008\001\009\001\010\001\050\001\003\001\011\001\051\001\
\011\001\011\001\032\001\031\001\011\001\001\001\011\001\011\001\
\035\001\004\001\019\001\051\001\009\001\011\001\051\001\017\001\
\019\001\017\001\035\001\005\001\051\001\000\000\016\001\007\001\
\017\001\017\001\051\001\007\001\040\001\030\001\003\001\001\001\
\011\001\011\001\050\001\001\001\019\001\011\001\009\001\007\001\
\005\001\002\000\160\000\007\001\102\000\102\000\157\000\184\000\
\103\000\155\000\056\000\176\000\185\000\180\000\024\000\013\000\
\088\000\087\000\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\003\001"

let yynames_const = "\
  Eof\000\
  OpenConditionalSect\000\
  CloseConditionalSect\000\
  OpenBracket\000\
  CloseBracket\000\
  OpenParen\000\
  CloseParen\000\
  CloseParenQuestion\000\
  CloseParenStar\000\
  CloseParenPlus\000\
  GT\000\
  Question\000\
  Star\000\
  Plus\000\
  Eq\000\
  Comma\000\
  Bar\000\
  XmlDeclStart\000\
  XmlDeclEnd\000\
  PCDATA\000\
  REQUIRED\000\
  IMPLIED\000\
  FIXED\000\
  INCLUDE\000\
  "

let yynames_block = "\
  ExternalIDSystem\000\
  ExternalIDPublic\000\
  PublicID\000\
  Ignore\000\
  VersionInfo\000\
  EncodingDecl\000\
  SDDecl\000\
  NDataDecl\000\
  AttlistName\000\
  NotationTypeName\000\
  Space\000\
  Comment\000\
  NotationName\000\
  GEDeclName\000\
  PEDeclName\000\
  AttValue\000\
  PEReference\000\
  CharRef\000\
  EntityRef\000\
  CData\000\
  CharDataIntercal\000\
  FirstSTag\000\
  FirstEmptyElemTag\000\
  STag\000\
  EmptyElemTag\000\
  ETag\000\
  Nmtoken\000\
  DocTypeName\000\
  ElementName\000\
  PI\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'xml_decl_opt) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'misc_star) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'doc_type_decl_opt) in
    Obj.repr(
# 338 "src/xml/parsing/xml_parser.mly"
  (
    let doc_type_decl, misc_star_trailing =
      match _3 with
      | None -> None, []
      | Some (doc_type_decl, misc_star_trailing) ->
        Some doc_type_decl, misc_star_trailing in
    {
      xml_decl = _1;
      xml_misc_star = _2;
      xml_doc_type_decl = doc_type_decl;
      xml_misc_star_trailing = misc_star_trailing;
    }
  )
# 621 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.prolog))
; (fun __caml_parser_env ->
    Obj.repr(
# 356 "src/xml/parsing/xml_parser.mly"
    ( None )
# 627 "src/xml/parsing/xml_parser.ml"
               : 'xml_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'xml_decl) in
    Obj.repr(
# 357 "src/xml/parsing/xml_parser.mly"
             ( Some _1 )
# 634 "src/xml/parsing/xml_parser.ml"
               : 'xml_decl_opt))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'encoding_decl_opt) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'sdd_decl_opt) in
    Obj.repr(
# 363 "src/xml/parsing/xml_parser.mly"
    ( make_xml_decl _2 _3 _4 )
# 643 "src/xml/parsing/xml_parser.ml"
               : 'xml_decl))
; (fun __caml_parser_env ->
    Obj.repr(
# 375 "src/xml/parsing/xml_parser.mly"
    ( None )
# 649 "src/xml/parsing/xml_parser.ml"
               : 'encoding_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 376 "src/xml/parsing/xml_parser.mly"
                 ( Some (make_enc_name _1) )
# 656 "src/xml/parsing/xml_parser.ml"
               : 'encoding_decl_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 384 "src/xml/parsing/xml_parser.mly"
    ( None )
# 662 "src/xml/parsing/xml_parser.ml"
               : 'sdd_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : bool) in
    Obj.repr(
# 386 "src/xml/parsing/xml_parser.mly"
    ( Some (make_sdd_decl _1) )
# 669 "src/xml/parsing/xml_parser.ml"
               : 'sdd_decl_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 392 "src/xml/parsing/xml_parser.mly"
    ( None )
# 675 "src/xml/parsing/xml_parser.ml"
               : 'doc_type_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'doc_type_decl) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'misc_star) in
    Obj.repr(
# 393 "src/xml/parsing/xml_parser.mly"
                            ( Some (_1, _2) )
# 683 "src/xml/parsing/xml_parser.ml"
               : 'doc_type_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Xml_basics_ast.tag_attribute_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'external_id_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'int_subset_outer_opt) in
    Obj.repr(
# 400 "src/xml/parsing/xml_parser.mly"
    (
      {
        doctype_name = _1;
        external_id = _2;
        int_subset = _3;
      }
    )
# 698 "src/xml/parsing/xml_parser.ml"
               : 'doc_type_decl))
; (fun __caml_parser_env ->
    Obj.repr(
# 410 "src/xml/parsing/xml_parser.mly"
    ( None )
# 704 "src/xml/parsing/xml_parser.ml"
               : 'external_id_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'external_id) in
    Obj.repr(
# 411 "src/xml/parsing/xml_parser.mly"
                ( Some _1 )
# 711 "src/xml/parsing/xml_parser.ml"
               : 'external_id_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 415 "src/xml/parsing/xml_parser.mly"
                     ( System _1 )
# 718 "src/xml/parsing/xml_parser.ml"
               : 'external_id))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string * string) in
    Obj.repr(
# 417 "src/xml/parsing/xml_parser.mly"
    ( let pl, sl = _1 in
      Public (pl, sl) )
# 726 "src/xml/parsing/xml_parser.ml"
               : 'external_id))
; (fun __caml_parser_env ->
    Obj.repr(
# 428 "src/xml/parsing/xml_parser.mly"
    ( None )
# 732 "src/xml/parsing/xml_parser.ml"
               : 'int_subset_outer_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'int_subset_outer) in
    Obj.repr(
# 429 "src/xml/parsing/xml_parser.mly"
                     ( Some _1 )
# 739 "src/xml/parsing/xml_parser.ml"
               : 'int_subset_outer_opt))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'int_subset) in
    Obj.repr(
# 433 "src/xml/parsing/xml_parser.mly"
                                      ( _2 )
# 746 "src/xml/parsing/xml_parser.ml"
               : 'int_subset_outer))
; (fun __caml_parser_env ->
    Obj.repr(
# 446 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 752 "src/xml/parsing/xml_parser.ml"
               : 'int_subset))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'int_subset_elem) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'int_subset) in
    Obj.repr(
# 447 "src/xml/parsing/xml_parser.mly"
                               ( _1 :: _2 )
# 760 "src/xml/parsing/xml_parser.ml"
               : 'int_subset))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'markup_decl) in
    Obj.repr(
# 452 "src/xml/parsing/xml_parser.mly"
                ( Markup_decl _1 )
# 767 "src/xml/parsing/xml_parser.ml"
               : 'int_subset_elem))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'decl_sep) in
    Obj.repr(
# 453 "src/xml/parsing/xml_parser.mly"
                ( Sep_decl _1 )
# 774 "src/xml/parsing/xml_parser.ml"
               : 'int_subset_elem))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.xml_name) in
    Obj.repr(
# 457 "src/xml/parsing/xml_parser.mly"
                ( Pe_reference_decl _1 )
# 781 "src/xml/parsing/xml_parser.ml"
               : 'decl_sep))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 458 "src/xml/parsing/xml_parser.mly"
          ( Space_decl _1 )
# 788 "src/xml/parsing/xml_parser.ml"
               : 'decl_sep))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'element_decl) in
    Obj.repr(
# 464 "src/xml/parsing/xml_parser.mly"
                 ( _1 )
# 795 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'attlist_decl) in
    Obj.repr(
# 465 "src/xml/parsing/xml_parser.mly"
                 ( _1 )
# 802 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'entity_decl) in
    Obj.repr(
# 466 "src/xml/parsing/xml_parser.mly"
                ( Entity_decl _1 )
# 809 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'notation_decl) in
    Obj.repr(
# 467 "src/xml/parsing/xml_parser.mly"
                  ( _1 )
# 816 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.pi_target * Xml_basics_ast.pi_instructions) in
    Obj.repr(
# 468 "src/xml/parsing/xml_parser.mly"
       ( Pi_decl (make_pi _1) )
# 823 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 469 "src/xml/parsing/xml_parser.mly"
            ( Comment_decl _1 )
# 830 "src/xml/parsing/xml_parser.ml"
               : 'markup_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.tag_attribute_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'content_spec) in
    Obj.repr(
# 475 "src/xml/parsing/xml_parser.mly"
    ( Element_decl (_1, _2) )
# 838 "src/xml/parsing/xml_parser.ml"
               : 'element_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.nm_token) in
    Obj.repr(
# 481 "src/xml/parsing/xml_parser.mly"
    (
      let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 1 in
      make_content_spec start_pos end_pos _1
    )
# 849 "src/xml/parsing/xml_parser.ml"
               : 'content_spec))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'children) in
    Obj.repr(
# 486 "src/xml/parsing/xml_parser.mly"
             ( Content_spec_children _1 )
# 856 "src/xml/parsing/xml_parser.ml"
               : 'content_spec))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'mixed) in
    Obj.repr(
# 487 "src/xml/parsing/xml_parser.mly"
          ( Content_spec_mixed _1 )
# 863 "src/xml/parsing/xml_parser.ml"
               : 'content_spec))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'choice) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_paren_spec_mod) in
    Obj.repr(
# 493 "src/xml/parsing/xml_parser.mly"
    ( make_content_spec_mod_choice _2 _3 )
# 871 "src/xml/parsing/xml_parser.ml"
               : 'children))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'seq) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_paren_spec_mod) in
    Obj.repr(
# 495 "src/xml/parsing/xml_parser.mly"
    ( make_content_spec_mod_seq _2 _3 )
# 879 "src/xml/parsing/xml_parser.ml"
               : 'children))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'cp) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'cp_alternative_plus) in
    Obj.repr(
# 500 "src/xml/parsing/xml_parser.mly"
    ( _1, _2 )
# 887 "src/xml/parsing/xml_parser.ml"
               : 'choice))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'cp) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'cp_comma_star) in
    Obj.repr(
# 505 "src/xml/parsing/xml_parser.mly"
    ( _1, _2 )
# 895 "src/xml/parsing/xml_parser.ml"
               : 'seq))
; (fun __caml_parser_env ->
    Obj.repr(
# 509 "src/xml/parsing/xml_parser.mly"
               ( None_mod )
# 901 "src/xml/parsing/xml_parser.ml"
               : 'close_paren_spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 510 "src/xml/parsing/xml_parser.mly"
                   ( Plus_mod )
# 907 "src/xml/parsing/xml_parser.ml"
               : 'close_paren_spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 511 "src/xml/parsing/xml_parser.mly"
                   ( Star_mod )
# 913 "src/xml/parsing/xml_parser.ml"
               : 'close_paren_spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 512 "src/xml/parsing/xml_parser.mly"
                       ( Opt_mod )
# 919 "src/xml/parsing/xml_parser.ml"
               : 'close_paren_spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 516 "src/xml/parsing/xml_parser.mly"
    ( None_mod )
# 925 "src/xml/parsing/xml_parser.ml"
               : 'spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 517 "src/xml/parsing/xml_parser.mly"
         ( Plus_mod )
# 931 "src/xml/parsing/xml_parser.ml"
               : 'spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 518 "src/xml/parsing/xml_parser.mly"
         ( Star_mod )
# 937 "src/xml/parsing/xml_parser.ml"
               : 'spec_mod))
; (fun __caml_parser_env ->
    Obj.repr(
# 519 "src/xml/parsing/xml_parser.mly"
             ( Opt_mod )
# 943 "src/xml/parsing/xml_parser.ml"
               : 'spec_mod))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'choice) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_paren_spec_mod) in
    Obj.repr(
# 524 "src/xml/parsing/xml_parser.mly"
    ( make_content_spec_particle_mod_choice _2 _3 )
# 951 "src/xml/parsing/xml_parser.ml"
               : 'cp))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'seq) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'close_paren_spec_mod) in
    Obj.repr(
# 526 "src/xml/parsing/xml_parser.mly"
    ( make_content_spec_particle_mod_seq _2 _3 )
# 959 "src/xml/parsing/xml_parser.ml"
               : 'cp))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Xml_basics_ast.nm_token) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'spec_mod) in
    Obj.repr(
# 528 "src/xml/parsing/xml_parser.mly"
    ( let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 1 in
      make_content_spec_particle_mod_name start_pos end_pos _1 _2 )
# 969 "src/xml/parsing/xml_parser.ml"
               : 'cp))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'cp) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cp_alternative_star) in
    Obj.repr(
# 534 "src/xml/parsing/xml_parser.mly"
                               ( _2, _3 )
# 977 "src/xml/parsing/xml_parser.ml"
               : 'cp_alternative_plus))
; (fun __caml_parser_env ->
    Obj.repr(
# 538 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 983 "src/xml/parsing/xml_parser.ml"
               : 'cp_alternative_star))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'cp) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cp_alternative_star) in
    Obj.repr(
# 539 "src/xml/parsing/xml_parser.mly"
                               ( _2 :: _3 )
# 991 "src/xml/parsing/xml_parser.ml"
               : 'cp_alternative_star))
; (fun __caml_parser_env ->
    Obj.repr(
# 543 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 997 "src/xml/parsing/xml_parser.ml"
               : 'cp_comma_star))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'cp) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cp_comma_star) in
    Obj.repr(
# 544 "src/xml/parsing/xml_parser.mly"
                           ( _2 :: _3 )
# 1005 "src/xml/parsing/xml_parser.ml"
               : 'cp_comma_star))
; (fun __caml_parser_env ->
    Obj.repr(
# 565 "src/xml/parsing/xml_parser.mly"
    ( PCData_empty )
# 1011 "src/xml/parsing/xml_parser.ml"
               : 'mixed))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'alternative_names1) in
    Obj.repr(
# 567 "src/xml/parsing/xml_parser.mly"
    ( PCData_alternatives _3 )
# 1018 "src/xml/parsing/xml_parser.ml"
               : 'mixed))
; (fun __caml_parser_env ->
    Obj.repr(
# 573 "src/xml/parsing/xml_parser.mly"
    ( PCData )
# 1024 "src/xml/parsing/xml_parser.ml"
               : 'mixed))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'alternative_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'alternative_names) in
    Obj.repr(
# 577 "src/xml/parsing/xml_parser.mly"
                                       ( _1 :: _2 )
# 1032 "src/xml/parsing/xml_parser.ml"
               : 'alternative_names1))
; (fun __caml_parser_env ->
    Obj.repr(
# 581 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1038 "src/xml/parsing/xml_parser.ml"
               : 'alternative_names))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'alternative_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'alternative_names) in
    Obj.repr(
# 582 "src/xml/parsing/xml_parser.mly"
                                       ( _1 :: _2 )
# 1046 "src/xml/parsing/xml_parser.ml"
               : 'alternative_names))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.nm_token) in
    Obj.repr(
# 587 "src/xml/parsing/xml_parser.mly"
    (
      let start_pos = Parsing.rhs_start_pos 2 in
      let end_pos = Parsing.rhs_end_pos 2 in
      make_xml_name start_pos end_pos _2
    )
# 1057 "src/xml/parsing/xml_parser.ml"
               : 'alternative_name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'att_def_list) in
    Obj.repr(
# 610 "src/xml/parsing/xml_parser.mly"
                                ( Attlist_decl (_1, _2) )
# 1065 "src/xml/parsing/xml_parser.ml"
               : 'attlist_decl))
; (fun __caml_parser_env ->
    Obj.repr(
# 614 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1071 "src/xml/parsing/xml_parser.ml"
               : 'att_def_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'att_def) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'att_def_list) in
    Obj.repr(
# 616 "src/xml/parsing/xml_parser.mly"
    ( _1 :: _2 )
# 1079 "src/xml/parsing/xml_parser.ml"
               : 'att_def_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Xml_basics_ast.nm_token) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'att_type) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'att_default) in
    Obj.repr(
# 623 "src/xml/parsing/xml_parser.mly"
    (
      let start_pos = Parsing.rhs_start_pos 2 in
      let end_pos = Parsing.rhs_end_pos 2 in
      make_att_def start_pos end_pos _2 _4 _6
    )
# 1095 "src/xml/parsing/xml_parser.ml"
               : 'att_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.nm_token) in
    Obj.repr(
# 632 "src/xml/parsing/xml_parser.mly"
    (
      let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 1 in
      make_att_type start_pos end_pos _1
    )
# 1106 "src/xml/parsing/xml_parser.ml"
               : 'att_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'enumerated_type) in
    Obj.repr(
# 637 "src/xml/parsing/xml_parser.mly"
                    ( Enumerated_type _1 )
# 1113 "src/xml/parsing/xml_parser.ml"
               : 'att_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'notation_type) in
    Obj.repr(
# 641 "src/xml/parsing/xml_parser.mly"
                  ( _1)
# 1120 "src/xml/parsing/xml_parser.ml"
               : 'enumerated_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'enumeration_type) in
    Obj.repr(
# 642 "src/xml/parsing/xml_parser.mly"
                     ( _1)
# 1127 "src/xml/parsing/xml_parser.ml"
               : 'enumerated_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'or_names) in
    Obj.repr(
# 647 "src/xml/parsing/xml_parser.mly"
    ( Notation_type (_1, _2) )
# 1135 "src/xml/parsing/xml_parser.ml"
               : 'notation_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 651 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1141 "src/xml/parsing/xml_parser.ml"
               : 'or_names))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : Xml_basics_ast.nm_token) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'or_names) in
    Obj.repr(
# 653 "src/xml/parsing/xml_parser.mly"
    ( let start_pos = Parsing.rhs_start_pos 2 in
      let end_pos = Parsing.rhs_end_pos 2 in
      make_xml_name start_pos end_pos _2 :: _3 )
# 1151 "src/xml/parsing/xml_parser.ml"
               : 'or_names))
; (fun __caml_parser_env ->
    Obj.repr(
# 659 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1157 "src/xml/parsing/xml_parser.ml"
               : 'or_nmtokens))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : Xml_basics_ast.nm_token) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'or_nmtokens) in
    Obj.repr(
# 661 "src/xml/parsing/xml_parser.mly"
    ( _2 :: _3 )
# 1165 "src/xml/parsing/xml_parser.ml"
               : 'or_nmtokens))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.nm_token) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'or_nmtokens) in
    Obj.repr(
# 667 "src/xml/parsing/xml_parser.mly"
    ( Enumeration_type (_2, _3) )
# 1173 "src/xml/parsing/xml_parser.ml"
               : 'enumeration_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 671 "src/xml/parsing/xml_parser.mly"
             ( Required )
# 1179 "src/xml/parsing/xml_parser.ml"
               : 'att_default))
; (fun __caml_parser_env ->
    Obj.repr(
# 672 "src/xml/parsing/xml_parser.mly"
            ( Implied )
# 1185 "src/xml/parsing/xml_parser.ml"
               : 'att_default))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'att_value) in
    Obj.repr(
# 673 "src/xml/parsing/xml_parser.mly"
                    ( Fixed_att_value _2 )
# 1192 "src/xml/parsing/xml_parser.ml"
               : 'att_default))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'att_value) in
    Obj.repr(
# 674 "src/xml/parsing/xml_parser.mly"
              ( Att_value _1 )
# 1199 "src/xml/parsing/xml_parser.ml"
               : 'att_default))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.att_value) in
    Obj.repr(
# 678 "src/xml/parsing/xml_parser.mly"
             ( make_att_value_literal _1 )
# 1206 "src/xml/parsing/xml_parser.ml"
               : 'att_value))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'gentity_def) in
    Obj.repr(
# 682 "src/xml/parsing/xml_parser.mly"
                              ( Ge_decl (_1, _2) )
# 1214 "src/xml/parsing/xml_parser.ml"
               : 'entity_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'pentity_def) in
    Obj.repr(
# 683 "src/xml/parsing/xml_parser.mly"
                              ( Pe_decl (_1, _2) )
# 1222 "src/xml/parsing/xml_parser.ml"
               : 'entity_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'entity_value) in
    Obj.repr(
# 687 "src/xml/parsing/xml_parser.mly"
                 ( Gentity_value _1 )
# 1229 "src/xml/parsing/xml_parser.ml"
               : 'gentity_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'external_id) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'ndata_decl_opt) in
    Obj.repr(
# 688 "src/xml/parsing/xml_parser.mly"
                               ( Gentity_external_id (_1, _2) )
# 1237 "src/xml/parsing/xml_parser.ml"
               : 'gentity_def))
; (fun __caml_parser_env ->
    Obj.repr(
# 692 "src/xml/parsing/xml_parser.mly"
    ( None )
# 1243 "src/xml/parsing/xml_parser.ml"
               : 'ndata_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'ndata_decl) in
    Obj.repr(
# 693 "src/xml/parsing/xml_parser.mly"
               ( Some _1 )
# 1250 "src/xml/parsing/xml_parser.ml"
               : 'ndata_decl_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.xml_name) in
    Obj.repr(
# 697 "src/xml/parsing/xml_parser.mly"
              ( _1 )
# 1257 "src/xml/parsing/xml_parser.ml"
               : 'ndata_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'entity_value) in
    Obj.repr(
# 701 "src/xml/parsing/xml_parser.mly"
                 ( Pentity_value _1 )
# 1264 "src/xml/parsing/xml_parser.ml"
               : 'pentity_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'external_id) in
    Obj.repr(
# 702 "src/xml/parsing/xml_parser.mly"
                ( Pentity_external_id _1 )
# 1271 "src/xml/parsing/xml_parser.ml"
               : 'pentity_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.att_value) in
    Obj.repr(
# 707 "src/xml/parsing/xml_parser.mly"
    (
      let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 1 in
      make_entity_value_of_att_value start_pos end_pos _1
    )
# 1282 "src/xml/parsing/xml_parser.ml"
               : 'entity_value))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'external_id) in
    Obj.repr(
# 717 "src/xml/parsing/xml_parser.mly"
                                ( Notation_decl (_1, External_id _2) )
# 1290 "src/xml/parsing/xml_parser.ml"
               : 'notation_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_basics_ast.xml_name) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 718 "src/xml/parsing/xml_parser.mly"
                             ( Notation_decl (_1, Public_id _2) )
# 1298 "src/xml/parsing/xml_parser.ml"
               : 'notation_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'ext_subset_decl) in
    Obj.repr(
# 725 "src/xml/parsing/xml_parser.mly"
                    ( None, _1 )
# 1305 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.raw_ext_subset))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'text_decl) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'ext_subset_decl) in
    Obj.repr(
# 726 "src/xml/parsing/xml_parser.mly"
                              ( Some _1, _2 )
# 1313 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.raw_ext_subset))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'version_info_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 732 "src/xml/parsing/xml_parser.mly"
    (
      {
         text_version_info = _2;
         text_encoding_decl = _3;
      }
    )
# 1326 "src/xml/parsing/xml_parser.ml"
               : 'text_decl))
; (fun __caml_parser_env ->
    Obj.repr(
# 741 "src/xml/parsing/xml_parser.mly"
    ( None )
# 1332 "src/xml/parsing/xml_parser.ml"
               : 'version_info_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 742 "src/xml/parsing/xml_parser.mly"
                ( Some _1 )
# 1339 "src/xml/parsing/xml_parser.ml"
               : 'version_info_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 747 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1345 "src/xml/parsing/xml_parser.ml"
               : 'ext_subset_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'ext_subset_decl_elem) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'ext_subset_decl) in
    Obj.repr(
# 748 "src/xml/parsing/xml_parser.mly"
                                         ( _1 :: _2 )
# 1353 "src/xml/parsing/xml_parser.ml"
               : 'ext_subset_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'markup_decl) in
    Obj.repr(
# 752 "src/xml/parsing/xml_parser.mly"
                ( Ext_markup_decl _1 )
# 1360 "src/xml/parsing/xml_parser.ml"
               : 'ext_subset_decl_elem))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'conditional_sect) in
    Obj.repr(
# 753 "src/xml/parsing/xml_parser.mly"
                     ( Ext_conditional_sect _1 )
# 1367 "src/xml/parsing/xml_parser.ml"
               : 'ext_subset_decl_elem))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'decl_sep) in
    Obj.repr(
# 754 "src/xml/parsing/xml_parser.mly"
             ( Ext_sep_decl _1 )
# 1374 "src/xml/parsing/xml_parser.ml"
               : 'ext_subset_decl_elem))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'include_sect) in
    Obj.repr(
# 760 "src/xml/parsing/xml_parser.mly"
                 ( Include_sect _1 )
# 1381 "src/xml/parsing/xml_parser.ml"
               : 'conditional_sect))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'ignore_sect) in
    Obj.repr(
# 761 "src/xml/parsing/xml_parser.mly"
                ( Ignore_sect _1 )
# 1388 "src/xml/parsing/xml_parser.ml"
               : 'conditional_sect))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'ext_subset_decl) in
    Obj.repr(
# 767 "src/xml/parsing/xml_parser.mly"
                                                ( _2 )
# 1395 "src/xml/parsing/xml_parser.ml"
               : 'include_sect))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 775 "src/xml/parsing/xml_parser.mly"
          ( _1 )
# 1402 "src/xml/parsing/xml_parser.ml"
               : 'ignore_sect))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_lexer_types.token_tag) in
    Obj.repr(
# 783 "src/xml/parsing/xml_parser.mly"
    ( make_empty_element_tag _1 )
# 1409 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_lexer_types.token_tag) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'content) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.tag_name) in
    Obj.repr(
# 786 "src/xml/parsing/xml_parser.mly"
    ( let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 3 in
      make_content_element_tag start_pos end_pos _1 _2 _3 )
# 1420 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_lexer_types.token_tag) in
    Obj.repr(
# 794 "src/xml/parsing/xml_parser.mly"
                      ( make_empty_element_tag _1 )
# 1427 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Xml_lexer_types.token_tag) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'content) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.tag_name) in
    Obj.repr(
# 797 "src/xml/parsing/xml_parser.mly"
    ( let start_pos = Parsing.rhs_start_pos 1 in
      let end_pos = Parsing.rhs_end_pos 3 in
      make_content_element_tag start_pos end_pos _1 _2 _3 )
# 1438 "src/xml/parsing/xml_parser.ml"
               : Xml_ast.element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'char_data_intercal_opt) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'content_tag_star) in
    Obj.repr(
# 808 "src/xml/parsing/xml_parser.mly"
    ( make_element_content (_1, _2) )
# 1446 "src/xml/parsing/xml_parser.ml"
               : 'content))
; (fun __caml_parser_env ->
    Obj.repr(
# 812 "src/xml/parsing/xml_parser.mly"
    ( None )
# 1452 "src/xml/parsing/xml_parser.ml"
               : 'char_data_intercal_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_lexer_types.char_data_intercal) in
    Obj.repr(
# 813 "src/xml/parsing/xml_parser.mly"
                     ( Some _1 )
# 1459 "src/xml/parsing/xml_parser.ml"
               : 'char_data_intercal_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 817 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1465 "src/xml/parsing/xml_parser.ml"
               : 'content_tag_star))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'content_tag_star_element) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'content_tag_star) in
    Obj.repr(
# 818 "src/xml/parsing/xml_parser.mly"
                                              ( _1 :: _2 )
# 1473 "src/xml/parsing/xml_parser.ml"
               : 'content_tag_star))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'content_element) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'char_data_intercal_opt) in
    Obj.repr(
# 823 "src/xml/parsing/xml_parser.mly"
    (
      _1, _2
    )
# 1483 "src/xml/parsing/xml_parser.ml"
               : 'content_tag_star_element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_ast.element) in
    Obj.repr(
# 830 "src/xml/parsing/xml_parser.mly"
            ( Child_element _1 )
# 1490 "src/xml/parsing/xml_parser.ml"
               : 'content_element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 831 "src/xml/parsing/xml_parser.mly"
          ( Child_cd_sect _1 )
# 1497 "src/xml/parsing/xml_parser.ml"
               : 'content_element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.pi_target * Xml_basics_ast.pi_instructions) in
    Obj.repr(
# 833 "src/xml/parsing/xml_parser.mly"
       ( Child_pi (make_pi _1) )
# 1504 "src/xml/parsing/xml_parser.ml"
               : 'content_element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 835 "src/xml/parsing/xml_parser.mly"
            ( Child_comment _1 )
# 1511 "src/xml/parsing/xml_parser.ml"
               : 'content_element))
; (fun __caml_parser_env ->
    Obj.repr(
# 840 "src/xml/parsing/xml_parser.mly"
    ( [] )
# 1517 "src/xml/parsing/xml_parser.ml"
               : 'misc_star))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'misc) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'misc_star) in
    Obj.repr(
# 841 "src/xml/parsing/xml_parser.mly"
                   ( _1 :: _2 )
# 1525 "src/xml/parsing/xml_parser.ml"
               : 'misc_star))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 846 "src/xml/parsing/xml_parser.mly"
            ( Misc_comment _1 )
# 1532 "src/xml/parsing/xml_parser.ml"
               : 'misc))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Xml_basics_ast.pi_target * Xml_basics_ast.pi_instructions) in
    Obj.repr(
# 847 "src/xml/parsing/xml_parser.mly"
       ( Misc_pi (make_pi _1) )
# 1539 "src/xml/parsing/xml_parser.ml"
               : 'misc))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 848 "src/xml/parsing/xml_parser.mly"
          ( Misc_space _1 )
# 1546 "src/xml/parsing/xml_parser.ml"
               : 'misc))
(* Entry prolog *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry element *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry element_after_prolog *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry ext_subset *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let prolog (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Xml_ast.prolog)
let element (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 2 lexfun lexbuf : Xml_ast.element)
let element_after_prolog (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 3 lexfun lexbuf : Xml_ast.element)
let ext_subset (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 4 lexfun lexbuf : Xml_ast.raw_ext_subset)
