(***********************************************************************)
(*                                                                     *)
(*                          Ocaml-xml Library                          *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2013-2018,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xml_to_ast.ml,v 1.1 2019-05-10 14:00:50 jpc Exp $ *)

open Xml_ast;;

let parse_prolog lexbuf = Xml_parser.prolog Xml_lexer.prolog lexbuf;;
let parse_element_after_prolog lexbuf =
  Xml_parser.element_after_prolog Xml_lexer.element lexbuf
;;
let parse_element lexbuf = Xml_parser.element Xml_lexer.element lexbuf;;

let empty_prolog = {
  xml_decl = None;
  xml_misc_star = [];
  xml_doc_type_decl = None;
  xml_misc_star_trailing = [];
}
;;

let parse_file fname =
  let ic, lexbuf = Location.lexing_from_file fname in
  let start_token =
    try
      let start_token = Xml_lexer.start lexbuf in

      Xml_say.fdebug
        (fun ppf ->
         Format.fprintf ppf
           "start token: %a"
           Xml_lexer_types_print.print_start_document start_token);

      Pervasives.close_in ic;
      start_token
    with
    | x ->
      Pervasives.close_in ic;
      raise x in

  let ic, lexbuf = Location.lexing_from_file fname in
  try
    let prolog, parse_element =
      match start_token with
      | Xml_lexer_types.Start_prolog ->
        parse_prolog lexbuf, parse_element_after_prolog
      | Xml_lexer_types.Start_element ->
        empty_prolog, parse_element in

    let element = parse_element lexbuf in
    let document = { prolog = prolog; element = element; misc_star = []; } in
    Pervasives.close_in ic;
    document
  with
  | x ->
    Pervasives.close_in ic;
    raise x
;;

let read_xml_file fname =
  let document =  parse_file fname in
  Xml_say.debug "xml: parsing phase done";
  Xml_say.fdebug (fun ppf -> Xml_ast_print.print_raw_document ppf document);
  document
;;

(*
 Local Variables:
  compile-command: "cd ../../..; make"
 End:
*)
