(***********************************************************************)
(*                                                                     *)
(*                               Simport                               *)
(*                                                                     *)
(*                   Pierre Weis, INRIA Rocquencourt                   *)
(*                                                                     *)
(*  Copyright 2010-2019,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved.                                               *)
(*                                                                     *)
(*  This file is distributed under the terms of the BSD License.       *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.5+dev0 generated code. *)

open Ident_print;;

open Ast_types_print;;

open Binding_print;;

open Mdl_block_types_print;;

let printer_model_ident ctx ppf = function
  | i_i -> Ident_print.printer_ident ctx ppf i_i

and printer_configuration_ident ctx ppf = function
  | i_i -> Ident_print.printer_ident ctx ppf i_i

and printer_tag_ident ctx ppf = function
  | i_i -> Ident_print.printer_ident ctx ppf i_i
;;

let printer_prop_name ctx ppf = function
  | i_i -> Ident_print.printer_ident ctx ppf i_i

and printer_class_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m
;;

let printer_system_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_block_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_block_source_path ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_path ctx ppf a_m

and printer_block_source_type ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_line_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_annotation_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_text ctx ppf a_m

and printer_port_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_port_number ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_port_number ctx ppf a_m

and printer_object_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_backup_class_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_array_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_ui_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m

and printer_matstruct_name ctx ppf = function
  | a_m -> Ast_types_print.printer_mdl_name ctx ppf a_m
;;

let printer_ast_node pp_a ctx ppf = function
  | { Mdl_ast.ast_loc = l_t; Mdl_ast.ast_desc = a; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>ast_loc =";
    Location_print.printer_t Og_precedence.Ck_record_field ppf l_t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>ast_desc =";
    pp_a Og_precedence.Ck_record_field ppf a; Format.fprintf ppf "%,;@]";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_model_version ctx ppf = function
  | Mdl_ast.Mdl_model_version s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mdl_model_version";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Slx_model_version s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Slx_model_version";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_enum ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let rec printer_configuration_longident ctx ppf = function
  | Mdl_ast.Configuration_lident c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_lident";
    printer_configuration_ident Og_precedence.Ck_argument ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Configuration_ldot (c, c0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_ldot";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_configuration_longident Og_precedence.Ck_tuple_item ppf c;
    Format.fprintf ppf "%,,@ ";
    printer_configuration_ident Og_precedence.Ck_tuple_item ppf c0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_line_type _ctx ppf = function
  | Mdl_ast.Connection ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Connection"

and printer_connection_type _ctx ppf = function
  | Mdl_ast.Dest_src ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Dest_src"
  | Mdl_ast.Dest_dest ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Dest_dest"
  | Mdl_ast.Src_dest ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Src_dest"
  | Mdl_ast.Src_src ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Src_src"

and printer_array_type ctx ppf = function
  | Mdl_ast.Struct ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Struct"
  | Mdl_ast.Cell ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Cell"
  | Mdl_ast.Handle ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Handle"
  | Mdl_ast.Array_type s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_type";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_object_type ctx ppf = function | m -> printer_mdl_name ctx ppf m

and printer_matstruct_type ctx ppf = function
  | m -> printer_mdl_name ctx ppf m

and printer_interface_block_parameter_type ctx ppf = function
  | i_i -> Ident_print.printer_ident ctx ppf i_i
;;

let printer_block_sf_block_type _ctx ppf = function
  | Mdl_ast.Block_sf_block_type_none ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Block_sf_block_type_none"
  | Mdl_ast.Block_sf_block_type_matlab_function ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Block_sf_block_type_matlab_function"
  | Mdl_ast.Block_sf_block_type_chart ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Block_sf_block_type_chart"
;;

let printer_custom_property_type ctx ppf = function
  | Mdl_ast.Custom_property_type_uint8 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_uint8"
  | Mdl_ast.Custom_property_type_uint16 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_uint16"
  | Mdl_ast.Custom_property_type_uint32 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_uint32"
  | Mdl_ast.Custom_property_type_uint64 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_uint64"
  | Mdl_ast.Custom_property_type_int8 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_int8"
  | Mdl_ast.Custom_property_type_int16 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_int16"
  | Mdl_ast.Custom_property_type_int32 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_int32"
  | Mdl_ast.Custom_property_type_int64 ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_int64"
  | Mdl_ast.Custom_property_type_slbool ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_slbool"
  | Mdl_ast.Custom_property_type_string ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Custom_property_type_string"
  | Mdl_ast.Custom_property_type_custom a_m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Custom_property_type_custom";
    Ast_types_print.printer_mdl_name Og_precedence.Ck_argument ppf a_m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_naturals ctx ppf = function
  | a_n_l ->
    Og_printer.printer_List Ast_types_print.printer_natural ctx ppf a_n_l
;;

let printer_integer_matrix ctx ppf = function
  | i_a_a ->
    Og_printer.printer_Array
      (Og_printer.printer_Array Og_printer.printer_Int)
      ctx
      ppf
      i_a_a
;;

let printer_object_id ctx ppf = function
  | (p_f, n) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Path_print.printer_file_name Og_precedence.Ck_tuple_item ppf p_f;
    Format.fprintf ppf "%,,@ ";
    printer_natural Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_zorder ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let rec printer_simple_values ctx ppf = function
  | s_l -> Og_printer.printer_List printer_simple_value ctx ppf s_l

and printer_simple_value ctx ppf = function
  | Mdl_ast.Bool s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Bool";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Float s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Float";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Int s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Int";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.String s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.String";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Ident i_i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Ident";
    Ident_print.printer_ident Og_precedence.Ck_argument ppf i_i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_simple_binding ctx ppf = function
  | s_b_i ->
    Binding_print.printer_ident_binding printer_simple_value ctx ppf s_b_i
;;

let rec printer_model_values ctx ppf = function
  | m_l -> Og_printer.printer_List printer_model_value ctx ppf m_l

and printer_model_value ctx ppf = function
  | Mdl_ast.Value_simple s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Value_simple";
    printer_simple_value Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Value_vector s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Value_vector";
    printer_simple_values Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Value_matrix m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Value_matrix";
    printer_model_values Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Value_bindings b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Value_bindings";
    printer_bindings Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_bindings ctx ppf = function
  | b_l -> Og_printer.printer_List printer_binding ctx ppf b_l

and printer_binding ctx ppf = function
  | m_b_i ->
    Binding_print.printer_ident_binding printer_model_value ctx ppf m_b_i
;;

let rec printer_poly_annotation_graphical_default pp_font_and_color ctx ppf =
  function
  | Mdl_ast.Font_and_color f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_and_color";
    pp_font_and_color Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Horizontal_alignment h ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Horizontal_alignment";
    printer_horizontal_alignment Og_precedence.Ck_argument ppf h;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Vertical_alignment v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Vertical_alignment";
    printer_vertical_alignment Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Text_as_click_cb m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Text_as_click_cb";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Click_fcn m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Click_fcn";
    printer_mdl_code Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Tex_mode m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Tex_mode";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Interpreter s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Interpreter";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_graphical_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_graphical_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_horizontal_alignment _ctx ppf = function
  | Mdl_ast.Alignment_Center ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Center"
  | Mdl_ast.Alignment_Left ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Left"
  | Mdl_ast.Alignment_Right ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Right"

and printer_vertical_alignment _ctx ppf = function
  | Mdl_ast.Alignment_Top ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Top"
  | Mdl_ast.Alignment_Middle ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Middle"
  | Mdl_ast.Alignment_Bottom ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Alignment_Bottom"
;;

let rec printer_font_items ctx ppf = function
  | f_l -> Og_printer.printer_List printer_font_item ctx ppf f_l

and printer_font_item ctx ppf = function
  | Mdl_ast.Font_angle s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_angle";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Font_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_name";
    printer_mdl_font_name Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Font_size n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_size";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Font_weight s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_weight";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_font_and_color ctx ppf = function
  | Mdl_ast.Background c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Background";
    printer_color Og_precedence.Ck_argument ppf c; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Drop_shadow m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Drop_shadow";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Font_item f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Font_item";
    printer_font_item Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Foreground c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Foreground";
    printer_color Og_precedence.Ck_argument ppf c; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_color ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let rec printer_block_graphical_defaults ctx ppf = function
  | b_l ->
    Og_printer.printer_List printer_block_graphical_default ctx ppf b_l

and printer_block_graphical_default ctx ppf = function
  | Mdl_ast.Block_mirror m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_mirror";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_name_placement s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_name_placement";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_orientation s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_orientation";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_rotation n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_rotation";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_show_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_show_name";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_hide_automatic_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_hide_automatic_name";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_zorder z ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_zorder";
    printer_zorder Og_precedence.Ck_argument ppf z;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_font_and_color f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_font_and_color";
    printer_font_and_color Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_block_parameter_defaults_list ctx ppf = function
  | b_l ->
    Og_printer.printer_List printer_block_parameter_defaults ctx ppf b_l

and printer_block_parameter_defaults ctx ppf = function
  | { Mdl_ast.block_type = b; Mdl_ast.parameter_defaults = b0; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>block_type =";
    printer_block_type Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>parameter_defaults =";
    printer_bindings Og_precedence.Ck_record_field ppf b0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_line_graphical_defaults ctx ppf = function
  | l_l -> Og_printer.printer_List printer_line_graphical_default ctx ppf l_l

and printer_line_graphical_default ctx ppf = function
  | Mdl_ast.Line_font_item f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_font_item";
    printer_font_item Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_zorder_item z ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_zorder_item";
    printer_zorder Og_precedence.Ck_argument ppf z;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_mask_graphical_defaults ctx ppf = function
  | m_l -> Og_printer.printer_List printer_mask_graphical_default ctx ppf m_l

and printer_mask_graphical_default ctx ppf = function
  | b -> printer_binding ctx ppf b

and printer_mask_parameter_defaults ctx ppf = function
  | m_l -> Og_printer.printer_List printer_mask_parameter_default ctx ppf m_l

and printer_mask_parameter_default ctx ppf = function
  | b -> printer_binding ctx ppf b
;;

let rec printer_annotation_graphical_defaults ctx ppf = function
  | a_l ->
    Og_printer.printer_List printer_annotation_graphical_default ctx ppf a_l

and printer_annotation_graphical_default ctx ppf = function
  | f_p ->
    printer_poly_annotation_graphical_default printer_font_and_color
      ctx
      ppf
      f_p
;;

let rec printer_lines ctx ppf = function
  | l_l -> Og_printer.printer_List printer_line ctx ppf l_l

and printer_line ctx ppf = function
  | Mdl_ast.Line l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line";
    printer_line_items Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_line_items ctx ppf = function
  | l_l -> Og_printer.printer_List printer_line_item ctx ppf l_l

and printer_line_item ctx ppf = function
  | Mdl_ast.Line_name l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_name";
    printer_line_name Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_source l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_source";
    printer_line_source Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target";
    printer_line_target Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_type l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_type";
    printer_line_type Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_labels l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_labels";
    printer_labels Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_points p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_points";
    printer_points Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_graphical_default l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_graphical_default";
    printer_line_graphical_default Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_line_source ctx ppf = function
  | Mdl_ast.Line_source_connector l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_source_connector";
    printer_line_connector Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_line_connector ctx ppf = function
  | { Mdl_ast.line_connector_block = b; Mdl_ast.line_connector_port = p; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>line_connector_block =";
    printer_block_name Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>line_connector_port =";
    printer_port_value Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_port_value ctx ppf = function
  | Mdl_ast.Port_trigger ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Port_trigger"
  | Mdl_ast.Port_enable ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Port_enable"
  | Mdl_ast.Port_state ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Port_state"
  | Mdl_ast.Port_ifaction ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Port_ifaction"
  | Mdl_ast.Port_number p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Port_number";
    printer_port_number Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Port_lconn p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Port_lconn";
    printer_port_number Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Port_rconn p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Port_rconn";
    printer_port_number Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_line_target ctx ppf = function
  | Mdl_ast.Line_target_connector l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_connector";
    printer_line_connector Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch";
    printer_line_target_branch_items Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_line_target_branch_items ctx ppf = function
  | l_l ->
    Og_printer.printer_List printer_line_target_branch_item ctx ppf l_l

and printer_line_target_branch_item ctx ppf = function
  | Mdl_ast.Line_target_branch_connection_type c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_connection_type";
    printer_connection_type Og_precedence.Ck_argument ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch_labels l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_labels";
    printer_labels Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch_points p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_points";
    printer_points Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch_source l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_source";
    printer_line_source Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch_target l ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_target";
    printer_line_target Og_precedence.Ck_argument ppf l;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Line_target_branch_zorder z ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Line_target_branch_zorder";
    printer_zorder Og_precedence.Ck_argument ppf z;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_points ctx ppf = function | i -> printer_integer_matrix ctx ppf i

and printer_labels ctx ppf = function | i -> printer_integer_matrix ctx ppf i
;;

let rec printer_annotations ctx ppf = function
  | a_l -> Og_printer.printer_List printer_annotation ctx ppf a_l

and printer_annotation ctx ppf = function
  | Mdl_ast.Annotation a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation";
    printer_annotation_items Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_annotation_items ctx ppf = function
  | a_l -> Og_printer.printer_List printer_annotation_item ctx ppf a_l

and printer_annotation_item ctx ppf = function
  | Mdl_ast.Annotation_name a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_name";
    printer_annotation_name Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_sid b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_sid";
    printer_block_sid Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_position p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_position";
    printer_position Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_auto_size m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_auto_size";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_word_wrap m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_word_wrap";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_zorder z ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_zorder";
    printer_zorder Og_precedence.Ck_argument ppf z;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_graphical_default a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_graphical_default";
    printer_annotation_graphical_default Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Annotation_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Annotation_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_block_sid ctx ppf = function
  | Mdl_ast.Sid_compound b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Sid_compound";
    printer_block_sid_compound Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Sid_string s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Sid_string";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_block_sid_compound ctx ppf = function
  | Mdl_ast.Sid_natural n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Sid_natural";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Sid_ref (n, b) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Sid_ref";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_natural Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    printer_block_sid_compound Og_precedence.Ck_tuple_item ppf b;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Sid_child (n, b) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Sid_child";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_natural Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    printer_block_sid_compound Og_precedence.Ck_tuple_item ppf b;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_position ctx ppf = function
  | i_l -> Og_printer.printer_List Og_printer.printer_Int ctx ppf i_l
;;

let rec printer_system_connectors ctx ppf = function
  | s_l -> Og_printer.printer_List printer_system_connector ctx ppf s_l

and printer_system_connector ctx ppf = function
  | b -> printer_bindings ctx ppf b
;;

let rec printer_array_description ctx ppf = function
  | {
      Mdl_ast.array_type = a;
      Mdl_ast.array_dimension = d;
      Mdl_ast.array_prop_name = p;
      Mdl_ast.array_items = a0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>array_type =";
    printer_array_type Og_precedence.Ck_record_field ppf a;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>array_dimension =";
    printer_dimension Og_precedence.Ck_record_field ppf d;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>array_prop_name =";
    printer_prop_name Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>array_items =";
    printer_array_items Og_precedence.Ck_record_field ppf a0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_dimension ctx ppf = function
  | s -> printer_simple_value ctx ppf s

and printer_array_items ctx ppf = function
  | a_l -> Og_printer.printer_List printer_array_item ctx ppf a_l

and printer_array_item ctx ppf = function
  | Mdl_ast.Array_name a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_name";
    printer_array_name Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_sub_array a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_sub_array";
    printer_array_description Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_custom_property c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_custom_property";
    printer_custom_property Og_precedence.Ck_argument ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_matstruct m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_matstruct";
    printer_matstruct_items Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_object o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_object";
    printer_object_description Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_vendor_binding v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_vendor_binding";
    printer_vendor_binding Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_vendor_configset (v, v0) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_vendor_configset";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_vendor_ident Og_precedence.Ck_tuple_item ppf v;
    Format.fprintf ppf "%,,@ ";
    printer_vendor_items Og_precedence.Ck_tuple_item ppf v0;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Array_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Array_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_vendor_binding ctx ppf = function
  | {
      Mdl_ast.vendor_ident = v;
      Mdl_ast.vendor_configuration_longident = c;
      Mdl_ast.vendor_items = v0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>vendor_ident =";
    printer_vendor_ident Og_precedence.Ck_record_field ppf v;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>vendor_configuration_longident =";
    printer_configuration_longident Og_precedence.Ck_record_field ppf c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>vendor_items =";
    printer_vendor_items Og_precedence.Ck_record_field ppf v0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_block_vendor ctx ppf = function
  | v -> printer_vendor_binding ctx ppf v

and printer_vendor_items ctx ppf = function
  | v_l -> Og_printer.printer_List printer_vendor_item ctx ppf v_l

and printer_vendor_item ctx ppf = function
  | Mdl_ast.Vendor_binding o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Vendor_binding";
    printer_object_item Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Vendor_dollar_binding d ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Vendor_dollar_binding";
    printer_dollar_binding Og_precedence.Ck_argument ppf d;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Vendor_vendor_binding v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Vendor_vendor_binding";
    printer_vendor_binding Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_dollar_binding ctx ppf = function
  | Mdl_ast.Dollar_object_id o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Dollar_object_id";
    printer_object_id Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Dollar_class_name c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Dollar_class_name";
    printer_class_name Og_precedence.Ck_argument ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Dollar_backup_class b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Dollar_backup_class";
    printer_backup_class_name Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Dollar_prop_name p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Dollar_prop_name";
    printer_prop_name Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_vendor_ident ctx ppf = function
  | Mdl_ast.Simulink ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Simulink"
  | Mdl_ast.Scicos ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Scicos"
  | Mdl_ast.Rtw ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Rtw"
  | Mdl_ast.Ui i_i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Ui";
    Ident_print.printer_ident Og_precedence.Ck_argument ppf i_i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_custom_property ctx ppf = function
  | {
      Mdl_ast.custom_property_type = c;
      Mdl_ast.custom_property_enum_str_pairs = e;
      Mdl_ast.custom_property_ident = p;
      Mdl_ast.custom_property_value = s;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>custom_property_type =";
    printer_custom_property_type Og_precedence.Ck_record_field ppf c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>custom_property_enum_str_pairs =";
    printer_enum_str_pairs Og_precedence.Ck_record_field ppf e;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>custom_property_ident =";
    printer_prop_name Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>custom_property_value =";
    printer_simple_value Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_matstruct_items ctx ppf = function
  | m_l -> Og_printer.printer_List printer_matstruct_item ctx ppf m_l

and printer_matstruct_item ctx ppf = function
  | Mdl_ast.Matstruct_type m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Matstruct_type";
    printer_matstruct_type Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Matstruct_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Matstruct_name";
    printer_matstruct_name Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Matstruct_array a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Matstruct_array";
    printer_array_description Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Matstruct_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Matstruct_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_enum_str_pairs ctx ppf = function
  | e_l -> Og_printer.printer_List printer_enum_str_pair ctx ppf e_l

and printer_enum_str_pair ctx ppf = function
  | (e, m) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_enum Og_precedence.Ck_tuple_item ppf e;
    Format.fprintf ppf "%,,@ ";
    printer_mdl_text Og_precedence.Ck_tuple_item ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"

and printer_object_description ctx ppf = function
  | o_l -> Og_printer.printer_List printer_object_item ctx ppf o_l

and printer_object_item ctx ppf = function
  | Mdl_ast.Object_type o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_type";
    printer_object_type Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_name o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_name";
    printer_object_name Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_dollar_binding d ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_dollar_binding";
    printer_dollar_binding Og_precedence.Ck_argument ppf d;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_array a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_array";
    printer_array_description Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_custom_property c ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_custom_property";
    printer_custom_property Og_precedence.Ck_argument ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_object o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_object";
    printer_object_description Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Object_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Object_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_block_mask_item ctx ppf = function
  | Mdl_ast.Mask_callback_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_callback_string";
    printer_mdl_pipe_expressions Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_capabilities m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_capabilities";
    printer_mdl_expression Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_description m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_description";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_display m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_display";
    printer_mdl_code Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_display_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_display_string";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_enable_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_enable_string";
    printer_mdl_comma_bools Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_help m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_help";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_hide_contents m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_hide_contents";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_icon_frame m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_icon_frame";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_icon_opaque m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_icon_opaque";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_icon_rotate i_i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_icon_rotate";
    Ident_print.printer_ident Og_precedence.Ck_argument ppf i_i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_icon_units i_i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_icon_units";
    Ident_print.printer_ident Og_precedence.Ck_argument ppf i_i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_initialization m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_initialization";
    printer_mdl_shallow_code Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_port_rotate i_i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_port_rotate";
    Ident_print.printer_ident Og_precedence.Ck_argument ppf i_i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_prompt_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_prompt_string";
    printer_mdl_pipe_texts Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_run_init_for_icon_redraw m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_run_init_for_icon_redraw";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_self_modifiable m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_self_modifiable";
    printer_mdl_bool Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_style_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_style_string";
    printer_mdl_comma_texts Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_tab_name_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_tab_name_string";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_tooltip_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_tooltip_string";
    printer_mdl_comma_bools Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_tunable_value_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_tunable_value_string";
    printer_mdl_comma_bools Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_type m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_type";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_value_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_value_string";
    printer_mdl_pipe_texts Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_var_alias_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_var_alias_string";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_variables m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_variables";
    printer_mdl_semi_texts Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Mask_visibility_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Mask_visibility_string";
    printer_mdl_comma_bools Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_expression ctx ppf = function
  | Mdl_ast.Expression m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Expression";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_comma_bools ctx ppf = function
  | Mdl_ast.Comma_bools m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Comma_bools";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_pipe_expressions ctx ppf = function
  | Mdl_ast.Pipe_expressions m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Pipe_expressions";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_pipe_texts ctx ppf = function
  | Mdl_ast.Pipe_texts m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Pipe_texts";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_comma_texts ctx ppf = function
  | Mdl_ast.Comma_texts m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Comma_texts";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mdl_semi_texts ctx ppf = function
  | Mdl_ast.Semi_texts m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Semi_texts";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_blocks ctx ppf = function
  | b_l -> Og_printer.printer_List printer_block ctx ppf b_l

and printer_block ctx ppf = function
  | Mdl_ast.Block b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block";
    printer_block_items Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_vendor b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_vendor";
    printer_block_vendor Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_block_items ctx ppf = function
  | b_l -> Og_printer.printer_List printer_block_item ctx ppf b_l

and printer_block_item ctx ppf = function
  | Mdl_ast.Block_type b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_type";
    printer_block_type Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_name b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_name";
    printer_block_name Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_source_path b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_source_path";
    printer_block_source_path Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_source_type b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_source_type";
    printer_block_source_type Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_port b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_port";
    printer_block_port Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_ports_description b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_ports_description";
    printer_block_ports_description Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_position p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_position";
    printer_position Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_tag m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_tag";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_sid b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_sid";
    printer_block_sid Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_state s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_state";
    printer_simple_value Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_sf_block_type b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_sf_block_type";
    printer_block_sf_block_type Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_array a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_array";
    printer_array_description Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_object o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_object";
    printer_object_description Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_system s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_system";
    printer_system Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_graphical_default b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_graphical_default";
    printer_block_graphical_default Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_mask_item b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_mask_item";
    printer_block_mask_item Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_mask m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_mask";
    printer_mask_description Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_vendor_binding v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_vendor_binding";
    printer_vendor_binding Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_instance_data b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_instance_data";
    printer_bindings Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_mask_description ctx ppf = function
  | {
      Mdl_ast.mask_type = m_o;
      Mdl_ast.mask_display = m;
      Mdl_ast.mask_description = m_o0;
      Mdl_ast.mask_initialisation = m_o1;
      Mdl_ast.mask_parameters = m_l;
      Mdl_ast.mask_bindings = b;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>mask_type =";
    Og_printer.printer_Option printer_mdl_text
      Og_precedence.Ck_record_field
      ppf
      m_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_display =";
    printer_mask_display Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_description =";
    Og_printer.printer_Option printer_mdl_text
      Og_precedence.Ck_record_field
      ppf
      m_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_initialisation =";
    Og_printer.printer_Option printer_mdl_code
      Og_precedence.Ck_record_field
      ppf
      m_o1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameters =";
    Og_printer.printer_List printer_mask_parameter
      Og_precedence.Ck_record_field
      ppf
      m_l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_bindings =";
    printer_bindings Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_mask_parameter ctx ppf = function
  | {
      Mdl_ast.mask_parameter_ident = i_i;
      Mdl_ast.mask_parameter_style = m;
      Mdl_ast.mask_parameter_prompt = m0;
      Mdl_ast.mask_parameter_callback = m_o;
      Mdl_ast.mask_parameter_tunable = m_o0;
      Mdl_ast.mask_parameter_value = m1;
      Mdl_ast.mask_parameter_evaluation = m_o1;
      Mdl_ast.mask_parameter_enable = m_o2;
      Mdl_ast.mask_parameter_visible = m_o3;
      Mdl_ast.mask_parameter_internal = m_o4;
      Mdl_ast.mask_parameter_never_save = m_o5;
      Mdl_ast.mask_parameter_read_only = m_o6;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_ident =";
    Ident_print.printer_ident Og_precedence.Ck_record_field ppf i_i;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_style =";
    printer_mdl_text Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_prompt =";
    printer_mdl_text Og_precedence.Ck_record_field ppf m0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_callback =";
    Og_printer.printer_Option printer_mdl_code
      Og_precedence.Ck_record_field
      ppf
      m_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_tunable =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_value =";
    printer_mdl_text Og_precedence.Ck_record_field ppf m1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_evaluation =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_enable =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_visible =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_internal =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_never_save =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_read_only =";
    Og_printer.printer_Option printer_mdl_bool
      Og_precedence.Ck_record_field
      ppf
      m_o6;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_mask_display ctx ppf = function
  | { Mdl_ast.mask_display_source = m_o; Mdl_ast.mask_display_bindings = b; } ->
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%, {@ " else
     Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
     Format.fprintf ppf "%,Mdl_ast.@,";
     Format.fprintf ppf "%,@[<hv 2>mask_display_source =";
     Og_printer.printer_Option printer_mdl_code
       Og_precedence.Ck_record_field
       ppf
       m_o;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
     Format.fprintf ppf "%,@[<hv 2>mask_display_bindings =";
     printer_bindings Og_precedence.Ck_record_field ppf b;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%,@;<1 -2>}" else
     Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_block_port ctx ppf = function
  | {
      Mdl_ast.block_port_number = p;
      Mdl_ast.block_port_name = p_o;
      Mdl_ast.block_port_bindings = b;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>block_port_number =";
    printer_port_number Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>block_port_name =";
    Og_printer.printer_Option printer_port_name
      Og_precedence.Ck_record_field
      ppf
      p_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>block_port_bindings =";
    printer_block_port_bindings Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_block_port_bindings ctx ppf = function
  | b_l -> Og_printer.printer_List printer_block_port_binding ctx ppf b_l

and printer_block_port_binding ctx ppf = function
  | Mdl_ast.Block_port_binding_array a ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_port_binding_array";
    printer_array_description Og_precedence.Ck_argument ppf a;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_port_binding_vendor v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_port_binding_vendor";
    printer_vendor_binding Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Block_port_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Block_port_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_block_ports_description ctx ppf = function
  | n -> printer_naturals ctx ppf n

and printer_system ctx ppf = function
  | {
      Mdl_ast.system_name = s;
      Mdl_ast.system_position = p;
      Mdl_ast.system_bindings = b;
      Mdl_ast.system_blocks = b0;
      Mdl_ast.system_lines = l;
      Mdl_ast.system_annotations = a;
      Mdl_ast.system_connectors = s0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>system_name =";
    printer_system_name Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_position =";
    printer_position Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_bindings =";
    printer_bindings Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_blocks =";
    printer_blocks Og_precedence.Ck_record_field ppf b0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_lines =";
    printer_lines Og_precedence.Ck_record_field ppf l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_annotations =";
    printer_annotations Og_precedence.Ck_record_field ppf a;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>system_connectors =";
    printer_system_connectors Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_model_section ctx ppf = function
  | {
      Mdl_ast.model_ident = m;
      Mdl_ast.model_version = m0;
      Mdl_ast.model_configuration_parameters = m1;
      Mdl_ast.model_configurations = m2;
      Mdl_ast.model_set_configuration_option = m3;
      Mdl_ast.model_workspace = m4;
      Mdl_ast.block_graphical_defaults_section = b;
      Mdl_ast.block_parameter_defaults_section = b_l;
      Mdl_ast.annotation_graphical_defaults_section = a;
      Mdl_ast.line_graphical_defaults_section = l;
      Mdl_ast.mask_graphical_defaults_section = m5;
      Mdl_ast.mask_parameter_defaults_section = m6;
      Mdl_ast.model_system = s;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>model_ident =";
    printer_model_ident Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_version =";
    printer_model_version Og_precedence.Ck_record_field ppf m0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_configuration_parameters =";
    printer_model_configuration_parameters
      Og_precedence.Ck_record_field
      ppf
      m1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_configurations =";
    printer_model_configurations Og_precedence.Ck_record_field ppf m2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_set_configuration_option =";
    printer_model_set_configuration_option
      Og_precedence.Ck_record_field
      ppf
      m3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_workspace =";
    printer_model_workspace Og_precedence.Ck_record_field ppf m4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>block_graphical_defaults_section =";
    printer_block_graphical_defaults Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>block_parameter_defaults_section =";
    Og_printer.printer_List printer_block_parameter_defaults
      Og_precedence.Ck_record_field
      ppf
      b_l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>annotation_graphical_defaults_section =";
    printer_annotation_graphical_defaults Og_precedence.Ck_record_field ppf a;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>line_graphical_defaults_section =";
    printer_line_graphical_defaults Og_precedence.Ck_record_field ppf l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_graphical_defaults_section =";
    printer_mask_graphical_defaults Og_precedence.Ck_record_field ppf m5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mask_parameter_defaults_section =";
    printer_mask_parameter_defaults Og_precedence.Ck_record_field ppf m6;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>model_system =";
    printer_system Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_model_configuration_parameters ctx ppf = function
  | m_l ->
    Og_printer.printer_List printer_model_configuration_parameter ctx ppf m_l

and printer_model_configuration_parameter ctx ppf = function
  | Mdl_ast.Configuration_graphical_interface g ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_graphical_interface";
    printer_graphical_interface_items Og_precedence.Ck_argument ppf g;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Configuration_object o ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_object";
    printer_object_description Og_precedence.Ck_argument ppf o;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Configuration_vendor_binding v ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_vendor_binding";
    printer_vendor_binding Og_precedence.Ck_argument ppf v;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Configuration_binding s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Configuration_binding";
    printer_simple_binding Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_graphical_interface_items ctx ppf = function
  | g_l ->
    Og_printer.printer_List printer_graphical_interface_item ctx ppf g_l

and printer_graphical_interface_item ctx ppf = function
  | Mdl_ast.Graphical_enableport i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_enableport";
    printer_io_port_binding_items Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_inport i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_inport";
    printer_io_port_binding_items Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_outport i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_outport";
    printer_io_port_binding_items Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_trigport i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_trigport";
    printer_io_port_binding_items Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_inports_number n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_inports_number";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_outports_number n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_outports_number";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_trigports_number n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_trigports_number";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Graphical_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Graphical_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_io_port_binding_items ctx ppf = function
  | i_l -> Og_printer.printer_List printer_io_port_binding_item ctx ppf i_l

and printer_io_port_binding_item ctx ppf = function
  | Mdl_ast.Io_port_name p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Io_port_name";
    printer_port_name Og_precedence.Ck_argument ppf p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Io_port_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Io_port_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_model_configurations ctx ppf = function
  | m_l -> Og_printer.printer_List printer_model_configuration ctx ppf m_l

and printer_model_configuration ctx ppf = function
  | a -> printer_array_description ctx ppf a

and printer_model_set_configuration_option ctx ppf = function
  | c_o -> Og_printer.printer_Option printer_configuration_set ctx ppf c_o

and printer_configuration_set ctx ppf = function
  | {
      Mdl_ast.configuration_vendor = v;
      Mdl_ast.configuration_longident = c;
      Mdl_ast.configuration_prop_name = p;
      Mdl_ast.configuration_object_id = o;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>configuration_vendor =";
    printer_vendor_ident Og_precedence.Ck_record_field ppf v;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>configuration_longident =";
    printer_configuration_longident Og_precedence.Ck_record_field ppf c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>configuration_prop_name =";
    printer_prop_name Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>configuration_object_id =";
    printer_object_id Og_precedence.Ck_record_field ppf o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_model_workspace ctx ppf = function
  | m_o ->
    Og_printer.printer_Option printer_model_workspace_datas ctx ppf m_o

and printer_model_workspace_datas ctx ppf = function
  | m_l -> Og_printer.printer_List printer_model_workspace_data ctx ppf m_l

and printer_model_workspace_data ctx ppf = function
  | Mdl_ast.Workspace_data_source_type w ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Workspace_data_source_type";
    printer_workspace_data_source_type Og_precedence.Ck_argument ppf w;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Workspace_data_mat_file_name p_f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Workspace_data_mat_file_name";
    Path_print.printer_file_name Og_precedence.Ck_argument ppf p_f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Workspace_data_source_file_name p_f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Workspace_data_source_file_name";
    Path_print.printer_file_name Og_precedence.Ck_argument ppf p_f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Workspace_data_mcode s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Workspace_data_mcode";
    printer_simple_value Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Workspace_data_mdl_file t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Workspace_data_mdl_file";
    printer_tag_ident Og_precedence.Ck_argument ppf t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_workspace_data_source_type _ctx ppf = function
  | Mdl_ast.M_Code ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.M_Code"
  | Mdl_ast.MATLAB_Code ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.MATLAB_Code"
  | Mdl_ast.MATLAB_File ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.MATLAB_File"
  | Mdl_ast.MAT_File ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.MAT_File"
;;

let rec printer_matdata_section ctx ppf = function
  | Mdl_ast.Matdata_section m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Matdata_section";
    printer_matdata_section_item Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"

and printer_matdata_section_item ctx ppf = function
  | { Mdl_ast.matdata_num_records = n; Mdl_ast.matdata_records = m; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>matdata_num_records =";
    printer_natural Og_precedence.Ck_record_field ppf n;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>matdata_records =";
    printer_matdata_records Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"

and printer_matdata_records ctx ppf = function
  | m_l -> Og_printer.printer_List printer_matdata_record ctx ppf m_l

and printer_matdata_record ctx ppf = function
  | { Mdl_ast.matdata_record_tag = t; Mdl_ast.matdata_record_data = m; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>matdata_record_tag =";
    printer_tag_ident Og_precedence.Ck_record_field ppf t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>matdata_record_data =";
    printer_mdl_text Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_stateflow_id ctx ppf = function
  | a_n -> Ast_types_print.printer_natural ctx ppf a_n
;;

let printer_stateflow_ref ctx ppf = function
  | s -> printer_stateflow_id ctx ppf s
;;

let rec printer_stateflow_numbers ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_number ctx ppf s_l

and printer_stateflow_number ctx ppf = function
  | Mdl_ast.Stateflow_int s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_int";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_float s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_float";
    Og_printer.printer_String Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_stateflow_src_dst_items ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_src_dst_item ctx ppf s_l

and printer_stateflow_src_dst_item ctx ppf = function
  | Mdl_ast.Stateflow_src_dst_ref s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_src_dst_ref";
    printer_stateflow_ref Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_intersection s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_intersection";
    printer_stateflow_numbers Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_stateflow_eml_items ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_eml_item ctx ppf s_l

and printer_stateflow_eml_item ctx ppf = function
  | Mdl_ast.Stateflow_eml_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_eml_name";
    printer_mdl_name Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_eml_script m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_eml_script";
    printer_mdl_code Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_eml_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_eml_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_stateflow_props_array_items ctx ppf = function
  | s_l ->
    Og_printer.printer_List printer_stateflow_props_array_item ctx ppf s_l

and printer_stateflow_props_array_item ctx ppf = function
  | Mdl_ast.Stateflow_props_array_item_size m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props_array_item_size";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_props_array_item_first_index m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf
      "%,@[<hv 2>Mdl_ast.Stateflow_props_array_item_first_index";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_props_array_item_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props_array_item_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_stateflow_props_type_items ctx ppf = function
  | s_l ->
    Og_printer.printer_List printer_stateflow_props_type_item ctx ppf s_l

and printer_stateflow_props_type_item ctx ppf = function
  | b -> printer_binding ctx ppf b
;;

let rec printer_stateflow_props_items ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_props_item ctx ppf s_l

and printer_stateflow_props_item ctx ppf = function
  | Mdl_ast.Stateflow_props_array s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props_array";
    printer_stateflow_props_array_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_props_type s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props_type";
    printer_stateflow_props_type_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_props_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_stateflow_items ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_item ctx ppf s_l

and printer_stateflow_item ctx ppf = function
  | Mdl_ast.Stateflow_binding b ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_binding";
    printer_binding Og_precedence.Ck_argument ppf b;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_eml s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_eml";
    printer_stateflow_eml_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_execution_order n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_execution_order";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_label_string m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_label_string";
    printer_mdl_text Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_chart_file_number n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_chart_file_number";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_name m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_name";
    printer_mdl_name Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_scope i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_scope";
    printer_ident Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_props s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_props";
    printer_stateflow_props_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_ssid_number n ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_ssid_number";
    printer_natural Og_precedence.Ck_argument ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_trigger i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_trigger";
    printer_ident Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_type i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_type";
    printer_ident Og_precedence.Ck_argument ppf i; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_src s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_src";
    printer_stateflow_src_dst_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_dst s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_dst";
    printer_stateflow_src_dst_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_plant_modeling_info ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Mdl_ast.Stateflow_plant_modeling_info"
;;

let rec printer_stateflow_junctions ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_junction ctx ppf s_l

and printer_stateflow_junction ctx ppf = function
  | {
      Mdl_ast.stateflow_junction_id = s;
      Mdl_ast.stateflow_junction_items = s0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_junction_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_junction_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_transitions ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_transition ctx ppf s_l

and printer_stateflow_transition ctx ppf = function
  | {
      Mdl_ast.stateflow_transition_id = s;
      Mdl_ast.stateflow_transition_items = s0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_transition_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_transition_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_events ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_event ctx ppf s_l

and printer_stateflow_event ctx ppf = function
  | { Mdl_ast.stateflow_event_id = s; Mdl_ast.stateflow_event_items = s0; } ->
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%, {@ " else
     Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
     Format.fprintf ppf "%,Mdl_ast.@,";
     Format.fprintf ppf "%,@[<hv 2>stateflow_event_id =";
     printer_stateflow_id Og_precedence.Ck_record_field ppf s;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
     Format.fprintf ppf "%,@[<hv 2>stateflow_event_items =";
     printer_stateflow_items Og_precedence.Ck_record_field ppf s0;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%,@;<1 -2>}" else
     Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_datas ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_data ctx ppf s_l

and printer_stateflow_data ctx ppf = function
  | { Mdl_ast.stateflow_data_id = s; Mdl_ast.stateflow_data_items = s0; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_data_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_data_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_states ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_state ctx ppf s_l

and printer_stateflow_state ctx ppf = function
  | {
      Mdl_ast.stateflow_state_id = s;
      Mdl_ast.stateflow_state_datas = s0;
      Mdl_ast.stateflow_state_events = s1;
      Mdl_ast.stateflow_state_junctions = s2;
      Mdl_ast.stateflow_state_states = s3;
      Mdl_ast.stateflow_state_transitions = s4;
      Mdl_ast.stateflow_state_items = s5;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_datas =";
    printer_stateflow_datas Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_events =";
    printer_stateflow_events Og_precedence.Ck_record_field ppf s1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_junctions =";
    printer_stateflow_junctions Og_precedence.Ck_record_field ppf s2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_states =";
    printer_stateflow_states Og_precedence.Ck_record_field ppf s3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_transitions =";
    printer_stateflow_transitions Og_precedence.Ck_record_field ppf s4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_state_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_targets ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_target ctx ppf s_l

and printer_stateflow_target ctx ppf = function
  | { Mdl_ast.stateflow_target_id = s; Mdl_ast.stateflow_target_items = s0; } ->
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%, {@ " else
     Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
     Format.fprintf ppf "%,Mdl_ast.@,";
     Format.fprintf ppf "%,@[<hv 2>stateflow_target_id =";
     printer_stateflow_id Og_precedence.Ck_record_field ppf s;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
     Format.fprintf ppf "%,@[<hv 2>stateflow_target_items =";
     printer_stateflow_items Og_precedence.Ck_record_field ppf s0;
     Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
     if ctx = Og_precedence.Ck_argument ||
        ctx = Og_precedence.Ck_record_field
       then Format.fprintf ppf "%,@;<1 -2>}" else
     Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_charts ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_chart ctx ppf s_l

and printer_stateflow_chart ctx ppf = function
  | {
      Mdl_ast.stateflow_chart_id = s;
      Mdl_ast.stateflow_chart_datas = s0;
      Mdl_ast.stateflow_chart_events = s1;
      Mdl_ast.stateflow_chart_junctions = s2;
      Mdl_ast.stateflow_chart_states = s3;
      Mdl_ast.stateflow_chart_transitions = s4;
      Mdl_ast.stateflow_chart_items = s5;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_datas =";
    printer_stateflow_datas Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_events =";
    printer_stateflow_events Og_precedence.Ck_record_field ppf s1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_junctions =";
    printer_stateflow_junctions Og_precedence.Ck_record_field ppf s2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_states =";
    printer_stateflow_states Og_precedence.Ck_record_field ppf s3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_transitions =";
    printer_stateflow_transitions Og_precedence.Ck_record_field ppf s4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_chart_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s5;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_stateflow_machine ctx ppf = function
  | {
      Mdl_ast.stateflow_machine_id = s;
      Mdl_ast.stateflow_machine_charts = s0;
      Mdl_ast.stateflow_machine_datas = s1;
      Mdl_ast.stateflow_machine_events = s2;
      Mdl_ast.stateflow_machine_targets = s3;
      Mdl_ast.stateflow_machine_items = s4;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_charts =";
    printer_stateflow_charts Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_datas =";
    printer_stateflow_datas Og_precedence.Ck_record_field ppf s1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_events =";
    printer_stateflow_events Og_precedence.Ck_record_field ppf s2;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_targets =";
    printer_stateflow_targets Og_precedence.Ck_record_field ppf s3;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_machine_items =";
    printer_stateflow_items Og_precedence.Ck_record_field ppf s4;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_stateflow_instance ctx ppf = function
  | {
      Mdl_ast.stateflow_instance_id = s;
      Mdl_ast.stateflow_instance_name = m;
      Mdl_ast.stateflow_instance_machine_ref = s0;
      Mdl_ast.stateflow_instance_chart_ref = s1;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>stateflow_instance_id =";
    printer_stateflow_id Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_instance_name =";
    printer_mdl_name Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_instance_machine_ref =";
    printer_stateflow_ref Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>stateflow_instance_chart_ref =";
    printer_stateflow_ref Og_precedence.Ck_record_field ppf s1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_stateflow_section_items ctx ppf = function
  | s_l -> Og_printer.printer_List printer_stateflow_section_item ctx ppf s_l

and printer_stateflow_section_item ctx ppf = function
  | Mdl_ast.Stateflow_section_machine s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_section_machine";
    printer_stateflow_machine Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Stateflow_section_instance s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_section_instance";
    printer_stateflow_instance Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_stateflow_section ctx ppf = function
  | Mdl_ast.Stateflow_section s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Stateflow_section";
    printer_stateflow_section_items Og_precedence.Ck_argument ppf s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let rec printer_interface_block_parameters ctx ppf = function
  | i_l ->
    Og_printer.printer_List printer_interface_block_parameter ctx ppf i_l

and printer_interface_block_parameter ctx ppf = function
  | i_i ->
    printer_ident_binding printer_interface_block_parameter_type ctx ppf i_i
;;

let rec printer_interface_blocks ctx ppf = function
  | i_l -> Og_printer.printer_List printer_interface_block ctx ppf i_l

and printer_interface_block ctx ppf = function
  | {
      Mdl_ast.interface_block_type = b;
      Mdl_ast.interface_block_source_type = b_o;
      Mdl_ast.interface_block_source_path = b_o0;
      Mdl_ast.interface_block_ports_description = b0;
      Mdl_ast.interface_block_parameters = i;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>interface_block_type =";
    printer_block_type Og_precedence.Ck_record_field ppf b;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>interface_block_source_type =";
    Og_printer.printer_Option printer_block_source_type
      Og_precedence.Ck_record_field
      ppf
      b_o;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>interface_block_source_path =";
    Og_printer.printer_Option printer_block_source_path
      Og_precedence.Ck_record_field
      ppf
      b_o0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>interface_block_ports_description =";
    printer_block_ports_description Og_precedence.Ck_record_field ppf b0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>interface_block_parameters =";
    printer_interface_block_parameters Og_precedence.Ck_record_field ppf i;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_interface ctx ppf = function
  | { Mdl_ast.interface_ident = i_i; Mdl_ast.interface_blocks = i; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Mdl_ast.@,";
    Format.fprintf ppf "%,@[<hv 2>interface_ident =";
    Ident_print.printer_ident Og_precedence.Ck_record_field ppf i_i;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>interface_blocks =";
    printer_interface_blocks Og_precedence.Ck_record_field ppf i;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let rec printer_poly_implementation_section_items pp_stateflow ctx ppf =
  function
  | s_p_l ->
    Og_printer.printer_List
      (printer_poly_implementation_section_item pp_stateflow)
      ctx
      ppf
      s_p_l

and printer_poly_implementation_section_item pp_stateflow ctx ppf = function
  | Mdl_ast.Implementation_library m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Implementation_library";
    printer_model_section Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Implementation_model m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Implementation_model";
    printer_model_section Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Implementation_matdata m ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Implementation_matdata";
    printer_matdata_section Og_precedence.Ck_argument ppf m;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Implementation_stateflow s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Implementation_stateflow";
    pp_stateflow Og_precedence.Ck_argument ppf s; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_poly_implementation pp_stateflow ctx ppf = function
  | s_p -> printer_poly_implementation_section_items pp_stateflow ctx ppf s_p
;;

let printer_poly_simulink_file_contents pp_stateflow ctx ppf = function
  | Mdl_ast.Implementation s_p ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Implementation";
    printer_poly_implementation pp_stateflow
      Og_precedence.Ck_argument
      ppf
      s_p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Mdl_ast.Interface i ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Mdl_ast.Interface";
    printer_interface Og_precedence.Ck_argument ppf i;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_poly_simulink_file pp_stateflow ctx ppf = function
  | (p_e, s_p) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Path_print.printer_explicit_file_name Og_precedence.Ck_tuple_item ppf p_e;
    Format.fprintf ppf "%,,@ ";
    printer_poly_simulink_file_contents pp_stateflow
      Og_precedence.Ck_tuple_item
      ppf
      s_p;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_simulink_file_contents ctx ppf = function
  | s_p ->
    printer_poly_simulink_file_contents printer_stateflow_section ctx ppf s_p
;;

let printer_simulink_file ctx ppf = function
  | s_p -> printer_poly_simulink_file printer_stateflow_section ctx ppf s_p
;;

(* Defining top printers. *)

let print_model_ident ppf = printer_model_ident Og_precedence.Ck_toplevel ppf

and print_configuration_ident ppf =
  printer_configuration_ident Og_precedence.Ck_toplevel ppf

and print_tag_ident ppf = printer_tag_ident Og_precedence.Ck_toplevel ppf
;;

let print_prop_name ppf = printer_prop_name Og_precedence.Ck_toplevel ppf

and print_class_name ppf = printer_class_name Og_precedence.Ck_toplevel ppf
;;

let print_system_name ppf = printer_system_name Og_precedence.Ck_toplevel ppf

and print_block_name ppf = printer_block_name Og_precedence.Ck_toplevel ppf

and print_block_source_path ppf =
  printer_block_source_path Og_precedence.Ck_toplevel ppf

and print_block_source_type ppf =
  printer_block_source_type Og_precedence.Ck_toplevel ppf

and print_line_name ppf = printer_line_name Og_precedence.Ck_toplevel ppf

and print_annotation_name ppf =
  printer_annotation_name Og_precedence.Ck_toplevel ppf

and print_port_name ppf = printer_port_name Og_precedence.Ck_toplevel ppf

and print_port_number ppf = printer_port_number Og_precedence.Ck_toplevel ppf

and print_object_name ppf = printer_object_name Og_precedence.Ck_toplevel ppf

and print_backup_class_name ppf =
  printer_backup_class_name Og_precedence.Ck_toplevel ppf

and print_array_name ppf = printer_array_name Og_precedence.Ck_toplevel ppf

and print_ui_name ppf = printer_ui_name Og_precedence.Ck_toplevel ppf

and print_matstruct_name ppf =
  printer_matstruct_name Og_precedence.Ck_toplevel ppf
;;

let print_ast_node pp_a ppf =
  printer_ast_node pp_a Og_precedence.Ck_toplevel ppf
;;

let print_model_version ppf =
  printer_model_version Og_precedence.Ck_toplevel ppf
;;

let print_enum ppf = printer_enum Og_precedence.Ck_toplevel ppf
;;

let print_configuration_longident ppf =
  printer_configuration_longident Og_precedence.Ck_toplevel ppf
;;

let print_line_type ppf = printer_line_type Og_precedence.Ck_toplevel ppf

and print_connection_type ppf =
  printer_connection_type Og_precedence.Ck_toplevel ppf

and print_array_type ppf = printer_array_type Og_precedence.Ck_toplevel ppf

and print_object_type ppf = printer_object_type Og_precedence.Ck_toplevel ppf

and print_matstruct_type ppf =
  printer_matstruct_type Og_precedence.Ck_toplevel ppf

and print_interface_block_parameter_type ppf =
  printer_interface_block_parameter_type Og_precedence.Ck_toplevel ppf
;;

let print_block_sf_block_type ppf =
  printer_block_sf_block_type Og_precedence.Ck_toplevel ppf
;;

let print_custom_property_type ppf =
  printer_custom_property_type Og_precedence.Ck_toplevel ppf
;;

let print_naturals ppf = printer_naturals Og_precedence.Ck_toplevel ppf
;;

let print_integer_matrix ppf =
  printer_integer_matrix Og_precedence.Ck_toplevel ppf
;;

let print_object_id ppf = printer_object_id Og_precedence.Ck_toplevel ppf
;;

let print_zorder ppf = printer_zorder Og_precedence.Ck_toplevel ppf
;;

let print_simple_values ppf =
  printer_simple_values Og_precedence.Ck_toplevel ppf

and print_simple_value ppf =
  printer_simple_value Og_precedence.Ck_toplevel ppf
;;

let print_simple_binding ppf =
  printer_simple_binding Og_precedence.Ck_toplevel ppf
;;

let print_model_values ppf =
  printer_model_values Og_precedence.Ck_toplevel ppf

and print_model_value ppf = printer_model_value Og_precedence.Ck_toplevel ppf

and print_bindings ppf = printer_bindings Og_precedence.Ck_toplevel ppf

and print_binding ppf = printer_binding Og_precedence.Ck_toplevel ppf
;;

let print_poly_annotation_graphical_default pp_font_and_color ppf =
  printer_poly_annotation_graphical_default pp_font_and_color
    Og_precedence.Ck_toplevel
    ppf

and print_horizontal_alignment ppf =
  printer_horizontal_alignment Og_precedence.Ck_toplevel ppf

and print_vertical_alignment ppf =
  printer_vertical_alignment Og_precedence.Ck_toplevel ppf
;;

let print_font_items ppf = printer_font_items Og_precedence.Ck_toplevel ppf

and print_font_item ppf = printer_font_item Og_precedence.Ck_toplevel ppf
;;

let print_font_and_color ppf =
  printer_font_and_color Og_precedence.Ck_toplevel ppf

and print_color ppf = printer_color Og_precedence.Ck_toplevel ppf
;;

let print_block_graphical_defaults ppf =
  printer_block_graphical_defaults Og_precedence.Ck_toplevel ppf

and print_block_graphical_default ppf =
  printer_block_graphical_default Og_precedence.Ck_toplevel ppf
;;

let print_block_parameter_defaults_list ppf =
  printer_block_parameter_defaults_list Og_precedence.Ck_toplevel ppf

and print_block_parameter_defaults ppf =
  printer_block_parameter_defaults Og_precedence.Ck_toplevel ppf
;;

let print_line_graphical_defaults ppf =
  printer_line_graphical_defaults Og_precedence.Ck_toplevel ppf

and print_line_graphical_default ppf =
  printer_line_graphical_default Og_precedence.Ck_toplevel ppf
;;

let print_mask_graphical_defaults ppf =
  printer_mask_graphical_defaults Og_precedence.Ck_toplevel ppf

and print_mask_graphical_default ppf =
  printer_mask_graphical_default Og_precedence.Ck_toplevel ppf

and print_mask_parameter_defaults ppf =
  printer_mask_parameter_defaults Og_precedence.Ck_toplevel ppf

and print_mask_parameter_default ppf =
  printer_mask_parameter_default Og_precedence.Ck_toplevel ppf
;;

let print_annotation_graphical_defaults ppf =
  printer_annotation_graphical_defaults Og_precedence.Ck_toplevel ppf

and print_annotation_graphical_default ppf =
  printer_annotation_graphical_default Og_precedence.Ck_toplevel ppf
;;

let print_lines ppf = printer_lines Og_precedence.Ck_toplevel ppf

and print_line ppf = printer_line Og_precedence.Ck_toplevel ppf

and print_line_items ppf = printer_line_items Og_precedence.Ck_toplevel ppf

and print_line_item ppf = printer_line_item Og_precedence.Ck_toplevel ppf

and print_line_source ppf = printer_line_source Og_precedence.Ck_toplevel ppf

and print_line_connector ppf =
  printer_line_connector Og_precedence.Ck_toplevel ppf

and print_port_value ppf = printer_port_value Og_precedence.Ck_toplevel ppf

and print_line_target ppf = printer_line_target Og_precedence.Ck_toplevel ppf

and print_line_target_branch_items ppf =
  printer_line_target_branch_items Og_precedence.Ck_toplevel ppf

and print_line_target_branch_item ppf =
  printer_line_target_branch_item Og_precedence.Ck_toplevel ppf

and print_points ppf = printer_points Og_precedence.Ck_toplevel ppf

and print_labels ppf = printer_labels Og_precedence.Ck_toplevel ppf
;;

let print_annotations ppf = printer_annotations Og_precedence.Ck_toplevel ppf

and print_annotation ppf = printer_annotation Og_precedence.Ck_toplevel ppf

and print_annotation_items ppf =
  printer_annotation_items Og_precedence.Ck_toplevel ppf

and print_annotation_item ppf =
  printer_annotation_item Og_precedence.Ck_toplevel ppf

and print_block_sid ppf = printer_block_sid Og_precedence.Ck_toplevel ppf

and print_block_sid_compound ppf =
  printer_block_sid_compound Og_precedence.Ck_toplevel ppf

and print_position ppf = printer_position Og_precedence.Ck_toplevel ppf
;;

let print_system_connectors ppf =
  printer_system_connectors Og_precedence.Ck_toplevel ppf

and print_system_connector ppf =
  printer_system_connector Og_precedence.Ck_toplevel ppf
;;

let print_array_description ppf =
  printer_array_description Og_precedence.Ck_toplevel ppf

and print_dimension ppf = printer_dimension Og_precedence.Ck_toplevel ppf

and print_array_items ppf = printer_array_items Og_precedence.Ck_toplevel ppf

and print_array_item ppf = printer_array_item Og_precedence.Ck_toplevel ppf

and print_vendor_binding ppf =
  printer_vendor_binding Og_precedence.Ck_toplevel ppf

and print_block_vendor ppf =
  printer_block_vendor Og_precedence.Ck_toplevel ppf

and print_vendor_items ppf =
  printer_vendor_items Og_precedence.Ck_toplevel ppf

and print_vendor_item ppf = printer_vendor_item Og_precedence.Ck_toplevel ppf

and print_dollar_binding ppf =
  printer_dollar_binding Og_precedence.Ck_toplevel ppf

and print_vendor_ident ppf =
  printer_vendor_ident Og_precedence.Ck_toplevel ppf

and print_custom_property ppf =
  printer_custom_property Og_precedence.Ck_toplevel ppf

and print_matstruct_items ppf =
  printer_matstruct_items Og_precedence.Ck_toplevel ppf

and print_matstruct_item ppf =
  printer_matstruct_item Og_precedence.Ck_toplevel ppf

and print_enum_str_pairs ppf =
  printer_enum_str_pairs Og_precedence.Ck_toplevel ppf

and print_enum_str_pair ppf =
  printer_enum_str_pair Og_precedence.Ck_toplevel ppf

and print_object_description ppf =
  printer_object_description Og_precedence.Ck_toplevel ppf

and print_object_item ppf = printer_object_item Og_precedence.Ck_toplevel ppf
;;

let print_block_mask_item ppf =
  printer_block_mask_item Og_precedence.Ck_toplevel ppf

and print_mdl_expression ppf =
  printer_mdl_expression Og_precedence.Ck_toplevel ppf

and print_mdl_comma_bools ppf =
  printer_mdl_comma_bools Og_precedence.Ck_toplevel ppf

and print_mdl_pipe_expressions ppf =
  printer_mdl_pipe_expressions Og_precedence.Ck_toplevel ppf

and print_mdl_pipe_texts ppf =
  printer_mdl_pipe_texts Og_precedence.Ck_toplevel ppf

and print_mdl_comma_texts ppf =
  printer_mdl_comma_texts Og_precedence.Ck_toplevel ppf

and print_mdl_semi_texts ppf =
  printer_mdl_semi_texts Og_precedence.Ck_toplevel ppf
;;

let print_blocks ppf = printer_blocks Og_precedence.Ck_toplevel ppf

and print_block ppf = printer_block Og_precedence.Ck_toplevel ppf

and print_block_items ppf = printer_block_items Og_precedence.Ck_toplevel ppf

and print_block_item ppf = printer_block_item Og_precedence.Ck_toplevel ppf

and print_mask_description ppf =
  printer_mask_description Og_precedence.Ck_toplevel ppf

and print_mask_parameter ppf =
  printer_mask_parameter Og_precedence.Ck_toplevel ppf

and print_mask_display ppf =
  printer_mask_display Og_precedence.Ck_toplevel ppf

and print_block_port ppf = printer_block_port Og_precedence.Ck_toplevel ppf

and print_block_port_bindings ppf =
  printer_block_port_bindings Og_precedence.Ck_toplevel ppf

and print_block_port_binding ppf =
  printer_block_port_binding Og_precedence.Ck_toplevel ppf

and print_block_ports_description ppf =
  printer_block_ports_description Og_precedence.Ck_toplevel ppf

and print_system ppf = printer_system Og_precedence.Ck_toplevel ppf
;;

let print_model_section ppf =
  printer_model_section Og_precedence.Ck_toplevel ppf

and print_model_configuration_parameters ppf =
  printer_model_configuration_parameters Og_precedence.Ck_toplevel ppf

and print_model_configuration_parameter ppf =
  printer_model_configuration_parameter Og_precedence.Ck_toplevel ppf

and print_graphical_interface_items ppf =
  printer_graphical_interface_items Og_precedence.Ck_toplevel ppf

and print_graphical_interface_item ppf =
  printer_graphical_interface_item Og_precedence.Ck_toplevel ppf

and print_io_port_binding_items ppf =
  printer_io_port_binding_items Og_precedence.Ck_toplevel ppf

and print_io_port_binding_item ppf =
  printer_io_port_binding_item Og_precedence.Ck_toplevel ppf

and print_model_configurations ppf =
  printer_model_configurations Og_precedence.Ck_toplevel ppf

and print_model_configuration ppf =
  printer_model_configuration Og_precedence.Ck_toplevel ppf

and print_model_set_configuration_option ppf =
  printer_model_set_configuration_option Og_precedence.Ck_toplevel ppf

and print_configuration_set ppf =
  printer_configuration_set Og_precedence.Ck_toplevel ppf

and print_model_workspace ppf =
  printer_model_workspace Og_precedence.Ck_toplevel ppf

and print_model_workspace_datas ppf =
  printer_model_workspace_datas Og_precedence.Ck_toplevel ppf

and print_model_workspace_data ppf =
  printer_model_workspace_data Og_precedence.Ck_toplevel ppf

and print_workspace_data_source_type ppf =
  printer_workspace_data_source_type Og_precedence.Ck_toplevel ppf
;;

let print_matdata_section ppf =
  printer_matdata_section Og_precedence.Ck_toplevel ppf

and print_matdata_section_item ppf =
  printer_matdata_section_item Og_precedence.Ck_toplevel ppf

and print_matdata_records ppf =
  printer_matdata_records Og_precedence.Ck_toplevel ppf

and print_matdata_record ppf =
  printer_matdata_record Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_id ppf =
  printer_stateflow_id Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_ref ppf =
  printer_stateflow_ref Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_numbers ppf =
  printer_stateflow_numbers Og_precedence.Ck_toplevel ppf

and print_stateflow_number ppf =
  printer_stateflow_number Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_src_dst_items ppf =
  printer_stateflow_src_dst_items Og_precedence.Ck_toplevel ppf

and print_stateflow_src_dst_item ppf =
  printer_stateflow_src_dst_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_eml_items ppf =
  printer_stateflow_eml_items Og_precedence.Ck_toplevel ppf

and print_stateflow_eml_item ppf =
  printer_stateflow_eml_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_props_array_items ppf =
  printer_stateflow_props_array_items Og_precedence.Ck_toplevel ppf

and print_stateflow_props_array_item ppf =
  printer_stateflow_props_array_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_props_type_items ppf =
  printer_stateflow_props_type_items Og_precedence.Ck_toplevel ppf

and print_stateflow_props_type_item ppf =
  printer_stateflow_props_type_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_props_items ppf =
  printer_stateflow_props_items Og_precedence.Ck_toplevel ppf

and print_stateflow_props_item ppf =
  printer_stateflow_props_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_items ppf =
  printer_stateflow_items Og_precedence.Ck_toplevel ppf

and print_stateflow_item ppf =
  printer_stateflow_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_junctions ppf =
  printer_stateflow_junctions Og_precedence.Ck_toplevel ppf

and print_stateflow_junction ppf =
  printer_stateflow_junction Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_transitions ppf =
  printer_stateflow_transitions Og_precedence.Ck_toplevel ppf

and print_stateflow_transition ppf =
  printer_stateflow_transition Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_events ppf =
  printer_stateflow_events Og_precedence.Ck_toplevel ppf

and print_stateflow_event ppf =
  printer_stateflow_event Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_datas ppf =
  printer_stateflow_datas Og_precedence.Ck_toplevel ppf

and print_stateflow_data ppf =
  printer_stateflow_data Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_states ppf =
  printer_stateflow_states Og_precedence.Ck_toplevel ppf

and print_stateflow_state ppf =
  printer_stateflow_state Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_targets ppf =
  printer_stateflow_targets Og_precedence.Ck_toplevel ppf

and print_stateflow_target ppf =
  printer_stateflow_target Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_charts ppf =
  printer_stateflow_charts Og_precedence.Ck_toplevel ppf

and print_stateflow_chart ppf =
  printer_stateflow_chart Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_machine ppf =
  printer_stateflow_machine Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_instance ppf =
  printer_stateflow_instance Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_section_items ppf =
  printer_stateflow_section_items Og_precedence.Ck_toplevel ppf

and print_stateflow_section_item ppf =
  printer_stateflow_section_item Og_precedence.Ck_toplevel ppf
;;

let print_stateflow_section ppf =
  printer_stateflow_section Og_precedence.Ck_toplevel ppf
;;

let print_interface_block_parameters ppf =
  printer_interface_block_parameters Og_precedence.Ck_toplevel ppf

and print_interface_block_parameter ppf =
  printer_interface_block_parameter Og_precedence.Ck_toplevel ppf
;;

let print_interface_blocks ppf =
  printer_interface_blocks Og_precedence.Ck_toplevel ppf

and print_interface_block ppf =
  printer_interface_block Og_precedence.Ck_toplevel ppf
;;

let print_interface ppf = printer_interface Og_precedence.Ck_toplevel ppf
;;

let print_poly_implementation_section_items pp_stateflow ppf =
  printer_poly_implementation_section_items pp_stateflow
    Og_precedence.Ck_toplevel
    ppf

and print_poly_implementation_section_item pp_stateflow ppf =
  printer_poly_implementation_section_item pp_stateflow
    Og_precedence.Ck_toplevel
    ppf
;;

let print_poly_implementation pp_stateflow ppf =
  printer_poly_implementation pp_stateflow Og_precedence.Ck_toplevel ppf
;;

let print_poly_simulink_file_contents pp_stateflow ppf =
  printer_poly_simulink_file_contents pp_stateflow
    Og_precedence.Ck_toplevel
    ppf
;;

let print_poly_simulink_file pp_stateflow ppf =
  printer_poly_simulink_file pp_stateflow Og_precedence.Ck_toplevel ppf
;;

let print_simulink_file_contents ppf =
  printer_simulink_file_contents Og_precedence.Ck_toplevel ppf
;;

let print_simulink_file ppf =
  printer_simulink_file Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.5+dev0 generated code. *)
